/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "addmedium.h"
#include "addmedium.moc"
#include "ui_addmedium.h"

#include "mediamodel.h"
#include "medium.h"
#include "sqlconstraint.h"
#include <esqldatabase.h>
#include "editmedium.h"

#include <QStringList>
#include <QSortFilterProxyModel>
#include <QSqlQuery>

AddMedium::AddMedium ( QWidget* parent, Qt::WindowFlags flags )
	: EDialog ( parent, flags ), ui(new Ui::AddMedium)
{
	init();
}

AddMedium::AddMedium( MediaModel* mediaModel, QWidget* parent, Qt::WindowFlags flags)
	: EDialog(parent, flags), ui(new Ui::AddMedium)
{
	init(mediaModel);
}

AddMedium::AddMedium(Medium* parentMedium, QWidget* parent, Qt::WindowFlags flags)
	: EDialog(parent, flags), ui(new Ui::AddMedium)
{
	if(parentMedium)
		init(parentMedium->childs());
	else
		init();
	this->parentMedium = parentMedium;
}

AddMedium::~AddMedium()
{
	this->close();
	ui->mediumView->setModel(NULL);
	if(searchModel)
		delete searchModel;
}

Medium* AddMedium::selectedMedium()
{
	return searchModel->mediumFromIndex(mediumIndex);
}

void AddMedium::search()
{
	if(!searchModel)
		return;
	
	searchModel->clear();
	
	QList<SqlConstraint> constraints;
	if(!ui->titleEdit->text().isEmpty()){
		SqlConstraint constraint;
		constraint.setConstraint("title LIKE ?");
		constraint.addBindValue(QString("%%1%" ).arg ( ui->titleEdit->text()) );
		constraints << constraint;
	}
	
	QList<SearchPlugin*> searchPlugins;
	searchModel->load(constraints, searchPlugins, myUsersId);
}

void AddMedium::newMedium()
{
	if(!searchModel)
		return;
	
	Medium *medium = searchModel->newSpecialMedium();
	
	if ( !medium ) {
		emit error ( tr ( "Could not add medium" ) );
		return;
	}
	
	if(parentMedium){
		medium->setManufacturer(parentMedium->manufacturer(), false);
		medium->setManufacturer(parentMedium->manufacturerName(), false);
		medium->setGenre(parentMedium->genre(), false);
		medium->setGenre(parentMedium->genreName(), false);
	}

	EditMedium *widget = medium->editWidget();
	if ( !widget ) {
		emit error ( tr ( "Could no get edit wigdet" ) );
		return;
	}
	connectByPass ( widget );
	widget->setNewMode(searchModel, medium);

	if ( widget->exec() != QDialog::Accepted )
		emit error ( tr ( "Could not add medium!" ) );
	
	delete widget;
}

void AddMedium::addMedium()
{
// 	if(!targetModel){
// 		emit error(tr("No target model specified!"));
// 		return;
// 	}
	Medium *medium = searchModel->mediumFromIndex(mediumIndex);
	if(!medium){
		emit error(tr("Please select a medium!"));
		return;
	}
	
	targetModel->addMedium(medium->copyMedium());
	
	accept();
}

void AddMedium::indexChanged(QModelIndex index)
{
	mediumIndex = proxyModel->mapToSource(index);
}


void AddMedium::init(MediaModel *targetModel)
{
	parentMedium = NULL;
	myUsersId = 0;
	searchModel = NULL;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		query.prepare("SELECT id FROM meganizer_users WHERE userName=:userName");
		query.bindValue(":userName", db.userName());
		query.exec();
		if(query.first())
			myUsersId = query.record().value("id").toUInt();
		if(closeDB)
			db.close();
	}
	
	ui->setupUi(this);
	this->targetModel = targetModel;
	if(!targetModel)
		searchModel = new MediaModel(NULL, this);
	else
		searchModel = targetModel->createNewModel();;

	searchModel->connectByPass(this, searchModel);
	connect(searchModel, SIGNAL(finished()), ui->mediumView, SLOT(resizeRowsToContents()));
	
	proxyModel = new QSortFilterProxyModel(this);
	proxyModel->setSourceModel(searchModel);
	// apply changes off the original model to the proxy model
  proxyModel->setDynamicSortFilter ( true );
//   proxyModel->sort ( 0 );
//   proxyModel->setFilterKeyColumn ( 0 );
//   proxyModel->setFilterCaseSensitivity ( Qt::CaseInsensitive );
	ui->mediumView->setModel(proxyModel);
	ui->mediumView->setSortingEnabled(true);
	searchModel->configureView(ui->mediumView);
	
	connect(ui->searchButton, SIGNAL(clicked()), this, SLOT(search()));
	connect(ui->mediumView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
	connect(ui->newButton, SIGNAL(clicked()), this, SLOT(newMedium()));
	connect(ui->addButton, SIGNAL(clicked()), this, SLOT(addMedium()));
	connect(ui->cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	ui->newButton->setIcon ( "document-new", ":/icons/filenew.png" );
	
	setHelp("keyword:Add Medium", ui->helpButton);
	
	search();
	loadGeometry();
}
