/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ADDMEDIUM_H
#define ADDMEDIUM_H

#include "libmeganizer.h"
#include <edialog.h>

namespace Ui {
	class AddMedium;
};
class MediaModel;
class Medium;

#include <QModelIndex>

class QSortFilterProxyModel;

/**
	* @brief Allows the user to add a medium to another medium
	*
	* (means the selected medium is added to a given MediaModel)
	* @todo use the specific search widget of the given MediaModel
	*/
class LIBMEGANIZER_EXPORT AddMedium : public EDialog
{
	Q_OBJECT
	public:
		AddMedium(QWidget *parent = 0, Qt::WindowFlags flags = 0);
		AddMedium(MediaModel *mediaModel, QWidget * parent = NULL, Qt::WindowFlags flags = 0);
		/**
			* @brief Uses the Medium::childs() model as searchModel.
			* 
			* The advantage of this function related to AddMedium(MediaModel*, QWidget*, Qt::WindowFlags)
			* is, that the propably common properties of a newly added medium (with newMedium())
			* are set to it's parents values (like manufacturer and genre of the new medium are mostly
			* the same as the parent ones).
			*/
		AddMedium(Medium *parentMedium, QWidget *parent = NULL, Qt::WindowFlags flags = 0);
		~AddMedium();
		Medium * selectedMedium();
		
	private slots:
		/**
			* @todo Use search plugins to find media
			*/
		void search();
		void newMedium();
		void addMedium();
		void indexChanged(QModelIndex index);
		
	private:
		void init(MediaModel *targetModel = NULL);
		/**
			* the model (given at the constructor as mediaModel)
			* is the model where the selected medium should be added
			*/
		MediaModel *targetModel;
		/**
			* this model is used to let the user search and select media
			*/
		MediaModel *searchModel;
		QSortFilterProxyModel *proxyModel;
		QModelIndex mediumIndex;
		unsigned int myUsersId;
		Medium *parentMedium;
		Ui::AddMedium *ui;
};

#endif // ADDMEDIUM_H
