/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "connectmediasearchsettings.h"
#include "connectmediasearchsettings.moc"
#include "ui_connectmediasearchsettings.h"

#include <mediaplugin.h>
#include <mediamodel.h>
#include <medium.h>
#include <searchplugin.h>
#include <mpluginloader.h>

ConnectMediaSearchSettings::ConnectMediaSearchSettings(SearchPlugin* searchPlugin, QWidget* parent, Qt::WindowFlags flags)
	: ESettingsWidget(parent, flags), ui(new Ui::ConnectMediaSearchSettings)
{
	this->searchPlugin = searchPlugin;
	
	setName(tr("Connect Media Attributes"));
	ui->setupUi(this);
	connMed = qobject_cast<ConnectMediaSearch*>(ui->connectMediaSearch);
	connectByPass(connMed);
	
	QList<MediaPlugin*> plugins = MPluginLoader<MediaPlugin>::loadPlugins();
	QStringList types;
  foreach ( MediaPlugin *plugin, plugins ) {
		mediaPlugins.append(plugin);
		QStringList untransTypes = plugin->supportedMaterialTypes();
		foreach(QString type, untransTypes){
			QStringList result = plugin->translateMaterialType(type);
			if(result.size() == 2 && !result[0].isEmpty())
				types << result[0];
		}
  }
  
  types.sort();
	ui->typeEdit->addItems(types);
	
	connect(ui->typeEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(changeType()));
	changeType();
}

bool ConnectMediaSearchSettings::saveSettings(QString& error)
{
	if(!connMed)
		return false;

	return connMed->saveSettings(error);
}

void ConnectMediaSearchSettings::changeType()
{
	if(!searchPlugin)
		return;

	QString msg;
	connMed->saveSettings(msg);
	
	QString type;
	foreach(MediaPlugin *plugin, mediaPlugins){
		type = plugin->untranslateMaterialType(ui->typeEdit->currentText());
		if(!plugin->isSupportedMaterialType(type))
			continue;
		
		Medium *medium = plugin->newMedium(type);
		connMed->setPlugins(medium, searchPlugin);
	}
}
