/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "delegateuser.h"
#include "delegateuser.moc"

#include <esqldatabase.h>

#include <QComboBox>
#include <QSettings>
#include <QAbstractItemView>
#include <QSqlRecord>
#include <QStyleOptionViewItemV2>

DelegateUser::DelegateUser(QWidget *parent)
  : QStyledItemDelegate(parent)
{
  QSqlQueryModel *resultModel = new QSqlQueryModel();
  ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if(db.isOpen()){
    resultModel->setQuery("SELECT id, name, givenName FROM meganizer_users");
    if(closeDB)
      db.close();
  }
  for(int i = 0; i < resultModel->rowCount(); i++){
    mapInOut.insert
    (
      resultModel->record(i).value("id").toUInt(),
      QString("%1, %2.")
	.arg(resultModel->record(i).value("name").toString())
	.arg(resultModel->record(i).value("givenName").toString().left(1))
    );
  }
  delete resultModel;
}

void DelegateUser::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const{
  if(!index.data().canConvert(QVariant::String))
    // all other than booleans should be painted by the standard delegates
    return QStyledItemDelegate::paint(painter, option, index);
  
  unsigned int data = index.data().toUInt();
 
	QBrush brush = option.palette.base();
	QVariant modelBrush = index.data(Qt::BackgroundRole);
  if (modelBrush.canConvert<QBrush>()){
      brush = modelBrush.value<QBrush>();
	}
	else{
		const QStyleOptionViewItemV2 *style2 = qstyleoption_cast<const QStyleOptionViewItemV2*>(&option);
		if(style2)
			if(style2->features.testFlag(QStyleOptionViewItemV2::Alternate))
				brush = style2->palette.alternateBase();
	}
	QColor foregroundColor = option.palette.text().color();
	QVariant foregroundBrush = index.data(Qt::ForegroundRole);
	if (foregroundBrush.canConvert<QBrush>())
      foregroundColor = foregroundBrush.value<QBrush>().color();
	
	painter->save();
	painter->setFont(option.font);
	painter->setPen(foregroundColor);
	painter->fillRect(option.rect, brush);
	if(option.state & QStyle::State_Selected){
		// its selected
		painter->fillRect(option.rect, option.palette.highlight());
		painter->setPen(option.palette.highlightedText().color());
	}

  QTextOption textOptions;
  textOptions.setAlignment(Qt::AlignCenter);
  painter->drawText(option.rect, mapInOut.value(data, QString()), textOptions);
	painter->restore();
}

QWidget *DelegateUser::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const{
  // called from model if the user want to edit a field
  if(!index.data().canConvert(QVariant::UInt))
    return QStyledItemDelegate::createEditor(parent, option, index);
  
  QComboBox *editor = new QComboBox(parent);
  editor->addItems(mapInOut.values());
  connect(editor, SIGNAL(currentIndexChanged(int)), this, SLOT(commitAndCloseEditor()));
  return editor;
}

void DelegateUser::commitAndCloseEditor(){
  // the user has changed data
  // so the editor can be closed (closeEditor()) and
  // the model is informed, that new data is present
  QComboBox *editor = qobject_cast<QComboBox *>(sender());
  if(editor){
    emit commitData(editor);
    emit closeEditor(editor);
  }
}

void DelegateUser::setEditorData(QWidget *editor, const QModelIndex &index) const{
  // used to initialize the data of the editor
  // this is mainly used to get the editor showing the current data of the model
  unsigned int data = index.data().toUInt();
  QComboBox *box = qobject_cast<QComboBox *>(editor);
  if(!box)
    return QStyledItemDelegate::setEditorData(editor, index);
    
  if(!box->findText(mapInOut.value(data)))
    return QStyledItemDelegate::setEditorData(editor, index);
  
  box->setCurrentIndex(box->findText(mapInOut.value(data)));
}

void DelegateUser::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const{
  QComboBox *box = qobject_cast<QComboBox *>(editor);
  if(!box)
    return QStyledItemDelegate::setModelData(editor, model, index);
    
  model->setData(index, mapInOut.key(box->currentText()));
}
