/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef EDITDIALOG_H
#define EDITDIALOG_H

#include <edialog.h>
#include "libmeganizer.h"

class DetailsWidget;
class EPushButton;

#include <QModelIndex>

class QSortFilterProxyModel;
class QSplitter;
class QLabel;
class QLineEdit;
class QAbstractItemView;
class QAbstractItemModel;

class LIBMEGANIZER_EXPORT EditDialog : public EDialog
{
	Q_OBJECT
	public:
		EditDialog ( QWidget *parent = 0, Qt::WindowFlags flags = 0 );
		virtual inline ~EditDialog(){ };
		/**
			* default is the Select mode, where a item must be selected before the
			* ok button can be clicked
			* in edit mode, no selection is needed
			*/
		void setEditMode();

  protected slots:
    virtual void addNew();
    virtual void editSelected();
    virtual void store() = 0;
    void finish();
    void cancel();
		/**
			* @param index QModelIndex of the ProxyModel, not the source model.
			* @param index This function translate it to the source model
			*/
    void selectionChanged ( QModelIndex index );
    void filter ( QString string );
    void closeEvent ( QCloseEvent* event );

  protected:
    /**
    	* set's up the UI
    	* @attention set a objectName before calling this method
    	* @attention the existingView->setModelColumn(), must be called at the class, which inherits from EditDialog, if needed
    	* @attention in general all other cosmetic stuff must be done also at the subclassed class (e.g. hide model columns in a QTableView)
    	*/
    void setupUi ( QAbstractItemView* existingView, DetailsWidget* detailsWidget, QString filterString = QString() );
		/**
			* tries to select the entered item
			*/
		void postStore(QString filterString);
    QSortFilterProxyModel *proxyModel;
    QAbstractItemModel *model;
    EPushButton *helpButton, *okButton, *cancelButton;
		EPushButton *newButton, *editButton, *saveButton;
    QLabel *existingLabel, *detailsLabel, *filterLabel;
    QSplitter *splitter;
    QLineEdit *filterEdit;
    /**
    	* The text which is displayed, if the finish process has failed
    	* @attention need's to be set by the class, which inherits from EditDialog
    	*/
    QString finishFailedText;
    QString existingLabelText;
		/**
			* index of the original model, not the ProxyModel
			*/
    QModelIndex curIndex;
    /**
    	* The number of the column, which should be used to search for filter String
    	* @attention must be set by the class, which inherits from EditDialog
    	*/
    int sortingColumn;
    QAbstractItemView *existingView;

  private:
    DetailsWidget *detailsWidget;
		bool editMode;
};

#endif // EDITDIALOG_H
