/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editmanufacturers.h"
#include "editmanufacturers.moc"

#include "editmanufacturersdetailswidget.h"
#include <epushbutton.h>
#include <esqldatabase.h>

#include <QSqlQueryModel>
#include <QSortFilterProxyModel>
#include <QListView>
#include <QSqlRecord>

EditManufacturers::EditManufacturers ( QString name, QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  init(name);
}

EditManufacturers::EditManufacturers ( QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  init();
}

EditManufacturers::~EditManufacturers()
{
  cancel();
	model = 0;
  delete sqlModel;
}

QString EditManufacturers::selectedName()
{
	if(!curIndex.isValid())
		return QString();
	
	if(!sqlModel)
		return QString();
	
	return sqlModel->record(curIndex.row()).value("name").toString();
}

unsigned int EditManufacturers::selectedId()
{
	if(!curIndex.isValid())
		return 0;
	
	if(!sqlModel)
		return 0;
	
	return sqlModel->record(curIndex.row()).value("id").toUInt();
}

void EditManufacturers::store()
{
	if(!myDetailsWidget)
		return;
	
	if(!myDetailsWidget->store())
		return;
	
  postStore(myDetailsWidget->name());
}

void EditManufacturers::init(QString selectName)
{
  sqlModel = 0;
	
	setObjectName("editManufacturers");
	setWindowTitle(tr("Add / Edit Manufacturers"));
	existingLabelText = tr("Existing Manufacturers");
	
	sqlModel = new QSqlQueryModel ( this );
	model = sqlModel;
	myDetailsWidget = new EditManufacturersDetailsWidget(sqlModel, this);
	QListView *listView = new QListView(this);
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		db.open();
	}
  if ( db.isOpen() ) {
    sqlModel->setQuery ( "SELECT * FROM meganizer_manufacturers" );
		
		if(closeDB)
			db.close();
  }
  
  sortingColumn = sqlModel->record().indexOf("name");
  setupUi(listView, myDetailsWidget, selectName);
  
  listView->setModelColumn ( sortingColumn );
  
  finishFailedText = tr("Please select a manufacturer or press %1 to abort").arg(cancelButton->text());
}
