/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "editmaterialpersons.h"
#include "editmaterialpersons.moc"

#include "materialperson.h"
#include "editmaterialpersonsdetailswidget.h"
#include <epushbutton.h>
#include <esqldatabase.h>
#include <etableview.h>

#include <QSqlQueryModel>
#include <QSqlQuery>
#include <QSqlError>
#include <QSortFilterProxyModel>
#include <QSqlRecord>

EditMaterialPersons::EditMaterialPersons ( QString name, QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  person = 0;
  init ( name );
}

EditMaterialPersons::EditMaterialPersons ( MaterialPerson *person, QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  if ( person ) {
    init ( person->name() );
  } else {
    init();
  }

  this->person = person;
}

EditMaterialPersons::EditMaterialPersons ( QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  person = 0;
  init();
}

EditMaterialPersons::~EditMaterialPersons()
{
  close();
  model = 0;
  delete sqlModel;
}

QString EditMaterialPersons::selectedName()
{
  if ( !curIndex.isValid() )
    return QString();

  if ( !sqlModel )
    return QString();

  return sqlModel->record ( curIndex.row() ).value ( "name" ).toString();
}

QString EditMaterialPersons::selectedGivenName()
{
  if ( !curIndex.isValid() )
    return QString();

  if ( !sqlModel )
    return QString();

  return sqlModel->record ( curIndex.row() ).value ( "givenName" ).toString();
}

unsigned int EditMaterialPersons::selectedId()
{
  if ( !curIndex.isValid() )
    return 0;

  if ( !sqlModel )
    return 0;

  return sqlModel->record ( curIndex.row() ).value ( "id" ).toUInt();
}

void EditMaterialPersons::addNew()
{
  // make curIndex invalid, since the new genres isn't at the model
  selectionChanged ( QModelIndex() );

  if(person){
    myDetailsWidget->setEnabled ( true, person );
  }
  else{
    myDetailsWidget->setEnabled( true, filterEdit->text());
  }
  saveButton->setEnabled ( true );
}

void EditMaterialPersons::store()
{
  if ( !myDetailsWidget )
    return;

  QString name = myDetailsWidget->name();

  QString givenName = myDetailsWidget->givenName();

  if ( name.isEmpty() ) {
    emit error ( tr ( "An empty name is not allowed!" ) );
    return;
  }

	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();

  if ( closeDB ) {
    if ( !db.open() ) {
      emit error ( tr ( "Could not connect to database" ) );
      return;
    }
  }

  QSqlQuery query;

  if ( curIndex.isValid() ) {
    // a present person was edited --> we must do a update
    query.prepare ( "UPDATE meganizer_materialpersons SET name=:name, givenName=:givenName WHERE id=:id" );
    query.bindValue ( ":id", sqlModel->record ( curIndex.row() ).value ( "id" ).toUInt() );
  } else {
    query.prepare ( "INSERT INTO meganizer_materialpersons (name, givenName) VALUES(:name, :givenName)" );
  }

  query.bindValue ( ":name", name );

  query.bindValue ( ":givenName", givenName );

  if ( !query.exec() ) {
    emit error ( tr ( "Could not create or update material person %1\nQuery: %2\nError: %3" ).arg ( name ).arg ( query.lastQuery() ).arg ( query.lastError().text() ) );

    if ( closeDB )
      db.close();

    return;
  }

  // reload model
  if ( sqlModel ) {
    sqlModel->setQuery ( sqlModel->query().lastQuery() );
  }

  if ( closeDB )
    db.close();
	
	postStore(name);
}

void EditMaterialPersons::init ( QString selectName )
{
  sqlModel = 0;
  person = 0;

  setObjectName ( "editMaterialPersons" );
  setWindowTitle ( "Add / Edit Material Persons" );
  existingLabelText = tr ( "Existing Persons:" );

  sqlModel = new QSqlQueryModel ( this );
  model = sqlModel;
  myDetailsWidget = new EditMaterialPersonsDetailsWidget ( sqlModel, this );
  ETableView *tableView = new ETableView ( this );

	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();

  if ( closeDB )
    db.open();

  if ( db.isOpen() ) {
    sqlModel->setQuery ( "SELECT * FROM meganizer_materialpersons" );

    if ( closeDB )
      db.close();
  }

  sortingColumn = sqlModel->record().indexOf ( "name" );

  setupUi ( tableView, myDetailsWidget, selectName );

  finishFailedText = tr ( "Please select a material person or press %1 to abort" ).arg ( cancelButton->text() );

  tableView->hideColumn ( sqlModel->record().indexOf ( "id" ) );
  sqlModel->setHeaderData ( sqlModel->record().indexOf ( "name" ), Qt::Horizontal, tr ( "Name" ) );
  sqlModel->setHeaderData ( sqlModel->record().indexOf ( "givenName" ), Qt::Horizontal, tr ( "Given Name" ) );
}
