/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef EDITMEDIUM_H
#define EDITMEDIUM_H

#include "libmeganizer.h"
#include <edialog.h>

class Medium;
class MediaModel;
class MediumEditWidget;

namespace Ui {
	class EditMedium;
};

/**
	* @brief The edit dialog for all media.
	*
	* To customize it, you can remove widgets / tabs and add widgets / tabs.
	*
	* @todo implement mediumChange and needSave functionality
	* @todo but think about this: The Medium*Widget's must emit if sum data is changed and not submitted
	*/
class LIBMEGANIZER_EXPORT EditMedium : public EDialog
{
	Q_OBJECT
	public:
		enum EditMode { Edit, Add, New };
		EditMedium(QWidget *parent = 0, Qt::WindowFlags flags = 0);
		/**
			* if called, the window title gets changed to "Add Medium to Database"
			*/
		void setAddMode(Medium *medium, bool newTransaction = true);
		/**
			* if called, the window title gets changed to "Edit Medium"
			* this is the standard
			*/
		void setEditMode(Medium *medium, bool newTransaction = true);
		void setNewMode(MediaModel *model, Medium *medium, bool newTransaction = true);

	public slots:
		void save();
		
	protected:
		/**
			* @brief Sets the text of the MediumManufacturerwidget.
			*
			*/
		void setManufacturerText(QString text);
		/**
			* @brief Sets the default role of the MediumPersonsWidget.
			*
			* @param name The name of the role, which should be set as default role.
			*     A ":" is automatically appended, if the last character is not a ":".
			*/
		void setPersonsDefaultRole(QString name);
		/**
			* @brief Use this function to add new widgets.
			*
			* It tries to insert @p newWidget into the layout of @p intoWidget after @p afterWidget
			* If @p afterWidget = 0, the new Widget is appended at the end.
			*
			* To add a new tab to a tab widget, use addTab().
			*
			* @todo insertItem seems to become public function in Qt5, review code than!
			* @todo change taborder 
			*/
		void insertWidget(MediumEditWidget *newWidget, QString intoWidget, QString afterWidget = QString());
		/**
			* @brief Adds a new tab with @p newWidget and @p title at @p tabWidget.
			*
			* To add a widget to an existing tab use insertWidget().
			* To remove a tab use removeTab().
			*/
		void addTab(MediumEditWidget *newWidget, QString title, QString tabWidget);
		/**
			* @brief Tries to remove @p tabbedWidget from @p tabWidget.
			*
			* To add a tab use addTab().
			*/
		void removeTab(QString tabWidget, QString tabbedWidget);
		QWidget *tabOrder(QWidget* lastWidget, QLayout* childlayout);
		void closeEvent ( QCloseEvent * event );

	private slots:
		void cancel();

	private:
		void cancel(bool noCancelButton);
		/**
			* this function loads the data of medium into the standard widgets
			* afterwards it calls loadData to allow further custom loading
			*/
		void setMedium(Medium* medium, bool newTransaction);

		Ui::EditMedium *ui;
		Medium *medium;
		MediaModel *model;
		EditMode editMode;
		bool startTransaction;
		QList<MediumEditWidget*> editWidgets;
};

#endif // EDITMEDIUM_H
