/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editroles.h"
#include "editroles.moc"

#include "detailswidgetnamedesc.h"
#include <epushbutton.h>
#include <esqldatabase.h>

#include <QSqlQueryModel>
#include <QSqlRecord>
#include <QSqlQuery>
#include <QSqlError>
#include <QSortFilterProxyModel>
#include <QListView>

EditRoles::EditRoles ( QString name, QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  init(name);

}

EditRoles::EditRoles ( QWidget* parent, Qt::WindowFlags flags )
    : EditDialog ( parent, flags )
{
  init();
}

EditRoles::~EditRoles()
{
  close();
	model = 0;
  delete sqlModel;
	delete proxyModel;
}

QString EditRoles::selectedName()
{
	if(!curIndex.isValid())
		return QString();
	
	if(!sqlModel)
		return QString();
	
	return sqlModel->record(curIndex.row()).value("name").toString();
}

unsigned int EditRoles::selectedId()
{
	if(!curIndex.isValid())
		return 0;
	
	if(!sqlModel)
		return 0;
	
	return sqlModel->record(curIndex.row()).value("id").toUInt();
}

void EditRoles::store()
{
	if(!myDetailsWidget)
		return;
	
	if ( myDetailsWidget->name().isEmpty() ) {
    emit error ( tr ( "An empty name is not allowed!" ) );
    return;
  }

	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if ( !db.open() ) {
			emit error ( tr ( "Could not connect to database" ) );
			return;
		}
	}

	QSqlQuery query;
	if(curIndex.isValid()){
		// a present manufacturer was edited --> we must do a update
		query.prepare("UPDATE meganizer_materialpersonsroles SET name=:name, description=:description WHERE id=:id");
		query.bindValue(":id", sqlModel->record(curIndex.row()).value("id").toUInt());
	}
  else{
		query.prepare("INSERT INTO meganizer_materialpersonsroles (name, description) VALUES(:name, :description)");
	}
	query.bindValue(":name", myDetailsWidget->name());
	query.bindValue(":description", myDetailsWidget->description());
	if ( !query.exec()){
	emit error ( tr ( "Could not create or update role %1\nQuery: %2\nError: %3" ).arg ( myDetailsWidget->name() ).arg(query.lastQuery()).arg(query.lastError().text()) );
	if(closeDB)
		db.close();
	return;
}
  
  // reload model
  if ( sqlModel ) {
    sqlModel->setQuery ( sqlModel->query().lastQuery() );
  }
  if(closeDB)
		db.close();
	
	// select the entered genres
	postStore(myDetailsWidget->name());
}

void EditRoles::init(QString selectName)
{
  sqlModel = 0;
	proxyModel = 0;

	setObjectName("editRoles");
	setWindowTitle(tr("Add / Edit Roles"));
	
	sqlModel = new QSqlQueryModel(this);
	model = sqlModel;
	myDetailsWidget = new DetailsWidgetNameDesc(this);
	QListView *listView = new QListView(this);

  ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if ( db.isOpen() ) {
    sqlModel->setQuery ( "SELECT * FROM meganizer_materialpersonsroles" );

		if(closeDB)
			db.close();
  }
  
  sortingColumn = sqlModel->record().indexOf("name");
	myDetailsWidget->setup(sqlModel, sortingColumn, sqlModel->record().indexOf("description"));
  
	setupUi(listView, myDetailsWidget, selectName);
  
	finishFailedText = tr("Please select a role or press %1 to abort").arg(cancelButton->text());
	
	listView->setModelColumn(sortingColumn);
}
