/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "edituser.h"
#include "edituser.moc"

#include "edituserdetailswidget.h"
#include <epushbutton.h>
#include <esqldatabase.h>
#include <etableview.h>

#include <QSqlQueryModel>
#include <QSqlRecord>
#include <QSqlQuery>
#include <QSqlError>
#include <QSortFilterProxyModel>
#include <QLineEdit>

EditUser::EditUser ( QString name, QWidget* parent, Qt::WindowFlags flags )
	: EditDialog(parent, flags)
{
	init(name);
}

EditUser::EditUser ( QWidget* parent, Qt::WindowFlags flags )
	: EditDialog(parent, flags)
{
	init();
}

EditUser::~EditUser()
{
  close();
  model = 0;
  delete sqlModel;
}

unsigned int EditUser::selectedId()
{
  if ( !curIndex.isValid() )
    return 0;

  if ( !sqlModel )
    return 0;

  return sqlModel->record ( curIndex.row() ).value ( "id" ).toUInt();
}

QString EditUser::selectedName()
{
  if ( !curIndex.isValid() )
    return QString();

  if ( !sqlModel )
    return QString();

  return sqlModel->record ( curIndex.row() ).value ( "name" ).toString();
}

QString EditUser::selectedGivenName()
{
  if ( !curIndex.isValid() )
    return QString();

  if ( !sqlModel )
    return QString();

  return sqlModel->record ( curIndex.row() ).value ( "givenName" ).toString();
}

void EditUser::store()
{
	if(!myDetailsWidget->store())
		return;
	
	QString name = myDetailsWidget->name();
	
	// reload model
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();

  if ( closeDB ) {
    if ( !db.open() ) {
      emit error ( tr ( "Could not connect to database" ) );
      return;
    }
  }
  if ( sqlModel ) {
    sqlModel->setQuery ( sqlModel->query().lastQuery() );
  }

  if ( closeDB )
    db.close();

  // select the entered genres
	postStore(name);
}

void EditUser::addNew()
{
	// make curIndex invalid, since the new genres isn't at the model
  selectionChanged ( QModelIndex() );

	myDetailsWidget->setEnabled( true, filterEdit->text());
  saveButton->setEnabled ( true );
}

void EditUser::init ( QString selectName )
{
  sqlModel = 0;

  setObjectName ( "editUser" );
  setWindowTitle ( "Add / Edit User" );
  existingLabelText = tr ( "Existing User:" );

  sqlModel = new QSqlQueryModel ( this );
  model = sqlModel;
  myDetailsWidget = new EditUserDetailsWidget ( this );
	myDetailsWidget->setModel(sqlModel);
	ETableView *tableView = new ETableView ( this );

  ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();

  if ( closeDB )
    db.open();

  if ( db.isOpen() ) {
    sqlModel->setQuery ( "SELECT * FROM meganizer_users" );

    if ( closeDB )
      db.close();
  }

  sortingColumn = sqlModel->record().indexOf ( "name" );

  setupUi ( tableView, myDetailsWidget, selectName );

  finishFailedText = tr ( "Please select a user or press %1 to abort" ).arg ( cancelButton->text() );

  tableView->hideColumn ( sqlModel->record().indexOf ( "id" ) );
	tableView->hideColumn ( sqlModel->record().indexOf ( "userName" ) );
	tableView->hideColumn ( sqlModel->record().indexOf ( "email" ) );
  sqlModel->setHeaderData ( sqlModel->record().indexOf ( "name" ), Qt::Horizontal, tr ( "Name" ) );
  sqlModel->setHeaderData ( sqlModel->record().indexOf ( "givenName" ), Qt::Horizontal, tr ( "Given Name" ) );
}
