/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "edituserdetailswidget.h"
#include "edituserdetailswidget.moc"
#include "ui_edituserdetailswidget.h"

#include <esqldatabase.h>

#include <QSqlQueryModel>
#include <QSqlRecord>
#include <QSqlQuery>
#include <QSqlError>

EditUserDetailsWidget::EditUserDetailsWidget ( QWidget* parent, Qt::WindowFlags flags )
	: DetailsWidget ( parent, flags ), ui(new Ui::EditUserDetailsWidget)
{
	model = NULL;
	userId = 0;
	userNameNeeded = false;
	ui->setupUi(this);
}

void EditUserDetailsWidget::setModel ( QSqlQueryModel* model )
{
	this->model = model;
}

void EditUserDetailsWidget::clear()
{
	userId = 0;
	ui->nameEdit->setText ( "" );
	ui->givenNameEdit->setText ( "" );
	ui->userNameEdit->setText ( "" );
	ui->userNameEdit->setReadOnly(false);
	ui->emailEdit->setText ( "" );
}

void EditUserDetailsWidget::setItem ( QModelIndex index )
{
	if(!model)
		return;
	
	QSqlRecord record = model->record(index.row());
	userId = record.value("id").toUInt();
	setName(record.value("name").toString());
	setGivenName(record.value("givenName").toString());
	setUserName(record.value("userName").toString());
	setEmail(record.value("email").toString());
}

void EditUserDetailsWidget::setEnabled ( bool show, QString defaultString )
{
	clear();
	setName(defaultString);
	EWidget::setEnabled(show);
	ui->nameEdit->setFocus();
}

bool EditUserDetailsWidget::store()
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if ( !db.isOpen() ) {
    emit error ( tr ( "Can not connect to database" ) );
    return false;
  }

	if((userName().isEmpty()) && (userNameNeeded)){
		emit error(tr("No username given!"));
		if(closeDB)
			db.close();
		return false;
	}

	if(name().isEmpty()){
		emit error(tr("Please enter your name"));
		if(closeDB)
			db.close();
		return false;
	}
	
	QSqlQuery query;
	if( userId != 0 ){
		query.prepare("UPDATE meganizer_users SET name=:name, givenName=:givenName, email=:email WHERE id=:id");
		query.bindValue(":id", userId);
	}
	else{
		query.prepare("INSERT INTO meganizer_users (name, givenName, userName, email) VALUES(:name, :givenName, :userName, :email)");
		query.bindValue(":userName", userName());
	}
	
	query.bindValue(":name", name());
	query.bindValue(":givenName", givenName());
	query.bindValue(":email", email());
   
  bool result = query.exec();
  if ( !result )
    emit error ( tr ( "Creation or update of user failed!\n%1" ).arg ( query.lastError().text() ) );
  if(closeDB)
			db.close();
  return result;
}

void EditUserDetailsWidget::setId ( unsigned int newId )
{
	userId = newId;
}

unsigned int EditUserDetailsWidget::id()
{
	return userId;
}

void EditUserDetailsWidget::setName ( QString name )
{
	ui->nameEdit->setText(name);
}

QString EditUserDetailsWidget::name()
{
	return ui->nameEdit->text();
}

void EditUserDetailsWidget::setGivenName ( QString name )
{
	ui->givenNameEdit->setText(name);
}

QString EditUserDetailsWidget::givenName()
{
	return ui->givenNameEdit->text();
}

void EditUserDetailsWidget::setUserName ( QString name )
{
	ui->userNameEdit->setText(name);
	// allow only the editing of the own username
	if(name != ESqlDatabase::database().userName())
		setUserNameReadOnly(true);
}

QString EditUserDetailsWidget::userName()
{
	return ui->userNameEdit->text();
}

void EditUserDetailsWidget::setUserNameReadOnly ( bool enable )
{
	ui->userNameEdit->setReadOnly(enable);
}

void EditUserDetailsWidget::setUserNameNeeded()
{
	userNameNeeded = true;
}

void EditUserDetailsWidget::setEmail ( QString mail )
{
	ui->emailEdit->setText(mail);
}

QString EditUserDetailsWidget::email()
{
	return ui->emailEdit->text();
}
