/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef CONNECTMEDIASEARCH_H
#define CONNECTMEDIASEARCH_H

#include "esettingswidget.h"

#include "medium.h"

namespace Ui {
	class ConnectMediaSearch;
};
class SearchPlugin;

#include <QModelIndex>
#include <QMap>

class QListWidgetItem;

/**
	* @brief Let the user connect the Attributes of the Medium and the SearchPlugin
	*
	*/
class ConnectMediaSearch : public ESettingsWidget
{
  Q_OBJECT
  public:
    ConnectMediaSearch(QWidget *parent = 0, Qt::WindowFlags flags = 0);
		~ConnectMediaSearch();
    void setPlugins(Medium *medium, SearchPlugin *searchPlugin);
    bool checkSettings();
		
  public slots:
    bool saveSettings(QString &error);
    
  private slots:
		/**
			* @brief Connects the currently selected attributes.
			*
			* It uses mediaIndex and searchIndex to find the
			* selected QListWidgetItems and calls
			* assign(QListWidgetItem*, QListWidgetItem*) afterwards.
			*/
    void assign();
		/**
			* @brief Connects the untranslated @p mediaAttr with the @p searchAttr.
			*
			* The first parameter @p mediaAttr should be the key from
			* a ColKeyTrans structure.
			* It tries to find the corresponding QListWidgetItems and calls
			* assign(QListWidgetItem*, QListWidgetItem*) afterwards.
			*/
		void assign(QString mediaAttr, QString searchAttr);
		/**
			* @brief Connects the @p mediaItem with @p searchItem.
			*
			* @note @p mediaItem and @p searchItem are removed from its view
			*    and deleted afterwards.
			*/
		void assign(QListWidgetItem *mediaItem, QListWidgetItem *searchItem);
    void remove();
    void indexChanged(QModelIndex index);
    
  private:
		void loadSettings();
		
    QModelIndex mediaIndex, searchIndex, connectionIndex;
    QString mediaGroup, mediaName, searchGroup;
		int numMediaAttrs, numSearchAttrs;
		QStringList mediaAttrs, searchAttrs;
		Medium *medium;
		SearchPlugin *searchPlugin;
		bool canceled, saved, deleteMedium;
		int numMediaModelAttrs, numSearchPluginAttrs;
		QMap<QString, QString> searchMap;
		QList<ColKeyTrans> mediaMap;
		Ui::ConnectMediaSearch *ui;
};

#endif // CONNECTMEDIASEARCH_H
