/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "delegateloanstati.h"
#include "delegateloanstati.moc"

#include <esqldatabase.h>
#include "mediumloanwidget.h"
#include "loanitem.h"

#include <QTimer>
#include <QComboBox>
#include <QSettings>
#include <QAbstractItemView>
#include <QSqlRecord>
#include <QStyleOptionViewItemV2>
#include <QSqlQueryModel>
#include <QSqlQuery>

DelegateLoanStati::DelegateLoanStati(QWidget *parent)
  : QStyledItemDelegate(parent)
{
	timer = new QTimer(this);

	resultModel = new QSqlQueryModel();
	
  ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if(db.isOpen()){
    resultModel->setQuery(QString("SELECT id, name FROM meganizer_loanstati ORDER BY name"));
    if(closeDB)
      db.close();
  }
  for(int i = 0; i < resultModel->rowCount(); i++){
		QString name = resultModel->record(i).value("name").toString();
		QStringList translation = MediumLoanWidget::statiMap(name);
		if(translation.size() != 3)
			translation << name << name << name;
		mapOutIn.insert
		(
			translation[0],
			resultModel->record(i).value("id").toUInt()
		);
	}
}

void DelegateLoanStati::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const{
  if(!index.data().canConvert(QVariant::String))
    // all other than booleans should be painted by the standard delegates
    return QStyledItemDelegate::paint(painter, option, index);
  
  unsigned int data = index.data().toUInt();

	QBrush brush = option.palette.base();
	QVariant modelBrush = index.data(Qt::BackgroundRole);
  if (modelBrush.canConvert<QBrush>()){
      brush = modelBrush.value<QBrush>();
	}
	else{
		const QStyleOptionViewItemV2 *style2 = qstyleoption_cast<const QStyleOptionViewItemV2*>(&option);
		if(style2)
			if(style2->features.testFlag(QStyleOptionViewItemV2::Alternate))
				brush = style2->palette.alternateBase();
	}
	QColor foregroundColor = option.palette.text().color();
	QVariant foregroundBrush = index.data(Qt::ForegroundRole);
	if (foregroundBrush.canConvert<QBrush>())
      foregroundColor = foregroundBrush.value<QBrush>().color();

	painter->save();
	painter->setFont(option.font);
	painter->setPen(foregroundColor);
	painter->fillRect(option.rect, brush);
	if(option.state & QStyle::State_Selected){
		// its selected
		painter->fillRect(option.rect, option.palette.highlight());
		painter->setPen(option.palette.highlightedText().color());
	}
	
  QTextOption textOptions;
  textOptions.setAlignment(Qt::AlignCenter);
  painter->drawText(option.rect, mapOutIn.key(data, QString()), textOptions);
	painter->restore();
}

QWidget *DelegateLoanStati::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const{
	Q_UNUSED(option);
	if(!index.isValid())
		return NULL;
	
	// called from model if the user want to edit a field
	QComboBox *editor = new QComboBox(parent);
	editor->addItems(mapOutIn.uniqueKeys());

	// remove not valid stati (except the currrent state).
	// The current state is needed, if the user decide, to not
	// change the state at all and to display the current state
	// in editing mode
	LoanItem *item = static_cast<LoanItem*>(index.internalPointer());
	QList<unsigned int> validStatis = item->validStatis();
	foreach(unsigned int status, mapOutIn.values()){
		if((status != item->status()) && !validStatis.contains(status))
			editor->removeItem(editor->findText(mapOutIn.key(status)));
	}

	connect(editor, SIGNAL(currentIndexChanged(int)), this, SLOT(commitAndCloseEditor()));
	return editor;
}

void DelegateLoanStati::setEditorData(QWidget *editor, const QModelIndex &index) const{
  // used to initialize the data of the editor
  // this is mainly used to get the editor showing the current data of the model
  unsigned int data = index.data().toUInt();
  if(mapOutIn.key(data, QString()).isEmpty())
    return QStyledItemDelegate::setEditorData(editor, index);
  
  QComboBox *box = qobject_cast<QComboBox *>(editor);
	if(box){    
		if(!box->findText(mapOutIn.key(data)))
			return QStyledItemDelegate::setEditorData(editor, index);
		box->setCurrentIndex(box->findText(mapOutIn.key(data)));
	}
}

void DelegateLoanStati::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const{
  if(!index.data().canConvert(QVariant::String))
    return QStyledItemDelegate::setModelData(editor, model, index);
  
  QComboBox *box = qobject_cast<QComboBox *>(editor);
	if(box){
		model->setData(index, mapOutIn.value(box->currentText()));
	}
}

void DelegateLoanStati::commitAndCloseEditor(){
  // the user has changed data
  // so the editor can be closed (closeEditor()) and
  // the model is informed, that new data is present
  QComboBox *editor = qobject_cast<QComboBox *>(sender());
  if(editor){
		emit commitData(editor);
//     emit closeEditor(editor);
		timer->singleShot(1000, this, SLOT(doCloseEditor()));
  }
}

void DelegateLoanStati::doCloseEditor()
{
	QComboBox *editor = qobject_cast<QComboBox *>(sender());
  if(editor)
		emit closeEditor(editor);
}
