/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "delegatematerialtypes.h"
#include "delegatematerialtypes.moc"

#include "mpluginloader.h"
#include "mediaplugin.h"
#include <esqldatabase.h>
#include <QSqlQuery>
#include <QSqlRecord>

#include <QTextOption>
#include <QPainter>

DelegateMaterialTypes::DelegateMaterialTypes(QWidget* parent): QStyledItemDelegate(parent)
{
	QMap<QString, unsigned int> mapTypeId;
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if(db.isOpen()){
		QSqlQuery query;
		query.exec("SELECT id, name FROM meganizer_materialtypes ORDER BY name");
		while(query.next()){
			mapTypeId.insert(
				query.record().value("name").toString(),
				query.record().value("id").toUInt()
			);
		}
    if(closeDB)
      db.close();
  }

	if(mapTypeId.size() > 0){
		QList<MediaPlugin*> plugins = MPluginLoader<MediaPlugin>::loadPlugins();
		foreach(MediaPlugin *plugin, plugins){
			foreach(QString type, plugin->supportedMaterialTypes()){
				QStringList trans = plugin->translateMaterialType(type);
				if(trans.size() == 2)
					mapTransToTypes.insert(trans[0], mapTypeId.value(type));
			}
		}
	}
}

void DelegateMaterialTypes::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
	if(!index.data().canConvert(QVariant::String))
		// all other than booleans should be painted by the standard delegates
		return QStyledItemDelegate::paint(painter, option, index);

	unsigned int data = index.data().toUInt();

	QBrush brush = option.palette.base();
	QVariant modelBrush = index.data(Qt::BackgroundRole);
	if (modelBrush.canConvert<QBrush>()){
			brush = modelBrush.value<QBrush>();
	}
	else{
		const QStyleOptionViewItemV2 *style2 = qstyleoption_cast<const QStyleOptionViewItemV2*>(&option);
		if(style2)
			if(style2->features.testFlag(QStyleOptionViewItemV2::Alternate))
				brush = style2->palette.alternateBase();
	}
	QColor foregroundColor = option.palette.text().color();
	QVariant foregroundBrush = index.data(Qt::ForegroundRole);
	if (foregroundBrush.canConvert<QBrush>())
      foregroundColor = foregroundBrush.value<QBrush>().color();

	painter->save();
	painter->setFont(option.font);
	painter->setPen(foregroundColor);
	painter->fillRect(option.rect, brush);
	if(option.state & QStyle::State_Selected){
		// its selected
		painter->fillRect(option.rect, option.palette.highlight());
		painter->setPen(option.palette.highlightedText().color());
	}

  QTextOption textOptions;
  textOptions.setAlignment(Qt::AlignCenter);
  painter->drawText(option.rect, mapTransToTypes.key(data, QString("Unkown")), textOptions);
	painter->restore();
}
