/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "delegatemin.h"
#include "delegatemin.moc"

#include "minvalidator.h"

#include <QLineEdit>
#include <QStyleOptionViewItemV2>
#include <QAbstractItemView>

DelegateMIN::DelegateMIN(QWidget *parent)
  : QStyledItemDelegate(parent)
{
	validator = new MINValidator(this);
}

DelegateMIN::~DelegateMIN()
{
	delete validator;
}

void DelegateMIN::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const{
	return QStyledItemDelegate::paint(painter, option, index);
}

QWidget *DelegateMIN::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const{
  // called from model if the user want to edit a field
  QLineEdit *editor = new QLineEdit(parent);
  editor->setValidator(validator);
  connect(editor, SIGNAL(editingFinished()), this, SLOT(commitAndCloseEditor()));
  return editor;
}

void DelegateMIN::commitAndCloseEditor(){
  // the user has changed data
  // so the editor can be closed (closeEditor()) and
  // the model is informed, that new data is present
  QLineEdit *editor = qobject_cast<QLineEdit *>(sender());
	if(editor){
		emit commitData(editor);
		emit closeEditor(editor);
	}
}

void DelegateMIN::setEditorData(QWidget *editor, const QModelIndex &index) const{
  // used to initialize the data of the editor
  // this is mainly used to get the editor showing the current data of the model
  QString data = index.data().toString();
  
  QLineEdit *box = qobject_cast<QLineEdit *>(editor);
  if(!box)
    return QStyledItemDelegate::setEditorData(editor, index);
    
	box->setText(data);
}

void DelegateMIN::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const{
	if(!index.data().canConvert(QVariant::String))
    return QStyledItemDelegate::setModelData(editor, model, index);
  
  QLineEdit *box = qobject_cast<QLineEdit *>(editor);
  if(!box)
    return QStyledItemDelegate::setModelData(editor, model, index);
   
	QString data = box->text();
  if(model->data(index).toString().isEmpty()){
		// if it is the first time, the min is entered,
		// we try to set the type accordingly
		QStringList types = validator->possibleTypes(data);
		if(types.size() == 1){
			data = validator->validate(data, types[0]);
			model->setData(model->index(index.row(), 1), types[0]);
		}
	}
	model->setData(index, data);
}
