/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editmanufacturersdetailswidget.h"
#include "editmanufacturersdetailswidget.moc"

#include <esqldatabase.h>
#include "manufacturermodel.h"
#include "manufacturermodelrootitem.h"
#include "manufacturer.h"

#include <QSqlQueryModel>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QCompleter>
#include <QMovie>
#include <QInputDialog>
#include <QSettings>
#include <QMessageBox>

EditManufacturersDetailsWidget::EditManufacturersDetailsWidget(QSqlQueryModel* model, QWidget* parent, Qt::WindowFlags flags)
	: DetailsWidget(parent, flags)
{
	setupUi(this);
	
	QSettings settings;
	sendMINRanges = false;
	if(settings.contains("sendMINRanges"))
		sendMINRanges = settings.value("sendMINRanges").toBool();
	else
		showSendMINRangesMessage();
		
	if(!sendMINRanges){			
		loadDataLabel->setVisible(false);
		loadDataButton->setVisible(false);
	}
	else{
		loadDataLabel->setMovie(new QMovie(":/icons/load.gif", QByteArray(), loadDataLabel));
		loadDataLabel->setVisible(false);
		loadDataButton->setIcon("view-refresh", ":/icons/reload.png");
		connect(nameEdit, SIGNAL(editingFinished()), this, SLOT(autoLoadData()));
		connect(loadDataButton, SIGNAL(clicked()), this, SLOT(loadData()));
	}
		
	this->model = model;
	
	connect(websiteEdit, SIGNAL(editingFinished()), this, SLOT(websiteFinished()));
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if ( db.isOpen() ) {
    QSqlQueryModel *complCity = new QSqlQueryModel ( this );
    complCity->setQuery ( "SELECT city FROM meganizer_manufacturers GROUP BY city" );
    if ( !complCity->lastError().isValid() ) {
      QCompleter *completer = new QCompleter ( complCity, this );
      completer->setCaseSensitivity ( Qt::CaseInsensitive );
      completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
      cityEdit->setCompleter ( completer );
    }

    QSqlQueryModel *complCountry = new QSqlQueryModel ( this );
    complCountry->setQuery ( "SELECT country FROM meganizer_manufacturers GROUP BY country" );
    if ( !complCountry->lastError().isValid() ) {
      QCompleter *completer = new QCompleter ( complCountry, this );
      completer->setCaseSensitivity ( Qt::CaseInsensitive );
      completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
      countryEdit->setCompleter ( completer );
    }
		if(closeDB)
			db.close();
  }
}

QString EditManufacturersDetailsWidget::name()
{
	return nameEdit->text();
}

bool EditManufacturersDetailsWidget::store()
{
	if ( nameEdit->text().isEmpty() ) {
    emit error ( tr ( "An empty name is not allowed!" ) );
    return false;
  }

	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if ( !db.open() ) {
			emit error ( tr ( "Could not connect to database" ) );
			return false;
		}
	}

	QSqlQuery query;
	if(curIndex.isValid()){
		// a present manufacturer was edited --> we must do a update
		query.prepare("UPDATE meganizer_manufacturers SET name=:name, street=:street, number=:number,"
																	" city=:city, postalCode=:postalCode, postOfficeBox=:postOfficeBox, country=:country, website=:website,"
																	" email=:email, phone=:phone, fax=:fax"
																	" WHERE id=:id");
		query.bindValue(":id", model->record(curIndex.row()).value("id").toUInt());
	}
  else{
		query.prepare("INSERT INTO meganizer_manufacturers (name, street, number, city, postalCode, postOfficeBox,"
									"  country, website, email, phone, fax)"
									" VALUES(:name, :street, :number, :city, :postalCode, :postOfficeBox,"
									"  :country, :website, :email, :phone, :fax)");
	}
	query.bindValue(":name", nameEdit->text());
	query.bindValue(":street", streetEdit->text());
	query.bindValue(":number", numberEdit->text());
	query.bindValue(":city", cityEdit->text());
	query.bindValue(":postalCode", postalCodeEdit->value());
	query.bindValue(":postOfficeBox", postOfficeBoxEdit->value());
	query.bindValue(":country", countryEdit->text());
	query.bindValue(":website", websiteEdit->text());
	query.bindValue(":email", emailEdit->text());
	query.bindValue(":phone", phoneEdit->text());
	query.bindValue(":fax", faxEdit->text());
	if ( !query.exec()){
		emit error ( tr ( "Could not insert or update manufacturer %1\nQuery: %2\nError: %3" )
			.arg ( nameEdit->text() )
			.arg(query.lastQuery())
			.arg(query.lastError().text()) );
		if(closeDB)
			db.close();
		return false;
	}
		
  // reload model
  if ( model ) {
    model->setQuery ( model->query().lastQuery() );
  }
  if(closeDB)
		db.close();
	
	return true;
}

void EditManufacturersDetailsWidget::clear()
{
	// setup the details widget
	nameEdit->setText ( "" );
  streetEdit->setText ( "" );
  numberEdit->setText ( "" );
  cityEdit->setText ( "" );
  postalCodeEdit->setValue ( 0 );
  countryEdit->setText ( "" );
  websiteEdit->setText ( "" );
  emailEdit->setText ( "" );
  phoneEdit->setText ( "" );
  faxEdit->setText ( "" );
	
	QAbstractItemModel *model = prefixView->model();		
	prefixView->setModel(NULL);
	if(model)
		delete model;
}

void EditManufacturersDetailsWidget::setItem(QModelIndex index)
{
	if(!model)
		return;
	
	curIndex = index;
	
	clear();
	
	if(!index.isValid())
		return;
	
	QSqlRecord record = model->record(curIndex.row());
	nameEdit->setText(record.value("name").toString());
	streetEdit->setText(record.value("street").toString());
	numberEdit->setText(record.value("number").toString());
	cityEdit->setText(record.value("city").toString());
	postalCodeEdit->setValue(record.value("postalCode").toInt());
	countryEdit->setText(record.value("country").toString());
	websiteEdit->setText(record.value("website").toString());
	emailEdit->setText(record.value("email").toString());
	phoneEdit->setText(record.value("phone").toString());
	faxEdit->setText(record.value("fax").toString());
	
	ManufacturerModel *model = new ManufacturerModel();
	Manufacturer *manu = new Manufacturer();
	manu->loadFromDatabase(record.value("id").toUInt());
	model->setRootItem(manu);
	if(prefixView->model()){
		QAbstractItemModel *oldModel = prefixView->model();
		prefixView->setModel(NULL);
		delete oldModel;
	}
	prefixView->setModel(model);
	prefixView->setRootIndex(model->indexFromItem(manu));
}

void EditManufacturersDetailsWidget::setEnabled(bool show, QString defaultString)
{
	clear();
	if(!defaultString.isEmpty())
		nameEdit->setText(defaultString);
	EWidget::setEnabled(show);
	nameEdit->setFocus();
}

void EditManufacturersDetailsWidget::websiteFinished()
{
	websiteEdit->setText(Manufacturer::prepareWebsite(websiteEdit->text()));
}

void EditManufacturersDetailsWidget::autoLoadData()
{
	if(curIndex.isValid())
		return;
	loadData();
}

void EditManufacturersDetailsWidget::loadData()
{
	if(!sendMINRanges)
		return;
	
	QSettings settings;
	if(!settings.contains("sendMINRanges"))
		showSendMINRangesMessage();
	if(!sendMINRanges)
		return;
	
	loadDataLabel->movie()->setScaledSize(loadDataButton->size());
	loadDataButton->setVisible(false);
	loadDataLabel->setVisible(true);
	loadDataLabel->movie()->start();
	
	ManufacturerModel *manuModel = new ManufacturerModel(this);
	ManufacturerModelRootItem *manuServer = new ManufacturerModelRootItem("manuserver");
	manuModel->setRootItem(manuServer);
	ManufacturerModelRootItem *getManu = new ManufacturerModelRootItem("getmanufacturersbyname");
	manuServer->addChild(manuServer->rowCount(), getManu);
	getManu->setName(nameEdit->text().left(120));
	
	if (!manuModel->isValidXML()){
		delete manuModel;
		loadDataCleanup();
		return;
	}
	
	connect ( manuModel, SIGNAL ( sendFinished(ManufacturerModel*)), this, SLOT ( loadDataRequestFinished(ManufacturerModel*) ) );
	if(!manuModel->send()){
		delete manuModel;
		loadDataCleanup();
		return;
	}
}

void EditManufacturersDetailsWidget::loadDataRequestFinished(ManufacturerModel* resultModel)
{
	if(sender())
		sender()->deleteLater();

	if(!resultModel){
		loadDataCleanup();
		return;
	}
	if(!resultModel->result()){
		delete resultModel;
		loadDataCleanup();
		return;
  }
  
  // get manufacturer
  ManufacturerModelRootItem *response = resultModel->response();
	if(!response){
		loadDataCleanup();
		delete resultModel;
		return;
	}
	
	if(response->rowCount() < 1){
		loadDataCleanup();
		delete resultModel;
		return;
	}

	Manufacturer *manu = dynamic_cast<Manufacturer*>(response->child(0));
	if(response->rowCount() > 1){
		// let the user choose one of the fetched entries
		QMap<QString, Manufacturer*> mapManus;
		for(int i = 0; i < response->rowCount(); i++){
			Manufacturer *tmp = dynamic_cast<Manufacturer*>(response->child(i));
			mapManus.insert(tmp->name(), tmp);
		}
		
		bool ok;
		// order the manufacturers according to their relevance (score())
		// and append the value to the name
		QMultiMap<uint, QString> mapScores;
		foreach(QString manu, mapManus.uniqueKeys()){
			mapScores.insert(
				mapManus.value(manu)->score(),
				QString("%1 (%2%)").arg(manu).arg(mapManus.value(manu)->score())
			);
		}
		// the multimap stores them in ascending order, therefore
		// we walk through the list of manufacturers from end to begin
		// and add them in this order to the new list.
		QStringList manus;
		QMultiMap<uint, QString>::const_iterator i;
		i = mapScores.constEnd();
		do {
			--i; // setting it to the first valid position (therefore size() must be greater 0)
			manus << i.value();
		} while ( i != mapScores.constBegin() );
		
		QString item = QInputDialog::getItem(this, tr("Found more than one manufacturer."),
                                          tr("Choose:"), manus, 0, false, &ok);
		if(!ok){
			loadDataCleanup();
			delete resultModel;
			return;
		}

		manu = mapManus.value(item.left(item.lastIndexOf("(") - 1));
	}
	if(!manu){
		loadDataCleanup();
		delete resultModel;
		return;
	}
	
	if(nameEdit->text().isEmpty() || overloadDataEdit->isChecked())
		nameEdit->setText(manu->name());
	if(streetEdit->text().isEmpty() || overloadDataEdit->isChecked())
		streetEdit->setText(manu->street());
	if(numberEdit->text().isEmpty() || overloadDataEdit->isChecked())
		numberEdit->setText(manu->number());
	if(postalCodeEdit->text().isEmpty() || overloadDataEdit->isChecked())
		postalCodeEdit->setValue(manu->zip());
	if(cityEdit->text().isEmpty() || overloadDataEdit->isChecked())
		cityEdit->setText(manu->city());
	if(postOfficeBoxEdit->text().isEmpty() || overloadDataEdit->isChecked())
		postOfficeBoxEdit->setValue(manu->postOfficeBox());
	if(countryEdit->text().isEmpty() || overloadDataEdit->isChecked())
		countryEdit->setText(manu->country());
	if(websiteEdit->text().isEmpty() || overloadDataEdit->isChecked())
		websiteEdit->setText(manu->website().toString());
	if(emailEdit->text().isEmpty() || overloadDataEdit->isChecked())
		emailEdit->setText(manu->email());
	if(phoneEdit->text().isEmpty() || overloadDataEdit->isChecked())
		phoneEdit->setText(manu->phone());
	if(faxEdit->text().isEmpty() || overloadDataEdit->isChecked())
		faxEdit->setText(manu->fax());
	
	if(prefixView->model()){
		QAbstractItemModel *oldModel = prefixView->model();
		prefixView->setModel(NULL);
		delete oldModel;
	}

	prefixView->setModel(resultModel);
	prefixView->setRootIndex(resultModel->indexFromItem(manu));
	
	loadDataCleanup();
}

void EditManufacturersDetailsWidget::loadDataCleanup()
{
	loadDataLabel->movie()->stop();
	loadDataButton->setVisible(true);
	loadDataLabel->setVisible(false);
}

void EditManufacturersDetailsWidget::showSendMINRangesMessage()
{
	QSettings settings;
	// only show the message, if the user has not choosen to disable them
	if(settings.contains("sendMINRanges"))
		return;
	
	// ask the user if he wants to enable this feature
	int result = QMessageBox::question(this, tr("Send and load MIN ranges"),
		tr("To recieve data of manufacturers stored at our central database at http://mins.engsas.de, you must enable the sending and recieving of manufacturer data. The data is submitted anonymously to our server and contains only the details of manufacturers and their calculated prefix.\n"
			" You can change revoke your approval at the settings dialog at any time.\n"
			"Do you want to recieve and send the descriped data?"
		),
		QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes
	);
	if(result == QMessageBox::Yes)
		sendMINRanges = true;
	settings.setValue("sendMINRanges", sendMINRanges);
}
