/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef LOANITEM_H
#define LOANITEM_H

#include <eobject.h>

#include <QSqlRecord>
#include <QDateTime>

class LoanItem : public EObject
{
	Q_OBJECT
	public:
		LoanItem(QObject *parent = 0);
		
		bool load(QSqlRecord record);
		bool store();
		bool modified() const;
		bool isValid() const;
		bool loanFinished() const;
		
		void setLoggedInUser(unsigned int id);
		/**
			* @returns the id of the database user
			* if current id is 0 (equal not set), it fetches it from db
			*/
		unsigned int loggedInUser();
		void setMaterial(unsigned int id);
		unsigned int material() const;
		QString materialTitle() const;
		/**
			* if called and @p setOwnerName is true, it tries to
			* set ownerUserName() accordingly
			*/
		void setOwner(unsigned int id, bool setOwnerName = true);
		unsigned int owner() const;
		void setOwnerUserName(QString userName);
		/**
			* the username is needed to detect wether a medium belongs
			* a user of this database or not. But it is not relevant for storing.
			* Therefore setModified is not called by setOwnerUserName().
			*/
		QString ownerUserName() const ;
		bool setRecipient(unsigned int id, bool ignoreRights = false);
		unsigned int recipient() const;
		/**
			* @return all stati, which can be setted by loggedInUser()
			*/
		QList<unsigned int> validStatis();
		bool setStatus(unsigned int id, bool ignoreRights = false);
		unsigned int status() const;
		bool setLoanedUntil(QDate until, bool ignoreRights = false);
		QDate loanedUntil() const;
		void setLastChange(QDate changeTime);
		QDate lastChange() const;
		
		bool operator== (const LoanItem &item) const;
		bool operator!= (const LoanItem &item) const;
		
		
	public slots:
		void setModified(bool mod = true);
		
	private:
		void setRow(unsigned int id);
		unsigned int row() const;
		
		unsigned int materialId, ownerId, rowId, recipientId, statusId, loggedInUserId;
		QDate myLoanedUntil, myLastChange;
		bool isModified;
		QString myOwnerUserName, myMaterialTitle;
		
	signals:
    void changed();
};

#endif // LOANITEM_H
