/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediummin.h"
#include "mediummin.moc"

#include "editmintypes.h"
#include <esqldatabase.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>

MediumMIN::MediumMIN(QString min, unsigned int typeId, unsigned int rowId, EQueryThread *thread, QObject* parent)
	: EObject(parent)
{
	init(thread);
	setMIN(min);
	setType(typeId);
	setRowId(rowId);
}

MediumMIN::MediumMIN(QString min, QString type, unsigned int rowId, EQueryThread *thread, QObject* parent)
	: EObject(parent)
{
	init(thread);
	setMIN(min);
	setType(type);
	setRowId(rowId);
}

MediumMIN::MediumMIN(EQueryThread *thread, QObject* parent)
	: EObject(parent)
{
	init(thread);
}

void MediumMIN::setThread(EQueryThread* thread)
{
	if(this->thread){
		this->thread->disconnect(this);
		this->disconnect(this->thread);
	}
	this->thread = thread;
	if(thread)
		connect(thread, SIGNAL(result(QObject*, QVariant, int, QSqlRecord)),
						this, SLOT(queryResult(QObject*, QVariant, int, QSqlRecord)));
}

bool MediumMIN::store(unsigned int materialId)
{
	bool checks = true;
	if(!prepareMIN())
		checks = false;
	
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return false;
  
  bool result = false;
  if(markForRemovel()){
		// if the checks failed, we do nothing and return true,
		// because the min couldn't be in the database
		if(!checks)
			return true;
    result = remove(materialId);
  }
  else{
		if(!checks)
			return false;
    result = insert(materialId);
  }
 
  if(closeDB)
    db.close();
  return result;
}

void MediumMIN::setRowId(unsigned int rowId)
{
	myRowId = rowId;
	emit itemChanged(this);
}

unsigned int MediumMIN::rowId() const
{
	return myRowId;
}

bool MediumMIN::prepareMIN()
{
	if(min().isEmpty()){
		setMarkForRemovel(true);
		return true;
	}
	
	if(type() > 0)
		return true;
	
	if(!typeName().isEmpty()){
		// so try to get it from database
		setType(typeName());
	}
	if(type() > 0)
		return true;
	
	// user must add a new min type
	EditMinTypes editMinTypes;
	if(editMinTypes.exec() != QDialog::Accepted)
		return false;
	
	if(editMinTypes.selectedId() < 1)
		return false;
	
	setType(editMinTypes.selectedId(), false);
	setType(editMinTypes.selectedName(), false);
	return true;
}

void MediumMIN::setMIN(QString min)
{
	myNumber = min;
	emit itemChanged(this);
}

QString MediumMIN::min() const
{
	return myNumber;
}

void MediumMIN::setType(unsigned int id, bool setTypeName)
{
	myTypeId = id;
	emit itemChanged(this);
	if(setTypeName){
		setType(QString(), false);
		if(myTypeId > 0){
			QString queryString = QString("SELECT name FROM meganizer_mintypes WHERE id=%1").arg(id);
			if(thread){
				thread->execQuery(this, "setType", queryString, "");
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.exec(queryString);
					if(query.first())
						// don't set the id afterwards (loop)
						setType(query.record().value("name").toString(), false);
					if(closeDB)
						db.close();
				}
			}
		}
	}
}

void MediumMIN::setType(QString name, bool setTypeId)
{
	myTypeName = name.trimmed();
	emit itemChanged(this);
	if(setTypeId){
		setType(0, false);
		if(!myTypeName.isEmpty()){
			QString queryString = "SELECT id FROM meganizer_mintypes WHERE name=?";
			if(thread){
				QList<QVariant> values;
				values << name;
				thread->execQuery(this, "setType", queryString, "", values);
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.prepare(queryString);
					query.addBindValue(name);
					query.exec();
					if((query.first()) && (query.size() == 1)){
						// don't set the id afterwards (loop)
						setType(query.record().value("id").toUInt(), false);
					}
					if(closeDB)
						db.close();
				}
			}
		}
	}
}

void MediumMIN::setType(QSqlRecord record)
{
	if(record.contains("id"))
		setType(record.value("id").toUInt(), false);
	else if(record.contains("name"))
		setType(record.value("name").toString(), false);
}

unsigned int MediumMIN::type() const
{
	return myTypeId;
}

QString MediumMIN::typeName() const
{
	return myTypeName;
}

void MediumMIN::setMarkForRemovel(bool remove)
{
	markedForRemovel = remove;
	emit itemChanged(this);
}

bool MediumMIN::markForRemovel() const
{
	return markedForRemovel;
}

MediumMIN* MediumMIN::copyMIN()
{
	MediumMIN *newMIN = new MediumMIN();
	newMIN->setRowId(rowId());
	newMIN->setType(type(), false);
	newMIN->setType(typeName(), false);
	newMIN->setMarkForRemovel(markForRemovel());
	newMIN->setMIN(min());
	return newMIN;
}

bool MediumMIN::operator==(const MediumMIN& otherMin) const
{
	if(otherMin.min() != min())
		return false;
	
	if((otherMin.type() != 0) && (type() != 0)){
		if(otherMin.type() != type())
			return false;
		return true;
	}
	
	if(otherMin.typeName() != typeName())
		return false;
	
	return true;
}

bool MediumMIN::operator!=(const MediumMIN& otherMin) const
{
	return !operator==(otherMin);
}

void MediumMIN::queryResult(QObject* client, QVariant requestId, int querySize, QSqlRecord record)
{
	if(client != this)
		return;
	if(requestId.toString() == "setType")
		if(querySize == 1)
			setType(record);
}

void MediumMIN::init(EQueryThread *thread)
{
	this->thread = NULL;
	setThread(thread);
	setRowId(0);
	setType(0, false);
	setMarkForRemovel(false);
}

bool MediumMIN::insert(unsigned int materialId)
{
  // check wether min material relation is already present
  QSqlQuery query;
	query.prepare("SELECT * FROM meganizer_materialhasmins"
								"  WHERE materialsId=:materialsId AND mintypesId=:mintypesId"
								"    AND min=:min");
	query.bindValue(":materialsId", materialId);
	query.bindValue(":mintypesId", type());
	query.bindValue(":min", min());
  if(!query.exec()){
    return false;
  }
  // if entry is already present nothing needs to be done
  if(query.first()){
    return true;
  }
  
  // the mintype or min has changed or the relation doesn't exists
	if(rowId() > 0){
		// mintype or min has changed
		query.prepare("UPDATE meganizer_materialhasmins SET mintypesId=:mintypesId, min=:min WHERE id=:id");
		query.bindValue(":mintypesId", type());
		query.bindValue(":min", min());
		query.bindValue(":id", rowId());
	}
	else{
		// the relation doesn't exist
		query.prepare("INSERT INTO meganizer_materialhasmins (materialsId, mintypesId, min)"
									" VALUES(:materialsId, :mintypesId, :min)");
		query.bindValue(":materialsId", materialId);
		query.bindValue(":mintypesId", type());
		query.bindValue(":min", min());
	}

	return query.exec();
}

bool MediumMIN::remove(unsigned int materialId)
{
  QSqlQuery query;
	if(rowId() > 0){
		query.prepare("DELETE FROM meganizer_materialhasmins WHERE id=:id");
		query.bindValue(":id", rowId());
	}
	else{
		query.prepare("DELETE FROM meganizer_materialhasmins WHERE materialsId=:materialsId"
									"  AND mintypesId=:mintypesId"
									"  AND min=:min");
		query.bindValue(":materialsId", materialId);
		query.bindValue(":mintypesId", type());
		query.bindValue(":min", min());
	}
	return query.exec();
}
