/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "manufacturer.h"

#include <exmlparseritem.h>
#include <esqldatabase.h>
#include "manufacturerprefix.h"

#include <QRegExp>
#include <QStringList>
#include <QSqlQuery>
#include <QSqlRecord>

Manufacturer::Manufacturer(EXmlItem* parent)
	: EXmlItem("manufacturer", parent)
{
	setId(0);
	setZIP(0);
	setPostOfficeBox(0);
	setScore(0);
}

EXmlItem* Manufacturer::create(EXmlModel*)
{
	return new Manufacturer();
}

QString Manufacturer::prepareWebsite(QString website)
{
	if(!website.contains(QRegExp("^https?://")))
		website = "http://"+ website;
	return website;
}

QVariant Manufacturer::data(int column, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	switch(column){
		case 0:
			return name();
		default:
			return "Not implemented";
	}
}

bool Manufacturer::load(QDomElement element, QMap< QString, EXmlParserItem* > parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		// processing the child elements, which are handled within this class
		if(child.tagName().toLower() == "name")
			setName(child.text());
		else if(child.tagName().toLower() == "id")
			setId(child.text().toUInt());
		else if(child.tagName().toLower() == "relevance")
			setScore(child.text().toUInt());
		else if(child.tagName().toLower() == "street"){
			QDomElement elem = child.firstChildElement("name");
			if(elem.isNull())
				return false;
			setStreet(elem.text());
			elem = elem.nextSiblingElement();
			if(elem.isNull())
				return false;
			setNumber(elem.text());
		}
		else if(child.tagName().toLower() == "zip")
			setZIP(child.text().toUInt());
		else if(child.tagName().toLower() == "city")
			setCity(child.text());
		else if(child.tagName().toLower() == "postofficebox")
			setPostOfficeBox(child.text().toUInt());
		else if(child.tagName().toLower() == "country")
			setCountry(child.text());
		else if(child.tagName().toLower() == "website")
			setWebsite(child.text());
		else if(child.tagName().toLower() == "email")
			setEmail(child.text());
		else if(child.tagName().toLower() == "phone")
			setPhone(child.text());
		else if(child.tagName().toLower() == "fax")
			setFax(child.text());
		else{
			// all other should be handled by others
			QStringList keys = parser.uniqueKeys();
			if(!keys.contains(child.tagName().toLower()))
				return false;

			if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser)))
				return false;
		}
		child = child.nextSiblingElement();
	}

	return true;
}

bool Manufacturer::save(QDomElement& useElem)
{
	// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;
	
	useElem.setTagName(tagName());
	QDomDocument doc = useElem.ownerDocument();
	QDomElement tmp = doc.createElement("name");
	tmp.appendChild(doc.createTextNode(name()));
	useElem.appendChild(tmp);
	if(id() > 0){
		tmp = doc.createElement("id");
		useElem.appendChild(tmp);
		tmp.appendChild(doc.createTextNode(QString::number(id())));
	}
	if(score() > 0){
		tmp = doc.createElement("relevance");
		useElem.appendChild(tmp);
		tmp.appendChild(doc.createTextNode(QString::number(score())));
	}
	if(!street().isEmpty()){
		QDomElement streetElem = doc.createElement("street");
		useElem.appendChild(streetElem);
		tmp = doc.createElement("name");
		tmp.appendChild(doc.createTextNode(street()));
		streetElem.appendChild(tmp);
		if(!number().isEmpty()){
			tmp = doc.createElement("number");
			tmp.appendChild(doc.createTextNode(number()));
			streetElem.appendChild(tmp);
		}
	}
	if(zip() > 0){
		tmp = doc.createElement("zip");
		tmp.appendChild(doc.createTextNode(QString::number(zip())));
		useElem.appendChild(tmp);
	}
	if(!city().isEmpty()){
		tmp = doc.createElement("city");
		tmp.appendChild(doc.createTextNode(city()));
		useElem.appendChild(tmp);
	}
	if(postOfficeBox() > 0){
		tmp = doc.createElement("postofficebox");
		tmp.appendChild(doc.createTextNode(QString::number(postOfficeBox())));
		useElem.appendChild(tmp);
	}
	if(!country().isEmpty()){
		tmp = doc.createElement("country");
		tmp.appendChild(doc.createTextNode(country()));
		useElem.appendChild(tmp);
	}
	if(!website().isEmpty()){
		tmp = doc.createElement("website");
		tmp.appendChild(doc.createTextNode(website().toString()));
		useElem.appendChild(tmp);
	}
	if(!email().isEmpty()){
		tmp = doc.createElement("email");
		tmp.appendChild(doc.createTextNode(email()));
		useElem.appendChild(tmp);
	}
	if(!phone().isEmpty()){
		tmp = doc.createElement("phone");
		tmp.appendChild(doc.createTextNode(phone()));
		useElem.appendChild(tmp);
	}
	if(!fax().isEmpty()){
		tmp = doc.createElement("fax");
		tmp.appendChild(doc.createTextNode(fax()));
		useElem.appendChild(tmp);
	}

	return EXmlItem::save(useElem);
}

void Manufacturer::setId(uint id)
{
	myId = id;
}

uint Manufacturer::id() const
{
	return myId;
}

void Manufacturer::setName(QString name)
{
	myName = name;
}

QString Manufacturer::name() const
{
	return myName;
}

void Manufacturer::setScore(uint score)
{
	myScore = score;
}

uint Manufacturer::score() const
{
	return myScore;
}

void Manufacturer::setStreet(QString street)
{
	myStreet = street;
}

QString Manufacturer::street() const
{
	return myStreet;
}

void Manufacturer::setNumber(QString number)
{
	myNumber = number;
}

QString Manufacturer::number() const
{
	return myNumber;
}

void Manufacturer::setCity(QString city)
{
	myCity = city;
}

QString Manufacturer::city() const
{
	return myCity;
}

void Manufacturer::setCountry(QString country)
{
	myCountry = country;
}

QString Manufacturer::country() const
{
	return myCountry;
}

void Manufacturer::setZIP(uint zip)
{
	myZIP = zip;
}

uint Manufacturer::zip() const
{
	return myZIP;
}

void Manufacturer::setPostOfficeBox(uint box)
{
	myPostOfficeBox = box;
}

uint Manufacturer::postOfficeBox() const
{
	return myPostOfficeBox;
}

void Manufacturer::setWebsite(QString website)
{
	myWebsite = QUrl(prepareWebsite(website));
}

QUrl Manufacturer::website() const
{
	return myWebsite;
}

void Manufacturer::setEmail(QString eMail)
{
	myEmail = eMail;
}

QString Manufacturer::email() const
{
	return myEmail;
}

void Manufacturer::setPhone(QString phone)
{
	myPhone = phone;
}

QString Manufacturer::phone() const
{
	return myPhone;
}

void Manufacturer::setFax(QString fax)
{
	myFax = fax;
}

QString Manufacturer::fax() const
{
	return myFax;
}

EXmlItem* Manufacturer::copy() const
{
	Manufacturer *res = dynamic_cast<Manufacturer*>(create(NULL));
	if(!res)
		return NULL;
	copyBase(res);
	res->setName(name());
	res->setScore(score());
	res->setStreet(street());
	res->setNumber(number());
	res->setZIP(zip());
	res->setCity(city());
	res->setPostOfficeBox(postOfficeBox());
	res->setCountry(country());
	res->setWebsite(website().toString());
	res->setEmail(email());
	res->setPhone(phone());
	res->setFax(fax());
	return res;
}

bool Manufacturer::loadFromDatabase(uint id)
{
	setId(id);
	setName("");
	setScore(0);
	setStreet("");
	setNumber("");
	setZIP(0);
	setCity("");
	setPostOfficeBox(0);
	setCountry("");
	setWebsite("");
	setEmail("");
	setPhone("");
	setFax("");
	
	// fetch data from database
	if(id < 1)
		return true;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return false;
		
	QSqlQuery query;
	query.prepare("SELECT * FROM meganizer_manufacturers WHERE id=:id");
	query.bindValue(":id", id);
	if(!query.exec()){
		if(closeDB)
			db.close();
		return false;
	}
	
	if(!query.first()){
		if(closeDB)
			db.close();
		return false;
	}
	
	setName(query.record().value("name").toString());
	setStreet(query.record().value("street").toString());
	setNumber(query.record().value("number").toString());
	setZIP(query.record().value("postalCode").toUInt());
	setCity(query.record().value("city").toString());
	setPostOfficeBox(query.record().value("postOfficeBox").toUInt());
	setCountry(query.record().value("country").toString());
	setWebsite(query.record().value("website").toString());
	setEmail(query.record().value("eMail").toString());
	setPhone(query.record().value("phone").toString());
	setFax(query.record().value("fax").toString());
	
	// load prefixes
	if(!query.exec(QString("SELECT mins.characteristic, types.name"
								" FROM meganizer_manufacturersmins mins, meganizer_mintypes types"
								" WHERE mins.mintypesId=types.id"
								"   AND mins.manufacturersId=%1").arg(id))){
		if(closeDB)
			db.close();
		return false;
	}
	while(query.next()){
		ManufacturerPrefix *prefix = new ManufacturerPrefix(this);
		addChild(rowCount(), prefix);
		prefix->setPrefix(query.record().value("characteristic").toString());
		prefix->setType(query.record().value("name").toString());
	}
	if(closeDB)
		db.close();
	return true;
}
