/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "manufacturermodel.h"
#include "manufacturermodel.moc"

#include "manufacturer.h"
#include "manufacturerprefix.h"
#include "manufacturermodelrootitem.h"
#include <enetworkaccessmanager.h>

#include <QNetworkReply>
#include <QNetworkRequest>

ManufacturerModel::ManufacturerModel(QObject* parent)
	: EXmlModel(QUrl("http://mins.engsas.de/manuserver.xsd"), parent)
{
	headerList << tr("Prefix") << tr("Type");
	
	// register parser
	registerParser(&Manufacturer::create);
	registerParser(&ManufacturerPrefix::create);
	registerParser(&ManufacturerModelRootItem::createManuServer);
	registerParser(&ManufacturerModelRootItem::createSubmitPrefixes);
	registerParser(&ManufacturerModelRootItem::createGetManufacturersByName);
	registerParser(&ManufacturerModelRootItem::createGetManufacturersByPrefix);
	registerParser(&ManufacturerModelRootItem::createResponse);
}

bool ManufacturerModel::loadFromReply(QNetworkReply* reply)
{
	if(!reply)
		return false;
	
	QNetworkReply::NetworkError netError = reply->error();
  if ( netError != QNetworkReply::NoError ) {
		if ( netError != QNetworkReply::SslHandshakeFailedError ) {
			reply->deleteLater();
			return false;
		}
  }
  
	// checking result
	QByteArray data = reply->readAll();
	reply->deleteLater();
	if(!load(data))
		return false;
	
	if(!isValidXML())
		return false;

	return true;
}

bool ManufacturerModel::send()
{
	// a response can not be send
	if(response())
		return false;
	
	QDomDocument doc = save();
	if(doc.isNull())
		return false;
	
	ENetworkAccessManager *http = new ENetworkAccessManager ( this );
	QNetworkRequest request;
	request.setUrl ( QUrl ( "http://mins.engsas.de/submit.php" ) );
	request.setHeader ( QNetworkRequest::ContentTypeHeader, QVariant ( "text/xml; charset=UTF-8" ) );
	QNetworkReply *reply = http->post ( request, doc.toByteArray() );
	connect ( reply, SIGNAL ( finished() ), this, SLOT ( sendRequestFinished() ) );
	return true;
}

bool ManufacturerModel::result() const
{
	ManufacturerModelRootItem *resp = response();
	if(!resp)
		return false;
	return resp->result();
}

QString ManufacturerModel::message() const
{
	ManufacturerModelRootItem *resp = response();
	if(!resp)
		return QString();
	return resp->message();
}

ManufacturerModelRootItem* ManufacturerModel::response() const
{
	EXmlItem *item = itemFromIndex(index(0,0));
	if(!item)
		return NULL;

	if(item->tagName() != "manuserver")
		return NULL;
	
	ManufacturerModelRootItem *response = dynamic_cast<ManufacturerModelRootItem*>(item->child(0));
	if(!response)
		return NULL;

	if(response->tagName() != "response")
		return NULL;
	return response;
}

void ManufacturerModel::sendRequestFinished()
{
	QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
	if(!reply){
		emit sendFinished(NULL);
		return;
	}
	
  if(reply->error() != QNetworkReply::NoError){
// 		emit information(tr("Could not fetch image from url: %1. Error: %2").arg(reply->url().toString()).arg(reply->errorString()));
		emit sendFinished(NULL);
		return;
	}
		
	ManufacturerModel *manuModel = new ManufacturerModel();
	if(!manuModel->loadFromReply(reply)){
		reply->deleteLater();
		emit sendFinished(NULL);
		return;
	}
	
	if(!manuModel->isValidXML()){
		emit sendFinished(NULL);
		return;
	}

	emit sendFinished(manuModel);
}
