/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef MANUFACTURERMODEL_H
#define MANUFACTURERMODEL_H

#include "libmeganizer.h"
#include <exmlmodel.h>

class ManufacturerModelRootItem;

class QNetworkReply;

/**
	* @brief Model related to manuserver.xsd.
	*/
class LIBMEGANIZER_EXPORT ManufacturerModel : public EXmlModel
{
	Q_OBJECT
	public:
    ManufacturerModel(QObject* parent = 0);
		
		/**
			* @brief Loads the model from the data of @p reply.
			* 
			* It also checks whether the contained data is a valid
			* XML document of manuserver.xsd.
			* 
			* @attention The @p reply is deleted later.
			*/
		bool loadFromReply(QNetworkReply *reply);
		/**
			* @brief Sends the current model as XML raw data to http://mins.engsas.de.
			* 
			* If the request was send, true is returned and the response is
			* populated with the signal sendFinished().
			* Use this code to connect to the result signal, delete (if possible) the sender
			* object (the one send() called on) and check for a NULL pointer.
			* The check for validaty is not needed, because in sendFinished() it is
			* already done.
			* @code
			* void sendYourModel(){
			*  // create your model
			*  ManufacturerModel *manuModel = new ManufacturerModel(this);
			*  // ...
			*  // prepare the model
			*  // ...
			*  connect ( manuModel, SIGNAL ( sendFinished(ManufacturerModel*)), this, SLOT ( yourResultSlot(ManufacturerModel*) ) );
			* }
			* 
			* void yourResultSlot(ManufacturerModel *model){
			*   if(sender())
			*     sender()->deleteLater(); // delete later only, because otherwise reply->deleteLater() gets into trouble.
			*   if(!model)
			*     return;
			*   // ...
			*   // process the model here
			*   // ...
			*   delete model;
			* }
			* @endcode
			*/
		bool send();
		/**
			* @brief Contains the content of the result tag, if the document is a response.
			* 
			* If the document is not a response defined in manuserver.xsd, it returns false
			* in any case.
			*/
		bool result() const;
		/**
			* @brief Contains the content of the message tag, if the document is a response.
			* 
			* If the document is not a response defined in manuserver.xsd, it returns an empty string
			* in any case.
			*/
		QString message() const;
		/**
			* @brief Returns the object belonging to the response tag, if present.
			* 
			* Otherwise NULL is returned.
			*/
		ManufacturerModelRootItem *response() const;
		
	private slots:
		void sendRequestFinished();
		
	signals:
		/**
			* @brief Emitted, if the send() request is replied.
			* 
			* The model is NULL, if an undefined error occured.
			*/
		emit void sendFinished(ManufacturerModel *model);
};

#endif // MANUFACTURERMODEL_H
