/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "manufacturermodelrootitem.h"

#include <exmlparseritem.h>

#include <QStringList>

ManufacturerModelRootItem::ManufacturerModelRootItem(QString tagName, EXmlItem* parent)
	: EXmlItem(tagName, parent)
{
	myResult = false;
}

EXmlItem* ManufacturerModelRootItem::createManuServer(EXmlModel*)
{
	return new ManufacturerModelRootItem("manuserver");
}

EXmlItem* ManufacturerModelRootItem::createSubmitPrefixes(EXmlModel*)
{
	return new ManufacturerModelRootItem("submitprefixes");
}

EXmlItem* ManufacturerModelRootItem::createGetManufacturersByName(EXmlModel*)
{
	return new ManufacturerModelRootItem("getmanufacturersbyname");
}

EXmlItem* ManufacturerModelRootItem::createGetManufacturersByPrefix(EXmlModel*)
{
	return new ManufacturerModelRootItem("getmanufacturersbyprefix");
}

EXmlItem* ManufacturerModelRootItem::createResponse(EXmlModel*)
{
	return new ManufacturerModelRootItem("response");
}

QVariant ManufacturerModelRootItem::data(int column, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	switch(column){
		case 0:
			return tagName();
		default:
			return QVariant();
	}
}

bool ManufacturerModelRootItem::load(QDomElement element, QMap< QString, EXmlParserItem* > parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		if((tagName().toLower() == "getmanufacturersbyname") && (child.tagName() == "name"))
			setName(child.text());
		else if((tagName().toLower() == "getmanufacturersbyprefix") && (child.tagName() == "prefix"))
			setPrefix(child.text());
		else if((tagName().toLower() == "response") && (child.tagName().toLower() == "result"))
			setResult(child.text());
		else if((tagName().toLower() == "response") && (child.tagName().toLower() == "message"))
			setMessage(child.text());
		else{
			// all other should be handled by others
			QStringList keys = parser.uniqueKeys();
			if(!keys.contains(child.tagName().toLower()))
				return false;

			if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser)))
				return false;
		}
		child = child.nextSiblingElement();
	}
		
	return true;
}

bool ManufacturerModelRootItem::save(QDomElement& useElem)
{
// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;
	
	useElem.setTagName(tagName());
	QDomDocument doc = useElem.ownerDocument();
	if(tagName() == "manuserver"){
		useElem.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		useElem.setAttribute("xmlns", "http://mins.engsas.de/manuserver");
		useElem.setAttribute("xsi:schemaLocation", "http://mins.engsas.de/manuserver http://mins.engsas.de/manuserver.xsd");
	}
	if(tagName() == "getmanufacturersbyname"){
		QDomElement tmp = doc.createElement("name");
		tmp.appendChild(doc.createTextNode(name()));
		useElem.appendChild(tmp);
	}
	if(tagName() == "getmanufacturersbyprefix"){
		QDomElement tmp = doc.createElement("prefix");
		tmp.appendChild(doc.createTextNode(prefix()));
		useElem.appendChild(tmp);
	}
	if(tagName() == "response"){
		QDomElement tmp = doc.createElement("result");
		QString text = "false";
		if(result())
			text = "true";
		tmp.appendChild(doc.createTextNode(text));
		useElem.appendChild(tmp);
		tmp = doc.createElement("message");
		tmp.appendChild(doc.createTextNode(message()));
		useElem.appendChild(tmp);
	}

	return EXmlItem::save(useElem);
}

void ManufacturerModelRootItem::setName(QString name)
{
	myName = name;
}

QString ManufacturerModelRootItem::name() const
{
	return myName;
}

void ManufacturerModelRootItem::setPrefix(QString prefix)
{
	myPrefix = prefix;
}

QString ManufacturerModelRootItem::prefix() const
{
	return myPrefix;
}

void ManufacturerModelRootItem::setResult(QString text)
{
	if(text.toLower() == "true")
		myResult = true;
	else
		myResult = false;
}

bool ManufacturerModelRootItem::result() const
{
	return myResult;
}

void ManufacturerModelRootItem::setMessage(QString message)
{
	myMessage = message;
}

QString ManufacturerModelRootItem::message() const
{
	return myMessage;
}

EXmlItem* ManufacturerModelRootItem::copy() const
{
	ManufacturerModelRootItem *res = new ManufacturerModelRootItem(tagName());
	copyBase(res);
	res->setName(name());
	if(result())
		res->setResult("true");
	else
		res->setResult("false");
	res->setMessage(message());
	return res;
}
