/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "manufacturerprefix.h"

#include <exmlparseritem.h>

#include <QStringList>

EXmlItem* ManufacturerPrefix::create(EXmlModel*)
{
	return new ManufacturerPrefix();
}

QVariant ManufacturerPrefix::data(int column, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	switch(column){
		case 0:
			return prefix();
		case 1:
			return type();
		default:
			return QVariant();
	}
}

bool ManufacturerPrefix::load(QDomElement element, QMap< QString, EXmlParserItem* > parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	setType(element.attribute("type"));
	setPrefix(element.text());
	
	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		// all other should be handled by others
		QStringList keys = parser.uniqueKeys();
		if(!keys.contains(child.tagName().toLower()))
			return false;
		if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser)))
			return false;
		child = child.nextSiblingElement();
	}

	return true;
}

bool ManufacturerPrefix::save(QDomElement& useElem)
{
// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;
	
	useElem.setTagName(tagName());
	useElem.setAttribute("type", type());
	QDomDocument doc = useElem.ownerDocument();
	useElem.appendChild(doc.createTextNode(prefix()));
	
	return EXmlItem::save(useElem);
}

void ManufacturerPrefix::setType(QString type)
{
	myType = type;
}

QString ManufacturerPrefix::type() const
{
	return myType;
}

void ManufacturerPrefix::setPrefix(QString prefix)
{
	myPrefix = prefix;
}

QString ManufacturerPrefix::prefix() const
{
	return myPrefix;
}

EXmlItem* ManufacturerPrefix::copy() const
{
	ManufacturerPrefix *res = dynamic_cast<ManufacturerPrefix*>(create(NULL));
	if(!res)
		return NULL;
	copyBase(res);
	res->setType(type());
	res->setPrefix(prefix());
	return res;
}
