/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "manuserverresponse.h"

#include <QNetworkReply>

ManuServerResponse::ManuServerResponse(QNetworkReply *reply)
{
	myIsValid = false;
	myResult = false;
	
	if(!reply)
		return;
	
	QNetworkReply::NetworkError netError = reply->error();
  if ( netError != QNetworkReply::NoError ) {
		if ( netError != QNetworkReply::SslHandshakeFailedError ) {
			reply->deleteLater();
			return;
		}
  }
  
	// checking result
	QByteArray data = reply->readAll();
	reply->deleteLater();
	QDomDocument result;
	if(!result.setContent(data, true))
		return;
	
	QUrl schemaUrl("http://mins.engsas.de/manuserver.xsd");
	QXmlSchema schema;
	schema.load(schemaUrl);
	if (!schema.isValid())
		return;

	QXmlSchemaValidator validator(schema);
	if (!validator.validate(doc.toByteArray()))
		return;
	myIsValid = true;
	
	QDomElement elem = result.documentElement();
	if(elem.isNull() || elem.tagName() != "manuServer")
		return;
	
	elem = elem.firstChildElement("response");
	if(elem.isNull())
		return;
	
	elem = elem.firstChildElement("result");
	if(elem.isNull())
		return;
	
	if(elem.text().toLower() == "true")
		myResult = true;
	
	elem = elem.nextSiblingElement("message");
	if(!elem.isNull())
		myMessage = elem.text();
}

bool ManuServerResponse::valid()
{
	return myIsValid;
}

bool ManuServerResponse::result()
{
	return myResult;
}

QString ManuServerResponse::message()
{
	return myMessage;
}
