/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "materialperson.h"
#include "materialperson.moc"

#include "editmaterialpersons.h"
#include "editroles.h"
#include <esqldatabase.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QStringList>
#include <QInputDialog>

MaterialPerson::MaterialPerson(EQueryThread *thread, QObject* parent)
	: Person(thread, parent)
{
  initMatP();
}

MaterialPerson::MaterialPerson(uint personId, uint roleId, uint rowId,
															 EQueryThread *thread, QObject *parent)
	: Person(personId, thread, parent)
{
	initMatP();
  setRole(roleId);
	setRowId(rowId);
}

MaterialPerson::MaterialPerson(QString newName, QString newGivenName, uint roleId,
															 EQueryThread *thread, QObject *parent)
	: Person(newName, newGivenName, thread, parent)
{
  initMatP();
  setRole(roleId);
}

MaterialPerson::MaterialPerson(QString newName, QString newGivenName, QString role,
															 EQueryThread *thread, QObject* parent)
	: Person(newName, newGivenName, thread, parent)
{
	initMatP();
	setRole(role);
}

bool MaterialPerson::store(uint materialId)
{
	bool checks = true;
	if(!preparePerson())
		checks = false;

	if(!prepareRole())
		checks = false;
		
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return false;
  
  bool result = false;
  if(markForRemovel()){
		// if the checks failed, we do nothing and return true,
		// because the person does not exist and so no relation can be found!
		if(!checks)
			return true;
    result = remove(materialId);
  }
  else{
		if(!checks)
			return false;
    result = insert(materialId);
  }
 
  if(closeDB)
    db.close();
  return result;
}

void MaterialPerson::setRowId(uint rowId)
{
	myRowId = rowId;
}

uint MaterialPerson::rowId() const
{
	return myRowId;
}

bool MaterialPerson::preparePerson()
{
	if(id() > 0)
		return true;
	
	if(!name().isEmpty()){
		// so try to get it from database
		setName(name());
	}
	if(id() > 0)
		return true;
	
	// use must add a new one
	EditMaterialPersons editManu(this);
	if(editManu.exec() != QDialog::Accepted)
		return false;
	
	if(editManu.selectedId() < 1)
		return false;
	
	setId(editManu.selectedId(), false);
	setName(editManu.selectedName(), false);
	setGivenName(editManu.selectedGivenName());
	return true;
}

void MaterialPerson::setRole(uint newId, bool setName)
{
  myRoleId = newId;
	// fetch name of the role
	if(setName){
		setRole(QString(), false);
		if(myRoleId > 0){
			QString queryString = QString("SELECT name FROM meganizer_materialpersonsroles WHERE id=%1").arg(newId);
			if(thread){
				thread->execQuery(this, "setRole", queryString, "");
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.exec( queryString );
					if(query.first())
						// don't set the id afterwards (loop)
						setRole(query.record().value("name").toString(), false);
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

void MaterialPerson::setRole(QString name, bool setId)
{
	myRole = name.trimmed();
	if(setId){
		setRole(0, false);
		if(!myRole.isEmpty()){
			QString queryString = "SELECT id FROM meganizer_materialpersonsroles WHERE name=?";
			if(thread){
				QList<QVariant> values;
				values << name;
				thread->execQuery(this, "setRole", queryString, "", values);
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.prepare(queryString);
					query.addBindValue(name);
					query.exec();
					if(query.first())
						if(query.size() == 1)
							// don't set the id afterwards (loop)
							setRole(query.record().value("id").toUInt(), false);
					if(closeDB)
						db.close();
				}
			}
		}
	}
	emit changed(this);
}

void MaterialPerson::setRole(QSqlRecord record)
{
	if(record.contains("id"))
		setRole(record.value("id").toUInt(), false);
	else if(record.contains("name"))
		setRole(record.value("name").toString(), false);
}

uint MaterialPerson::role() const
{
  return myRoleId;
}

QString MaterialPerson::roleName() const
{
	return myRole;
}

bool MaterialPerson::prepareRole()
{
	if(role() > 0)
		return true;
	
	if(!roleName().isEmpty()){
		// so try to get it from database
		setRole(name());
	}
	if(role() > 0)
		return true;
	
	// use must add a new one
	EditRoles editRoles(roleName());
	if(editRoles.exec() != QDialog::Accepted)
		return false;
	
	if(editRoles.selectedId() < 1)
		return false;
	
	setRole(editRoles.selectedId(), false);
	setName(editRoles.selectedName(), false);
	return true;
}

MaterialPerson* MaterialPerson::copyPerson()
{
	MaterialPerson *newPerson = new MaterialPerson();
	newPerson->setId(id(), false);
	newPerson->setName(name(), false);
	newPerson->setGivenName(givenName());
	newPerson->setRole(role(), false);
	newPerson->setRole(roleName(), false);
	newPerson->setRowId(rowId());
	if(markForRemovel())
		newPerson->setMarkForRemovel();

	return newPerson;
}

bool MaterialPerson::operator==(const MaterialPerson &matPerson) const
{
  if(name() != matPerson.name())
    return false;
  if(givenName() != matPerson.givenName())
    return false;
  if(role() != matPerson.role())
    return false;
  return true;
}

void MaterialPerson::customQueryResult(QVariant requestId, int querySize, QSqlRecord record)
{
	if(requestId.toString() == "setRole")
		if(querySize == 1)
			setRole(record);
}

void MaterialPerson::initMatP()
{
  myRoleId = 0;
  myRowId = 0;
}

bool MaterialPerson::insert(uint materialId)
{
  // check wether person material relation is already present
  QSqlQuery query;
	query.prepare("SELECT * FROM meganizer_materialhaspersons"
								"  WHERE materialsId=:materialsId AND materialPersonsId=:materialPersonsId"
								"    AND materialPersonsRolesId=:materialPersonsRolesId");
	query.bindValue(":materialsId", materialId);
	query.bindValue(":materialPersonsId", id());
	query.bindValue(":materialPersonsRolesId", role());
  if(!query.exec()){
    return false;
  }
  // if entry is already present nothing needs to be done
  if(query.first()){
    return true;
  }
  
  // the role has changed or the relation doesn't exists
	if(rowId() > 0){
		// relation has changed
		query.prepare("UPDATE meganizer_materialhaspersons SET materialPersonsRolesId=:materialPersonsRolesId WHERE id=:id");
		query.bindValue(":materialPersonsRolesId", role());
		query.bindValue(":id", rowId());
	}
	else{
		// the relation doesn't exist
		query.prepare("INSERT INTO meganizer_materialhaspersons (materialsId, materialPersonsId, materialPersonsRolesId)"
									" VALUES(:materialsId, :materialPersonsId, :materialPersonsRolesId)");
		query.bindValue(":materialsId", materialId);
		query.bindValue(":materialPersonsId", id());
		query.bindValue(":materialPersonsRolesId", role());
	}

	return query.exec();
}

bool MaterialPerson::remove(uint materialId)
{
  QSqlQuery query;
	if(rowId() > 0){
		query.prepare("DELETE FROM meganizer_materialhaspersons WHERE id=:id");
		query.bindValue(":id", rowId());
	}
	else{
		query.prepare("DELETE FROM meganizer_materialhaspersons WHERE materialsId=:materialsId"
									"  AND materialPersonsId=:materialPersonsId"
									"  AND materialPersonsRolesId=:materialPersonsRolesId");
		query.bindValue(":materialsId", materialId);
		query.bindValue(":materialPersonsId", id());
		query.bindValue(":materialPersonsRolesId", role());
	}
	return query.exec();
}
