/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MATERIALPERSON_H
#define MATERIALPERSON_H

#include "libmeganizer.h"
#include "person.h"

/**
	* @brief A person related to a medium.
	*
	* A person is associated to a medium with a role.
	* So Mr. X (the person) can be an author (role) for medium Y.
	*/
class LIBMEGANIZER_EXPORT MaterialPerson : public Person
{
  Q_OBJECT
  public:
    MaterialPerson(EQueryThread *thread = NULL, QObject *parent = NULL);
    MaterialPerson(uint personId, uint roleId, uint rowId = 0,
									 EQueryThread *thread = NULL, QObject *parent = NULL);
    MaterialPerson(QString newName, QString newGivenName, uint roleId,
									 EQueryThread *thread = NULL, QObject *parent = NULL);
		MaterialPerson(QString newName, QString newGivenName, QString role,
									 EQueryThread *thread = NULL, QObject *parent = NULL);
    
    bool store(uint materialId);
		void setRowId(uint rowId);
		uint rowId() const;
		bool preparePerson();
    void setRole(uint newId, bool setName = true);
		void setRole(QString name, bool setId = true);
		void setRole(QSqlRecord record);
    uint role() const;
		bool prepareRole();
		QString roleName() const;
		MaterialPerson *copyPerson();
		
    bool operator== ( const MaterialPerson &matPerson) const;

	protected:
		void customQueryResult(QVariant requestId, int querySize, QSqlRecord record);
    
  private:
    void initMatP();
    bool insert(uint materialId);
    bool remove(uint materialId);
    
    QString myRole;
    uint myRoleId, myRowId;
};

#endif // MATERIALPERSON_H
