/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "mediumchildswidget.h"
#include "mediumchildswidget.moc"

#include "medium.h"
#include "mediamodel.h"
#include <elistaddbutton.h>
#include <elistremovebutton.h>
#include <addmedium.h>
#include <etableview.h>

#include <QVBoxLayout>

MediumChildsWidget::MediumChildsWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	model = NULL;
	
	QHBoxLayout *ml = new QHBoxLayout(this);
	setLayout(ml);
	view = new ETableView(this);
	ml->addWidget(view);
	connect(view, SIGNAL(clicked(QModelIndex)), this, SLOT(selectMedium(QModelIndex)));
	
	QVBoxLayout *bl = new QVBoxLayout();
	ml->addLayout(bl);
	
	addButton = new EListAddButton(this);
	bl->addWidget(addButton);
	connect ( addButton, SIGNAL(clicked()), this, SLOT(addMedium()));
	
	delButton = new EListRemoveButton(this);
	bl->addWidget(delButton);
	connect ( delButton, SIGNAL(clicked()), this, SLOT(removeMedium()));
	bl->addStretch();
	
	selectMedium(QModelIndex());
}

void MediumChildsWidget::setMedium(Medium* medium)
{
	setEnabled(false);
	setModified(false);
	if(model){
		model->disconnect(this);
		disconnect(model);
	}
	selectMedium(QModelIndex());
	view->setModel(NULL);
	this->medium = medium;
	if(!medium)
		return;
	model = medium->childs();
	if(!model)
		return;

	connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(setModified()));
	model->configureView(view);
	view->setSelectionBehavior(QTableView::SelectItems);
	view->resizeColumnsToContents();
	selectMedium(QModelIndex());
	setEnabled(true);
	setModified(false);
}

bool MediumChildsWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);
	if(!medium){
		emit error(tr("Could not store the childs. (No medium)"));
		return false;
	}
	// since we're only doing things on the model of the medium,
	// we do not need to store anything here.
	setModified(false);
	return true;
}

void MediumChildsWidget::addMedium(unsigned int mediumId)
{
  if(!model)
    return;

	model->addMedium(mediumId);
}

void MediumChildsWidget::addMedium()
{
  if(!model)
    return;

  AddMedium addMedium(medium, this);
  connectByPass(&addMedium);
  addMedium.exec();
}

void MediumChildsWidget::removeMedium()
{
	if(!index.isValid())
		return;
	Medium *med = model->mediumFromIndex(index);
	if(!med){
		emit error(tr("Could not remove medium!"));
		return;
	}
	med->setMarkForRemovel();
}

void MediumChildsWidget::selectMedium(QModelIndex index)
{
	this->index = index;
	addButton->setDisabled(true);
	delButton->setDisabled(true);
	if(model){
		addButton->setEnabled(true);
		if(index.isValid())
			delButton->setEnabled(true);
	}
}
