/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIUMEDITWIDGET_H
#define MEDIUMEDITWIDGET_H

#include "libmeganizer.h"
#include <ewidget.h>

class Medium;

/**
	* @brief Base class of all widgets, used at the EditMedium dialog.
	*
	* To load all needed stuff, implement the setMedium() function.
	* In that function, you need to take care of NULL media, enable
	* the widget only, if everything is setup correctly and call
	* setModified(false) before exiting the function to reset the
	* modification state.
	* The changes can be stored to the medium in store(). There you
	* must first check whether the medium is NULL and in this case
	* return false.
	*/
class LIBMEGANIZER_EXPORT MediumEditWidget : public EWidget
{
	Q_OBJECT
	public:
		MediumEditWidget(QWidget *parent = 0, Qt::WindowFlags flags = 0);
		/**
			* @brief Called to setup this widget with new medium @p newMedium.
			*
			*/
		virtual void setMedium(Medium *newMedium) = 0;
		/**
			* @brief Called to store all made changes.
			*
			* Emitted error messages in here should be of the form:
			* Could not store "What do you store?" (The reason)
			* Should return false in any case, if no medium is given.
			*/
		virtual bool store(bool useExistingTransaction = false) = 0;
		/**
			* @brief Returns true if the user has done modifications.
			*
			* To indicate, that the current medium is modified, call
			* setModifed().
			*/
		bool modified() const;

	protected slots:
		/**
			* @brief Sets the modifed state to true.
			*
			* Overloaded function to provide a slot without attributes.
			* It calls setModified(true).
			*/
		void setModified();
		
	protected:
		/**
			* @brief Sets the modified state to @p status.
			*
			* If you need a slot without attributes, use setModified().
			* Overload this function, if you need to do something special
			* on changes.
			*/
		virtual void setModified(bool status);
		
		Medium *medium;

	private:
		bool isModified;
};

#endif // MEDIUMEDITWIDGET_H
