/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumevaluationwidget.h"
#include "mediumevaluationwidget.moc"

#include "medium.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QComboBox>

MediumEvaluationWidget::MediumEvaluationWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	setupUi();
	setMedium(medium);
}

MediumEvaluationWidget::MediumEvaluationWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	setupUi();
	setMedium(0);
}

void MediumEvaluationWidget::setMedium(Medium* medium)
{
	setModified(false);
	setEnabled(false);
	this->medium = medium;
	if(!medium)
		return;
	
	setEnabled(true);
	setEvaluationFromMedium();
	setModified(false);
}

bool MediumEvaluationWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);
	if(!medium){
		emit error(tr("Could not store the evaluation. (No medium)"));
		return false;
	}
	
	if(!modified())
		return true;

	medium->setEvaluation(evalEdit->currentText().toInt());
	
	setModified(false);
	return true;
}

void MediumEvaluationWidget::setEvaluationFromMedium()
{
	if(medium)
		evalEdit->setCurrentIndex(evalEdit->findText(QString::number(medium->evaluation())));
}

void MediumEvaluationWidget::setupUi()
{
	this->medium = 0;
	QHBoxLayout *mainLayout = new QHBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	QLabel *label = new QLabel(tr("Evaluation:"), this);
	mainLayout->addWidget(label);
	label->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	label->setMinimumSize(150, 23);
	
	evalEdit = new QComboBox(this);
	evalEdit->setObjectName("evalEdit");
	mainLayout->addWidget(evalEdit);
	evalEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	evalEdit->setMinimumSize(150, 23);
	evalEdit->setMaximumHeight(30);
	QStringList items;
	items << "0" << "1" << "2" << "3" << "4" << "5";
	evalEdit->addItems(items);
	connect(evalEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(setModified()));
}
