/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIUMEXPORT_H
#define MEDIUMEXPORT_H

#include "libmeganizer.h"
#include <eobject.h>
#include "medium.h"

class MediaModel;

class QTextDocument;

#include <QModelIndex>
#include <QMap>
#include <QFileInfo>
#include <QTextTableFormat>

/**
	* @brief Exports one Medium or a list of media in different formats.
	*/
class LIBMEGANIZER_EXPORT MediumExport : public EObject
{
	Q_OBJECT
	public:
		MediumExport(QObject *parent = 0);
		~MediumExport();
		
		/**
			* @return A list with all supported formats in format: name, extensions
			*/
		static QMap<QString, QStringList> supportedFileFormats();

		bool exportMedium(Medium *medium, Medium::ExportFormat layout = Medium::Table, QString fileFormat = QString(), QString file = QString());
		/**
			* @brief Exports all media which are into indexes (if indexes is empty, all media are exported)
			* @param model the model which holds the media which should be exported
			* @param indexes per selected row one index should be into the list, if the list is empty, all media are exported
			* @param layout the layout of the export Medium::ExportFormat
			* @param fileFormat the target format of the export, if empty, it will be extracted from file
			* @param fileFormat if this is also not successfull, a dialog is shown
			* @param file the target file, if not specified, a selection dialog is presented
			* @return true on success
			*/
		bool exportMedium(MediaModel *model, QModelIndexList indexes, Medium::ExportFormat layout = Medium::Table, QString fileFormat = QString(), QString file = QString());
	
	private:
		QFileInfo selectFile(QString file = QString(), QString format = QString());
		QString selectFormat(QString format, QFileInfo file);
		bool addHeader(QTextDocument *document);

		QTextTableFormat tableFormat;
};

#endif // MEDIUMEXPORT_H
