/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumgenrewidget.h"
#include "mediumgenrewidget.moc"

#include "editgenres.h"
#include <esqldatabase.h>

#include <QLineEdit>
#include <QSqlQueryModel>
#include <QCompleter>

MediumGenreWidget::MediumGenreWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumSimpleEditWidget(LineEdit, tr("Genre"), "genre", parent, flags)
{
	complModel = NULL;
	genreEdit = qobject_cast<QLineEdit*>(editWidget());
	addSearchButton(SLOT(searchGenre()));
	loadCompleter();
}

void MediumGenreWidget::searchGenre()
{
  EditGenres editGenres( genreEdit->text(), this );
  connectByPass ( &editGenres );
  if ( editGenres.exec() != QDialog::Accepted )
		return;

	loadCompleter();
	genreEdit->setText(editGenres.selectedName());
}

void MediumGenreWidget::loadCompleter()
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if(!db.open())
			return;
	}
		
	QString query = "SELECT name FROM meganizer_genres ORDER BY name";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		genreEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
}
