/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumgiftwidget.h"
#include "mediumgiftwidget.moc"
#include "ui_mediumgiftwidget.h"

#include "medium.h"
#include "personsmodel.h"
#include "person.h"
#include <esqldatabase.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlDriver>
#include <QItemSelectionModel>
#include <QMenu>
#include <QAction>
#include <QDate>
#include <QSettings>

MediumGiftWidget::MediumGiftWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags), ui(new Ui::MediumGiftWidget)
{
	init();
	setMedium(NULL);
}

void MediumGiftWidget::setMedium(Medium* medium)
{
	setEnabled(false);
	setModified(false);
	this->medium = medium;

	userSelected(QModelIndex());
	presenteeSelected(QModelIndex());
	ui->presenteesView->setModel(NULL);
	ui->usersView->setModel(NULL);
	if(presenteesModel){
		delete presenteesModel;
		presenteesModel = NULL;
	}
	if(userModel){
		delete userModel;
		userModel = NULL;
	}
	qDeleteAll(givenGift);
	givenGift.clear();
	
	if(!medium)
		return;

	setEnabled(true);
	ui->giftOnlyEdit->setChecked(medium->giftOnly());

	presenteesModel = new PersonsModel(NULL, this);
	userModel = new PersonsModel(NULL, this);
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	QSqlQuery query;
	// get own user id
	unsigned int myUserId = 0;
	query.prepare("SELECT id FROM meganizer_users WHERE userName=:userName");
	query.bindValue(":userName", db.userName());
	query.exec();
	if(query.first())
		myUserId = query.record().value("id").toUInt();
	
	// find owner
	QList<int> presentIds;
	if(!query.exec(QString("SELECT presenteeId FROM meganizer_gifts WHERE materialsId=%1").arg(medium->id()))){
		if(closeDB)
			db.close();
		return;
	}
	while(query.next()){
		presentIds << query.record().value("presenteeId").toUInt();
	}
	
	// hide the users, who has already recieved this materials as a gift
	// this is needed, because they can use it as a present of someone else
	// or from whatever reasons, they does not own it anymore
	QList<int> finishedIds;
	if(!query.exec(QString("SELECT presenteeId FROM meganizer_gifts WHERE materialsId=%1 AND finished<>0000-00-00").arg(medium->id()))){
		if(closeDB)
			db.close();
		return;
	}
	while(query.next()){
		finishedIds << query.record().value("presenteeId").toUInt();
	}
	
	// hide the users, who ownes already the same material
	QList<int> ownerIds;
	if(!query.exec(QString("SELECT usersId FROM meganizer_userownmaterials WHERE materialsId=%1").arg(medium->id()))){
		if(closeDB)
			db.close();
		return;
	}
	while(query.next())
		ownerIds << query.record().value("usersId").toUInt();

	query.exec("SELECT * FROM meganizer_users");
	while(query.next()){
		unsigned int userId = query.record().value("id").toUInt();
		// hide own user from all lists
		if(userId == myUserId)
			continue;
		// the user has recieved it already as a present
		if(finishedIds.indexOf(userId) != -1)
			continue;
		if(ownerIds.indexOf(userId) != -1)
			continue;
		
		if(presentIds.indexOf(userId) != -1){
			presenteesModel->addPerson(
												query.record().value("id").toUInt(),
												query.record().value("name").toString(),
												query.record().value("givenName").toString()
											);
		}
		else{
			userModel->addPerson(
												query.record().value("id").toUInt(),
												query.record().value("name").toString(),
												query.record().value("givenName").toString()
											);
		}
	}
	if(closeDB)
		db.close();
	ui->presenteesView->setModel(presenteesModel);
	presenteesModel->hideColumns(ui->presenteesView);
	ui->usersView->setModel(userModel);
	userModel->hideColumns(ui->usersView);
}

bool MediumGiftWidget::store( bool useExistingTransaction )
{
	// we have nothing to do here, because Medium handle all stuff
	if(!medium){
		emit error(tr("Could not store the gifts. (No medium)"));
		return false;
	}
	if(!modified())
		return true;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	// we can enable transactions, if useExistingTransaction is false
	bool transactions = false;
	if(useExistingTransaction)
		transactions = db.driver()->hasFeature(QSqlDriver::Transactions);
	if(transactions)
		db.transaction();
	
	QSqlQuery query;
	
	// finish presentees
	foreach(Person *person, givenGift){
		if(!person)
			continue;
		query.prepare("UPDATE meganizer_gifts SET finished=:finished"
									"  WHERE materialsId=:materialsId AND donerId=:donerId"
									"     AND presenteeId=:presenteeId");
		query.bindValue(":finished", QDate::currentDate());
		query.bindValue(":materialsId", medium->id());
		query.bindValue(":donerId", medium->user());
		query.bindValue(":presenteeId", person->id());
		if(!query.exec()){
			if(transactions)
				db.rollback();
			if(closeDB)
				db.close();
			emit error(tr("Could not store gift changes. (Could not mark gift as given)"));
			return false;
		}
		
		// make the presentee owner of the medium
		query.prepare("INSERT INTO meganizer_userownmaterials"
									"  (materialsId, usersId, boughtPrice, boughtPriceCurrenciesId)"
									"  VALUES(:materialsId, :usersId, :boughtPrice, :boughtPriceCurrenciesId)");
		query.bindValue(":materialsId", medium->id());
		query.bindValue(":usersId", person->id());
		query.bindValue(":boughtPrice", medium->boughtPrice());
		query.bindValue(":boughtPriceCurrenciesId", medium->boughtPriceCurrency());
		if(!query.exec()){
			if(transactions)
				db.rollback();
			if(closeDB)
				db.close();
			emit error(tr("Could not store gift changes. (Could not mark gift as given)"));
			return false;
		}
		
		// remove finished presentees from list
		presenteesModel->takePerson(person);
		givenGift.removeAll(person);
		delete person;
	}

	// remove presentees
	QStringList removeConstraints;
	removeConstraints << QString("materialsId=%1").arg(medium->id());
	removeConstraints << QString("finished=0000-00-00"); //
	for(int i = 0; i < presenteesModel->rowCount(); i++){
		Person *person = presenteesModel->personFromIndex(presenteesModel->index(i,0));
		if(!person)
			continue;
		removeConstraints << QString("presenteeId<>%1").arg(person->id());
	}
	if(!query.exec(QString("DELETE FROM meganizer_gifts WHERE %1").arg(removeConstraints.join(" AND ")))){
		if(transactions)
			db.rollback();
		if(closeDB)
			db.close();
		emit error(tr("Could not store gift changes. (Could not remove presentee)"));
		return false;
	}
	
	// add new users
	QList<int> presentIds;
	if(!query.exec(QString("SELECT * FROM meganizer_gifts WHERE materialsId=%1").arg(medium->id()))){
		if(transactions)
			db.rollback();
		if(closeDB)
			db.close();
		emit error(tr("Could not store gift changes. (Could not add presentee)"));
		return false;
	}
	while(query.next())
		presentIds << query.record().value("presenteeId").toUInt();


	for(int i = 0; i < presenteesModel->rowCount(); i++){
		Person *person = presenteesModel->personFromIndex(presenteesModel->index(i,0));
		if(!person)
			continue;
		if(presentIds.indexOf(person->id()) == -1){
			query.prepare("INSERT INTO meganizer_gifts (materialsId, donerId, presenteeId) VALUES(:materialsId, :donerId, :presenteeId)");
			query.bindValue(":materialsId", medium->id());
			query.bindValue(":donerId", medium->user());
			query.bindValue(":presenteeId", person->id());
			if(!query.exec()){
				if(transactions)
					db.rollback();
				if(closeDB)
					db.close();
				emit error(tr("Could not store gift changes. (Could not add presentee)"));
				return false;
			}
		}
	}
	
	// if no presentee is set and giftOnly is true, remove owner from the owner list
	unsigned int oldUser = medium->user();
	if((medium->giftOnly()) && (presenteesModel->rowCount() == 0)){
		query.prepare("DELETE FROM meganizer_userownmaterials WHERE materialsId=:materialsId AND usersId=:usersId");
		query.bindValue(":materialsId", medium->id());
		query.bindValue(":usersId", medium->user());
		if(!query.exec()){
			if(transactions)
				db.rollback();
			if(closeDB)
				db.close();
			emit error(tr("Could not store gift changes. (Could not remove owner from the list)"));
			return false;
		}
		medium->setUser(0);
	}
	
	if(transactions)
		if(!db.commit())
			medium->setUser(oldUser);
	if(closeDB)
		db.close();
	return true;
}

void MediumGiftWidget::setGiftOnly()
{
	if(!medium)
		return;
	
	medium->setGiftOnly(ui->giftOnlyEdit->isChecked());
	setModified();
}

void MediumGiftWidget::addPresentees()
{
	if(!userModel)
		return;
	if(!presenteesModel)
		return;
	
	QItemSelectionModel *selModel = ui->usersView->selectionModel();
	if(!selModel)
		return;
	
	QModelIndexList indexes = selModel->selectedRows();
	foreach(QModelIndex index, indexes){
		Person *person = userModel->personFromIndex(index);
		if(!person)
			continue;
		userModel->takePerson(person);
		presenteesModel->addPerson(person);
	}
	userSelected(QModelIndex());
	presenteeSelected(QModelIndex());
	setModified();
}

void MediumGiftWidget::removePresentees()
{
	if(!userModel)
		return;
	if(!presenteesModel)
		return;
	
	QItemSelectionModel *selModel = ui->presenteesView->selectionModel();
	if(!selModel)
		return;
	
	QModelIndexList indexes = selModel->selectedRows();
	foreach(QModelIndex index, indexes){
		Person *person = presenteesModel->personFromIndex(index);
		if(!person)
			continue;
		presenteesModel->takePerson(person);
		userModel->addPerson(person);
	}
	userSelected(QModelIndex());
	presenteeSelected(QModelIndex());
	setModified();
}

void MediumGiftWidget::presenteeContextMenu ( QPoint pos )
{
	if(!presenteesModel)
		return;
	
	Person *person = presenteesModel->personFromIndex(ui->presenteesView->indexAt(pos));
	if(!person)
		return;
	
	QMenu menu(this);
	QAction *tmp;
	tmp = menu.addAction(QIcon(), tr("Already given"), this, SLOT(giftGiven()));
	if(person->markForRemovel())
		tmp->setEnabled(false);
	tmp = menu.addAction(QIcon(), tr("Not given"), this, SLOT(giftNotGiven()));
	if(!person->markForRemovel())
		tmp->setEnabled(false);
	menu.addAction(QIcon(), tr("Remove"), this, SLOT(removePresentees()));
	menu.exec(QCursor::pos());
}

void MediumGiftWidget::giftGiven()
{
	if(!presenteesModel)
		return;
	
	QItemSelectionModel *selModel = ui->presenteesView->selectionModel();
	if(!selModel)
		return;
	
	QModelIndexList indexes = selModel->selectedRows();
	if(indexes.size() != 1)
		return;
	
	Person *person = presenteesModel->personFromIndex(indexes[0]);
	if(!person)
		return;
	
	// person is already in the list
	if(givenGift.indexOf(person) != -1)
		return;
	
	givenGift << person;
	person->setMarkForRemovel();
	setModified();
}

void MediumGiftWidget::giftNotGiven()
{
	if(!presenteesModel)
		return;
	
	QItemSelectionModel *selModel = ui->presenteesView->selectionModel();
	if(!selModel)
		return;
	
	QModelIndexList indexes = selModel->selectedRows();
	if(indexes.size() != 1)
		return;
	
	Person *person = presenteesModel->personFromIndex(indexes[0]);
	if(!person)
		return;
	givenGift.removeAll(person);
	if(givenGift.indexOf(person) == -1)
		person->setMarkForRemovel(false);
	setModified();
}

void MediumGiftWidget::userSelected ( QModelIndex index )
{
	userIndex = index;
	
	ui->addPresenteeButton->setEnabled( false );
	if(userIndex.isValid())
		ui->addPresenteeButton->setEnabled(true);
}

void MediumGiftWidget::userDoubleClicked(QModelIndex index)
{
	userSelected(index);
	addPresentees();
}

void MediumGiftWidget::presenteeSelected ( QModelIndex index )
{
	presenteesIndex = index;
	
	ui->removePresenteeButton->setEnabled( false );
	if(presenteesIndex.isValid())
		ui->removePresenteeButton->setEnabled(true);
}

void MediumGiftWidget::presenteeDoubleClicked(QModelIndex index)
{
	presenteeSelected(index);
	removePresentees();
}

void MediumGiftWidget::init()
{
	presenteesModel = NULL;
	userModel = NULL;
	medium = NULL;
	
	ui->setupUi(this);
	connect(ui->addPresenteeButton, SIGNAL(clicked()), this, SLOT(addPresentees()));
	connect(ui->removePresenteeButton, SIGNAL(clicked()), this, SLOT(removePresentees()));
	connect(ui->presenteesView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(presenteeContextMenu(QPoint)));
	connect(ui->presenteesView, SIGNAL(clicked(QModelIndex)), this, SLOT(presenteeSelected(QModelIndex)));
	connect(ui->presenteesView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(presenteeDoubleClicked(QModelIndex)));
	connect(ui->usersView, SIGNAL(clicked(QModelIndex)), this, SLOT(userSelected(QModelIndex)));
	connect(ui->usersView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(userDoubleClicked(QModelIndex)));
	ui->presenteesView->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(ui->giftOnlyEdit, SIGNAL(stateChanged(int)), this, SLOT(setGiftOnly()));
	ui->usersView->setSelectionMode(QAbstractItemView::ExtendedSelection);
	ui->usersView->setSelectionBehavior(QAbstractItemView::SelectRows);
	ui->presenteesView->setSelectionMode(QAbstractItemView::ExtendedSelection);
	ui->presenteesView->setSelectionBehavior(QAbstractItemView::SelectRows);
	
	ui->addPresenteeButton->setIcon(":/icons/1left.png", "arrow-left");
	ui->removePresenteeButton->setIcon(":/icons/1right.png", "arrow-right");

	setEnabled(false);
	userSelected(QModelIndex());
	presenteeSelected(QModelIndex());
}
