/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumimagewidget.h"
#include "mediumimagewidget.moc"

#include "medium.h"
#include <escaledimage.h>
#include <esearchbutton.h>
#include <efiledialog.h>

#include <QSettings>
#include <QVBoxLayout>
#include <QHBoxLayout>

MediumImageWidget::MediumImageWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	setupUi();
	setMedium(medium);
}

MediumImageWidget::MediumImageWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	setupUi();
	setMedium(0);
}

void MediumImageWidget::setMedium(Medium* medium)
{
	setModified(false);
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
// 	if(medium)
// 		connect(medium, SIGNAL(changed()), this, SLOT(setImageFromMedium()));
	
	bool enable = true;
	if(!medium)
		enable = false;
	setEnabled(enable);
	setImageFromMedium();
}

bool MediumImageWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction)
	if(!modified())
		return true;
	if(!medium){
		emit error(tr("Could not store the image. (No medium)"));
		return false;
	}
	medium->setImage(scaledImage->image());
	setModified(false);
	return true;
}

void MediumImageWidget::setImageFromMedium()
{
	if(medium)
		scaledImage->setImage(medium->image());
}

void MediumImageWidget::searchImage()
{
	if(!medium)
		return;

	QStringList filters;
	QString defaultFilter = QString("%1 (*.jpg *.png)").arg(tr("Images"));
	filters << defaultFilter;
	filters << QString("%1 (*.*)").arg(tr("All Files"));

	QSettings settings;
	QString lastDir = settings.value("lastExportDir", "").toString();
	QString wTitle = tr("Search image");
	QString result = EFileDialog::getOpenFileName(this, wTitle, lastDir, filters.join(";;"), &defaultFilter);

	if(result.isEmpty())
		return;
	QImage image(result);
	if(image.isNull()){
		emit error(tr("Could not load selected image"));
		return;
	}
	scaledImage->setImage ( image );
	setModified();
}

void MediumImageWidget::setupUi()
{
	this->medium = 0;
	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	scaledImage = new EScaledImage(this);
	scaledImage->setObjectName("scaledImage");
	mainLayout->addWidget(scaledImage);
// 	scaledImage->setMinimumSize(100, 100);
	
	QHBoxLayout *buttonLayout = new QHBoxLayout();
	mainLayout->addLayout(buttonLayout);
	buttonLayout->addStretch();
	
	ESearchButton *button = new ESearchButton(this);
	buttonLayout->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(searchImage()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	button->setMaximumHeight(30);
}
