/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumkeywordwidget.h"
#include "mediumkeywordwidget.moc"
#include "ui_mediumkeywordwidget.h"

#include <esqldatabase.h>
#include "medium.h"

#include <QSqlQueryModel>
#include <QCompleter>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QStringListModel>

MediumKeywordWidget::MediumKeywordWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags), ui(new Ui::MediumKeywordWidget)
{
	keywordModel = NULL;
	completerModel = new QSqlQueryModel(this);
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		completerModel->setQuery("SELECT * FROM meganizer_keywords ORDER BY keyword");
		if(closeDB)
			db.close();
	}
	ui->setupUi(this);

	QCompleter *completer = new QCompleter(completerModel, this);
	completer->setCaseSensitivity ( Qt::CaseInsensitive );
	completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
	completer->setCompletionColumn(completerModel->record().indexOf("keyword"));
	ui->keywordEdit->setCompleter ( completer );

	setEnabled(false);

	connect(ui->keywordList, SIGNAL(clicked(QModelIndex)), this, SLOT(selectedKeyword(QModelIndex)));
	connect(ui->addButton, SIGNAL(clicked()), this, SLOT(addKeyword()));
	connect(ui->removeButton, SIGNAL(clicked()), this, SLOT(removeKeyword()));
}

bool MediumKeywordWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);

	// we have nothing to do here, because Medium handle all stuff
	if(!medium){
		emit error(tr("Could not store the keywords. (No medium)"));
		return false;
	}
	
	if(!modified())
		return true;

	// add newly added keywords
	// this can be done by adding all stored keywords,
	// because already inserted ones are ignored by medium->addKeyword()
	QStringList keywords = keywordModel->stringList();
	foreach(QString keyword, keywords){
		medium->addKeyword(keyword);
	}

	// remove keywords
	foreach(QString keyword, removeKeywords){
		// it was present at the medium, so we must remove it,
		// all keywords which are added within this run,
		// wasn't submitted to the medium and therefore they are
		// not into the oldKeywords list.
		if(oldKeywords.contains(keyword, Qt::CaseInsensitive)){
			if(!medium->removeKeyword(keyword)){
				emit error(tr("Could not remove keyword %1.").arg(keyword));
				return false;
			}
		}
	}

	setModified(false);
	return true;
}

void MediumKeywordWidget::setMedium(Medium* newMedium)
{
	setModified(false);
	selectedKeyword(QModelIndex());
	oldKeywords.clear();
	removeKeywords.clear();
	keywordModel = NULL;

	medium = newMedium;
	if(!medium){
		setEnabled(false);
		return;
	}

	QStringListModel *model = medium->keywords();
	if(!model)
		return;
	
	oldKeywords = model->stringList();
	keywordModel = new QStringListModel(oldKeywords);
	ui->keywordList->setModel(keywordModel);
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	completerModel->setQuery(completerModel->query().lastQuery());
	if(closeDB)
		db.close();
	
	setEnabled(true);
}

void MediumKeywordWidget::addKeyword()
{
	if(ui->keywordEdit->text().isEmpty() || !keywordModel)
		return;

	setModified();
	keywordModel->insertRow(keywordModel->rowCount());
	keywordModel->setData(keywordModel->index(keywordModel->rowCount() - 1, 0), ui->keywordEdit->text());
	ui->keywordEdit->setText(QString());
}

void MediumKeywordWidget::removeKeyword()
{
	if(!keywordModel)
		return;

	QModelIndexList indexes = ui->keywordList->selectionModel()->selectedRows();
	if(indexes.size() < 1)
		return;

	foreach(QModelIndex index, indexes){
		QString keyword = ui->keywordList->model()->data(index).toString();
		if(!keyword.isEmpty()){
			for(int i = 0; i < keywordModel->rowCount(); i++){
				if(keywordModel->data(keywordModel->index(i, 0), Qt::DisplayRole).toString().toUpper() == keyword.toUpper()){
					if(!keywordModel->removeRow(i)){
						emit warning(tr("Could not remove keyword %1!").arg(keyword));
						return;
					}
					removeKeywords << keyword;
				}
			}
			setModified();
		}
	}
}

void MediumKeywordWidget::selectedKeyword(QModelIndex index)
{
	this->index = index;

	ui->removeButton->setEnabled(false);
	if(!index.isValid())
		return;
	ui->removeButton->setEnabled(true);
}
