/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumloanwidget.h"
#include "mediumloanwidget.moc"

#include "medium.h"
#include "loanmodel.h"
#include "loanitem.h"
#include <esqldatabase.h>
#include <etableview.h>

#include <QSqlQueryModel>
#include <QSqlRecord>
#include <QSqlQuery>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QSqlError>

MediumLoanWidget::MediumLoanWidget ( QWidget* widget, Qt::WindowFlags flags )
	: MediumEditWidget ( widget, flags )
{
	model = NULL;
	medium = NULL;
	myUserId = 0;
	
	setupUi();
}

MediumLoanWidget::~MediumLoanWidget()
{
	delete model;
}

void MediumLoanWidget::setMedium ( Medium* medium )
{
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
	// load model
	loanStateView->setModel(0);
	if(model)
		delete model;
	
	model = new LoanModel(medium, myUserId, this);
	connect(model, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
	connect(model, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
	connect(model, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(setModified()));
	loanStateView->setModel(model);
	model->addItemDelegates(loanStateView);
	model->hideAndResizeColumns(loanStateView);
	
	bool enable = true;
	if(!medium)
		enable = false;
	setEnabled(enable);
	
	selectionChanged(QModelIndex());
	setModified(false);
}

bool MediumLoanWidget::store(bool useExistingTransaction)
{
	if(!model){
		emit error(tr("Could not store loan changes. (No loan model)"));
		return false;
	}
	bool result = model->store();
	if(result)
		setModified(false);
	return result;
}

QStringList MediumLoanWidget::statiMap(QString name)
{
	// first setup translations
	QMap<QString, QStringList> map;
	QStringList translations;
	translations << tr("Loan Requested") << tr("Request Loan") << tr ( "The medium is requested for loaning" );
	map.insert("Loan Requested", translations);
	translations.clear();
	translations << tr ( "Loan Denied" ) << tr("Denie Loan") << tr ( "The loan request was denied" );
	map.insert("Loan Denied", translations);
	translations.clear();
	translations << tr ( "Loaned" ) << tr("Loan") << tr ( "The medium is loaned to the user" );
	map.insert("Loaned", translations);
	translations.clear();
	translations << tr ( "Return Requested" ) << tr("Request Return") << tr( "The owner requested the recipient, to give the medium back" );
	map.insert("Return Requested", translations);
	translations.clear();
	translations << tr ( "Returned" ) << tr("Returned") << tr( "The medium was given back to owner" );
	map.insert("Returned", translations);
	translations.clear();
	translations << tr ( "Request Canceled" ) << tr("Cancel Request") << tr( "The request was canceled" );
	map.insert("Request Canceled", translations);
	
	return map.value(name, QStringList());
}

void MediumLoanWidget::selectionChanged ( QModelIndex index )
{
	this->index = index;
	
	foreach(QPushButton *button, buttonList)
		button->setEnabled(false);

	if(buttonList.size() < 6)
		return;
	
	if(myUserId == 0)
		return;

	LoanItem *item = model->loanItemFromIndex(index);
	if(!item)
		return;

	QList<unsigned int> allowedStati = item->validStatis();
	foreach(unsigned int status, allowedStati)
		buttonList[status - 1]->setEnabled(true);
}

void MediumLoanWidget::changeState()
{
	// determine status
	QPushButton *button = qobject_cast<QPushButton*>(sender());
	if(!button)
		return;
	// add 1 to the index of button in buttonList, because id start's at 1
	LoanItem *item = model->loanItemFromIndex(index);
	if(!item)
		return;
	item->setStatus(buttonList.indexOf(button) + 1);
	setModified();
}

void MediumLoanWidget::setupUi()
{
	this->medium = 0;
		
	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	QHBoxLayout *viewLayout = new QHBoxLayout();
	mainLayout->addLayout(viewLayout);
	
	loanStateView = new ETableView(this);
	viewLayout->addWidget(loanStateView);
	connect(loanStateView, SIGNAL(clicked(QModelIndex)), this, SLOT(selectionChanged(QModelIndex)));
	
	// setup the state buttons
	
	QVBoxLayout *buttonLayout = new QVBoxLayout();
	viewLayout->addLayout(buttonLayout);
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		query.exec("SELECT * FROM meganizer_loanstati ORDER BY id");
		while(query.next()){
			QString name = query.record().value("name").toString();
			QStringList translations = statiMap(name);
			if(translations.size() != 3)
				translations << name << name << name;
			
			QPushButton *button = new QPushButton(translations[1], this);
			buttonList << button;
			buttonLayout->addWidget(button);
			connect(button, SIGNAL(clicked()), this, SLOT(changeState()));
			button->setToolTip(translations[2]);
			button->setObjectName("status"+ query.record().value("id").toString());
		}
		buttonLayout->addStretch();
		
		// set our own userId
		query.prepare("SELECT * FROM meganizer_users WHERE userName=:userName");
		query.bindValue(":userName", db.userName());
		query.exec();
		if(query.first())
			myUserId = query.record().value("id").toUInt();
		
		if(closeDB)
			db.close();
	}
	
	connect(loanStateView, SIGNAL(clicked(QModelIndex)), this, SLOT(selectionChanged(QModelIndex)));
}
