/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumownerwidget.h"
#include "mediumownerwidget.moc"
#include "ui_mediumownerwidget.h"

#include "personsmodel.h"
#include "medium.h"
#include "person.h"
#include <esqldatabase.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlDriver>

MediumOwnerWidget::MediumOwnerWidget ( Medium* newMedium, QWidget* widget, Qt::WindowFlags flags )
	: MediumEditWidget ( widget, flags ), ui(new Ui::MediumOwnerWidget)
{
	init();
	setMedium(newMedium);
}

MediumOwnerWidget::MediumOwnerWidget ( QWidget* widget, Qt::WindowFlags flags )
	: MediumEditWidget ( widget, flags ), ui(new Ui::MediumOwnerWidget)
{
	init();
}

void MediumOwnerWidget::setMedium ( Medium* newMedium )
{
	medium = newMedium;

	setModified(false);
	setEnabled(false);
	ownerSelected(QModelIndex());
	userSelected(QModelIndex());
	ui->ownerView->setModel(0);
	ui->userView->setModel(0);
	delete ownerModel;
	delete userModel;
	ownerModel = 0;
	userModel = 0;
	
	if(!medium)
		return;
	
	ownerModel = new PersonsModel(NULL, this);
	userModel = new PersonsModel(NULL, this);
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();

	QSqlQuery query;
	// find owner
	QList<int> presentIds;
	if(!query.exec(QString("SELECT * FROM meganizer_userownmaterials WHERE materialsId=%1").arg(medium->id()))){
		if(closeDB)
			db.close();
		return;
	}
	while(query.next()){
		presentIds << query.record().value("usersId").toUInt();
	}
	
	query.exec("SELECT * FROM meganizer_users");
	while(query.next()){
		unsigned int userId = query.record().value("id").toUInt();
		if(presentIds.indexOf(userId) != -1){
			ownerModel->addPerson(
				query.record().value("id").toUInt(),
				query.record().value("name").toString(),
				query.record().value("givenName").toString()
			);
		}
		else{
			userModel->addPerson(
				query.record().value("id").toUInt(),
				query.record().value("name").toString(),
				query.record().value("givenName").toString()
			);
		}
	}
	if(closeDB)
		db.close();

	ui->ownerView->setModel(ownerModel);
	ownerModel->hideColumns(ui->ownerView);
	ui->userView->setModel(userModel);
	userModel->hideColumns(ui->userView);
	setModified(false);
	setEnabled(true);
}

bool MediumOwnerWidget::store(bool useOldTransaction)
{
	if(!medium){
		emit error(tr("Could not store the owner changes. (No medium)"));
		return false;
	}
	
	if(!modified())
		return true;
		
	if(!ownerModel){
		emit error(tr("Could not store the owner changes. (No owner model)"));
		return false;
	}
	
	if(!userModel){
		emit error(tr("Could not store the owner changes. (No user model)"));
		return false;
	}
	
	QStringList removeConstraints;
	removeConstraints << QString("materialsId=%1").arg(medium->id());
	for(int i = 0; i < ownerModel->rowCount(); i++){
		Person *person = ownerModel->personFromIndex(ownerModel->index(i,0));
		if(!person)
			continue;
		removeConstraints << QString("usersId<>%1").arg(person->id());
	}
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	bool transactions = false;
	if(!useOldTransaction){
		transactions =  db.driver()->hasFeature(QSqlDriver::Transactions);
		if(transactions)
			db.transaction();
	}
	
	// remove not needed users
	QSqlQuery query;
	if(!query.exec(QString("DELETE FROM meganizer_userownmaterials WHERE %1").arg(removeConstraints.join(" AND ")))){
		if(transactions)
			db.rollback();
		if(closeDB)
			db.close();
		emit error(tr("Could not store owner changes. (Could not remove not needed users)"));
		return false;
	}
	
	// add new users
	QList<int> presentIds;
	if(!query.exec(QString("SELECT * FROM meganizer_userownmaterials WHERE materialsId=%1").arg(medium->id()))){
		if(transactions)
			db.rollback();
		if(closeDB)
			db.close();
		emit error(tr("Could not store owner changes. (Could not get existing owners)"));
		return false;
	}
	while(query.next())
		presentIds << query.record().value("usersId").toUInt();

	for(int i = 0; i < ownerModel->rowCount(); i++){
		Person *person = ownerModel->personFromIndex(ownerModel->index(i,0));
		if(!person)
			continue;
		if(presentIds.indexOf(person->id()) == -1){
			query.prepare("INSERT INTO meganizer_userownmaterials (materialsId, usersId, boughtPriceCurrenciesId)"
										"  VALUES(:materialsId, :usersId, :boughtPriceCurrenciesId)");
			query.bindValue(":materialsId", medium->id());
			query.bindValue(":usersId", person->id());
			query.bindValue(":boughtPriceCurrenciesId", medium->boughtPriceCurrency());
			if(!query.exec()){
				if(transactions)
					db.rollback();
				if(closeDB)
					db.close();
				emit error(tr("Could not store owner changes. (Could not insert new owner)"));
				return false;
			}
		}
	}
	if(transactions)
		db.commit();
	if(closeDB)
		db.close();

	setModified(false);
	return true;
}

void MediumOwnerWidget::ownerSelected ( QModelIndex index )
{
	ownerIndex = index;
	
	ui->removeButton->setEnabled( false );
	if(ownerIndex.isValid())
		ui->removeButton->setEnabled(true);
}

void MediumOwnerWidget::ownerDoubleClicked(QModelIndex index)
{
	ownerSelected(index);
	removeOwner();
}

void MediumOwnerWidget::userSelected ( QModelIndex index )
{
	userIndex = index;
	
	ui->addButton->setEnabled( false );
	if(userIndex.isValid())
		ui->addButton->setEnabled(true);
}

void MediumOwnerWidget::userDoubleClicked(QModelIndex index)
{
	userSelected(index);
	addOwner();
}

void MediumOwnerWidget::addOwner()
{
	setModified();
	QModelIndexList selected = ui->userView->selectionModel()->selectedRows();
	foreach(QModelIndex index, selected){
		Person *person = userModel->takePerson(index);
		if(!person)
			return;
	
		ownerModel->addPerson(person);
		userSelected(QModelIndex());
		ownerSelected(QModelIndex());
	}
}

void MediumOwnerWidget::removeOwner()
{
	setModified();
	QModelIndexList selected = ui->ownerView->selectionModel()->selectedRows();
	foreach(QModelIndex index, selected){
		Person *person = ownerModel->takePerson(index);
		if(!person)
			return;
		
		userModel->addPerson(person);
		userSelected(QModelIndex());
		ownerSelected(QModelIndex());
	}
}

void MediumOwnerWidget::init()
{
	medium = NULL;
	ownerModel = NULL;
	userModel = NULL;
	
	ui->setupUi(this);
	connect(ui->addButton, SIGNAL(clicked()), this, SLOT(addOwner()));
	connect(ui->removeButton, SIGNAL(clicked()), this, SLOT(removeOwner()));
	connect(ui->ownerView, SIGNAL(clicked(QModelIndex)), this, SLOT(ownerSelected(QModelIndex)));
	connect(ui->ownerView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(ownerDoubleClicked(QModelIndex)));
	connect(ui->userView, SIGNAL(clicked(QModelIndex)), this, SLOT(userSelected(QModelIndex)));
	connect(ui->userView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(userDoubleClicked(QModelIndex)));
	
	ownerSelected(QModelIndex());
	userSelected(QModelIndex());
	
	ui->addButton->setIcon("arrow-left", ":/icons/1left.png");
	ui->removeButton->setIcon("arrow-right", ":/icons/1right.png");
}
