/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "mediumparentswidget.h"
#include "mediumparentswidget.moc"
#include "ui_mediumparentswidget.h"

#include "medium.h"
#include "mediamodel.h"
#include <esqldatabase.h>

#include <QSqlQuery>

MediumParentsWidget::MediumParentsWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags), ui(new Ui::MediumParentsWidget)
{
	ui->setupUi(this);
}

void MediumParentsWidget::setMedium(Medium* child)
{
	setEnabled(false);
	setModified(false);
	medium = child;
	if(!child)
		return;
	ui->view->setModel(NULL);

	if(child->id() < 1)
		return;

	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
	
	MediaModel *childs = child->childs();
	// build up list of child id's
	QStringList whereStrings;
	whereStrings << QString("hasMaterialsId=%1").arg(child->id());
	if(childs){
		for(int i = 0; i < childs->rowCount(); i++){
			Medium *med = childs->mediumFromIndex(childs->index(i, 0));
			if(med)
				whereStrings << QString("hasMaterialsId=%1").arg(med->id());
		}
	}
	QString inString = " AND id IN (SELECT materialsId FROM meganizer_materialhasmaterials WHERE "+ whereStrings.join(" OR ") +")";
	QSqlQuery query;
	// exclude ourselve, beacuse otherwise, we may list as related media, because we are listed at our child's
	// as a parent
	query.exec(QString("SELECT * FROM viewmaterials WHERE id<>%1"
		" AND id IN (SELECT materialsId FROM meganizer_materialhasmaterials WHERE %2)")
		.arg(child->id()).arg(whereStrings.join(" OR ")));
	MediaModel *model = new MediaModel(NULL, this);
	while(query.next())
		model->addMedium(query.record());
	if(closeDB)
		db.close();
	
	model->configureView(ui->view);
	setEnabled(true);
	setModified(false);
}

bool MediumParentsWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);
	if(!medium){
		emit error(tr("Could not store the parents changes. (No medium)"));
		return false;
	}
	if(!modified())
		return true;

	setModified(false);
	return true;
}
