/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumpersonswidget.h"
#include "mediumpersonswidget.moc"

#include "medium.h"
#include "editmaterialpersons.h"
#include "editroles.h"
#include "materialpersonsmodel.h"
#include "materialperson.h"
#include <elistaddbutton.h>
#include <elistremovebutton.h>
#include <etableview.h>

#include <QHBoxLayout>
#include <QVBoxLayout>

MediumPersonsWidget::MediumPersonsWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	setupUi();
	setMedium(NULL);
}

void MediumPersonsWidget::setDefaultRole(unsigned int roleId)
{
	defaultRole = roleId;
}

void MediumPersonsWidget::setMedium(Medium* medium)
{
	setEnabled(false);
	this->medium = medium;
	
	clicked(QModelIndex());
	personsView->setModel(0);
	
	if(!medium)
		return;

	MaterialPersonsModel *personModel = medium->materialPersons();
	if(personModel){
		personsView->setModel(personModel);
		personModel->addItemDelegates(personsView);
		personModel->hideColumns(personsView);
		setEnabled(true);
		connect(personModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(setModified()));
	}
	setModified(false);
}

bool MediumPersonsWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);
	// we have nothing to do here, because Medium handle all stuff
	if(!medium){
		emit error(tr("Could not store the material persons. (No medium)"));
		return false;
	}

	// since we're only doing things on the model of the medium,
	// we do not need to store anything here.
	setModified(false);
	return true;
}

void MediumPersonsWidget::addPerson()
{
	if(!medium)
		return;
	
	EditMaterialPersons *editPersons = new EditMaterialPersons(this);
	connectByPass(editPersons);
	if(editPersons->exec() != QDialog::Accepted){
		delete editPersons;
		return;
	}
	
	MaterialPersonsModel *personModel = medium->materialPersons();
	if(!personModel){
		delete editPersons;
		return;
	}

	MaterialPerson *person = qobject_cast<MaterialPerson*>(personModel->addPerson());
	if(!person){
		delete editPersons;
		return;
	}
	person->setId(editPersons->selectedId(), false);
	person->setName(editPersons->selectedName(), false);
	person->setGivenName(editPersons->selectedGivenName());
	person->setRole(defaultRole);
	
	delete editPersons;
}

void MediumPersonsWidget::removePerson()
{
  QModelIndexList indexes = personsView->selectionModel()->selectedIndexes();
	if(indexes.size() < 1)
		return;

	foreach(QModelIndex index, indexes){
		medium->removeMaterialPerson(index);
		clicked(QModelIndex());
	}
}

void MediumPersonsWidget::newRole()
{
	EditRoles *editRoles = new EditRoles(this);
	connectByPass(editRoles);
	if(editRoles->exec() == QDialog::Accepted)
		personsView->reloadDelegates();

	delete editRoles;
}

void MediumPersonsWidget::clicked(QModelIndex index)
{
	this->index = index;
	
	removeButton->setEnabled(false);
	if(index.isValid())
		removeButton->setEnabled(true);
}

void MediumPersonsWidget::setupUi()
{
	this->medium = 0;
	setDefaultRole(0);
	
	QHBoxLayout *mainLayout = new QHBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	personsView = new ETableView(this);
	mainLayout->addWidget(personsView);
	personsView->setObjectName("personsView");
	connect(personsView, SIGNAL(clicked(QModelIndex)), this, SLOT(clicked(QModelIndex)));
	personsView->setSelectionBehavior(QTableView::SelectRows);
	personsView->setSelectionMode(QTableView::ExtendedSelection);
	personsView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	
	QVBoxLayout *buttonLayout = new QVBoxLayout();
	mainLayout->addLayout(buttonLayout);

	EListAddButton *button = new EListAddButton(this);
	buttonLayout->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(addPerson()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	
	removeButton = new EListRemoveButton(this);
	buttonLayout->addWidget(removeButton);
	connect(removeButton, SIGNAL(clicked()), this, SLOT(removePerson()));
	removeButton->setObjectName("removeButton");
	removeButton->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	removeButton->setEnabled(false);
	
	EPushButton *button1 = new EPushButton(tr("New Role"), this);
	buttonLayout->addWidget(button1);
	connect(button1, SIGNAL(clicked()), this, SLOT(newRole()));
	button1->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	
	buttonLayout->addStretch();

	setTabOrder(personsView, button);
	setTabOrder(button, removeButton);
	setTabOrder(removeButton, button1);
}
