/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumpricewidget.h"
#include "mediumpricewidget.moc"
#include "ui_mediumpricewidget.h"

#include "medium.h"
#include <esqldatabase.h>

#include <QSqlQueryModel>
#include <QSqlRecord>
#include <QCompleter>

MediumPriceWidget::MediumPriceWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags), ui(new Ui::MediumPriceWidget)
{
	init();
	setPrice(ListPrice);
	setMedium(0);
}

void MediumPriceWidget::setMedium(Medium* medium)
{
	this->medium = medium;

	setEnabled(false);
	if(!medium)
		return;
	
	if(price() == BoughtPrice){
		ui->priceEdit->setValue(medium->boughtPrice());
		ui->currencyEdit->setText(mapCurrencies.value(medium->boughtPriceCurrency()));
	}
	else{
		ui->priceEdit->setValue(medium->listPrice());
		ui->currencyEdit->setText(mapCurrencies.value(medium->listPriceCurrency()));
	}
	setModified(false);
	setEnabled(true);
}

bool MediumPriceWidget::store ( bool useExistingTransaction )
{
	Q_UNUSED(useExistingTransaction);
	if(!medium){
		emit error(tr("Could not store price. (No medium given)"));
		return false;
	}
	if(price() == BoughtPrice){
		medium->setBoughtPrice(ui->priceEdit->value());
		medium->setBoughtPriceCurrency(mapCurrencies.key(ui->currencyEdit->text()));
	}
	else{
		medium->setListPrice(ui->priceEdit->value());
		medium->setListPriceCurrency(mapCurrencies.key(ui->currencyEdit->text()));
	}
	return true;
}

void MediumPriceWidget::setPrice ( MediumPriceWidget::Price price )
{
	myPrice = price;
	if(price == BoughtPrice)
		ui->priceLabel->setText(tr("Bought Price:"));
	else
		ui->priceLabel->setText(tr("List Price:"));
}

MediumPriceWidget::Price MediumPriceWidget::price() const
{
	return myPrice;
}

void MediumPriceWidget::init()
{
	this->medium = NULL;
	complModel = NULL;
	
	ui->setupUi(this);
	connect(ui->priceEdit, SIGNAL(editingFinished()), this, SLOT(setModified()));
	connect(ui->currencyEdit, SIGNAL(editingFinished()), this, SLOT(setModified()));
	loadCompleter();
}

void MediumPriceWidget::loadCompleter()
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
		
	QString query = "SELECT id, shortName FROM meganizer_currencies ORDER BY shortName";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		ui->currencyEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
	
	for(int i = 0; i < complModel->rowCount(); i++){
		QSqlRecord result = complModel->record(i);
		mapCurrencies.insert(result.value("id").toUInt(), result.value("shortName").toString());
	}
}
