/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumtitlewidget.h"
#include "mediumtitlewidget.moc"

#include "medium.h"
#include <esqldatabase.h>

#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSqlQueryModel>
#include <QCompleter>


MediumTitleWidget::MediumTitleWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumSimpleEditWidget(LineEdit, tr("Title"), "title", parent, flags)
{
	titleEdit = qobject_cast<QLineEdit*>(editWidget());
	connect(titleEdit, SIGNAL(textChanged(QString)), this, SLOT(checkLength()));
	complModel = NULL;
	loadCompleter();
}

void MediumTitleWidget::checkLength()
{
	if(titleEdit->text().length() <= 120)
		return;
	
	titleEdit->setText(titleEdit->text().left(120));
	emit information(tr("Title can not be longer than 120 characters"));
}

void MediumTitleWidget::loadCompleter()
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
		
	QString query = "SELECT title FROM meganizer_materials ORDER BY name LIMIT 500";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		titleEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
}
