/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "mediumwidget.h"

#include <medium.h>
#include <materialpersonsmodel.h>
#include <searchplugin.h>

#include <QPainter>
#include <QPalette>

MediumWidget::MediumWidget(QWidget* parent, Qt::WindowFlags flags)
	: EWidget(parent, flags)
{
	medium = 0;
	
	borderPen = QPen ( QColor ( 0, 0, 0 ) );
  textPen = QPen ( QColor ( 0, 0, 0 ) );
  
  notFoundBrush = QBrush ( QColor ( 244, 12, 13, 60 ) );
  notEnoughBrush = QBrush ( QColor ( 255, 132, 0, 60 ) );
  foundBrush = QBrush ( QColor ( 20, 96, 0, 60 ) );

  rowHeight = 20;
  titleHeight = 25;
  padding = 5;
  margin = 4;
	maxWidth = 200;
	
	setMode(Standard);
}

void MediumWidget::setMode(Mode mode)
{
	myMode = mode;
	update();
}

void MediumWidget::setMedium(Medium* medium)
{
	this->medium = medium;
	update();
}

void MediumWidget::setBuyData(BuyData buyData, unsigned int quantity)
{
	this->buyData = buyData;
	this->quantity = quantity;
	update();
}

void MediumWidget::paintMedium(QPainter* painter, const QStyleOptionViewItem& option)
{
	QFont titleFont = QFont ( option.font );
  titleFont.setBold ( true );
	QFont normalFont = option.font;
	paintMedium(painter, option.rect, titleFont, normalFont, option.palette.window());
}

void MediumWidget::setMaxWidth(unsigned int maxWidth)
{
	this->maxWidth = maxWidth;
}

QSize MediumWidget::sizeHint() const
{
	int height = 120;
	if(myMode == Extended)
		height = 500;
	if(maxWidth)
		return QSize ( maxWidth - margin, height );
	return QSize ( 0, height );
}

void MediumWidget::paintEvent(QPaintEvent* event)
{
	QFont normalFont = font();
	QFont titleFont = font();
	titleFont.setBold ( true );
	
  QPainter *painter = new QPainter(this);
	if(!paintMedium(painter, rect(), titleFont, normalFont, palette().window()))
		QWidget::paintEvent(event);
	delete painter;
}

bool MediumWidget::paintMedium(QPainter* painter, QRect drawInRect, QFont titleFont, QFont normalFont, QBrush backgroundBrush)
{
	if(!medium)
		return false;
	
	if(!painter)
		return false;
	
	QString persons;
	MaterialPersonsModel *personModel = medium->materialPersons();
	if(personModel)
		persons = personModel->combinePersons();
  

  float listPrice = medium->listPrice();

  QString currency;
	if(buyData.isValid() || myMode == Shopping){
		backgroundBrush = notFoundBrush;
		if ( buyData.inCatalog ) {
			listPrice = buyData.price;
	    currency = buyData.currencyShortName;
	    if ( buyData.available >= quantity ) {
	      backgroundBrush = foundBrush;
	    } else {
	      backgroundBrush = notEnoughBrush;
	    }
		}
	}
  
  // if it is a used item, display it with a dense pattern
  if(buyData.isUsed)
    backgroundBrush.setStyle(Qt::Dense1Pattern);
  
  QRect border = drawInRect.adjusted ( margin, margin, -margin, -margin );

  QRect image = QRect ( border.left() + padding, border.top() + padding, border.width() - padding, 3 * (rowHeight + padding) );
	
	QPixmap pixmap;
	if ( !medium->image().isNull() ) {
		pixmap = QPixmap::fromImage(medium->image().scaled ( image.width(), image.height(), Qt::KeepAspectRatio ));
		image.setWidth ( pixmap.width() + padding );
	} else {
		image.setWidth ( 0 );
	}

  QRect rectTitle = QRect ( image.right() + padding, image.top(), border.width() - image.width() - padding, titleHeight );

//   QFont titleFont = QFont ( option.font );
  QRect rectPerson = QRect ( rectTitle.left(), rectTitle.bottom() + padding, rectTitle.width(), rowHeight );
  QRect rectNumber = QRect ( rectPerson.left(), rectPerson.bottom() + padding, rectPerson.width(), rowHeight );
	QRect rectSource = QRect ( rectNumber.right() - rowHeight, border.bottom() - 2 *  padding - rowHeight, rowHeight, rowHeight );
  QRect rectPrice = QRect ( rectNumber.left(), border.bottom() - 2 * padding - rowHeight, rectSource.left() - padding, rowHeight );
	if(myMode == Extended )
		rectPrice = QRect ( image.left(), image.bottom() + padding, border.width() - 2 * padding, border.bottom() - 2 * padding - image.bottom() );
	
  painter->save();
//   painter->setFont ( option.font );
	painter->setFont ( normalFont );
  painter->setPen ( borderPen );
  painter->fillRect ( border, backgroundBrush );

  if ( !pixmap.isNull() )
    painter->drawPixmap ( image, pixmap );

  painter->setPen ( textPen );

  painter->setFont ( titleFont );

  painter->drawText ( rectTitle, Qt::AlignLeft | Qt::AlignVCenter, truncateString ( painter->fontMetrics(), medium->title(), rectTitle ) );

  painter->setFont ( normalFont );

  painter->drawText ( rectPerson, Qt::AlignLeft | Qt::AlignVCenter, truncateString ( painter->fontMetrics() , persons, rectPerson ) );

  painter->drawText ( rectNumber, Qt::AlignLeft | Qt::AlignVCenter, truncateString ( painter->fontMetrics(), "EAN: "+ medium->min("EAN"), rectNumber ) );

	if(buyData.isValid() || myMode == Shopping){
		if(buyData.shop)
			buyData.shop->icon().paint(painter, rectSource);
		
		painter->drawText ( rectPrice, Qt::AlignLeft | Qt::AlignVCenter,
												truncateString ( painter->fontMetrics(), QString ( "%1 x %2 %3 = %4 %3" )
																				.arg ( quantity )
																				.arg ( QString::number ( listPrice, 'f', 2 ) )
																				.arg ( currency )
																				.arg ( QString::number ( float(quantity) * listPrice, 'f', 2 ) ),
																				rectPrice )
											);
	}
	else{
		painter->drawText ( rectPrice, Qt::AlignLeft | Qt::AlignTop | Qt::TextWordWrap, medium->description() );
	}
  painter->restore();
	
	return true;
}

QString MediumWidget::truncateString ( QFontMetrics metrics, QString string, QRect rect ) const
{
  QString orig = string;
  bool truncated = false;

  while ( ( metrics.width ( string ) > rect.width() ) && ( string.length() > 0 ) ) {
    string = string.left ( string.length() - 1 );
    truncated = true;
  }

  if ( !truncated )
    return orig;

  string = string.left ( string.length() - 3 ) + "...";

  return string;
}

#include "mediumwidget.moc"
