/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MINMODEL_H
#define MINMODEL_H

#include "libmeganizer.h"
#include <eabstractitemmodel.h>

class MediumMIN;
class EDelegateForeignKey;
class DelegateMIN;
class EQueryThread;

#include <QModelIndex>
#include <QPointer>
#include <QSqlRecord>

class QAbstractItemView;
class QTableView;

class LIBMEGANIZER_EXPORT MINModel : public EAbstractItemModel
{
	Q_OBJECT
	public:
		MINModel(EQueryThread *thread = NULL, QObject *parent = NULL);
    ~MINModel();
    
    // own API
		void setThread(EQueryThread *thread);
    MediumMIN *addMIN();
		MediumMIN *addMIN(QString min, QString type);
    MediumMIN *addMIN(QString min, unsigned int typeId);
    bool addMIN(QString min, unsigned int typeId, unsigned int rowId);
		bool addMIN(MediumMIN *min);
		bool addMIN(QSqlRecord record);
		bool removeMIN(MediumMIN *min);
		bool removeMIN(QModelIndex index);
    bool store(unsigned int materialId);
    bool loadAllMINs(unsigned int materialId);
		void setupView(QAbstractItemView *view);
		void reloadDelegates();
    void hideColumns(QTableView *view);
		MINModel *copyModel();
		QString combineMINs();
		MediumMIN * min(QString minType);
		QStringList mins(QString minType);
		
		MediumMIN *minFromIndex(QModelIndex index) const;
		QModelIndex indexFromMIN(MediumMIN *min);
		
    // QT API
    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
		bool setData ( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    int columnCount(const QModelIndex& parent = QModelIndex()) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
		Qt::ItemFlags flags(const QModelIndex& index) const;
    int rowCount(const QModelIndex& parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex& child) const;
    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
    
	private slots:
		void queryResult(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
    void itemChanged(MediumMIN *min);
		
	private:
		QList<QPointer<EDelegateForeignKey> > delegates;
    QList<MediumMIN*> items;
		EQueryThread *thread;
};

#endif // MINMODEL_H
