/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "person.h"
#include "person.moc"

#include <esqldatabase.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QVariant>

Person::Person ( EQueryThread *thread, QObject* parent )
	: QObject ( parent )
{
	init(thread);
}

Person::Person ( uint personId, EQueryThread *thread, QObject* parent )
	: QObject ( parent )
{
	init(thread, personId);
}

Person::Person ( QString newName, QString newGivenName, EQueryThread* thread, QObject* parent )
	: QObject ( parent )
{
	init(thread);
	setName(newName, newGivenName);
}

void Person::setThread(EQueryThread* thread)
{
	if(this->thread){
		this->thread->disconnect(this);
		disconnect(this->thread);
	}
	this->thread = thread;
	if(thread)
		connect(thread, SIGNAL(result(QObject*, QVariant, int, QSqlRecord)),
						this, SLOT(queryResult(QObject*, QVariant, int, QSqlRecord)));
}

bool Person::isValid()
{
	if(name().isEmpty())
		return false;
	return true;
}

void Person::setId ( uint personId, bool setPName )
{
	myPersonId = personId;
	if(setPName){
// 		setName(QString(), false);
// 		setGivenName("");
		if(personId > 0){
			QString queryString = QString("SELECT * FROM meganizer_materialpersons WHERE id=%1").arg(personId);
			if(thread){
				thread->execQuery(this, "setName", queryString, "");
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.exec(queryString);
					if(query.first())
						setName(query.record());
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

uint Person::id() const
{
	return myPersonId;
}

void Person::setName ( QString newName, QString givenName, bool setPId )
{
	setGivenName(givenName);
	setName(newName, setPId);
}

void Person::setName(QSqlRecord record)
{
	if(record.contains("givenName"))
		setGivenName(record.value("givenName").toString());
	if(record.contains("name"))
		setName(record.value("name").toString(), false);
}

void Person::setName ( QString newName, bool setPId )
{
	myName = newName.trimmed();
	if(setPId){
		setId(0, false);
		if(!myName.isEmpty()){
			QString queryString = "SELECT * FROM meganizer_materialpersons WHERE name=?, givenName=?";
			if(thread){
				QList<QVariant> values;
				values << newName << givenName();
				thread->execQuery(this, "setId", queryString, "", values);
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.prepare(queryString);
					query.addBindValue(newName);
					query.addBindValue(givenName());
					query.exec();
					if(query.size() == 1){
						if(query.first()){
							setId(query.record().value("id").toUInt(), false);
							setGivenName(query.record().value("givenName").toString());
						}
					}
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

QString Person::name() const
{
	return myName;
}

void Person::setGivenName ( QString newName )
{
	myGivenName = newName.trimmed();
  emit changed(this);
}


QString Person::givenName() const
{
	return myGivenName;
}

QString Person::combine() const
{
	if(givenName().isEmpty())
		return name();
	
	return QString("%1, %2").arg(name()).arg(givenName().left(1));
}

void Person::setMarkForRemovel( bool remove )
{
	shouldBeRemoved = remove;
	emit changed(this);
}

bool Person::markForRemovel() const
{
	return shouldBeRemoved;
}

Person* Person::copyPerson()
{
	Person *newP = new Person();
	newP->setId(id(), false);
	newP->setName(name(), false);
	newP->setGivenName(givenName());
	if(markForRemovel())
		newP->setMarkForRemovel();
	return newP;
}

bool Person::operator== ( const Person& person ) const
{
	if((id() > 0) || (person.id() > 0)){
		if(id() == person.id())
			return true;
		return false;
	}
	
	if(name() != person.name())
		return false;
	
	if(givenName() != person.givenName())
		return false;
	
	return true;
}

bool Person::operator!= ( const Person& person ) const
{
	return !operator==(person);
}

void Person::queryResult(QObject* client, QVariant requestId, int querySize, QSqlRecord record)
{
	if(client != this)
		return;
	
	QString request = requestId.toString();
	if(request == "setName")
		setName(record);
	else if(request == "setId"){
		if(querySize == 1){
			setId(record.value("id").toUInt(), false);
			setGivenName(record.value("givenName").toString());
		}
	}
	else
		customQueryResult(requestId, querySize, record);
}

void Person::customQueryResult(QVariant requestId, int querySize, QSqlRecord record)
{
	Q_UNUSED(requestId);
	Q_UNUSED(querySize);
	Q_UNUSED(record);
}

void Person::init(EQueryThread* thread, int personId)
{
	this->thread = NULL;
	setThread(thread);
	shouldBeRemoved = false;
  setId(personId);
}
