/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PERSON_H
#define PERSON_H

#include "libmeganizer.h"

class EQueryThread;

#include <QObject>
#include <QString>
#include <QSqlRecord>
#include <QVariant>

class LIBMEGANIZER_EXPORT Person : public QObject
{
  Q_OBJECT
  public:
    Person(EQueryThread *thread = NULL, QObject *parent = NULL);
    Person(uint personId, EQueryThread *thread = NULL, QObject *parent = NULL);
    Person(QString newName, QString newGivenName, EQueryThread *thread = NULL, QObject *parent = NULL);
    virtual inline ~Person(){};

		void setThread(EQueryThread *thread);
		virtual bool isValid();
    void setId(uint personId, bool setPName = true);
    uint id() const;
		/**
			* @attention if the whole name  (both name and givenname) of the person should be changed, use this function
			* @attention it takes care, that the givenName is set before the name, so that the correct person is searched from database
			* @attention (Think of Thomas Mann, Heinrich Mann, Klaus Mann
			*/
		void setName(QString newName, QString givenName, bool setPId = true);
		void setName(QSqlRecord record);
		/**
			* @attention if you intend to set the givenname to, use setName(name, givenName) (function above)
			*/
    void setName(QString newName, bool setPId = true);
    QString name() const;
    void setGivenName(QString newName);
    QString givenName() const;
		QString combine() const;
    void setMarkForRemovel( bool remove = true );
    bool markForRemovel() const;
		Person *copyPerson();
		
    bool operator== ( const Person &person ) const;
		bool operator!= ( const Person &person ) const;

	protected slots:
		/**
			* @brief Used to process results from EQueryThread.
			*
			* To process your own requests, implement customQueryResult().
			*/
		void queryResult(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
		
	protected:
		virtual void customQueryResult(QVariant requestId, int querySize, QSqlRecord record);
		void init(EQueryThread *thread, int personId = 0);

		EQueryThread *thread;
		
  private:
    QString myName, myGivenName;
    uint myPersonId;
    bool shouldBeRemoved;
    
  signals:
    void changed(Person *person);
};

#endif // PERSON_H
