/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PERSONSMODEL_H
#define PERSONSMODEL_H

#include "libmeganizer.h"
#include <eabstractitemmodel.h>

class Person;
class EQueryThread;

#include <QModelIndex>
#include <QSqlRecord>

class QTableView;

class LIBMEGANIZER_EXPORT PersonsModel : public EAbstractItemModel
{
  Q_OBJECT
  public:
    PersonsModel(EQueryThread *thread = NULL, QObject *parent = 0);
    virtual inline ~PersonsModel(){};
    
    // own API
		void setThread(EQueryThread *thread);
		virtual Person *addPerson();
		void addPerson(Person *person);
		Person *addPerson(uint personId);
		Person *addPerson(QString name, QString givenName);
		Person *addPerson(uint personId, QString name, QString givenName);
    Person *personFromIndex(QModelIndex index) const;
		virtual QModelIndex indexFromPerson(Person *person);
		virtual void hideColumns(QTableView *view);
		Person *takePerson(Person *person);
		Person *takePerson(QModelIndex index);
		QString combinePersons();
		QStringList persons();
    
    // QT API
    virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
		virtual bool setData ( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
    int rowCount(const QModelIndex& parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex& child) const;
    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
  
	protected slots:
    void itemChanged(Person *person);
		/**
			* @brief Used to process results from EQueryThread.
			*
			* To process your own requests, implement customQueryResult().
			*/
		void queryResult(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
		
	protected:
		virtual void customQueryResult(QVariant requestId, int querySize, QSqlRecord record);
		
    QList<Person*> items;
		EQueryThread *thread;
		int maxCombinedPersons;
};

#endif // PERSONSMODEL_H
