/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SEARCHPLUGIN_H
#define SEARCHPLUGIN_H

#include "libmeganizer.h"
#include "meganizerplugin.h"

#include "buydata.h"
#include "sqlconstraint.h"

#include <QMap>

class QIcon;
class QAbstractItemModel;
class QStringList;

class ESettingsModelItem;
class ESetupWizardObject;
class EWidget;
class MediaModel;
class Medium;
class SearchPlugin;

/**
	* @brief An interface for all search providers.
	*
	* If you want to implement your own SearchPlugin, implement it
	* with the following base code:
	* @code
	* class YourSearchPlugin : public SearchPlugin
	* {
	*   Q_OBJECT
	*   // Both interfaces are needed
	*   Q_INTERFACES ( SearchPlugin MeganizerPlugin )
	*   // Register properties, needed by EPluginLoader to
	*   // access information about this plugin.
	*   Q_PROPERTY(QString name READ name)
	*   Q_PROPERTY(QString description READ description)
	*   Q_PROPERTY(QIcon icon READ icon)
	*   public:
	*     ...
	* };
	* @endcode
	*
	* or copy and adapt for example the Amazon plugin.
	* @see MediaPlugin
	* @see MeganizerPlugin
	*/ 
class LIBMEGANIZER_EXPORT SearchPlugin : public MeganizerPlugin
{
  public:
    virtual ~SearchPlugin() {}

    virtual QString settingsGroup() = 0;

		/**
			* a list of supported attributes (key, translation)
			* This list should contain only special attributes, not supported by Medium.
			*/
    virtual QMap<QString, QString> supportedAttributes() = 0;
    /**
    	* A list of supported Categories which can be searched (key, translation).
    	* The categories should represent different material types.
    	* @see Amazon
    	*/
    virtual QMap<QString, QString> supportedCategories() = 0;
		virtual ESettingsModelItem *settingsModelItem() = 0;
    virtual ESetupWizardObject *wizardObject() = 0;
		/**
			* If any medium is created, the source should be setted to Medium::SearchPlugin using
			* setSource(Medium::SearchPlugin)
			*/
    virtual bool search ( MediaModel *resultModel, QList<SqlConstraint> sqlConstraints ) = 0;
    virtual void recieveBuyData ( Medium *medium ) = 0;
    virtual bool purchase ( QList<BuyData> items, QMap<Medium*, int> quantities ) = 0;
    virtual bool operator== (const SearchPlugin &plugin) const = 0;
    virtual bool operator!= (const SearchPlugin &plugin) const = 0;

  signals:
    virtual void foundBuyData ( const BuyData &data ) = 0;
};

Q_DECLARE_INTERFACE ( SearchPlugin,
											"Meganizer.SearchPlugin/0.4.0" );

#endif // SEARCHPLUGIN_H
