/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SQLCONSTRAINT_H
#define SQLCONSTRAINT_H

#include "libmeganizer.h"

#include <QStringList>
#include <QVariant>

/**
	* @brief Represents an constraint at the WHERE part of an SQL term.
	*
	* The constraint string should use the prepare functionality
	* and should look like this:
	* columnName=?
	* any combination with OR AND is possible also any other condition as =
	* the list returned by bindValues() must contain as many entries as the constraint has placeholders (the ?)
	* and they must have the correct order from first to last placeholder (using positional placeholders here)
	*/
class LIBMEGANIZER_EXPORT SqlConstraint
{
	public:
		SqlConstraint();
		bool isValid();
		QString constraint() const ;
		void setConstraint(QString condition);
		QList<QVariant> bindValues() const;
		void addBindValue(QVariant value);
		bool operator==( const SqlConstraint &sqlConstraint ) const;
		bool operator!=( const SqlConstraint &sqlConstraint ) const;
		
	private:
		QString myConstraint;
		QList<QVariant> myBindValues;
};

#endif // SQLCONSTRAINT_H
