/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "connectmediasearchtest.h"
#include "connectmediasearchtest.moc"

#include <testplugindir.h>
#include "../lib/internal/connectmediasearch.h"
#include <epluginloader.h>
#include <mediaplugin.h>
#include <searchplugin.h>
#include <mediamodel.h>
#include <medium.h>

#include <QtTest>
#include <QSettings>

void ConnectMediaSearchTest::testConnectMediaSearch()
{
	QApplication::setOrganizationName("EngSaS");
	QApplication::setApplicationName("Testing");
	QSettings settings;
	settings.clear();
	
	EModuleListWidgetItem* item = new EModuleListWidgetItem();
	MediaPlugin* books = EPluginLoader<MediaPlugin>::loadPlugin("books", TestPluginDir::libDir(), item);
	if(!books)
		QSKIP("Could not load Books plugin\n"+ item->errorMessage().toLatin1(), SkipSingle);
	SearchPlugin* amazon = EPluginLoader<SearchPlugin>::loadPlugin("amazon", TestPluginDir::libDir(), item);
	if(!amazon)
		QSKIP("Could not load Amazon plugin\n"+ item->errorMessage().toLatin1(), SkipSingle);

	ConnectMediaSearch *search = new ConnectMediaSearch();
	search->setPlugins(0, 0);
	QCOMPARE(search->checkSettings(), false);
	QString error;
	QCOMPARE(search->saveSettings(error), false);

	// find book model
	MediaModel *model = books->newModel("book");
	QVERIFY(model);
	Medium *med = model->newMedium();
	QVERIFY(med);
	search->setPlugins(med, amazon);
	QCOMPARE(search->checkSettings(), false);
	QCOMPARE(search->saveSettings(error), true);
	settings.beginGroup("mapping"+ amazon->settingsGroup() + med->group());
	QCOMPARE(settings.value("numMediaAttrs").toInt(), 4);
	QCOMPARE(settings.value("numSearchAttrs").toInt(), 5);
	QStringList mediaAttrs = settings.value("mediaAttrs").toStringList();
	QStringList searchAttrs = settings.value("searchAttrs").toStringList();
	settings.endGroup();
	
	QCOMPARE(mediaAttrs.size(), searchAttrs.size());
	QCOMPARE(mediaAttrs.size(), 3);
	QVERIFY(mediaAttrs.contains("edition"));
	QVERIFY(mediaAttrs.contains("pages"));
	QVERIFY(mediaAttrs.contains("pubDate"));
	QVERIFY(searchAttrs.contains("edition"));
	QVERIFY(searchAttrs.contains("pages"));
	QVERIFY(searchAttrs.contains("pubDate"));
}

void ConnectMediaSearchTest::testConnectMediaSearchSettings()
{

}

QTEST_MAIN(ConnectMediaSearchTest)
