/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editmediumtest.h"

#include <editmedium.h>
#include <esqldatabase.h>
#include <dbtestinit.h>
#include <medium.h>

#include <QtTest/QtTest>

void EditMediumTest::initTestCase()
{
	editMed = 0;
	dbTest = new DBTestInit(this);
	if(!dbTest->initTestDB())
		QSKIP("Could not setup database", SkipAll);
	
	keys = dbTest->prepareMediumKeys("Testype");
	if(!keys.valid)
		QSKIP("Could not prepare foreignkeys", SkipAll);
	
	editMed = new EditMedium();
}

void EditMediumTest::cleanupTestCase()
{
	if(editMed)
		delete editMed;
}

void EditMediumTest::addMode()
{
	ESqlDatabase db = ESqlDatabase::database();
	db.open();
	
	Medium *med = new Medium();
	editMed->setAddMode(med, false);
	// saving should fail, because medium is not setup up
// 	editMed->save();
	QCOMPARE(db.isOpen(), true);
	editMed->setAddMode(0);
	delete med;
	db.close();
}

void EditMediumTest::editMode()
{
	ESqlDatabase db = ESqlDatabase::database();
	db.open();
	
	Medium *med = new Medium();
	med->setTitle(keys.invalid);
	med->setType(keys.testTypeId);
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	editMed->setEditMode(med, false);
	editMed->save();
	QCOMPARE(db.isOpen(), true);
	editMed->setAddMode(0);
	delete med;
	db.close();
}

void EditMediumTest::newMode()
{
	ESqlDatabase db = ESqlDatabase::database();
	db.open();
	
	Medium *med = new Medium();
	editMed->setNewMode(0, med, false);
	// saving should fail, because medium is not setup up
// 	editMed->save();
	QCOMPARE(db.isOpen(), true);
	editMed->setAddMode(0);
	delete med;
	db.close();
}

QTEST_MAIN(EditMediumTest)

#include "editmediumtest.moc"
