/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "loanmodeltest.h"
#include "loanmodeltest.moc"

#include <loanmodel.h>
#include <modeltest.h>
#include "../lib/internal/loanitem.h"
#include <esqldatabase.h>
#include <medium.h>

#include <QtTest>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QSignalSpy>

void LoanModelTest::initTestCase()
{
	testInit = new DBTestInit(this);
	if(!testInit->initTestDB())
		QSKIP("Could not create test database", SkipAll);
	
	keys = testInit->prepareMediumKeys("Testtype");
	if(!keys.valid)
		QSKIP("Could not create test database", SkipAll);
	
	ESqlDatabase db = ESqlDatabase::database();
	db.open();
	QSqlQuery query;
	
	// prepare user entries
	dbUserName1 = "max";
	if(!query.exec("INSERT INTO meganizer_users (name, givenName, userName) VALUES('Mustermann', 'Max', '"+ dbUserName1 +"')"))
		QFAIL("Could not insert "+ dbUserName1.toLatin1());
	query.exec("SELECT id FROM meganizer_users WHERE userName='"+ dbUserName1 +"'");
	if(!query.first())
		QFAIL("Could not select "+ dbUserName1.toLatin1() +"'s id");
	dbUser1 = query.record().value("id").toUInt();
	QVERIFY(dbUser1 > 0);

	dbUserName2 = "annette";
	if(!query.exec("INSERT INTO meganizer_users (name, givenName, userName) VALUES('Musterfrau', 'Annette', '"+ dbUserName2 +"')"))
		QFAIL("Could not insert "+ dbUserName2.toLatin1());
	query.exec("SELECT id FROM meganizer_users WHERE userName='"+ dbUserName2 +"'");
	if(!query.first())
		QFAIL("Could not select "+ dbUserName2.toLatin1() +"'s id");
	dbUser2 = query.record().value("id").toUInt();
	QVERIFY(dbUser2 > 0);
		
	if(!query.exec("INSERT INTO meganizer_users (name, givenName, userName) VALUES('Mustermann', 'Manuela', '')"))
		QFAIL("Could not insert manuela");
	query.exec("SELECT id FROM meganizer_users WHERE givenName='Manuela'");
	if(!query.first())
		QFAIL("Could not select Manuela's id");
	user1 = query.record().value("id").toUInt();
	QVERIFY(user1 > 0);
	
	
	// prepare loan stati
	QStringList stati;
	stati << "stat 1" << "stat 2" << "stat 3" << "stat 4" << "stat 5" << "stat 6";
	query.exec("SET foreign_key_checks = 1");
	bool result = true;
	for(int i = 0; i < stati.size(); i++)
		if(!query.exec("INSERT INTO meganizer_loanstati (id, name) VALUES("+ QString::number(i) +", '"+ stati[i] +"')"))
			result = false;

	query.exec("SET foreign_key_checks = 1");
	if(!result)
		QFAIL("Could not select insert status");

	// prepare test media
	Medium *medium = createNewMedium(user1);
	QVERIFY(medium);
	loanedNever.ownerId = user1;
	loanedNever.mediumId = medium->id();
	loanedNever.loanStatus = 0;
	loanedNever.recipientId = keys.testUserId;
	loanedNever.loanedUntil = QDate();
	delete medium;
	
	QDate date = QDate::currentDate();
	date.addYears(-1);
	loanedBefore.ownerId = keys.testUserId;
	loanedBefore.mediumId = 0;
	loanedBefore.loanStatus = 5;
	loanedBefore.recipientId = dbUser2;
	loanedBefore.loanedUntil = date;
	QVERIFY(storeLoanedData(loanedBefore));
	
	date = QDate::currentDate();
	date.addYears(2);
	loaned.ownerId = user1;
	loaned.mediumId = 0;
	loaned.loanStatus = 3;
	loaned.recipientId = keys.testUserId;
	loaned.loanedUntil = date;
	QVERIFY(storeLoanedData(loaned));

	date = QDate::currentDate();
	date.addYears(-2);
	loanedOver.ownerId = keys.testUserId;
	loanedOver.mediumId = 0;
	loanedOver.loanStatus = 3;
	loanedOver.recipientId = dbUser1;
	loanedOver.loanedUntil = date;
	QVERIFY(storeLoanedData(loanedOver));
}

void LoanModelTest::loanItemTest(){
	// test set Functions
	LoanItem *item = new LoanItem(this);
	QCOMPARE(item->modified(), false);
	item->setLoggedInUser(keys.testUserId);
	QVERIFY(item->loggedInUser() == keys.testUserId);
	QCOMPARE(item->modified(), false);
	
	delete item;
	item = new LoanItem(this);
	QVERIFY(item->loggedInUser() == keys.testUserId);
	
	delete item;
	item = new LoanItem(this);
	item->setMaterial(loaned.mediumId);
	QVERIFY(item->material() == loaned.mediumId);
	QCOMPARE(item->modified(), true);
	
	delete item;
	item = new LoanItem(this);
	item->setOwner(keys.testUserId, false);
	QVERIFY(item->owner() == keys.testUserId);
	QCOMPARE(item->ownerUserName(), QString());
	QCOMPARE(item->modified(), true);
	delete item;
	item = new LoanItem(this);
	item->setOwner(keys.testUserId);
	QVERIFY(item->owner() == keys.testUserId);
	QCOMPARE(item->ownerUserName(), keys.testUser);
	QCOMPARE(item->modified(), true);
	item->setOwner(user1);
	QVERIFY(item->owner() == user1);
	QCOMPARE(item->ownerUserName(), QString());
	item->setOwner(dbUser1);
	QVERIFY(item->owner() == dbUser1);
	QCOMPARE(item->ownerUserName(), dbUserName1);
	item->setOwner(keys.testUserId, false);
	QVERIFY(item->owner() == keys.testUserId);
	QCOMPARE(item->ownerUserName(), dbUserName1);
	item->setOwner(0);
	QVERIFY(item->owner() == 0);
	QCOMPARE(item->ownerUserName(), QString());
	
	delete item;
	item = new LoanItem(this);
	item->setOwnerUserName(keys.testUser);
	QVERIFY(item->owner() == 0);
	QCOMPARE(item->ownerUserName(), keys.testUser);
	QCOMPARE(item->modified(), false);
	
	delete item;
	item = new LoanItem(this);
	// set recipient is allowed, because ownerUserName is empty
	QCOMPARE(item->setRecipient(dbUser1), true);
	QVERIFY(item->recipient() == dbUser1);
	QCOMPARE(item->modified(), true);
	// cause ownerUserName is set, it is not allowed anymore
	item->setOwnerUserName(keys.testUser);
	QCOMPARE(item->setRecipient(dbUser1), false);
	// cause owner is set to us, we are allowed to do it
	item->setOwner(keys.testUserId);
	QCOMPARE(item->setRecipient(user1), true);
	QVERIFY(item->recipient() == user1);
	// cause owner is set to us, we can not loan to ourself
	item->setOwner(keys.testUserId);
	QCOMPARE(item->setRecipient(keys.testUserId), false);
	QVERIFY(item->recipient() == user1);
	// set recipient is allowed, because ownerUserName is empty
	item->setOwnerUserName(QString());
	QCOMPARE(item->setRecipient(dbUser1), true);
	QVERIFY(item->recipient() == dbUser1);
	
	delete item;
	item = new LoanItem(this);
	item->setLastChange(QDate::currentDate());
	QCOMPARE(item->lastChange(), QDate::currentDate());
	QCOMPARE(item->modified(), true);
	
	// test is valid
	delete item;
	item = new LoanItem(this);
	QCOMPARE(item->isValid(), false);
	item->setOwner(keys.testUserId);
	QCOMPARE(item->isValid(), false);
	item->setMaterial(loaned.mediumId);
	QCOMPARE(item->isValid(), false);
	item->setRecipient(dbUser1);
	QCOMPARE(item->isValid(), true);
	QCOMPARE(item->setRecipient(keys.testUserId, true), true);
	QCOMPARE(item->isValid(), false);
	delete item;
}

void LoanModelTest::validStati_data()
{
	QTest::addColumn<unsigned int>("owner");
	QTest::addColumn<QString>("ownerUserName");
	QTest::addColumn<unsigned int>("recipient");
	QTest::addColumn<unsigned int>("curStat");
	QTest::addColumn<QStringList>("results");
	
	unsigned int status0 = 0;
	unsigned int status1 = 1;
	unsigned int status2 = 2;
	unsigned int status3 = 3;
	unsigned int status4 = 4;
	unsigned int status5 = 5;
	unsigned int status6 = 6;
	QStringList results;
	
	// owner, ownerUserName, recipient, curStat
	// not testUserId, is set, not testUserId
	QTest::newRow("not cur, set, not cur, 0") << dbUser1 << dbUserName1 << user1 << status0 << results;
	QTest::newRow("not cur, set, not cur, 1") << dbUser1 << dbUserName1 << user1 << status1 << results;
	QTest::newRow("not cur, set, not cur, 2") << dbUser1 << dbUserName1 << user1 << status2 << results;
	QTest::newRow("not cur, set, not cur, 3") << dbUser1 << dbUserName1 << user1 << status3 << results;
	QTest::newRow("not cur, set, not cur, 4") << dbUser1 << dbUserName1 << user1 << status4 << results;
	QTest::newRow("not cur, set, not cur, 5") << dbUser1 << dbUserName1 << user1 << status5 << results;
	QTest::newRow("not cur, set, not cur, 6") << dbUser1 << dbUserName1 << user1 << status6 << results;
	
	// not testUserId, set, not set
	results << "1";
	QTest::newRow("not cur, set, not set, 0") << dbUser1 << dbUserName1 << status0 << status0 << results;
	results.clear();
	QTest::newRow("not cur, set, not set, 1") << dbUser1 << dbUserName1 << status0 << status1 << results;
	QTest::newRow("not cur, set, not set, 2") << dbUser1 << dbUserName1 << status0 << status2 << results;
	QTest::newRow("not cur, set, not set, 3") << dbUser1 << dbUserName1 << status0 << status3 << results;
	QTest::newRow("not cur, set, not set, 4") << dbUser1 << dbUserName1 << status0 << status4 << results;
	QTest::newRow("not cur, set, not set, 5") << dbUser1 << dbUserName1 << status0 << status5 << results;
	QTest::newRow("not cur, set, not set, 6") << dbUser1 << dbUserName1 << status0 << status6 << results;
	
	// not testUserId, not set, not set
	results << "1" << "2" << "3";
	QTest::newRow("not cur, not set, not set, 0") << dbUser1 << "" << status0 << status0 << results;
	results.clear();
	results << "2" << "3" << "6";
	QTest::newRow("not cur, not set, not set, 1") << dbUser1 << "" << status0 << status1 << results;
	results.clear();
	QTest::newRow("not cur, not set, not set, 2") << dbUser1 << "" << status0 << status2 << results;
	results.clear();
	results << "4" << "5";
	QTest::newRow("not cur, not set, not set, 3") << dbUser1 << "" << status0 << status3 << results;
	results.clear();
	results << "5" << "6";
	QTest::newRow("not cur, not set, not set, 4") << dbUser1 << "" << status0 << status4 << results;
	results.clear();
	QTest::newRow("not cur, not set, not set, 5") << dbUser1 << "" << status0 << status5 << results;
	results.clear();
	QTest::newRow("not cur, not set, not set, 6") << dbUser1 << "" << status0 << status6 << results;
	
	// testUserId, doesn't matter, not cur
	results << "2" << "3";
	QTest::newRow("cur, set, not cur, 0") << keys.testUserId << dbUserName1 << dbUser1 << status0 << results;
	QTest::newRow("cur, not set, not cur, 0") << keys.testUserId << "" << dbUser1 << status0 << results;
	results.clear();
	results << "2" << "3";
	QTest::newRow("cur, set, not cur, 1") << keys.testUserId << dbUserName1 << dbUser1 << status1 << results;
	QTest::newRow("cur, not set, not cur, 1") << keys.testUserId << "" << dbUser1 << status1 << results;
	results.clear();
	QTest::newRow("cur, set, not cur, 2") << keys.testUserId << dbUserName1 << dbUser1 << status2 << results;
	QTest::newRow("cur, not set, not cur, 2") << keys.testUserId << "" << dbUser1 << status2 << results;
	results.clear();
	results << "4" << "5";
	QTest::newRow("cur, set, not cur, 3") << keys.testUserId << dbUserName1 << dbUser1 << status3 << results;
	QTest::newRow("cur, not set, not cur, 3") << keys.testUserId << "" << dbUser1 << status3 << results;
	results.clear();
	results << "5" << "6";
	QTest::newRow("cur, set, not cur, 4") << keys.testUserId << dbUserName1 << dbUser1 << status4 << results;
	QTest::newRow("cur, not set, not cur, 4") << keys.testUserId << "" << dbUser1 << status4 << results;
	results.clear();
	QTest::newRow("cur, set, not cur, 5") << keys.testUserId << dbUserName1 << dbUser1 << status5 << results;
	QTest::newRow("cur, not set, not cur, 5") << keys.testUserId << "" << dbUser1 << status5 << results;
	results.clear();
	QTest::newRow("cur, set, not cur, 6") << keys.testUserId << dbUserName1 << dbUser1 << status6 << results;
	QTest::newRow("cur, not set, not cur, 6") << keys.testUserId << "" << dbUser1 << status6 << results;
	
	// not cur, set, cur
	results << "1";
	QTest::newRow("not cur, set, cur, 0") << dbUser1 << dbUserName1 << keys.testUserId << status0 << results;
	results.clear();
	results << "6";
	QTest::newRow("not cur, set, cur, 1") << dbUser1 << dbUserName1 << keys.testUserId << status1 << results;
	results.clear();
	QTest::newRow("not cur, set, cur, 2") << dbUser1 << dbUserName1 << keys.testUserId << status2 << results;
	QTest::newRow("not cur, set, cur, 3") << dbUser1 << dbUserName1 << keys.testUserId << status3 << results;
	QTest::newRow("not cur, set, cur, 4") << dbUser1 << dbUserName1 << keys.testUserId << status4 << results;
	QTest::newRow("not cur, set, cur, 5") << dbUser1 << dbUserName1 << keys.testUserId << status5 << results;
	QTest::newRow("not cur, set, cur, 6") << dbUser1 << dbUserName1 << keys.testUserId << status6 << results;
	
	// not cur, not set, cur
	results << "1" << "2" << "3";
	QTest::newRow("not cur, not set, cur, 0") << dbUser1 << "" << keys.testUserId << status0 << results;
	results.clear();
	results << "6";
	QTest::newRow("not cur, not set, cur, 1") << dbUser1 << "" << keys.testUserId << status1 << results;
	results.clear();
	QTest::newRow("not cur, not set, cur, 2") << dbUser1 << "" << keys.testUserId << status2 << results;
	results << "4" << "5";
	QTest::newRow("not cur, not set, not cur, 3") << dbUser1 << "" << keys.testUserId << status3 << results;
	results.clear();
	results << "5" << "6";
	QTest::newRow("not cur, not set, cur, 4") << dbUser1 << "" << keys.testUserId << status4 << results;
	results.clear();
	QTest::newRow("not cur, not set, cur, 5") << dbUser1 << "" << keys.testUserId << status5 << results;
	QTest::newRow("not cur, not set, cur, 6") << dbUser1 << "" << keys.testUserId << status6 << results;
}

void LoanModelTest::validStati()
{
	QFETCH(unsigned int, owner);
	QFETCH(QString, ownerUserName);
	QFETCH(unsigned int, recipient);
	QFETCH(unsigned int, curStat);
	QFETCH(QStringList, results);
	
	LoanItem item(this);
	// we set ownerusername afterwards
	item.setOwner(owner, false);
	item.setOwnerUserName(ownerUserName);
	// force to set current status
	QCOMPARE(item.setStatus(curStat, true), true);
	// set recipient after status to reset setted recip by setStatus
	QCOMPARE(item.setRecipient(recipient, true), true);
	
	QList<unsigned int> validStati = item.validStatis();
	QCOMPARE(validStati.size(), results.size());
	foreach(QString result, results)
		QVERIFY(validStati.contains(result.toUInt()));
}

void LoanModelTest::loanedUntil_data()
{
	QTest::addColumn<QDate>("date");
	QTest::addColumn<unsigned int>("owner");
	QTest::addColumn<unsigned int>("recipient");
	QTest::addColumn<bool>("ignoreRights");
	QTest::addColumn<bool>("modified");
	
	QDate date = QDate::currentDate();
	QTest::newRow("owner, user1, false") << date << keys.testUserId << user1 << false << true;
	QTest::newRow("owner, user1, true") << date << keys.testUserId << user1 << true << true;
	
	QTest::newRow("not owner, user1, false") << date << dbUser1 << user1 << false << false;
	QTest::newRow("not owner, user1, true") << date << dbUser1 << user1 << true << true;
	
	QTest::newRow("not owner, testUser, false") << date << dbUser1 << keys.testUserId << false << false;
	QTest::newRow("not owner, testUser, true") << date << dbUser1 << keys.testUserId << true << true;

	QTest::newRow("user1, testUser, false") << date << user1 << keys.testUserId << true << true;
	QTest::newRow("user1, testUser, true") << date << user1 << keys.testUserId << true << true;

	QTest::newRow("user1, dbUser1, false") << date << user1 << dbUser1 << false << false;
	QTest::newRow("user1, dbUser1, true") << date << user1 << dbUser1 << true << true;
}

void LoanModelTest::loanedUntil()
{	
	QFETCH(QDate, date);
	QFETCH(unsigned int, owner);
	QFETCH(unsigned int, recipient);
	QFETCH(bool, ignoreRights);
	QFETCH(bool, modified);
	
	LoanItem item;
	item.setLoggedInUser(keys.testUserId);
	item.setOwner(owner);
	item.setRecipient(recipient);
	QCOMPARE(item.setLoanedUntil(date, ignoreRights), modified);

	if(modified)
		QCOMPARE(item.loanedUntil(), date);
	else
		QCOMPARE(item.loanedUntil(), QDate());
}

void LoanModelTest::loanStatus_data()
{
	QTest::addColumn<unsigned int>("owner");
	QTest::addColumn<QString>("ownerUserName");
	QTest::addColumn<unsigned int>("recipient");
	QTest::addColumn<unsigned int>("nextStat");
	QTest::addColumn<bool>("result");
	
	unsigned int status0 = 0;
	unsigned int status1 = 1;
	unsigned int status2 = 2;
	unsigned int status3 = 3;
	unsigned int status4 = 4;
	unsigned int status5 = 5;
	unsigned int status6 = 6;
	QStringList results;
	
	// setRecipient should return false
	QTest::newRow("cur, set, cur, 1") << keys.testUserId << keys.testUser << keys.testUserId << status1 << false;
	// should be good and recipient should be our recipient afterwards
	QTest::newRow("not cur, set, cur, 1") << dbUser1 << dbUserName1 << keys.testUserId << status1 << true;
	QTest::newRow("not cur, not set, cur, 3") << user1 << "" << keys.testUserId << status3 << true;
	QTest::newRow("not cur, not set, not cur 1") << user1 << "" << dbUser1 << status1 << false;
	
	QTest::newRow("cur, set, not cur, 3") << keys.testUserId << keys.testUser << dbUser1 << status5 << false;
}

void LoanModelTest::loanStatus()
{
	QFETCH(unsigned int, owner);
	QFETCH(QString, ownerUserName);
	QFETCH(unsigned int, recipient);
	QFETCH(unsigned int, nextStat);
	QFETCH(bool, result);
	
	LoanItem item(this);
	item.setOwner(owner);
	item.setOwnerUserName(ownerUserName);
	item.setRecipient(recipient, true);
	
	QCOMPARE(item.setStatus(nextStat, false), result);
	if(result)
		QVERIFY(item.status() == nextStat);
	else
		QVERIFY(item.status() == 0);
	if(owner != keys.testUserId && result)
		QVERIFY(item.recipient() == keys.testUserId);
}

void LoanModelTest::loanModelTest()
{
	// test the loading of medias, which are
	// never loaned before
	Medium *med = new Medium();
	med->load(loanedNever.mediumId, loanedNever.ownerId);
	LoanModel *model = new LoanModel(med, keys.testUserId, this);
	ModelTest *modeltest = new ModelTest(model, this);
	QCOMPARE(model->rowCount(), 1);
	LoanItem *item = model->loanItemFromIndex(model->index(0, 0));
	QVERIFY(item);
	QCOMPARE(item->material(), med->id());
	QCOMPARE(item->materialTitle(), med->title());
	QCOMPARE(item->owner(), med->user());
	QCOMPARE(item->recipient(), loanedNever.recipientId);
	QCOMPARE(item->status(), loanedNever.loanStatus);
	QCOMPARE(item->loanedUntil(), loanedNever.loanedUntil);
	delete modeltest;
	delete model;
	
	// test the loading from actually loaned items
	med->load(loaned.mediumId, loaned.ownerId);
	model = new LoanModel(med, keys.testUserId, this);
	modeltest = new ModelTest(model, this);
	QCOMPARE(model->rowCount(), 1);
	item = model->loanItemFromIndex(model->index(0, 0));
	QVERIFY(item);
	QCOMPARE(item->material(), med->id());
	QCOMPARE(item->materialTitle(), med->title());
	QCOMPARE(item->owner(), med->user());
	QCOMPARE(item->recipient(), loaned.recipientId);
	QCOMPARE(item->status(), loaned.loanStatus);
	QCOMPARE(item->loanedUntil(), loaned.loanedUntil);
	delete modeltest;
	delete model;

	// test the loading from media, which are loaned in past
	med->load(loanedBefore.mediumId, loanedBefore.ownerId);
	model = new LoanModel(med, keys.testUserId, this);
	modeltest = new ModelTest(model, this);
	QCOMPARE(model->rowCount(), 1);
	item = model->loanItemFromIndex(model->index(0, 0));
	QVERIFY(item);
	QCOMPARE(item->material(), med->id());
	QCOMPARE(item->materialTitle(), QString());
	QCOMPARE(item->owner(), med->user());
	// it must have the state of loanedNever, because it
	// should be a new empty item
	QCOMPARE(item->recipient(), 0u);
	QCOMPARE(item->status(), 0u);
	QCOMPARE(item->loanedUntil(), QDate());


	// testing data and setData
	QCOMPARE(model->columnCount(), 5);
	QCOMPARE(model->data(model->index(0,0)).toUInt(), med->user());
	QCOMPARE(model->data(model->index(0,1)).toUInt(), 0u);
	QVERIFY(model->data(model->index(0,2)).toUInt() ==  0u);
	QCOMPARE(model->data(model->index(0,3)).toDate(), QDate());

	// change data
	QCOMPARE(model->setData(model->index(0,0), dbUser1), false);
	QCOMPARE(model->setData(model->index(0,1), user1), true);
	QCOMPARE(model->setData(model->index(0,2), 2), true);
	QDate date = QDate::currentDate();
	date.addDays(60);
	QCOMPARE(model->setData(model->index(0,3), date), true);
	// check changes
	QCOMPARE(model->data(model->index(0,0)).toUInt(), keys.testUserId);
	QCOMPARE(model->data(model->index(0,1)).toUInt(), user1);
	QVERIFY(model->data(model->index(0,2)).toUInt() ==  2);
	QCOMPARE(model->data(model->index(0,3)).toDate(), date);

	delete modeltest;
	delete model;
	
	delete med;
}

void LoanModelTest::loanSummaryTest()
{
	LoanModel *model = new LoanModel(keys.testUserId, this);
	QCOMPARE(model->columnCount(), 6);
	QCOMPARE(model->rowCount(), 2);

	QList<LoanedData> loanedData;
	loanedData << loaned << loanedOver;
	QCOMPARE(model->rowCount(), loanedData.size());
	for(int i = 0; i < model->rowCount(); i++){
		Medium *med = new Medium();
		med->load(loanedData[i].mediumId, loanedData[i].ownerId);
		QCOMPARE(model->data(model->index(i, 0)).toString(), med->title());
		QCOMPARE(model->data(model->index(i, 1)).toUInt(), med->user());
		QCOMPARE(model->data(model->index(i, 2)).toUInt(), loanedData[i].recipientId);
		QCOMPARE(model->data(model->index(i, 3)).toUInt(), loanedData[i].loanStatus);
		QCOMPARE(model->data(model->index(i, 4)).toDate(), loanedData[i].loanedUntil);
		if(loanedData[i].loanedUntil < QDate::currentDate() && loanedData[i].loanStatus == 3){
			QVariant varBrush = model->data(model->index(i, 0), Qt::BackgroundRole);
			QVERIFY(varBrush.canConvert<QBrush>());
			QBrush brush = varBrush.value<QBrush>();
			QCOMPARE(brush, QBrush(QColor ( 244, 12, 13, 60 )));
		}
		delete med;
	}
	delete model;
}

Medium* LoanModelTest::createNewMedium(unsigned int ownerId)
{
	Medium *medium = new Medium();

	medium->setTitle("'SELECT * FROM meganizer_user;"+ QDateTime::currentDateTime().toString("yyyyMMddhhmmsszzz"));
	medium->setDescription("This is greate show; INSERT INTO test test = 10;");
	medium->setGenre(keys.testGenreId);
	medium->setManufacturer(keys.testManuId);
	medium->setImage(QImage(":/images/meganizer.png"));
	medium->setListPriceCurrency(keys.testCurId);
	medium->setListPrice(11.11);
	medium->setBoughtPriceCurrency(keys.testCurId);
	medium->setBoughtPrice(22.22);
	medium->setEvaluation(5);
	medium->setType(keys.testTypeId);
	if(!medium->store())
		return NULL;
	if(medium->id() < 1)
		return NULL;
	if(ownerId != keys.testUserId){
		// we must change the user relation manually
		QSqlQuery query;
		if(!query.exec(QString("UPDATE meganizer_userownmaterials SET usersId=%1 WHERE materialsId=%2").arg(ownerId).arg(medium->id()))){
			qDebug() << "Could not change owner: "+ query.lastError().text();
			return NULL;
		}

		unsigned int id = medium->id();
		if(!medium->load(medium->id(), ownerId)){
			qDebug() << QString("Could not select updated medium->select(%1, %2)").arg(id).arg(ownerId);
			return NULL;
		}
		
		if(medium->user() != ownerId){
			qDebug() << "Owner is not changed!";
			return NULL;
		}
	}
	
	return medium;
}

bool LoanModelTest::storeLoanedData(LoanedData &data)
{
	Medium *med = NULL;
	if(data.mediumId < 1){
		med = createNewMedium(data.ownerId);
		if(!med)
			return false;
		data.mediumId = med->id();
	}
	else{
		Medium *med = new Medium();
		if(!med->load(data.mediumId, data.ownerId)){
			qDebug() << "storeLoanedData: Could not select medium!";
			delete med;
			return false;
		}
	}
	LoanItem item;
	item.setMaterial(data.mediumId);
	item.setOwner(med->user(), true);
	delete med;
	item.setRecipient(data.recipientId, true);
	item.setStatus(data.loanStatus, true);
	item.setLoanedUntil(data.loanedUntil);
	QSignalSpy spy(&item, SIGNAL(error(QString)));
	if(!item.store()){
		qDebug() << "storeLoanedData: Could not store loan data!";
		if(spy.count() > 1){
			QList<QVariant> arguments = spy.takeFirst();
			qDebug() << arguments.at(0).toString();
		}
		return false;
	}
	return true;
}

QTEST_MAIN(LoanModelTest)
