/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "mediumeditwidgettests.h"
#include "mediumeditwidgettests.moc"

#include <medium.h>
#include <mediumeditwidget.h>
#include <escaledimage.h>
#include <esqldatabase.h>
#include <materialperson.h>
#include <materialpersonsmodel.h>

#include <mediumownerwidget.h>
#include <mediumchildswidget.h>
#include <mediumimagewidget.h>
#include <mediumpricewidget.h>
#include <mediumevaluationwidget.h>
#include <mediumgiftwidget.h>
#include <mediumkeywordwidget.h>
// #include <mediumloanwidget.h>
// #include <mediumminwidget.h>
#include <mediumpersonswidget.h>
#include <mediumparentswidget.h>
// #include <mediumfileswidget.h>

#include <QTest>
#include <QSignalSpy>
#include <QSqlQuery>
#include <QByteArray>
#include <QBuffer>
#include <QDoubleSpinBox>
#include <QComboBox>
#include <QLineEdit>
#include <QStringListModel>
#include <QListView>
#include <QCheckBox>
#include <QTableView>
#include <QPushButton>

void MediumEditWidgetTests::initTestCase()
{
	MediumEditWidgetBaseTests::initTestCase();
}

bool MediumEditWidgetTests::ownerWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	// find needed gui elements
	QTableView *ownerView = widget->findChild<QTableView*>("ownerView");
	if(!ownerView)
		return false;

	QPushButton *addButton = widget->findChild<QPushButton*>("addButton");
	if(!addButton)
		return false;
	
	QPushButton *removeButton = widget->findChild<QPushButton*>("removeButton");
	if(!removeButton)
		return false;
	
	QTableView *userView = widget->findChild<QTableView*>("userView");
	if(!userView)
		return false;

	// remove owner
	ownerView->selectRow(0);
	removeButton->setEnabled(true);
 	QTest::mouseClick(removeButton, Qt::LeftButton);
	if(ownerView->model()->rowCount() != 0)
		return false;

	// add new owner
	bool rowSelected = false;
	for(int i = 0; i < userView->model()->rowCount(); i++){
		if(userView->model()->data(userView->model()->index(i, 0)).toString() == "User"){
			userView->selectRow(i);
			rowSelected = true;
			break;
		}
	}
	if(!rowSelected)
		return false;
	addButton->setEnabled(true);
 	QTest::mouseClick(addButton, Qt::LeftButton);
	if(ownerView->model()->rowCount() != 1)
		return false;
	return true;
}

bool MediumEditWidgetTests::ownerWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(med);
	Q_UNUSED(widget);
	Medium *medium = new Medium();
	if(!medium->load(caller->medId, caller->newUserId))
		return false;

	// we must rollback the user change, to let all other widgets prepare it medium correctly
	medium->setUser(caller->keys.testUserId);
	if(!medium->store(true))
		return false;
	delete medium;
	return true;
}

bool MediumEditWidgetTests::childsWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	return true;
}

bool MediumEditWidgetTests::childsWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	Q_UNUSED(med);
	return true;
}

bool MediumEditWidgetTests::imageWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	QImage image(":/images/meganizer.png");
	if(image.isNull())
		return false;
	
	EScaledImage *img = widget->findChild<EScaledImage*>("scaledImage");
	if(!img)
		return false;
	img->setImage(image);
	
	return true;
}

bool MediumEditWidgetTests::imageWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget *widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	QByteArray bytes;
	QBuffer buffer(&bytes);
	buffer.open(QIODevice::WriteOnly);
	QPixmap(":/images/meganizer.png").save(&buffer, "JPG");
	
	if(med->imageToByteArray() != bytes)
		return false;
	return true;
}

bool MediumEditWidgetTests::priceWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	QDoubleSpinBox *priceEdit = widget->findChild<QDoubleSpinBox*>("priceEdit");
	if(!priceEdit)
		return false;
	QLineEdit *curEdit = widget->findChild<QLineEdit*>("currencyEdit");
	if(!curEdit)
		return false;

	QString value = "22,22";
	if(widget->property("price").value<MediumPriceWidget::Price>() == MediumPriceWidget::BoughtPrice)
		value = "11,11";
	
	// to enter edit mode of the spinbox
	QTest::keyClick(priceEdit, Qt::Key_Enter);
	QTest::keyClicks(priceEdit, value);
	// changing the currency
	// clean up old stuff
	QTest::keyClick(curEdit, 'a', Qt::ControlModifier);
	QTest::keyClick(curEdit, Qt::Key_Delete);
	QTest::keyClicks(curEdit, caller->newCur);

	return true;
}

bool MediumEditWidgetTests::priceWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	MediumPriceWidget::Price type = widget->property("price").value<MediumPriceWidget::Price>();
	if(type == MediumPriceWidget::ListPrice){
		if(med->listPrice() != 22.22){
			QTest::qWarn("List Price isn't ok");
			return false;
		}
		if(med->listPriceCurrency() != caller->newCurId){
			QTest::qWarn("List Price currency isn't ok");
			return false;
		}
		return true;
	}
	else if(type == MediumPriceWidget::BoughtPrice){
		if(med->boughtPrice() != 11.11){
			QTest::qWarn("Bought Price isn't ok");
			return false;
		}
		if(med->boughtPriceCurrency() != caller->newCurId){
			QTest::qWarn("Bought Price currency isn't ok");
			return false;
		}
		return true;
	}
	QTest::qWarn("No Price type given");
	return false;
}

bool MediumEditWidgetTests::evaluationWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	QComboBox *evalEdit = widget->findChild<QComboBox*>("evalEdit");
	if(!evalEdit)
		return false;

	QTest::keyClick(evalEdit, Qt::Key_Down);
	QTest::keyClick(evalEdit, Qt::Key_Down);

	return true;
}

bool MediumEditWidgetTests::evaluationWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	if(med->evaluation() != 2)
		return false;

	return true;
}

bool MediumEditWidgetTests::keywordWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	QListView *keywordList = widget->findChild<QListView*>("keywordList");
	if(!keywordList){
		QTest::qWarn("keywordList not found");
		return false;
	}

	QPushButton *addButton = widget->findChild<QPushButton*>("addButton");
	if(!addButton){
		QTest::qWarn("addButton not found");
		return false;
	}

	QPushButton *removeButton = widget->findChild<QPushButton*>("removeButton");
	if(!removeButton){
		QTest::qWarn("removeButton not found");
		return false;
	}

	QLineEdit *keywordEdit = widget->findChild<QLineEdit*>("keywordEdit");
	if(!keywordEdit){
		QTest::qWarn("keywordEdit not found");
		return false;
	}

	// check wether one keyword exists
	if(keywordList->model()->rowCount() != 1){
		QTest::qWarn("Related keyword was not loaded");
		return false;
	}

	// remove keyword
	keywordList->selectionModel()->select(keywordList->model()->index(0, 0), QItemSelectionModel::Select);
	removeButton->setEnabled(true);
 	QTest::mouseClick(removeButton, Qt::LeftButton);
	if(keywordList->model()->rowCount() != 0){
		QTest::qWarn("Could not remove related keyword (from database)");
		return false;
	}

	// add keyword
	// empty keywords should not be added
	QTest::mouseClick(addButton, Qt::LeftButton);
	if(keywordList->model()->rowCount() != 0){
		QTest::qWarn("Added empty keyword");
		return false;
	}
	// not empty keywords should be allowed
	QTest::keyClicks(keywordEdit, "SuperKeyword");
	QTest::mouseClick(addButton, Qt::LeftButton);
	if(keywordList->model()->rowCount() != 1){
		QTest::qWarn("Added new Keyword");
		return false;
	}
	if(!keywordEdit->text().isEmpty()){
		QTest::qWarn("keywordEdit is not cleared after adding keyword");
		return false;
	}

	// removed added keywords (which are not stored at the database)
	QTest::keyClicks(keywordEdit, "Will Be Removed");
	QTest::mouseClick(addButton, Qt::LeftButton);
	if(keywordList->model()->rowCount() != 2){
		QTest::qWarn("Second keyword not added");
		return false;
	}
	keywordList->selectionModel()->select(keywordList->model()->index(1, 0), QItemSelectionModel::Select);
	removeButton->setEnabled(true);
 	QTest::mouseClick(removeButton, Qt::LeftButton);
	if(keywordList->model()->rowCount() != 1){
		QTest::qWarn("Second keyword not removed");
		return false;
	}

	return true;
}

bool MediumEditWidgetTests::keywordWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	if(med->keywords()->rowCount() != 1)
		return false;
	if(med->keywords()->stringList()[0] != "SuperKeyword")
		return false;
	return true;
}

bool MediumEditWidgetTests::personsWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	
	QTableView *personsView = widget->findChild<QTableView*>("personsView");
	if(!personsView){
		QTest::qWarn("personsView not found");
		return false;
	}
	
	QPushButton *removeButton = widget->findChild<QPushButton*>("removeButton");
	if(!removeButton){
		QTest::qWarn("removeButton not found");
		return false;
	}
	
	// check wether all three persons exists
	if(personsView->model()->rowCount() != 3){
		QTest::qWarn("Related persons were not loaded");
		return false;
	}

	// remove person
	personsView->selectionModel()->select(personsView->model()->index(1, 0), QItemSelectionModel::Select);
	if(personsView->selectionModel()->selectedIndexes().size() != 1){
		QTest::qWarn("Selected rows are not correct");
		return false;
	}
	removeButton->setEnabled(true);
 	QTest::mouseClick(removeButton, Qt::LeftButton);
	return true;
}

bool MediumEditWidgetTests::personsWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	
	MaterialPersonsModel *model = med->materialPersons();
	// check that person 2 is mark for removal now
	for(int i = 0; i < model->rowCount(); i++){
		Person *person = model->personFromIndex(model->index(i, 0));
		if(!person){
			QTest::qWarn("Could not find removed person");
			return false;
		}
		if((!person->markForRemovel()) && (i == 1)){
			QTest::qWarn("Second person isn't marked for removal.");
			return false;
		}
		else if((person->markForRemovel()) && (i != 1)){
			QTest::qWarn("Other person is marked for removal.");
			return false;
		}
	}
	return true;
}


bool MediumEditWidgetTests::giftWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	QCheckBox *giftOnlyEdit = widget->findChild<QCheckBox*>("giftOnlyEdit");
	if(!giftOnlyEdit)
		return false;

	QTest::keyClick(giftOnlyEdit, Qt::Key_Space);
	return true;
}

bool MediumEditWidgetTests::giftWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	if(!med->giftOnly())
		return false;
	// giftonly without any presentees, removes the current owner from the owner list.
	// Therefore, we must add him again
	if(!med->store(true))
		return false;
	return true;
}

bool MediumEditWidgetTests::parentsWidgetModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	// nothing todo since it is readonly
	return true;
}

bool MediumEditWidgetTests::parentsWidgetCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(caller);
	Q_UNUSED(widget);
	Q_UNUSED(med);
	return true;
}

void MediumEditWidgetTests::test()
{
	registerWidget(new MediumOwnerWidget(),
		&MediumEditWidgetTests::ownerWidgetModify,
		&MediumEditWidgetTests::ownerWidgetCheck);

	registerWidget(new MediumChildsWidget(),
		&MediumEditWidgetTests::childsWidgetModify,
		&MediumEditWidgetTests::childsWidgetCheck,
		NULL,
		"MediumChildsWidget - Current medium has no childs");

	// we must test the listprice and the boughtprice
	MediumPriceWidget *price = new MediumPriceWidget();
	price->setProperty("price", "ListPrice");
	registerWidget(price,
		&MediumEditWidgetTests::priceWidgetModify,
		&MediumEditWidgetTests::priceWidgetCheck);
	price = new MediumPriceWidget();
	price->setProperty("price", "BoughtPrice");
	registerWidget(price,
		&MediumEditWidgetTests::priceWidgetModify,
		&MediumEditWidgetTests::priceWidgetCheck);

	registerWidget(new MediumEvaluationWidget(),
		&MediumEditWidgetTests::evaluationWidgetModify,
		&MediumEditWidgetTests::evaluationWidgetCheck);

	registerWidget(new MediumKeywordWidget(),
		&MediumEditWidgetTests::keywordWidgetModify,
		&MediumEditWidgetTests::keywordWidgetCheck);

	// add some persons for the material to test
	// MediumPersonsWidget
	if(medId == 0){
		Medium *med = prepareMedium();
		delete med;
	}
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	QSqlQuery query;
	query.prepare("INSERT INTO meganizer_materialpersonsroles (name) VALUES(:name)");
	query.bindValue(":name", "Author");
	QCOMPARE(query.exec(), true);
	query.exec("SELECT id FROM meganizer_materialpersonsroles WHERE name='Author'");
	QCOMPARE(query.first(), true);
	uint roleId = query.record().value("id").toUInt();
	QVERIFY(roleId > 0);
	QVERIFY(addPerson("Test 1", "First", medId, roleId));
	QVERIFY(addPerson("Test 2", "Second", medId, roleId));
	QVERIFY(addPerson("Test 3", "Third", medId, roleId));
	if(closeDB)
		db.close();
	registerWidget(new MediumPersonsWidget(),
		&MediumEditWidgetTests::personsWidgetModify,
		&MediumEditWidgetTests::personsWidgetCheck);

	registerWidget(new MediumGiftWidget(),
		&MediumEditWidgetTests::giftWidgetModify,
		&MediumEditWidgetTests::giftWidgetCheck);

	registerWidget(new MediumParentsWidget(),
		&MediumEditWidgetTests::parentsWidgetModify,
		&MediumEditWidgetTests::parentsWidgetCheck,
		NULL,
		"MediumParentsWidget can not be modifed at all");
	
	testWidgets();
}


bool MediumEditWidgetTests::addPerson(QString name, QString givenName, uint medId, uint roleId)
{
	if(medId < 1){
		QTest::qWarn("No valid medId");
		return false;
	}
	if(roleId < 1){
		QTest::qWarn("No valid roleId");
		return false;
	}
	if(name.isEmpty()){
		QTest::qWarn("Name is empty");
		return false;
	}
	if(givenName.isEmpty()){
		QTest::qWarn("GivenName is empty");
		return false;
	}
	QSqlQuery query;
	query.prepare("INSERT INTO meganizer_materialpersons (name, givenName) VALUES(:name, :givenName)");
	query.bindValue(":name", name);
	query.bindValue(":givenName", givenName);
	if(!query.exec()){
		QTest::qWarn("Could not insert person");
		return false;
	}
	query.prepare("SELECT id FROM meganizer_materialpersons WHERE name=:name AND givenName=:givenName");
	query.bindValue(":name", name);
	query.bindValue(":givenName", givenName);
	query.exec();
	if(!query.first()){
		QTest::qWarn("Could not fetch id of person (SELECT)");
		return false;
	}
	uint personId = query.record().value("id").toUInt();
	if(personId < 1){
		QTest::qWarn("Could not fetch id of person");
		return false;
	}
	query.prepare("INSERT INTO meganizer_materialhaspersons"
								" (materialsId, materialPersonsId, materialPersonsRolesId)"
								" VALUES(:materialsId, :materialPersonsId, :materialPersonsRolesId)");
	query.bindValue(":materialsId", medId);
	query.bindValue(":materialPersonsId", personId);
	query.bindValue(":materialPersonsRolesId", roleId);
	return query.exec();
}

QTEST_MAIN(MediumEditWidgetTests)
