/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumsimpleeditwidgettest.h"
#include "mediumsimpleeditwidgettest.moc"

#include <mediumtitlewidget.h>
#include <mediumwishwidget.h>
#include <mediumliteraturewidget.h>
#include <mediummanufacturerwidget.h>
#include <mediumgenrewidget.h>
#include <mediumdescriptionwidget.h>
#include <medium.h>
#include <mediumsimpleeditwidget.h>

#include <QTest>
#include <QSignalSpy>
#include <QSpinBox>
#include <QDateEdit>
#include <QTimeEdit>
#include <QLineEdit>
#include <QPlainTextEdit>
#include <QCheckBox>

Q_DECLARE_METATYPE(MediumSimpleEditWidget*);

void MediumSimpleEditWidgetTest::testBasicUsage()
{
	Medium med;

	// test with empty property
	MediumSimpleEditWidget *edit = new MediumSimpleEditWidget(MediumSimpleEditWidget::DateEdit, "", "");
	QSignalSpy spy(edit, SIGNAL(error(QString)));
	edit->setMedium(&med);
	QCOMPARE(edit->isEnabled(), false);
	QCOMPARE(edit->store(), false);
	QCOMPARE(spy.count(), 1);
	delete edit;

	// test with NULL medium pointer
	edit = new MediumSimpleEditWidget(MediumSimpleEditWidget::LineEdit, "", "title");
	QSignalSpy spy2(edit, SIGNAL(error(QString)));
	edit->setMedium(NULL);
	QCOMPARE(edit->isEnabled(), false);
	QCOMPARE(edit->store(), false);
	QCOMPARE(spy2.count(), 1);
	delete edit;
	
	// property does not exists
	edit = new MediumSimpleEditWidget(MediumSimpleEditWidget::DateEdit, "", "adasdsad");
	QSignalSpy spy1(edit, SIGNAL(error(QString)));
	edit->setMedium(&med);
	QCOMPARE(edit->isEnabled(), false);
	QCOMPARE(edit->store(), false);
	QCOMPARE(spy1.count(), 1);
	delete edit;

	// test everything is correct and modified is false
	edit = new MediumSimpleEditWidget(MediumSimpleEditWidget::CheckBox, "", "wish");
	QSignalSpy spy3(edit, SIGNAL(error(QString)));
	edit->setMedium(&med);
	QCOMPARE(edit->isEnabled(), true);
	QCOMPARE(edit->modified(), false);
	QCOMPARE(edit->store(), true);
	QCOMPARE(spy3.count(), 0);
	delete edit;
	
	// test set property with wrong data type
	edit = new MediumSimpleEditWidget(MediumSimpleEditWidget::DateEdit, "", "wish");
	QSignalSpy spy4(edit, SIGNAL(error(QString)));
	edit->setMedium(&med);
	QCOMPARE(edit->isEnabled(), true);
	setValue(edit, "wish", &med);
	QCOMPARE(edit->store(), false);
	QCOMPARE(spy4.count(), 1);
	delete edit;

	// everything is correctyl configured and modified is true
	edit = new MediumSimpleEditWidget(MediumSimpleEditWidget::LineEdit, "", "title");
	QSignalSpy spy5(edit, SIGNAL(error(QString)));
	edit->setMedium(&med);
	QCOMPARE(edit->isEnabled(), true);
	setValue(edit, "title", &med);
	QCOMPARE(edit->modified(), true);
	QCOMPARE(edit->store(), true);
	QCOMPARE(edit->modified(), false);
	QCOMPARE(spy5.count(), 0);
	delete edit;
}

void MediumSimpleEditWidgetTest::testLineEditWidget()
{
	QString testTitle = "This is a real greate title, but it is much to long, so that only"
											"the first 120 characters are used as title! But this is not enough,"
											"so now we should have more than 120 characters.";
	Medium med;
	MediumTitleWidget titleWidget;
	titleWidget.setMedium(&med);
	QLineEdit *titleEdit = titleWidget.findChild<QLineEdit*>("titleEdit");
	QVERIFY(titleEdit);
	QTest::keyClicks(titleEdit, testTitle);
	QVERIFY(titleWidget.modified());
	QVERIFY(titleWidget.store());
	QCOMPARE(med.title(), testTitle.left(120));
}

void MediumSimpleEditWidgetTest::testPlainTextEditWidget()
{
	QString text = "This is a real greate title, but it is much to long, so that only"
									"the first 120 characters are used as title! But this is not enough,"
									"so now we should have more than 120 characters.";
	Medium med;
	MediumDescriptionWidget descriptionWidget;
	descriptionWidget.setMedium(&med);
	QPlainTextEdit *descriptionEdit = descriptionWidget.findChild<QPlainTextEdit*>("descriptionEdit");
	QVERIFY(descriptionEdit);
	QTest::keyClicks(descriptionEdit, text);
	QVERIFY(descriptionWidget.modified());
	QVERIFY(descriptionWidget.store());
	QCOMPARE(med.description(), text);
}

void MediumSimpleEditWidgetTest::testCheckBoxWidget()
{
	bool wish = true;
	Medium med;
	MediumWishWidget wishWidget;
	wishWidget.setMedium(&med);
	QCheckBox *wishEdit = wishWidget.findChild<QCheckBox*>("wishEdit");
	QVERIFY(wishEdit);
	QTest::keyClick(wishEdit, Qt::Key_Space);
	QVERIFY(wishWidget.modified());
	QVERIFY(wishWidget.store());
	QCOMPARE(med.wish(), wish);
}

void MediumSimpleEditWidgetTest::testDerivedClasses()
{
	addTestObject(new MediumTitleWidget(), "title");
	addTestObject(new MediumWishWidget(), "wish");
	addTestObject(new MediumLiteratureWidget(), "literatureOnly");
	addTestObject(new MediumManufacturerWidget(), "manufacturer");
	addTestObject(new MediumGenreWidget(), "genre");
	addTestObject(new MediumDescriptionWidget(), "description");

	setTestMedium(new Medium());
	baseTest();
}

QTEST_MAIN(MediumSimpleEditWidgetTest)
