/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumtest.h"
#include "mediumtest.moc"

#include <medium.h>
#include <materialpersonsmodel.h>
#include <materialperson.h>
#include <mediamodel.h>

#include <esqldatabase.h>
#include <efileinfo.h>
#include <efileservice.h>
#include <eclosemodalwidgetthread.h>

#include <QtTest/QtTest>
#include <QSqlQuery>
#include <QSqlError>
#include <QStringListModel>

void MediumTest::initTestCase()
{
	testMedId = 0;
	
	testInit = new DBTestInit(this);
	if(!testInit->initTestDB())
		QSKIP("Could not create test database", SkipAll);
	
	keys = testInit->prepareMediumKeys("Testtype");
	if(!keys.valid)
		QSKIP("Could not create test database", SkipAll);
}

void MediumTest::fetchMessages(QString msg)
{
	qDebug() << msg;
}

void MediumTest::operators()
{
	Medium med;
	med.setId(1);
	Medium med1;
	med1.setId(1);
	QCOMPARE(med, med1);
	// test operator == and !=
	QVERIFY(med == med1);
	med1.setId(0);
	QVERIFY(med != med1);
}

void MediumTest::genre()
{
	Medium *medium = new Medium();
	connectMedium(medium);

	// testing valid genre name
	medium->setGenre(keys.testGenre);
	QCOMPARE(medium->genre(), keys.testGenreId);
	QCOMPARE(medium->genreName(), keys.testGenre);
	// testing wrong genre name
	medium->setGenre(keys.invalid);
	QCOMPARE(medium->genreName(), keys.invalid);
	QVERIFY(medium->genre() == 0);
	// valid genre id
	medium->setGenre(keys.testGenreId);
	QCOMPARE(medium->genre(), keys.testGenreId);
	QCOMPARE(medium->genreName(), keys.testGenre);
	// testing wrong genre id
	medium->setGenre(keys.invalidId);
	QCOMPARE(medium->genre(), keys.invalidId);
	QCOMPARE(medium->genreName(), QString());
	// invalid genre name, with not set id
	medium->setGenre(keys.invalid, false);
	QCOMPARE(medium->genreName(), keys.invalid);
	QCOMPARE(medium->genre(), keys.invalidId);
	// not set name
	medium->setGenre(keys.testGenreId, false);
	QCOMPARE(medium->genreName(), keys.invalid);
	QCOMPARE(medium->genre(), keys.testGenreId);
	
	delete medium;
}

void MediumTest::manufacturer()
{
	Medium *med = new Medium();
	connectMedium(med);
	
	// testing correct name
	med->setManufacturer(keys.testManu);
	QCOMPARE(med->manufacturer(), keys.testManuId);
	QCOMPARE(med->manufacturerName(), keys.testManu);
	// testing wrong manu name
	med->setManufacturer(keys.invalid);
	QVERIFY(med->manufacturer() == 0);
	QCOMPARE(med->manufacturerName(), keys.invalid);
	// testing correct id
	med->setManufacturer(keys.testManuId);
	QCOMPARE(med->manufacturer(), keys.testManuId);
	QCOMPARE(med->manufacturerName(), keys.testManu);
	// testing wrong id
	med->setManufacturer(keys.invalidId);
	QCOMPARE(med->manufacturer(), keys.invalidId);
	QCOMPARE(med->manufacturerName(), QString());
	// testing wrong name with setId = false
	med->setManufacturer(keys.invalid, false);
	QCOMPARE(med->manufacturer(), keys.invalidId);
	QCOMPARE(med->manufacturerName(), keys.invalid);
	// testing correct id with setName false
	med->setManufacturer(keys.testManuId, false);
	QCOMPARE(med->manufacturer(), keys.testManuId);
	QCOMPARE(med->manufacturerName(), keys.invalid);
	
	delete med;
}


void MediumTest::readAndSetBasicProperties()
{
	QString title = "'SELECT * FROM meganizer_user;";
	QString description = "This is great Medium!; INSERT INTO test;";
	int eval =  5;
	
	Medium *medium = new Medium();
	connectMedium(medium);
	
	// checking init status
	QVERIFY(medium->id() == 0);
	QCOMPARE(medium->title(), QString());
	QCOMPARE(medium->image(), QImage());
	QCOMPARE(medium->description(), QString());
	QVERIFY(medium->listPrice() == 0);
	QVERIFY(medium->listPriceCurrency() == 1);
	QVERIFY(medium->boughtPrice() == 0);
	QVERIFY(medium->boughtPriceCurrency() == 1);
	QVERIFY(medium->evaluation() == 0);
	QCOMPARE(medium->wish(), false);
	QCOMPARE(medium->giftOnly(), false);
	QCOMPARE(medium->literatureOnly(), false);
	QCOMPARE(medium->materialPersonsCombined(), QString());
	QCOMPARE(medium->minsCombined(), QString());
	QVERIFY(medium->genre() == 0);
	QVERIFY(medium->manufacturer() == 0);
	
	medium->setTitle(title);
	QCOMPARE(medium->title(), title);
	
	QImage image(":/images/meganizer.png");
	medium->setImage(image);
	QCOMPARE(medium->image(), image);
	
	medium->setDescription(description);
	QCOMPARE(medium->description(), description);
	
	medium->setListPriceCurrency(keys.testCurShort);
	QCOMPARE(medium->listPriceCurrency(), keys.testCurId);
	medium->setListPriceCurrency(keys.invalid);
	QEXPECT_FAIL("", "Validation tests on Id are not done!", Continue);
	QVERIFY(medium->listPriceCurrency() == 0);
	medium->setListPriceCurrency(keys.testCur);
	QCOMPARE(medium->listPriceCurrency(), keys.testCurId);
	medium->setListPriceCurrency(keys.invalidId);
	// qcompare has problems with unsigned int and int
	QVERIFY(medium->listPriceCurrency() == keys.invalidId);
	
	medium->setListPrice(11.11);
	QCOMPARE(medium->listPrice(), 11.11);
	
	medium->setBoughtPriceCurrency(keys.testCurShort);
	QCOMPARE(medium->boughtPriceCurrency(), keys.testCurId);
	medium->setBoughtPriceCurrency(keys.invalid);
	QEXPECT_FAIL("", "Validation tests on Id are not done!", Continue);
	QVERIFY(medium->boughtPriceCurrency() == 0);
	medium->setBoughtPriceCurrency(keys.testCur);
	QCOMPARE(medium->boughtPriceCurrency(), keys.testCurId);
	medium->setBoughtPriceCurrency(0);
	QVERIFY(medium->boughtPriceCurrency() == 0);
	
	medium->setEvaluation(eval);
	QCOMPARE(medium->evaluation(), eval);

	medium->setWish(true);
	QCOMPARE(medium->wish(), true);
	medium->setWish(false);
	QCOMPARE(medium->wish(), false);
	
	medium->setLiteratureOnly(true);
	QCOMPARE(medium->literatureOnly(), true);
	medium->setLiteratureOnly(false);
	QCOMPARE(medium->literatureOnly(), false);
	
	medium->setGiftOnly(true);
	QCOMPARE(medium->giftOnly(), true);
	medium->setGiftOnly(false);
	QCOMPARE(medium->giftOnly(), false);

	// testing type
	medium->setType(keys.testTypeId);
	QCOMPARE(medium->type(), keys.testTypeId);
	QCOMPARE(medium->typeName(), keys.testType);
	medium->setType(0);
	QCOMPARE(medium->type(), 0u);
	QCOMPARE(medium->typeName(), QString());
	medium->setType(keys.testType);
	QCOMPARE(medium->type(), keys.testTypeId);
	QCOMPARE(medium->typeName(), keys.testType);
	medium->setType(QString());
	QCOMPARE(medium->type(), 0u);
	QCOMPARE(medium->typeName(), QString());
	medium->setType(keys.testTypeId, false);
	QCOMPARE(medium->type(), keys.testTypeId);
	QCOMPARE(medium->typeName(), QString());
	medium->setType("test", false);
	QCOMPARE(medium->type(), keys.testTypeId);
	QCOMPARE(medium->typeName(), QString("test"));
	delete medium;

	medium = new Medium("test", keys.testType, "");
	QCOMPARE(medium->type(), keys.testTypeId);
	QCOMPARE(medium->typeName(), keys.testType);
}

void MediumTest::copyMedium()
{
	Medium *med = new Medium();
	connectMedium(med);
	
	QString title = "Title test";
	QImage image(":/images/meganizer.png");
	QString desc = "Super Description";
	int eval = 5;
	double listPrice = 11.11;
	double boughtPrice = 22.22;
	
	med->setTitle(title);
	med->setGenre(keys.testGenre);
	med->setType(keys.testTypeId);
	med->setImage(image);
	med->setDescription(desc);
	med->setListPriceCurrency(keys.testCurId);
	med->setListPrice(listPrice);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setBoughtPrice(boughtPrice);
	med->setEvaluation(5);
	med->setWish(true);
	med->setLiteratureOnly(true);
	med->setGiftOnly(true);
	med->setManufacturer(keys.testManuId);
	med->setUser(keys.invalidId);
	med->addKeyword("test");
// 	med->setMaterialPerson();
// 	med->setMIN();

	Medium *copy = med->copyMedium();
	QVERIFY(copy);
	
	QCOMPARE(copy->title(), title);
	QCOMPARE(copy->genreName(), keys.testGenre);
	QCOMPARE(copy->genre(), keys.testGenreId);
	QCOMPARE(copy->type(), keys.testTypeId);
	QCOMPARE(copy->image(), image);
	QCOMPARE(copy->description(), desc);
	QCOMPARE(copy->listPriceCurrency(), keys.testCurId);
	QCOMPARE(copy->listPrice(), listPrice);
	QCOMPARE(copy->boughtPriceCurrency(), keys.testCurId);
	QCOMPARE(copy->boughtPrice(), boughtPrice);
	QCOMPARE(copy->evaluation(), eval);
	QCOMPARE(copy->wish(), true);
	QCOMPARE(copy->literatureOnly(), true);
	QCOMPARE(copy->giftOnly(), true);
	QCOMPARE(copy->manufacturer(), keys.testManuId);
	QCOMPARE(copy->user(), keys.invalidId);
	QCOMPARE(copy->keywords()->rowCount(), 1);
	
	delete med;
	delete copy;
}

void MediumTest::insertAndLoadBasicProperties()
{
	// prepare medium
	Medium *medium = new Medium();
	connectMedium(medium);

	medium->setTitle("'SELECT * FROM meganizer_user;");
	medium->setDescription("This is greate show; INSERT INTO test test = 10;");
	medium->setGenre(keys.testGenreId);
	medium->setManufacturer(keys.testManuId);
	medium->setImage(QImage(":/images/meganizer.png"));
	medium->setListPriceCurrency(keys.testCurId);
	medium->setListPrice(11.11);
	medium->setBoughtPriceCurrency(keys.testCurId);
	medium->setBoughtPrice(22.22);
	medium->setEvaluation(5);
	
	// storing should not be successfull here, because we have no media type
	ESqlDatabase db = ESqlDatabase::database();
	db.close();
	ECloseModalWidgetThread thread;
	thread.start();
	if(!thread.waitForStart())
		QSKIP("Could not start thread", SkipSingle);
	QCOMPARE(medium->store(), false);
	thread.stopThread();
	QCOMPARE(db.isOpen(), false);
	
	medium->setType(keys.testTypeId);
	Medium *checkMed = medium->copyMedium();
	QVERIFY(checkMed);
	db.open();
	QCOMPARE(medium->store(), true);
	QVERIFY(medium->id() > 0);
	QCOMPARE(db.isOpen(), true);
	db.close();
	
	testMedId = medium->id();
	delete medium;
	medium = new Medium();
	connectMedium(medium);
	QCOMPARE(medium->load(testMedId), true);
	QCOMPARE(medium->id(), testMedId);
	
	QCOMPARE(medium->title(), checkMed->title());
	QCOMPARE(medium->genreName(), checkMed->genreName());
	QCOMPARE(medium->manufacturerName(), checkMed->manufacturerName());
	QCOMPARE(medium->type(), checkMed->type());
	QEXPECT_FAIL("", "Should work normally", Continue);
	QCOMPARE(medium->image(), checkMed->image());
	QCOMPARE(medium->evaluation(), checkMed->evaluation());
	QCOMPARE(medium->wish(), checkMed->wish());
	QCOMPARE(medium->literatureOnly(), checkMed->literatureOnly());
	QCOMPARE(medium->giftOnly(), checkMed->giftOnly());
	QCOMPARE(medium->listPriceCurrency(), checkMed->listPriceCurrency());
	QCOMPARE(medium->listPrice(), checkMed->listPrice());
	QCOMPARE(medium->boughtPriceCurrency(), checkMed->boughtPriceCurrency());
	QCOMPARE(medium->boughtPrice(), checkMed->boughtPrice());
	
	// update
	medium->setWish(true);
	medium->setGiftOnly(true);
	medium->setLiteratureOnly(true);
	QCOMPARE(medium->store(), true);
	delete checkMed;
	checkMed = medium->copyMedium();
	medium->setId(0);
	medium->load(testMedId);
	QCOMPARE(medium->id(), testMedId);
	QCOMPARE(medium->wish(), checkMed->wish());
	QCOMPARE(medium->giftOnly(), checkMed->giftOnly());
	QCOMPARE(medium->literatureOnly(), checkMed->literatureOnly());
	
	delete medium;
	delete checkMed;
}

void MediumTest::addOwner()
{
	if(testMedId <= 0)
		QSKIP("No inserted medium present", SkipSingle);
	
	Medium *med = new Medium();
	connectMedium(med);
	med->load(testMedId);
	QCOMPARE(med->id(), testMedId);
	
	// creating new owner
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open())
		QSKIP("Could not connect to database", SkipSingle);
	QSqlQuery query;
	if(!query.exec("INSERT INTO meganizer_users (name, givenName, userName) VALUES('Mustermann', 'Max', 'max')"))
		QSKIP("Could not create Mr. Mustermann", SkipSingle);
	query.exec("SELECT id FROM meganizer_users WHERE name='Mustermann'");
	query.first();
	unsigned int userId = query.record().value("id").toUInt();
	if(userId == 0)
		QSKIP("Could not fetch Mr. Mustermann's userId", SkipSingle);
	db.close();
	
	med->setUser(userId);
	QCOMPARE(med->user(), userId);
	QCOMPARE(med->store(true), true);
	
	delete med;
}

void MediumTest::personsHandling()
{
	if(testMedId <= 0)
		QSKIP("No inserted medium present", SkipSingle);
	
	Medium *med = new Medium();
	med->load(testMedId);
	QCOMPARE(med->id(), testMedId);
	
	// add test persons into database
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open())
		QSKIP("Could not connect to database", SkipSingle);
	QList<QStringList> persons;
	QStringList personString;
	personString << "Mustermann" << "Max";
	persons << personString;
	personString.clear();
	personString << "Mustermann" << "Erna";
	persons << personString;
	personString.clear();
	personString << "Mustermann" << "Manfred Maria";
	persons << personString;
	personString.clear();
	personString << "Dicht" << "Haucke";
	persons << personString;
	personString.clear();
	
	QSqlQuery query;
	QList<unsigned int> personIds;
	foreach(QStringList person, persons){
		query.prepare("INSERT INTO meganizer_materialpersons (name, givenName) VALUES(:name, :givenName)");
		query.bindValue(":name", person[0]);
		query.bindValue(":givenName", person[1]);
		if(!query.exec())
			QSKIP("Could not insert test person", SkipSingle);
		query.prepare("SELECT id FROM meganizer_materialpersons WHERE name=:name AND givenName=:givenName");
		query.bindValue(":name", person[0]);
		query.bindValue(":givenName", person[1]);
		query.exec();
		query.first();
		unsigned int id = query.record().value("id").toUInt();
		if(id <= 0)
			QSKIP("Could not fetch person id", SkipSingle);
		personIds << id;
	}
	
	
	// insert role
	QString testRole = "Testrole";
	if(!query.exec("INSERT INTO meganizer_materialpersonsroles (name, description) VALUES('"+ testRole +"', 'Role for testing purposes')"))
		QSKIP("Could not create Testrole", SkipSingle);
	query.exec("SELECT id FROM meganizer_materialpersonsroles WHERE name='"+ testRole +"'");
	query.first();
	unsigned int testRoleId = query.record().value("id").toUInt();
	QVERIFY(testRoleId > 0);
	
	db.close();
	
 	
	MaterialPersonsModel *model = med->materialPersons();
	QVERIFY(model);
	foreach(unsigned int personId, personIds){
		QCOMPARE(model->addPerson(personId, testRoleId), true);
	}
	
	// check storing and loading
	QCOMPARE(med->store(), true);
	delete med;
	med = new Medium();
	connectMedium(med);
	med->load(testMedId);
	QCOMPARE(med->id(), testMedId);
	model = med->materialPersons();
	QCOMPARE(personIds.size(), model->rowCount());
	for(int i = 0; i < model->rowCount(); i++){
		MaterialPerson *person = qobject_cast<MaterialPerson*>(model->personFromIndex(model->index(i, 0)));
		QVERIFY(person);
		personIds.removeAll(person->id());
	}
	QCOMPARE(personIds.size(), 0);
	
	delete med;
}

void MediumTest::minHandling()
{
	QSKIP("Not implemented yet", SkipSingle);
}

void MediumTest::valueAndSetValueString_data()
{
	QTest::addColumn<QString>("attribute");
	QTest::addColumn<QVariant>("value");
	QTest::addColumn<bool>("result");
	QTest::addColumn<QVariant>("dataResult");
	
	QTest::newRow("invalid attribute") << keys.invalid << QVariant(keys.invalidId) << false << QVariant();
	
	QTest::newRow("type invalidId") << "type" << QVariant(keys.invalidId) << true << QVariant(keys.invalidId);
	QTest::newRow("type invalid") << "type" << QVariant(keys.invalid) << false << QVariant(0);
	
	QTest::newRow("manufacturer invalid") << "manufacturer" << QVariant(keys.invalid) << true << QVariant(keys.invalid);
	QTest::newRow("manufacturer invalidId") << "manufacturer" << QVariant(keys.invalidId) << true << QVariant(QString());
	QTest::newRow("manufacturer testManuId") << "manufacturer" << QVariant(keys.testManuId) << true << QVariant(keys.testManu);
	QTest::newRow("manufacturer testManu") << "manufacturer" << QVariant(keys.testManu) << true << QVariant(keys.testManu);
	
	QTest::newRow("genre invalid") << "genre" << QVariant(keys.invalid) << true << QVariant(0);
	QTest::newRow("genre invalidId") << "genre" << QVariant(keys.invalidId) << true << QVariant(QString());
	QTest::newRow("genre testGenreId") << "genre" << QVariant(keys.testGenreId) << true << QVariant(keys.testGenre);
	QTest::newRow("genre testGenre") << "genre" << QVariant(keys.testGenre) << true << QVariant(keys.testGenre);
	
	QTest::newRow("owner invalid") << "owner" << QVariant(keys.invalid) << false << QVariant(0);
	QTest::newRow("owner invalidId") << "owner" << QVariant(keys.invalidId) << true << QVariant(keys.invalidId);
// 	QTest::newRow("owner testUserId") << "genre" << QVariant(testGenreId) << true << QVariant(testGenre);
// 	QTest::newRow("owner testUser") << "genre" << QVariant(testGenre) << true << QVariant(testGenre);
}

void MediumTest::valueAndSetValueString()
{
	Medium *med = new Medium();
	connectMedium(med);
	
	QFETCH(QString, attribute);
	QFETCH(QVariant, value);
	QFETCH(bool, result);
	QFETCH(QVariant, dataResult);

	QStringList abortRows;
	abortRows << "manufacturer invalid" << "manufacturer testManu" << "genre invalid" << "genre testGenre";
	// Abort skips this data row, not the whole test
	if(abortRows.contains(QTest::currentDataTag()))
		QEXPECT_FAIL(QTest::currentDataTag(), "Setting "+ attribute.toLatin1() +" names through setValue is not supported right now", Abort);

	QCOMPARE(med->setValue(attribute, value), result);
	QCOMPARE(med->value(attribute, Qt::DisplayRole), dataResult);
	
	delete med;
}

void MediumTest::keywords()
{
	Medium *med = new Medium();
	connectMedium(med);

	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->keywords()->rowCount(), 0);

	QStringList keywords;
	keywords << "Keyword 1" << "Keyword 2" << "Keyword 3";

	// add keywords
	foreach(QString keyword, keywords)
		med->addKeyword(keyword);
	QCOMPARE(med->keywords()->rowCount(), keywords.size());

	// store and load keywords
	QCOMPARE(med->store(), true);
	delete med;
	med = new Medium();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->keywords()->rowCount(), keywords.size());

	// removing keywords
	QVERIFY(keywords.size() > 1);
	QCOMPARE(med->removeKeyword(keywords[1]), true);
	QCOMPARE(med->store(), true);
	delete med;
	med = new Medium();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->keywords()->rowCount(), keywords.size() - 1);

	// test that keywords are case insensitiv
	med->addKeyword("kEywORD 1");
	QCOMPARE(med->keywords()->rowCount(), keywords.size() - 1);
	QCOMPARE(med->removeKeyword("kEywORD 1"), true);
	QCOMPARE(med->store(), true);
	delete med;
	med = new Medium();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->keywords()->rowCount(), keywords.size() - 2);

	delete med;
}

void MediumTest::files()
{
	QApplication::setOrganizationName("EngSaS");
	QApplication::setApplicationName("Testing");
	QSettings settings;
	settings.setValue("fileServiceId", "efilesystem");
	
	EFileService service(this);
	QCOMPARE(service.serviceLoaded(), true);
	
	QTemporaryFile file1, file2, file3;
	QCOMPARE(file1.open(), true);
	QCOMPARE(file2.open(), true);
	QCOMPARE(file3.open(), true);
	
	EFileInfo info1, info2, info3;
	info1 = service.info(file1.fileName());
	info2 = service.info(file2.fileName());
	info3 = service.info(file3.fileName());
	QCOMPARE(info1.isValid(), true);
	QCOMPARE(info2.isValid(), true);
	QCOMPARE(info3.isValid(), true);

	Medium *med = new Medium();
	connectMedium(med);

	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->files().size(), 0);
	
	QCOMPARE(med->addFile(info1), true);
	QCOMPARE(med->addFile(info2), true);
	QCOMPARE(med->addFile(info3), true);
	QCOMPARE(med->files().size(), 3);
	
	// double entries should be avoided
	QCOMPARE(med->addFile(info2), false);
	QCOMPARE(med->files().size(), 3);
	
	// store and load keywords
	QCOMPARE(med->store(), true);
	delete med;
	med = new Medium();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->files().size(), 3);

	// removing keywords
	QCOMPARE(med->removeFile(info2.id()), true);
	QCOMPARE(med->files().size(), 2);
	QVERIFY(med->files().indexOf(info1) > -1);
	QVERIFY(med->files().indexOf(info2) == -1);
	QVERIFY(med->files().indexOf(info3) > -1);
	QCOMPARE(med->store(), true);
	delete med;
	med = new Medium();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->files().size(), 2);
	QVERIFY(med->files().indexOf(info1) > -1);
	QVERIFY(med->files().indexOf(info2) == -1);
	QVERIFY(med->files().indexOf(info3) > -1);
}

void MediumTest::childs()
{
	// prepare three child media
	Medium *child1 = new Medium();
	child1->setTitle("Child 1");
	child1->setGenre(keys.testGenre);
	child1->setType(keys.testTypeId);
	child1->setListPriceCurrency(keys.testCurId);
	child1->setBoughtPriceCurrency(keys.testCurId);
	child1->setManufacturer(keys.testManuId);
	QCOMPARE(child1->store(), true);
	Medium *child2 = new Medium();
	child2->setTitle("Child 2");
	child2->setGenre(keys.testGenre);
	child2->setType(keys.testTypeId);
	child2->setListPriceCurrency(keys.testCurId);
	child2->setBoughtPriceCurrency(keys.testCurId);
	child2->setManufacturer(keys.testManuId);
	QCOMPARE(child2->store(), true);
	Medium *child3 = new Medium();
	child3->setTitle("Child 3");
	child3->setGenre(keys.testGenre);
	child3->setType(keys.testTypeId);
	child3->setListPriceCurrency(keys.testCurId);
	child3->setBoughtPriceCurrency(keys.testCurId);
	child3->setManufacturer(keys.testManuId);
	QCOMPARE(child3->store(), true);
	
	Medium *med = new Medium();
	connectMedium(med);
	QCOMPARE(med->load(testMedId), true);

	// setting null model
	QVERIFY(!med->childs());
	med->setChildModel(NULL);
	QVERIFY(!med->childs());
	QVERIFY(!med->removeChild(QModelIndex()));

	// test the storing of the childs
	MediaModel *model = new MediaModel(NULL, NULL);
	med->setChildModel(model);
	QCOMPARE(med->childs(), model);
	model->addMedium(child1->copyMedium());
	model->addMedium(child2->copyMedium());
	model->addMedium(child3->copyMedium());
	QCOMPARE(model->rowCount(), 3);
	// removed not stored media
	// the medium is removed an storing the changes
	QCOMPARE(med->removeChild(model->index(1, 0)), true);
	QCOMPARE(model->mediumFromIndex(model->index(1, 0))->markForRemovel(), true);
	QCOMPARE(model->rowCount(), 3);
	QCOMPARE(med->store(), true);
	QCOMPARE(model->rowCount(), 2);
	uint mathasmatId1 = model->mediumFromIndex(model->index(0,0))->materialHasMaterialsId();
	uint mathasmatId3 = model->mediumFromIndex(model->index(1,0))->materialHasMaterialsId();
	QVERIFY(mathasmatId1 > 0);
	QVERIFY(mathasmatId3 > 0);
	delete med;

	// test loading
	med = new Medium();
	connectMedium(med);
	model = new MediaModel(NULL, NULL);
	med->setChildModel(model);
	QCOMPARE(med->load(testMedId), true);
	QVERIFY(med->childs());
	QCOMPARE(model->rowCount(), 2);
	QCOMPARE(model->mediumFromIndex(model->index(0,0))->id(), child1->id());
	QCOMPARE(model->mediumFromIndex(model->index(1,0))->id(), child3->id());
	QCOMPARE(model->mediumFromIndex(model->index(0,0))->materialHasMaterialsId(), mathasmatId1);
	QCOMPARE(model->mediumFromIndex(model->index(1,0))->materialHasMaterialsId(), mathasmatId3);

	// remove childs, which are already stored at the database
	QCOMPARE(med->removeChild(model->index(0,0)), true);
	QCOMPARE(model->mediumFromIndex(model->index(0, 0))->markForRemovel(), true);
	QCOMPARE(med->store(), true);
	QCOMPARE(model->rowCount(), 1);
	delete med;
	med = new Medium();
	connectMedium(med);
	model = new MediaModel(NULL, NULL);;
	med->setChildModel(model);
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(model->rowCount(), 1);
	QCOMPARE(model->mediumFromIndex(model->index(0, 0))->id(), child3->id());
	QCOMPARE(model->mediumFromIndex(model->index(0, 0))->materialHasMaterialsId(), mathasmatId3);

	delete child1;
	delete child2;
	delete child3;
	delete med;
}

void MediumTest::connectMedium(Medium* med)
{
	connect(med, SIGNAL(error(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(warning(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(information(QString)), this, SLOT(fetchMessages(QString)));
}

QTEST_MAIN(MediumTest)
