/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "minmodeltest.h"
#include "minmodeltest.moc"

#include <minmodel.h>
#include "modeltest.h"
#include "../lib/internal/mediummin.h"
#include <dbtestinit.h>
#include <esqldatabase.h>

#include <QtTest/QtTest>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>

void MINModelTest::initTestCase()
{
	testMINTypeId = 0;
	testInit = new DBTestInit(this);
	testInit->initTestDB();
	keys = testInit->prepareMediumKeys("Testtype");
	
	if(keys.valid){
		ESqlDatabase db = ESqlDatabase::database();
		db.open();
		QSqlQuery query;
		
		// we need the track type also to test track handling
		testMINType = "Test Type";
		if(!query.exec("INSERT INTO meganizer_mintypes (name) VALUES('"+ testMINType +"')"))
			QFAIL("Could not insert test MIN");
		
		query.exec("SELECT id FROM meganizer_mintypes WHERE name='"+ testMINType +"'");
		if(!query.first())
			QFAIL("Could not select test MIN id");
		testMINTypeId = query.record().value("id").toUInt();
		QVERIFY(testMINTypeId > 0);
	}
}

void MINModelTest::mediumMINTest()
{
	// testing constructors and default values
	MediumMIN min;
	QVERIFY(min.rowId() == 0);
	QCOMPARE(min.min(), QString());
	QVERIFY(min.type() == 0);
	QCOMPARE(min.typeName(), QString());
	QCOMPARE(min.markForRemovel(), false);
	
	QString nr = "1234567";
	MediumMIN min2(nr, testMINType, 1);
	QVERIFY(min2.rowId() == 1);
	QCOMPARE(min2.min(), nr);
	QVERIFY(min2.type() == testMINTypeId);
	QCOMPARE(min2.typeName(), testMINType);
	QCOMPARE(min2.markForRemovel(), false);
	
	MediumMIN min3(nr, testMINTypeId, 1);
	QVERIFY(min3.rowId() == 1);
	QCOMPARE(min3.min(), nr);
	QVERIFY(min3.type() == testMINTypeId);
	QCOMPARE(min3.typeName(), testMINType);
	QCOMPARE(min3.markForRemovel(), false);
	
	// testing set functions
	MediumMIN min4;
	
	// rowId
	min4.setRowId(1);
	QVERIFY(min4.rowId() == 1);
	
	// type typeName and setType
	// setting via id
	min4.setType(testMINTypeId, false);
	QVERIFY(min4.type() == testMINTypeId);
	QCOMPARE(min4.typeName(), QString());
	min4.setType(0);
	QVERIFY(min4.type() == 0);
	min4.setType(testMINTypeId);
	QVERIFY(min4.type() == testMINTypeId);
	QCOMPARE(min4.typeName(), testMINType);
	min4.setType(0);
	QVERIFY(min4.type() == 0);
	QCOMPARE(min4.typeName(), QString());
	// setting via name
	min4.setType(testMINType, false);
	QVERIFY(min4.type() == 0);
	QCOMPARE(min4.typeName(), testMINType);
	min4.setType(QString());
	QCOMPARE(min4.typeName(), QString());
	min4.setType(testMINType);
	QVERIFY(min4.type() == testMINTypeId);
	QCOMPARE(min4.typeName(), testMINType);
	min4.setType(QString());
	QVERIFY(min4.type() == 0);
	QCOMPARE(min4.typeName(), QString());
	
	// markForRemoval and setMarkForRemoval
	MediumMIN min5;
	min5.setMarkForRemovel(true);
	QCOMPARE(min5.markForRemovel(), true);
	min5.setMarkForRemovel(false);
	QCOMPARE(min5.markForRemovel(), false);
	
	// setMIN and min
	MediumMIN min6;
	min6.setMIN(nr);
	QCOMPARE(min6.min(), nr);
	
	MediumMIN *orig = new MediumMIN(nr, testMINTypeId, 1);
	orig->setMarkForRemovel(false);
	MediumMIN *copy = orig->copyMIN();
	QCOMPARE(copy->rowId(), orig->rowId());
	QCOMPARE(copy->type(), orig->type());
	QCOMPARE(copy->typeName(), orig->typeName());
	QCOMPARE(copy->min(), orig->min());
	QCOMPARE(copy->markForRemovel(), orig->markForRemovel());
	delete orig;
	delete copy;
	
	MediumMIN orig1(nr, testMINTypeId);
	MediumMIN copy1(nr, testMINTypeId);
	QVERIFY(orig1 == copy1);
	
	orig1.setType("asdasd");
	QVERIFY(orig1 != copy1);
	orig1.setType(testMINTypeId);
	
	orig1.setMIN("asdasd");
	QVERIFY(orig1 != copy1);
	orig1.setMIN(nr);
	
	orig1.setMarkForRemovel(true);
	QVERIFY(orig1 == copy1);
}

void MINModelTest::addingAndRemovingMINs()
{
	MINModel *model = new MINModel(NULL, this);
	
	MediumMIN *min = model->addMIN();
	QCOMPARE(min->min(), QString());
	QVERIFY(min->rowId() == 0);
	QVERIFY(min->type() == 0);
	QCOMPARE(min->typeName(), QString());
	QCOMPARE(min->markForRemovel(), false);
	QCOMPARE(model->rowCount(), 1);
	
	QCOMPARE(model->removeMIN(min), true);
	QCOMPARE(model->rowCount(), 0);
	
	QString nr = "12345";
	min = model->addMIN(nr, testMINType);
	QCOMPARE(min->min(), nr);
	QVERIFY(min->type() == testMINTypeId);
	QCOMPARE(min->typeName(), testMINType);
	QCOMPARE(model->rowCount(), 1);
	// testing data function
	QCOMPARE(model->data(model->index(0, 0)).toString(), nr);
	QVERIFY(model->data(model->index(0, 1)).toUInt() == testMINTypeId);
	QCOMPARE(model->removeMIN(model->indexFromMIN(min)), true);
	QCOMPARE(model->rowCount(), 0);
	
	min = model->addMIN(nr, testMINTypeId);
	QCOMPARE(min->min(), nr);
	QVERIFY(min->type() == testMINTypeId);
	QCOMPARE(min->typeName(), testMINType);
	QCOMPARE(model->rowCount(), 1);
	QCOMPARE(model->removeMIN(model->indexFromMIN(min)), true);
	QCOMPARE(model->rowCount(), 0);
	
	min = new MediumMIN(nr, testMINTypeId);
	QCOMPARE(model->addMIN(min), true);
	QCOMPARE(model->rowCount(), 1);
	QCOMPARE(model->removeMIN(min), true);
	QCOMPARE(model->rowCount(), 0);
	
	QCOMPARE(model->addMIN(nr, testMINTypeId, 0), true);
	QCOMPARE(model->rowCount(), 1);
	QCOMPARE(model->removeMIN(model->index(0, 0)), true);
	QCOMPARE(model->rowCount(), 0);
	
	delete model;
}

void MINModelTest::standardTests()
{
	MINModel *model = new MINModel(NULL, this);
	ModelTest *test = new ModelTest(model, this);
	
	QList<QStringList> items;
	QStringList tmp;
	
	tmp << "123456" << "0" << "Test";
	items << tmp;
	tmp.clear();
	
	tmp << "" << "0" << "";
	items << tmp;
	tmp.clear();
	
	tmp << "ABCDE" << "0" << "Aha";
	items << tmp;
	tmp.clear();
	
	tmp << "IJKL" << "20" << "";;
	items << tmp;
	tmp.clear();
	
	tmp << "" << "20" << "";
	items << tmp;
	tmp.clear();
	
	tmp << "" << "0" << "test";
	items << tmp;
	
	// insert mins
	foreach(QStringList list, items){
		MediumMIN *min = model->addMIN();
		min->setMIN(list[0]);
		min->setType(list[1].toUInt());
		if(!list[2].isEmpty())
			min->setType(list[2], false);
	}
	
	QCOMPARE(model->rowCount(), items.size());
	
	// remove a min from the beginning, the middle and the end
	if(model->rowCount() > 0)
		model->removeRow(0);
	
	if(model->rowCount() > 0)
		model->removeRow(model->rowCount() - 1);
	
	if(model->rowCount() > 2){
		int half = model->rowCount() / 2;
		model->removeRow(half);
	}
	
	delete test;
	delete model;
}

void MINModelTest::minAndMinsTest()
{
	if(!keys.valid)
		QSKIP("Database not working, but needed", SkipSingle);
	
	// prepare test case
	MINModel *model = new MINModel(NULL, this);
	QStringList mins;
	mins << "9783981019452";
	mins << "1232134";
	foreach(QString min, mins){
		model->addMIN(min, testMINType);
	}
	QCOMPARE(model->rowCount(), mins.size());

	// testing mins();
	QStringList result = model->mins(QString());
	QCOMPARE(result.size(), 0);
	result = model->mins("asdasd");
	QCOMPARE(result.size(), 0);
	result = model->mins(testMINType);
	QCOMPARE(result.size(), mins.size());
	foreach(QString min, result){
		QVERIFY(mins.contains(min));
	}

	// testing min();
	QVERIFY(!model->min(QString()));
	QCOMPARE(model->min(testMINType)->min(), mins[0]);

	delete model;
}

QTEST_MAIN(MINModelTest)
