/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "minvalidatortests.h"
#include "minvalidatortests.moc"

#include "minvalidator.h"
#include "testplugindir.h"
#include <epluginloader.h>
#include "minvalidatorplugin.h"

#include <QtTest/QtTest>

void MINValidatorTests::initTestCase()
{
	QList<EModuleListWidgetItem*> pluginProtocol;
	QList<MINValidatorPlugin*> plugins = EPluginLoader<MINValidatorPlugin>::loadPlugins(TestPluginDir::libDir(), pluginProtocol);
	if(plugins.size() < 1)
		QSKIP("Could not find MINValidatorPlugins", SkipAll);
	validator = new MINValidator(plugins);
}

void MINValidatorTests::cleanupTestCase()
{
	delete validator;
}

void MINValidatorTests::type_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("inputType");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("855937325x") << "855937325x" << "ISBN-10" << "ISBN-10";
	QTest::newRow("855937325X") << "855937325X" << "ISBN-10" << "ISBN-10";
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << "ISBN-10" << "";
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << "ISBN-10" << "";
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << "ISBN-10" << "ISBN-10";
	QTest::newRow("3-77kk23-7325-9") << "3-77kk23-7325-9" << "ISBN-10" << "";
	QTest::newRow("3-7723-73-9") << "3-7723-73-9" << "ISBN-10" << "";
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << "EAN" << "EAN";
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1" << "EAN" << "";
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1" << "EAN" << "";
	
	// instore EAN-13
	QTest::newRow("2404105 001722") << "2404105 001722" << "EAN" << "EAN";
	// in store EAN-8 (Aldi Schovit ;-) )
	QTest::newRow("29033805") << "29033805" << "EAN" << "EAN";
	QTest::newRow("29023805") << "29023805" << "EAN" << "";
	QTest::newRow("2?./0dÄa") << "2?./0dÄa" << "EAN" << "";
	
	QTest::newRow("5 449000 09624-123") << "5 449000 09624-123" << "EAN" << "";
	QTest::newRow("123456") << "123456" << "EAN" << "";

	// ISSN
	QTest::newRow("0028 - 9604") << "0028 - 9604" << "ISSN" << "ISSN";
	QTest::newRow("00287604") << "00287604" << "ISSN" << "";
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << "ISSN" << "";
	QTest::newRow("1432640X") << "1432640X" << "ISSN" << "ISSN";
	QTest::newRow("1432640x") << "1432640x" << "ISSN" << "ISSN";
	QTest::newRow("14326402") << "14326402" << "ISSN" << "";
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << "ISSN" << "";
}

void MINValidatorTests::type()
{
	QFETCH(QString, min);
	QFETCH(QString, inputType);
	QFETCH(QString, result);
	
	// if type is not supported, "" should be the result
	if((!inputType.isEmpty()) && (!validator->isSupported(inputType))){
		QCOMPARE(validator->type(min), QString());
	}
	else{
		QCOMPARE(validator->type(min), result);
	}
}

void MINValidatorTests::possibleTypes_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QStringList>("result");
	
	QStringList isbnEan;
	isbnEan << "ISBN-10" << "EAN";
	
	QTest::newRow("855937325x") << "855937325x" << isbnEan;
	QTest::newRow("855937325X") << "855937325X" << isbnEan;
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << isbnEan;
	QTest::newRow("3-7723-7325-81") << "3-7723-7325-8" << isbnEan;
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << isbnEan;
	QTest::newRow("3-77kk23-7325-9") << "3-77kk23-7325-9" << isbnEan;
	QTest::newRow("3-772223-7325-9") << "3-77kk23-7325-9" << QStringList("EAN");
	QTest::newRow("3-7723-73-9") << "3-7723-73-9" << isbnEan;
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << QStringList("EAN");
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1" << QStringList("EAN");
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1" << QStringList("EAN");
	QTest::newRow("5 449000 k9624-1123") << "5 449000 k9624-1123" << QStringList();

	// ISSN
	QTest::newRow("0028 - 9604") << "0028 - 9604" << QStringList("ISSN");
	QTest::newRow("00287604") << "00287604" << QStringList("ISSN");
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << QStringList("ISSN");
	QTest::newRow("1432640X") << "1432640X" << QStringList("ISSN");
	QTest::newRow("1432640x") << "1432640x" << QStringList("ISSN");
	QTest::newRow("14326402") << "14326402" << QStringList("ISSN");
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << QStringList("ISSN");
	QTest::newRow("02894") << "02894" << QStringList("ISSN");
	QTest::newRow("0212312894") << "028912334" << QStringList();
}

void MINValidatorTests::possibleTypes()
{
	QFETCH(QString, min);
	QFETCH(QStringList, result);
	
	// if type is not supported, "" should be the result
	QStringList results = validator->possibleTypes(min);
	foreach(QString res, result){
		if(validator->isSupported(res)){
			QVERIFY(results.contains(res));
		}
		else{
			QVERIFY(!results.contains(res));
		}
	}
}

void MINValidatorTests::isValid_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("inputType");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("ölksAadasd-9") << "ölksAadasd-9" << "sdfsdf" << false;
	
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << "ISBN-10" << true;
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << "ISBN-10" << false;
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << "ISBN-10" << false;
	QTest::newRow("855937325X") << "855937325X" << "ISBN-10" << true;
	QTest::newRow("855937325x") << "855937325x" << "ISBN-10" << true;
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << "EAN" << true;
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1"<< "EAN" << false;
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1"<< "EAN" << false;
	QTest::newRow("9788559373257") << "9788559373257" << "EAN" << true;
	
	// instore EAN-13
	QTest::newRow("2404105 001722") << "2404105 001722" << "EAN" << true;
	// in store EAN-8 (Aldi Schovit ;-) )
	QTest::newRow("29033805") << "29033805" << "EAN" << true;
	QTest::newRow("29023805") << "29023805" << "EAN" << false;
	QTest::newRow("2?./0dÄa") << "2?./0dÄa" << "EAN" << false;
	
	QTest::newRow("5 449000 09624-123") << "5 449000 09624-123" << "EAN" << false;
	QTest::newRow("123456") << "123456" << "EAN" << false;

	// ISSN
	QTest::newRow("0028 - 9604") << "0028 - 9604" << "ISSN" << true;
	QTest::newRow("00287604") << "00287604" << "ISSN" << false;
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << "ISSN" << false;
	QTest::newRow("1432640X") << "1432640X" << "ISSN" << true;
	QTest::newRow("1432640x") << "1432640x" << "ISSN" << true;
	QTest::newRow("14326402") << "14326402" << "ISSN" << false;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << "ISSN" << false;
}

void MINValidatorTests::isValid()
{
	QFETCH(QString, min);
	QFETCH(QString, inputType);
	QFETCH(bool, result);
	
	// if type is not supported, false should be the result
	if((!inputType.isEmpty()) && (!validator->isSupported(inputType))){
		QCOMPARE(validator->isValid(min), false);
		QCOMPARE(validator->isValid(min, inputType), false);
	}
	else{
		QCOMPARE(validator->isValid(min), result);
		QCOMPARE(validator->isValid(min, inputType), result);
	}
}

void MINValidatorTests::validate_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("inputType");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("3-7723-7325-9 ISBN-10") << "3-7723-7325-9" << "ISBN-10" << "3772373259";
	QTest::newRow("3-7723-7325-9 EAN") << "3-7723-7325-9" << "EAN" << QString();
	QTest::newRow("3-7723-7325-9 na") << "3-7723-7325-9" << "" << QString();
	QTest::newRow("3-7723-7325-9 ISBN-10") << "3-7723-7325-9" << "ISBN-10" << "3772373259";
	QTest::newRow("3-7723-7325 ISBN-10") << "3-7723-7325" << "ISBN-10" << "3772373259";
	QTest::newRow("377237325 ISBN-10") << "377237325" << "ISBN-10" << "3772373259";
	QTest::newRow("3-7723-7325-8 ISBN-10") << "3-7723-7325-8" << "ISBN-10" << "3772373259";
	QTest::newRow("3-77k3-7325-9 ISBN-10") << "3-77k3-7325-9" << "ISBN-10" << "3773732597";
	QTest::newRow("3-77kk-7325-9 ISBN-10") << "3-77kk-7325-9" << "ISBN-10" << QString();
	// this test should better not give a good result (because a malformed ISBN-10 is changed into a correct EAN-8)
	QTest::newRow("3-77kk-7325-9 EAN") << "3-77kk-7325-9" << "EAN" << "";
	QTest::newRow("3-77kk-7325-9 na") << "3-77kk-7325-9" << "" << QString();
	QTest::newRow("3-77k3-7325 ISBN-10") << "3-77k3-7325" << "ISBN-10" << QString();
	QTest::newRow("3-5723-7325-9 ISBN-10") << "3-5773-7325-9" << "ISBN-10" << "3577373253";
	QTest::newRow("3-5723-adbc-9 ISBN-10") << "3-5773-adbc-9" << "ISBN-10" << QString();
	QTest::newRow("12313123123213123213 ISBN-10") << "12313123123213123213" << "ISBN-10" << QString();
	
	QTest::newRow("5 449000 09624 EAN") << "5 449000 09624" << "EAN" << "5449000096241";
	QTest::newRow("5 449000 09624 ISBN-10") << "5 449000 09624" << "ISBN-10" << QString();
	QTest::newRow("5 449000 09624 na") << "5 449000 09624" << "" << QString();
	QTest::newRow("5 449000 09624-1 EAN") << "5 449000 09624-1" << "EAN" << "5449000096241";
	QTest::newRow("5 449000 09624-1 ISBN-10") << "5 449000 09624-1" << "ISBN-10" << QString();
	QTest::newRow("5 449000 09624-1 na") << "5 449000 09624-1" << "" << "5449000096241";
	QTest::newRow("5449000096241 EAN") << "5449000096241" << "EAN" << "5449000096241";
	QTest::newRow("5 449000 09623-1 EAN") << "5 449000 09623-1" << "EAN" << "5449000096234";
	QTest::newRow("5 449000 k9624-1 EAN") << "5 449000 k9624-1" << "EAN" << "5449000962416";
	QTest::newRow("5 449000 kk624-1 EAN") << "5 449000 kk624-1" << "EAN" << QString();
	QTest::newRow("5 4900 k624 EAN") << "5 44900 k624" << "EAN" << QString();
	// this test should better not give a good result (because a malformed EAN-13 is changed into a correct ISBN-10) 
	QTest::newRow("5 4900 k624 ISBN-10") << "5 44900 k624" << "ISBN-10" << "";
	QTest::newRow("5 4900 k624 na") << "5 44900 k624" << "" << QString();
	QTest::newRow("5 449 EAN") << "5 449" << "EAN" << QString();
	QTest::newRow("5 449123123123123123 EAN") << "5 449123123123123123" << "EAN" << QString();
	
	QTest::newRow("29033805 EAN") << "29033805" << "EAN" << "29033805";
	QTest::newRow("29033805 ISBN-10") << "29033805" << "ISBN-10" << "";
	QTest::newRow("29033805 na") << "29033805" << "" << "";
	QTest::newRow("2903380 EAN") << "2903380" << "EAN" << "29033805";
	QTest::newRow("2903X805 EAN") << "2903X805" << "EAN" << "29038059";
	QTest::newRow("2903XX05 EAN") << "2903XX05" << "EAN" << QString();
	QTest::newRow("29033805 IBSN-10") << "2903XX05" << "ISBN-10" << QString();
	QTest::newRow("29033805 na") << "2903XX05" << "" << QString();
	
	QTest::newRow("0-12345-67890-5 UPC-A") << "0-12345-67890-5" << "UPC-A" << "012345678905";
	// since it can be an EAN two, the result is QString()
	QTest::newRow("0-12345-67890-7") << "0-12345-67890-7" << "" << QString();
	QTest::newRow("0-12345-67890-7 UPC-A") << "0-12345-67890-7" << "UPC-A" << "012345678905";
	QTest::newRow("0-12345-67890") << "0-12345-67890" << "" << QString();
	QTest::newRow("0-12345-67890 UPC-A") << "0-12345-67890" << "UPC-A" << "012345678905";
	QTest::newRow("72435420682") << "72435420682" << "" << QString();
	QTest::newRow("72435420682 UPC-A") << "72435420682" << "UPC-A" << "724354206825";
	QTest::newRow("72435k4206825") << "72435k4206825" << "" << QString();
	QTest::newRow("72435k4206825 UPC-A") << "72435k4206825" << "UPC-A" << "724354206825";
	QTest::newRow("7243kk206825") << "7243kk206825" << "" << QString();
	QTest::newRow("724354206") << "724354206" << "" << QString();
	QTest::newRow("12313123123213123213") << "12313123123213123213" << "" << QString();

	// ISSN
	QTest::newRow("0028 - 9604") << "0028 - 9604" << "ISSN" << "00289604";
	QTest::newRow("00287604") << "00287604" << "ISSN" << "00287601";
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << "ISSN" << "1432640x";
	QTest::newRow("14& 3- / 64kj0x") << "14& 3- / 64kj0x" << "ISSN" << QString();
	QTest::newRow("14320x") << "14340x" << "ISSN" << QString();
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << "ISSN" << "02896044";
}

void MINValidatorTests::validate()
{
	QFETCH(QString, min);
	QFETCH(QString, inputType);
	QFETCH(QString, result);
	
	// if output is not supported, "" should be the result
	if((!inputType.isEmpty()) && (!validator->isSupported(inputType))){
		QCOMPARE(validator->validate(min, inputType), QString());
	}
	else if((inputType.isEmpty()) && (validator->possibleTypes(min).size() == 0)){
		QCOMPARE(validator->validate(min, inputType), QString());
	}
	else{
		QEXPECT_FAIL("5 4900 k624 ISBN-10", "Convert malformed input to a complete other correct number", Continue);
		QEXPECT_FAIL("3-77kk-7325-9 EAN", "Convert malformed input to a complete other correct number", Continue);
		QCOMPARE(validator->validate(min, inputType), result);
	}
}

void MINValidatorTests::convert_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("inputType");
	QTest::addColumn<QString>("outputType");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("lkskjw12 na EAN") << "lkskjw12" << "" << "EAN" << QString();
	QTest::newRow("lkskjw12 ISBN-10 EAN") << "lkskjw12" << "ISBN-10" << "EAN" << QString();
	QTest::newRow("lkskjw12 EAN EAN") << "lkskjw12" << "EAN" << "EAN" << QString();
	QTest::newRow("lkskjw12 na ISBN-10") << "lkskjw12" << "" << "ISBN-10" << QString();
	QTest::newRow("lkskjw12 EAN ISBN-10") << "lkskjw12" << "EAN" << "ISBN-10" << QString();
	QTest::newRow("lkskjw12 ISBN-10 ISBN-10") << "lkskjw12" << "ISBN-10" << "ISBN-10" << QString();
	
	QTest::newRow("3-7723-7325-9 na na") << "3-7723-7325-9" << "" << "" << QString();
	QTest::newRow("3-7723-7325-9 ean ean") << "3-7723-7325-9" << "EAN" << "EAN" << QString();
	QTest::newRow("3-7723-7325-9 ISBN-10 na") << "3-7723-7325-9" << "ISBN-10" << "" << QString();
	QTest::newRow("3-7723-7325-9 na EAN") << "3-7723-7325-9" << "" << "EAN" << "9783772373251";
	QTest::newRow("5 449000 09624-1 ISBN-10 EAN") << "5 449000 09624-1" << "ISBN-10" << "EAN" << QString();
	QTest::newRow("5 449000 09624-1 na na") << "5 449000 09624-1" << "" << "" << QString();
	
	QTest::newRow("3-7723-7325-9 ISBN-10 EAN") << "3-7723-7325-9" << "ISBN-10" << "EAN" << "9783772373251";
	QTest::newRow("3-7723-73-9 ISBN-10 EAN") << "3-7723-73-9" << "ISBN-10" << "EAN" << QString();
	QTest::newRow("3-7723-73123-9 ISBN-10 EAN") << "3-7723-73123-9" << "ISBN-10" << "EAN" << QString();
	QTest::newRow("3-7723-7325-8 ISBN-10 EAN") << "3-7723-7325-8" << "ISBN-10" << "EAN" << QString();
	QTest::newRow("3548609821 ISBN-10 EAN") << "3548609821" << "ISBN-10" << "EAN" << "9783548609829";
	QTest::newRow("3442374928 ISBN-10 EAN") << "3442374928" << "ISBN-10" << "EAN" << "9783442374922";
	QTest::newRow("855937325x ISBN-10 EAN") << "855937325x" << "ISBN-10" << "EAN" << "9788559373257";
	QTest::newRow("855937325X ISBN-10 EAN") << "855937325X" << "ISBN-10" << "EAN" << "9788559373257";
	
	QTest::newRow("7 24354 20682 5 UPC-A EAN") << "724354206825" << "UPC-A" << "EAN" << "0724354206825";
	QTest::newRow("7 24354 20682 5 na EAN") << "724354206825" << "" << "EAN" << "0724354206825";
	QTest::newRow("7 24354 20682 5 UPC-A ISBN-10") << "724354206825" << "UPC-A" << "ISBN-10" << QString();
	QTest::newRow("7 24354 20682 5 UPC-A na") << "724354206825" << "UPC-A" << "" << QString();
	QTest::newRow("7 24354 20682 5 na EAN") << "724354206825" << "" << "EAN" << "0724354206825";
	QTest::newRow("0-12345-67890-5 na EAN") << "0-12345-67890-5" << "UPC-A" << "EAN" << "0012345678905";
	QTest::newRow("012345678905 UPC-A EAN") << "012345678905" << "UPC-A" << "EAN" << "0012345678905";
	QTest::newRow("012345678904 UPC-A EAN") << "012345678904" << "UPC-A" << "EAN" << QString();
	QTest::newRow("0123456k8905 UPC-A EAN") << "0123456k8905" << "UPC-A" << "EAN" << QString();
	QTest::newRow("012345605 UPC-A EAN") << "012345905" << "UPC-A" << "EAN" << QString();

	QTest::newRow("1432640X ISSN EAN") << "1432640X" << "ISSN" << "EAN" << "9771432640003";
	QTest::newRow("146402 ISSN EAN") << "146402" << "ISSN" << "EAN" << QString();
	QTest::newRow("00289601 ISSN EAN") << "00289601" << "ISSN" << "EAN" << QString();
	QTest::newRow("00289604 ISSN EAN") << "00289604" << "ISSN" << "EAN" << "9770028960006";
	
	QTest::newRow("9783548609829 ISBN-13 ISBN-10") << "9783548609829" << "EAN" << "ISBN-10" << "3548609821";
	QTest::newRow("9793548609828 ISBN-13 ISBN-10") << "9793548609828" << "EAN" << "ISBN-10" << "3548609821";
	QTest::newRow("9771432640003 EAN (ISSN) ISSN") << "9771432640003" << "EAN" << "ISSN" << "1432640X";
	QTest::newRow("9770028960006 EAN (ISSN) ISSN") << "9770028960006" << "EAN" << "ISSN" << "00289604";
	QTest::newRow("1230028960006 EAN ISSN") << "1230028960006" << "EAN" << "ISSN" << QString();
	QTest::newRow("1230028960006 EAN ISBN-10") << "1230028960006" << "EAN" << "ISBN-10" << QString();
}

void MINValidatorTests::convert()
{
	QFETCH(QString, min);
	QFETCH(QString, inputType);
	QFETCH(QString, outputType);
	QFETCH(QString, result);
	
	// if output is not supported, "" should be the result
	if(((!inputType.isEmpty()) && (!validator->isSupported(inputType))) || ((!outputType.isEmpty()) && (!validator->isSupported(outputType)))){
		QCOMPARE(validator->convert(min, outputType, inputType), QString());
	}
	else{
		QCOMPARE(validator->convert(min, outputType, inputType), result);
	}
}

void MINValidatorTests::strip_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("inputType");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("5 449000 09624-1 EAN") << "5 449000 09624-1" << "EAN" << "5449000096241";
	QTest::newRow("5 449000 09623-1 EAN") << "5449000096231" << "EAN" << "5449000096231";
	QTest::newRow("5 449000 k9624-1 EAN") << "544  900adb _?& 41" << "EAN" << "54490041";
	QTest::newRow("9788559373257 EAN") << "9788559373257" << "EAN" << "9788559373257";
	QTest::newRow("855937325X EAN") << "855937325X" << "EAN" << "855937325";
	QTest::newRow("855937325x EAN") << "855937325x" << "EAN" << "855937325";
	QTest::newRow("85  ak 593 7325x EAN") << "85  ak 593 7325x" << "EAN" << "855937325";
	
	QTest::newRow("5 449000 09624-1 ISBN-10") << "5 449000 09624-1" << "ISBN-10" << "5449000096241";
	QTest::newRow("5 449000 09623-1 ISBN-10") << "5449000096231" << "ISBN-10" << "5449000096231";
	QTest::newRow("5 449000 k9624-1 ISBN-10") << "544  900adb _?& 41" << "ISBN-10" << "54490041";
	QTest::newRow("5 449000 k9624-1 na") << "544  900adb _?& 41" << "" << "";
	QTest::newRow("9788559373257 ISBN-10") << "9788559373257" << "ISBN-10" << "9788559373257";
	QTest::newRow("855937325X ISBN-10") << "855937325X" << "ISBN-10" << "855937325X";
	QTest::newRow("855937325x ISBN-10") << "855937325x" << "ISBN-10" << "855937325x";
	QTest::newRow("85  ak 593 7325x ISBN-10") << "85  ak 593 7325x" << "ISBN-10" << "855937325x";

	// ISSN
	QTest::newRow("0028-9604") << "0028-9604" << "ISSN" << "00289604";
	QTest::newRow("00289604") << "00289604" << "ISSN" << "00289604";
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << "ISSN" << "1432640x";
	QTest::newRow("14& 32 - / 64kj0X") << "14& 32 - / 64kj0X" << "ISSN" << "1432640X";
	QTest::newRow("1432640X") << "1432640X" << "ISSN" << "1432640X";
	QTest::newRow("1432640x") << "1432640x" << "ISSN" << "1432640x";
}

void MINValidatorTests::strip()
{
	QFETCH(QString, min);
	QFETCH(QString, inputType);
	QFETCH(QString, result);
	
	// if output is not supported, "" should be the result
	if((!inputType.isEmpty()) && (!validator->isSupported(inputType))){
		QCOMPARE(validator->strip(min), QString());
	}
	else{
		QCOMPARE(validator->strip(min, inputType), result);
	}
}

void MINValidatorTests::commonPrefixes_data()
{
	QTest::addColumn<QStringList>("mins");
	QTest::addColumn<QStringList>("prefixes");
	
	QStringList mins, prefixes;
	// test if not correct EAN's are stripped out
	mins << "97831234" << "97831234567";
	QTest::newRow("Strip out not valid mins") << mins << QStringList();
	
	// remove duplicates and stripping mins
	mins.clear();
	mins << "9783 426775899" << "9783 426196816" << "97834261-96816";
	prefixes.clear();
	prefixes << "9783426";
	QTest::newRow("Duplicates and stripping") << mins << prefixes;
}

void MINValidatorTests::commonPrefixes()
{
	QFETCH(QStringList, mins);
	QFETCH(QStringList, prefixes);
	
	QCOMPARE(validator->commonPrefixes(mins), prefixes);
}

QTEST_MAIN(MINValidatorTests)
