/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DBTESTINIT_H
#define DBTESTINIT_H

#include "libmeganizertest.h"
#include <eobject.h>

class SearchPlugin;
class MediaPlugin;

struct MediumKeys {
	bool valid;
	unsigned int testTypeId;
	QString testType;
	unsigned int testManuId;
	QString testManu;
	unsigned int testCurId;
	QString testCur;
	QString testCurShort;
	unsigned int testGenreId;
	QString testGenre;
	unsigned int testUserId;
	QString testUser;
	unsigned int invalidId;
	QString invalid;
};

/**
	* @brief Initialize a test databasefor Meganizer and cleans it up.
	*
	* Use it like this:
	* @code
	* private slots:
	*   void YourTest::initTestCase()
	*   {
	*     testMedId = 0;
	*     DBTestInit *testInit = new DBTestInit(this);
	*     if(!testInit->initTestDB())
	*       QSKIP("Could not create test database", SkipAll);
	*     keys = testInit->prepareMediumKeys("Testtype");
	*     if(!keys.valid)
	*       QSKIP("Could not create test database", SkipAll);
	*   }
	* private:
	*   MediumKeys keys;
	* @endcode
	*/
class LIBMEGANIZER_TEST_EXPORT DBTestInit : public EObject
{
	Q_OBJECT
	public:
		/**
			* parent is a needed to make sure, that the destructor is called and it removed the
			* database again
			*/
		DBTestInit(QObject *parent);
    ~DBTestInit();
		/**
			* initialize the test database and returns the connection name
			* with the configured database connection. @return QString() if something went wrong
			*/
		bool initTestDB();
		bool cleanupTestDB();
		/**
			* use this function to prepare all needed foreignkeys to add a Medium (only real Medium are covered)
			* @return MediumKeys contains the id and the name of all needed foreignkeys
			* the prepared testType gets the name @p materialTypeName
			*/
		MediumKeys prepareMediumKeys(QString materialTypeName);
	
	private:
		QStringList sortViews(QStringList views);
		
		QString connName, sourceConnName;
};

#endif // DBTESTINIT_H
