/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediamodelbasetests.h"

#include <mediamodel.h>
#include "modeltest.h"

#include "mediamodelbasetests.h"
#include "mediamodelbasetests.moc"

#include <mediamodel.h>
#include "modeltest.h"
#include <medium.h>

#include <QtTest/QtTest>

void MediaModelBaseTests::baseTests(MediaModel *model)
{
	QVERIFY(model);
	new ModelTest(model, this);
	
	// test name() and group()
	QVERIFY(!model->name().isEmpty());
	QVERIFY(!model->group().isEmpty());
	// Because group is used as settings group, it shouldn't contain spaces
	QVERIFY(!model->group().contains(" "));
	
	QList<Medium*> items;
	items << model->newMedium();
	items << model->newMedium();
	items[items.size() - 1]->setTitle("test");
	items << model->newMedium();
	items[items.size() - 1]->setTitle("Super");
	items[items.size() - 1]->setId(0);
	items[items.size() - 1]->setDescription("ASdasd");
	items << model->newMedium();
	
	// insert mins
	foreach(Medium *med, items){
		model->addMedium(med);
	}
	
	QCOMPARE(model->rowCount(), items.size());
	
	// test item from index and index from item
	QModelIndex index = model->indexFromMedium(items[1]);
	QCOMPARE(index, model->index(1, 0, QModelIndex()));
	Medium *returnMed = model->mediumFromIndex ( index );
	QCOMPARE(returnMed, items[1]);
		
	// remove a min from the beginning, the middle and the end
	if(model->rowCount() > 0)
		model->removeRow(0);
	
	if(model->rowCount() > 0)
		model->removeRow(model->rowCount() - 1);
	
	if(model->rowCount() > 2){
		int half = model->rowCount() / 2;
		model->removeRow(half);
	}

	// check the createNewModel() function
	// (It must create a model of the same type, if
	// not, you forget the Q_INVOKEABLE makro for the
	// right constructor)
	MediaModel *newModel = model->createNewModel();
	QVERIFY(newModel);
	QCOMPARE(newModel->metaObject()->className(), model->metaObject()->className());
	delete newModel;
}
