/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "mediumeditwidgetbasetests.h"
#include "mediumeditwidgetbasetests.moc"

#include <medium.h>
#include <esqldatabase.h>
#include <mediumeditwidget.h>
#include "dbtestinit.h"

#include <QTest>
#include <QSqlQuery>
#include <QSignalSpy>

TestData::TestData()
{
	widget = NULL;
	testMedium = NULL;
}

TestData::TestData(MediumEditWidget* widget, 
									 ModifyFunctionPointer modifyCallback, CheckFunctionPointer checkCallback,
									 Medium *testMedium,
									 QString skipMsg)
{
	this->widget = widget;
	this->modifyCallback = modifyCallback;
	this->checkCallback = checkCallback;
	this->testMedium = testMedium;
	this->skipMsg = skipMsg;
}

MediumEditWidgetBaseTests::MediumEditWidgetBaseTests()
{
	medId = 0;
}

void MediumEditWidgetBaseTests::initTestCase()
{
	DBTestInit *testInit = new DBTestInit(this);
	if(!testInit->initTestDB())
		QSKIP("Could not create test database", SkipAll);

	keys = testInit->prepareMediumKeys("Testtype");
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);

	ESqlDatabase db = ESqlDatabase::database();
	// add new user
	db.open();
	QSqlQuery query;
	QString name = "User";
	newUserId = 0;
	if(!query.exec("INSERT INTO meganizer_users (name, givenName) VALUES('"+ name +"', 'New')")){
		db.close();
		QSKIP("Could not create new owner", SkipSingle);
	}
	query.exec("SELECT id FROM meganizer_users WHERE name='"+ name +"'");
	query.first();
	newUserId = query.record().value("id").toUInt();
	if(newUserId == 0){
		db.close();
		QSKIP("Could not create new owner", SkipSingle);
	}


	newCur = "USD";
	newCurId = 0;
	if(!query.exec("INSERT INTO meganizer_currencies (shortName, name) VALUES('"+ newCur +"', 'long name')")){
		db.close();
		QSKIP("Could not create new currency", SkipSingle);
	}
	query.exec("SELECT id FROM meganizer_currencies WHERE shortName='"+ newCur +"'");
	query.first();
	newCurId = query.record().value("id").toUInt();
	if(newCurId == 0){
		db.close();
		QSKIP("Cold not create new currency", SkipSingle);
	}

	db.close();
}

Medium* MediumEditWidgetBaseTests::prepareMedium()
{
	Medium *med = new Medium();
	if(medId == 0){
		med->setTitle("Testtitle");
		med->setType(keys.testTypeId);
		med->setGenre(keys.testGenreId);
		med->setManufacturer(keys.testManuId);
		med->setBoughtPriceCurrency(keys.testCurId);
		med->setListPriceCurrency(keys.testCurId);

		// we need a keyword to test the removal
		med->addKeyword("Test Keyword");
		
		QSignalSpy spy(med, SIGNAL(error(QString)));
		if(!med->store()){
			QString msg;
			if(spy.count() > 0)
				msg = spy.takeFirst().at(0).toString();
			QTest::qWarn("Could not prepare medium (Store failed: "+ msg.toLatin1() +")");
			return NULL;
		}
		medId = med->id();
	}
	else{
		QSignalSpy spy(med, SIGNAL(error(QString)));
		if(!med->load(medId)){
			QString msg;
			if(spy.count() > 0)
				msg = spy.takeFirst().at(0).toString();
			QTest::qWarn("Could not prepare medium (Load failed: "+ msg.toLatin1() +")");

			return NULL;
		}
	}
	return med;
}

void MediumEditWidgetBaseTests::registerWidget(MediumEditWidget* widget,
																							 ModifyFunctionPointer modifyCallback,
																							 CheckFunctionPointer checkCallback,
																							 Medium *testMedium,
																							 QString skipMsg)
{
	testData << TestData(widget, modifyCallback, checkCallback, testMedium, skipMsg);
}

void MediumEditWidgetBaseTests::testWidgets()
{
	foreach(TestData data, testData){
		QVERIFY(data.widget);
		QTest::qWarn(QString("Testing: %1").arg(data.widget->metaObject()->className()).toLatin1());
		if(!data.skipMsg.isEmpty()){
			QTest::qWarn(data.skipMsg.toLatin1());
		}
		else{
			if(!data.testMedium)
				data.testMedium = prepareMedium();
			QVERIFY(data.testMedium);
			QVERIFY(data.modifyCallback);
			QVERIFY(data.checkCallback);

			ESqlDatabase db = ESqlDatabase::database();
			db.close();

			// test NULL media
			QCOMPARE(db.isOpen(), false);
			data.widget->setMedium(NULL);
			QCOMPARE(data.widget->isEnabled(), false);
			QCOMPARE(db.isOpen(), false);
			QSignalSpy *spy = new QSignalSpy(data.widget, SIGNAL(error(QString)));
			QCOMPARE(data.widget->store(), false);
			QCOMPARE(spy->count(), 1);
			// setMedium(NULL) a second time
			QCOMPARE(db.isOpen(), false);
			data.widget->setMedium(NULL);
			QCOMPARE(data.widget->isEnabled(), false);
			QCOMPARE(db.isOpen(), false);
			delete spy;
			spy = new QSignalSpy(data.widget, SIGNAL(error(QString)));
			QCOMPARE(data.widget->store(), false);
			QCOMPARE(spy->count(), 1);

			// test store with modified() == false
			QCOMPARE(db.isOpen(), false);
			QCOMPARE(data.widget->isEnabled(), false);
			data.widget->setMedium(data.testMedium);
			QCOMPARE(data.widget->isEnabled(), true);
			QCOMPARE(db.isOpen(), false);
			delete spy;
			spy = new QSignalSpy(data.widget, SIGNAL(error(QString)));
			QCOMPARE(data.widget->store(), true);
			QCOMPARE(db.isOpen(), false);
			QCOMPARE(spy->count(), 0);

			QVERIFY(data.modifyCallback(this, data.widget));
			QCOMPARE(data.widget->modified(), true);
			// store changes
			spy = new QSignalSpy(data.widget, SIGNAL(error(QString)));
			// check wether all work with closed db connection
			QCOMPARE(db.isOpen(), false);
			bool result = data.widget->store();
			if(spy->count() > 0)
				QTest::qWarn(spy->takeFirst().at(0).toString().toLatin1());
			// if storign has failed, an error should be emitted
			if(!result)
				QCOMPARE(spy->count(), 1);
			QVERIFY(data.widget);
			QCOMPARE(db.isOpen(), false);

			QCOMPARE(data.checkCallback(this, data.widget, data.testMedium), true);

			// set the medium to NULL again, to test disabling
			// the widget after a valid medium was set
			QCOMPARE(db.isOpen(), false);
			data.widget->setMedium(NULL);
			QCOMPARE(data.widget->isEnabled(), false);
			QCOMPARE(db.isOpen(), false);
			delete spy;
			spy = new QSignalSpy(data.widget, SIGNAL(error(QString)));
			QCOMPARE(data.widget->store(), false);
			QCOMPARE(spy->count(), 1);
			
			delete data.widget;
			delete data.testMedium;
			delete spy;
		}
	}
}
