/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef MEDIUMEDITWIDGETBASETESTS_H
#define MEDIUMEDITWIDGETBASETESTS_H

#include "libmeganizertest.h"
#include <QObject>

#include "dbtestinit.h"

class MediumEditWidgetBaseTests;
class MediumEditWidget;
class Medium;

typedef bool(*ModifyFunctionPointer)(MediumEditWidgetBaseTests*, MediumEditWidget*);
typedef bool(*CheckFunctionPointer)(MediumEditWidgetBaseTests*, MediumEditWidget*, Medium*);

/**
	* @brief Structure to store all data related to test one MediumEditWidget.
	* 
	* If TestData::testMedium is NULL (default), the test is done with a plain
	* Medium. To skip the test, set the TestData::skipMsg to something useful.
	*/
class TestData {
	public:
		TestData();
		TestData(MediumEditWidget *widget, 
						 ModifyFunctionPointer modifyCallback, CheckFunctionPointer checkCallback,
						 Medium *testMedium = NULL, QString skipMsg = QString());
		MediumEditWidget *widget;
		Medium *testMedium;
		ModifyFunctionPointer modifyCallback;
		CheckFunctionPointer checkCallback;
		QString skipMsg;
};

/**
	* @brief Does standard tests on all registered MediumEditWidgets.
	* 
	* To initialize the test case, implement your subclass as follows:
	* @code
	* class YourTestClass : public MediumEditWidgetBaseTests
	* 	private slots:
	* 		void initTestCase(){
	*				// create and setup the database (connection)
	* 			MediumEditWidgetBaseTests::initTestCase();
	* 		}
	* @endcode
	* 
	* Call registerWidget() to add a widget to the test list. After
	* registering all widgets, call testWidgets() to run the tests.
	*/
class LIBMEGANIZER_TEST_EXPORT MediumEditWidgetBaseTests : public QObject
{
	Q_OBJECT
	public:
		MediumEditWidgetBaseTests();
		virtual inline ~MediumEditWidgetBaseTests(){ };

		void initTestCase();
		Medium *prepareMedium();
		void registerWidget(MediumEditWidget *widget,
												ModifyFunctionPointer modifyCallback,
												CheckFunctionPointer checkCallback,
												Medium *testMedium = NULL,
												QString skipMsg = QString());
		void testWidgets();
		
		MediumKeys keys;
		uint medId, newUserId, newCurId;
		QString newCur;
		
	private:
		QList<TestData> testData;
};

#endif // MEDIUMEDITWIDGETBASETESTS_H
