/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIUMSIMPLEEDITWIDGETBASETEST_H
#define MEDIUMSIMPLEEDITWIDGETBASETEST_H

#include "libmeganizertest.h"
#include <QObject>

class MediumSimpleEditWidget;
class Medium;

#include <QMap>

/**
	* @brief Subclass this class to test your MediumSimpleEditWidgets.
	*
	* To do so, implement it as follows:
	* @code
	*  private slots:
	*    void doTests(){
	*      addTestObject(new YourSimpleEditWidgetClass(), "usedproperty");
	*      baseTest("yourmaterialtype");
	*    };
	* @endcode
	*/
class LIBMEGANIZER_TEST_EXPORT MediumSimpleEditWidgetBaseTest : public QObject
{
	Q_OBJECT
	public:
		MediumSimpleEditWidgetBaseTest();
		~MediumSimpleEditWidgetBaseTest();
		
	protected:
		/**
			* @brief Adds a widget and property combination to the test list.
			*
			* Call baseTest() to test all registered objects.
			*/
		void addTestObject(MediumSimpleEditWidget* widget, const char* property);
		/**
			* @brief Call this function to test all objects add with addTestObject().
			*
			* @param materialType specifies the type of the material, which should be used
			*    for testing. If @p materialType is empty, you must call setTestMedium() your self
			*    before.
			*/
		void baseTest(QString materialType = QString());
		void baseTest(MediumSimpleEditWidget* widget, const char* property);
		/**
			* @brief Sets the value of editWidget() of @p editWidget to the returned value.
			*/
		QVariant setValue(MediumSimpleEditWidget* editWidget, const char* property, Medium *testedMed);
		void setTestMedium(QString materialType);
		void setTestMedium(Medium *med);
		Medium *testMedium();

	private:
		Medium *testMed;
		QMap<const char*, MediumSimpleEditWidget*> map;
};

#endif // MEDIUMSIMPLEEDITWIDGETBASETEST_H
