<?php

$usersFile = "users.inc.php";
$lang = "de";

session_start();
header( 'Content-Type: text/html; charset=UTF-8' );
mb_internal_encoding( 'UTF-8' );

/**
	* tries include the class defintion automatically, if the class is used
	*/
function __autoload($className) {
    require_once $className . '.php';
}

/**
	* @brief Stores the given @p user with @p password in the usersFile.
	*
	* If @p encrypt is true (default), the password is encrypted with sha512.
	* Otherwise it is stored as is (Used to restore already stored and therefore
	* already encrypted passwords.
	*/
function addUser($user, $password, $encrypt = true){
	global $usersFile;

	// strip out linebreaks and unneded spaces
	$user = preg_replace("/\r|\n|\\\\|\|/s", "", $user);
	$user = trim($user);
	$password = preg_replace("/\r|\n|\\\\|\|/s", "", $password);
	$password = trim($password);

	echo "	<div class=\"logLine\">Checking user string";
	flush();
	if(empty($user) || empty($password)){
		echo "<span class=\"failed\">FAILED</span></div>\n";
		return false;
	}
	echo "<span class=\"successfull\">OK</span></div>\n";

	echo "	<div class=\"logLine\">Adding ". $user;
	flush();
	$creation = !is_file($usersFile);
	$fd = @ fopen($usersFile, "a+");
	if($fd == false){
		echo "<span class=\"failed\">FAILED</span></div>\n";
		return false;
	}
	if($creation)
		fwrite($fd, "<?php die(); ?>\n");
	if($encrypt)
		$password = hash("sha512", $password);
	fwrite($fd, $user ."|". $password ."\n");
	fclose($fd);
	echo "<span class=\"successfull\">OK</span></div>\n";
	return true;
}

// show login screen, if needed
if((!empty($_POST['username'])) && (!empty($_POST['password']))){
	// load user
	if(!file_exists($usersFile)){
		// create the first new user
		addUser($_POST['username'], $_POST['password']);
	}
	$users = @ file($usersFile, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
	if(is_array($users)){
		array_shift($users);
		foreach($users as $user){
			$tmp = explode("|", $user);
			if(count($tmp) == 2){
				if(($_POST['username'] == $tmp[0]) && (hash("sha512", $_POST['password']) == $tmp[1])){
					$_SESSION['Username'] = $_POST['username'];
					$_SESSION['loggedIn'] = 1;
				}
			}
		}
	}
}

$loggedIn = 0;
$user = "";
if(array_key_exists("loggedIn", $_SESSION) && array_key_exists("Username", $_SESSION)){
	$loggedIn = $_SESSION['loggedIn'];
	$user = $_SESSION['Username'];
}

$title = "ManuServer - Administration";
include("header.php");
echo "<div class=\"content\">\n";

if(($loggedIn != 1) || (empty($user))){
	# if $usersFile contains no users, let the user create one
	if(!file_exists($usersFile))
		echo "<h3>Enter a username and a password to create the first user!</h3>";
	?>
		<form action="<?php echo $_SERVER['PHP_SELF']; ?>" method=post>
			<table>
				<tr>
					<td>Username:</td>
					<td><input type="text" name="username" autofocus></td>
				</tr>
				<tr>
					<td>Password:</td>
					<td><input type="password" name="password"></td>
				</tr>
				<tr>
					<td><input type="submit" value="Login"></td>
					<td></td>
				</tr>
			</table>
		</form>
	<?php
	include("footer.php");
	die();
}
else{
	if(array_key_exists("action", $_POST)){
		if($_POST['action'] == "addUser"){
			echo "<div class=\"log\">\n";
			addUser($_POST['newUsername'], $_POST['newPassword']);
			echo "</div>\n";
		}
		elseif($_POST['action'] == "storeGeneral"){
			Settings::setValue("SUBMITUSER", $_POST["submitUser"]);
			Settings::setValue("SUBMITPASSWORD", $_POST["submitPassword"]);
			Settings::setValue("ADMINUSER", $_POST["adminUser"]);
			Settings::setValue("ADMINPASSWORD", $_POST["adminPassword"]);
			Settings::setValue("DBNAME", $_POST["dbName"]);

			$piwikId = intval($_POST['piwikId']);
			Settings::setValue("PIWIKID", $piwikId);
			$piwikURL = $_POST['piwikURL'];
			$piwikURL = preg_replace("/^(https?:\/\/)/", "", $piwikURL);
			$piwikURL = preg_replace("/(\/$)/", "", $piwikURL);
			Settings::setValue("PIWIKURL", $piwikURL);
		}
		elseif($_POST['action'] == "deleteUser"){
			$delUsers = $_POST["delUsers"];
			
			echo "<div class=\"log\">\n";
			echo "	<div class=\"logLine\">Checking selected users";
			flush();
			if(count($delUsers) == 0 || !is_array($delUsers)){
				echo "<span class=\"failed\">FAILED</span></div>\n";
			}
			else{
				echo "<span class=\"successfull\">OK</span></div>\n";
				// first cleanup all
				echo "	<div class=\"logLine\">Reading users list";
				flush();
				$users = @ file($usersFile, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
				array_shift($users);
				if(count($users) == 0){
					echo "<span class=\"failed\">FAILED</span></div>\n";
				}
				else{
					// clean up old user list file
					echo "<span class=\"successfull\">OK</span></div>\n";
					echo "	<div class=\"logLine\">Cleanup old user list";
					flush();
					$fd = @ fopen($usersFile, "w+");
					if($fd == false){
						echo "<span class=\"failed\">FAILED</span></div>\n";
					}
					else{
						fwrite($fd, "<?php die(); ?>\n");
						echo "<span class=\"successfull\">OK</span></div>\n";
						// add not deleted users to the emptied user list
						foreach($users as $user){
							$tmp = explode("|", $user);
							if(count($tmp) != 2){
								echo "<span class=\"failed\">FAILED</span></div>\n";
							}
							else{
								$name = trim($tmp[0]);
								$password = trim($tmp[1]);
								if(empty($name) || empty($password)){
									echo "<span class=\"failed\">FAILED</span></div>\n";
								}
								else{
									if(!in_array($name, $delUsers)){
										addUser($name, $password, false);
									}
								}
							}
						}
					}
				}
			}
			$userList = @ file($usersFile, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
			array_shift($userList);
			echo "</div>\n";
		}
	}

	# if any queued manufacturer are present, display the link
	$count = 0;
	$connId = @ mysql_connect("localhost", Settings::value("SUBMITUSER"), Settings::value("SUBMITPASSWORD"));
	if(!$connId){
		echo "<div class=\"log\">\n";
		echo "<div class=\"logLine\">Please specify the database submit user</div>\n";
		echo "</div>\n";
	}
	else{
		if(!mysql_select_db(Settings::value("DBNAME"))){
			echo "<div class=\"log\">\n";
			echo "<div class=\"logLine\">Please specify the database</div>\n";
			echo "</div>\n";
			mysql_close($connId);
		}
		else{
			$res = mysql_query("SELECT id FROM manufacturerhasmins_queue");
			$count = mysql_num_rows($res);
			if($count < 1){
				$res = mysql_query("SELECT id FROM mintypes_queue");
				$count = mysql_num_rows($res);
				if($count < 1){
					$res = mysql_query("SELECT id FROM manufacturers_queue");
					$count = mysql_num_rows($res);
				}
			}
			mysql_close($connId);
		}
	}
	if($count > 0){
		echo "<div class=\"queueLinks\">\n";
		echo "  <a href=\"processqueue.php\">Process ". $count ." queued items</a>\n";
		echo "</div>\n";
	}
?>
	<div class="userAdministration">
		<h3>Users</h3>
		<div class="userAdmin">
			<form action="<?php echo $_SERVER['PHP_SELF']; ?>" method="post">
				<input type="hidden" name="action" value="addUser">
				<table class="General">
					<tr>
						<td>Username:</td>
						<td><input class="input" type="text" name="newUsername"></td>
					</tr>
					<tr>
						<td>Password:</td>
						<td><input class="input" type="password" name="newPassword"></td>
					</tr>
					<tr>
						<td><input type="submit" name="addUser" value="Add"></td>
						<td></td>
					</tr>
				</table>
			</form>
		</div>
		<form action="<?php echo $_SERVER['PHP_SELF']; ?>" method="post">
			<table class="userList">
				<?php
					if(is_file($usersFile)){
						$userList = @ file($usersFile, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
						array_shift($userList);
						$i = 1;
						foreach($userList as $user){
							$tmp = explode("|", $user);
							$user = $tmp[0];
							echo "<tr><td><input type=\"checkbox\" name=\"delUsers[". $i ."]\" value=\"". $user ."\"></td><td>". $user ."</td></tr>\n";
							$i++;
						}
					}
				?>
			</table>
			<input type="hidden" name="action" value="deleteUser">
			<input type="submit" name="deleteUsers" value="Delete Selected">
		</form>
	</div>
	<form class="generalSettings" action="<?php echo $_SERVER['PHP_SELF']; ?>" method="post">
		<h3>General Settings</h3>
		<input type="hidden" name="action" value="storeGeneral">
		<table class="General">
			<tr>
				<th colspan="2">Database</th>
			</tr>
			<tr>
				<td colspan="2">General</td>
			</tr>
			<tr>
				<td>Database name:</td>
				<td><input class="input" type="dbname" name="dbName" value="<?php echo Settings::value("DBNAME"); ?>"></td>
			</tr>
			<tr>
				<td colspan="2">Submit Database User:
					<br>(Should be able to SELECT all tables and to write only on _queue tables)</td>
			</tr>
			<tr>
				<td>Username:</td>
				<td><input class="input" type="text" name="submitUser" value="<?php echo Settings::value("SUBMITUSER"); ?>"></td>
			</tr>
			<tr>
				<td>Password:</td>
				<td><input class="input" type="password" name="submitPassword" value="<?php echo Settings::value("SUBMITPASSWORD"); ?>"></td>
			</tr>
			<tr>
				<td colspan="2">Admin Database User:</td>
			</tr>
			<tr>
				<td>Username:</td>
				<td><input class="input" type="text" name="adminUser" value="<?php echo Settings::value("ADMINUSER"); ?>"></td>
			</tr>
			<tr>
				<td>Password:</td>
				<td><input class="input" type="password" name="adminPassword" value="<?php echo Settings::value("ADMINPASSWORD"); ?>"></td>
			</tr>
			<tr>
				<th colspan="2">Piwik</th>
			</tr>
			<tr>
				<td>Piwik URL:</td>
				<td><input class="input" type="text" name="piwikURL" value="<?php echo Settings::value("PIWIKURL"); ?>"></td>
			</tr>
			<tr>
				<td>Piwik Id:</td>
				<td><input class="input" type="text" name="piwikId" value="<?php echo Settings::value("PIWIKID"); ?>"></td>
			</tr>
			<tr>
				<td><input type="submit" name="storeGeneral" value="Store"></td>
				<td></td>
			</tr>
		</table>
	</form>
	<?php
}

echo "</div>\n";
include("footer.php");
?>