/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "booksdb.h"
#include "booksdb.moc"

#include "books.h"
#include "book.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>

BooksDB::BooksDB ( QObject *parent ) : ESqlInit ( parent )
{
	setObjectName("BooksDB");
}

void BooksDB::scheme()
{
	setup(Books::group(), "Books", 9);

  ESqlTable *books = addTable ( "books" );

  ESqlField *field = books->appendField ( "materialsId", ESqlField::Integer, true );
  field->setIsUnsigned ( true );
  books->addForeignKey ( field->name(), "meganizer_materials", "id" );

  field = books->appendField ( "pages", ESqlField::Integer );
  field->setIsUnsigned ( true );

  field = books->appendField ( "edition", ESqlField::Integer );
  field->setIsUnsigned ( true );

  field = books->appendField ( "publicationPlace", ESqlField::Varchar );
  field->setLength ( 120 );

  field = books->appendField ( "publicationDate", ESqlField::Date );

  ESqlTable *table = addTable ( "meganizer_materialtypes", true, false );
  table->appendField ( "name", ESqlField::Varchar );

	Book book;
  QStringList insert;
  insert << "" << book.group();
  table->appendValues ( insert );
  
  table = addTable("meganizer_materialpersonsroles", true, false);
  table->appendField("name", ESqlField::Varchar);
  table->appendField("description", ESqlField::Varchar);
  
  insert.clear();
  insert << "" << tr("Author") << tr("The author of the material");
  table->appendValues(insert);
  insert.clear();
  insert << "" << tr("Editor") << tr("The editor of a book (abbr.: Ed., ed.)");
  table->appendValues(insert);
	
	addView ( "viewbooks",
						"SELECT  mats.*, books.pages, books.edition, books.publicationPlace, books.publicationDate"
						"  FROM  "+ books->name() +" books"
						"  LEFT JOIN viewmaterials mats ON mats.id=books.materialsId");

	addUpdateRoutine(6, &BooksDB::updateFrom5To6);
}

bool BooksDB::updateFrom5To6(ESqlInit* caller, QList< ESqlTable* > tables, QString& errorString)
{
	Q_UNUSED(caller);
	Q_UNUSED(tables);

	Book book;
	QSqlQuery query;
	// update old entry
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Book" ));
	query.exec();
	if(!query.first())
		return false;
	unsigned int id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find old book entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("UPDATE meganizer_materialtypes SET name=:name WHERE id=:id");
	query.bindValue(":name", book.group());
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not update old book entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}

	// remove newly added double entry (we must leave the old one,
	// because there may be materials with this type)
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name AND id<>:id");
	query.bindValue(":name", book.group());
	query.bindValue(":id", id);
	query.exec();
	if(!query.first())
		return false;
	id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find new book entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("DELETE FROM meganizer_materialtypes WHERE id=:id");
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not delete new book entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	return true;
}
