/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "booksearchwidget.h"

#include "books.h"

BookSearchWidget::BookSearchWidget ( QWidget* parent, Qt::WindowFlags flags )
    : SearchDetailsWidget ( parent, flags )
{
  setupUi ( this );
	connect(pubPlaceEdit, SIGNAL(returnPressed()), this, SLOT(startSearchByPass()));
	connect(pubDateEdit, SIGNAL(dateChanged(QDate)), this, SLOT(dateChanged(QDate)));
}

QString BookSearchWidget::searchTable()
{
	return Books::group() +"_viewbooks";
}

QList<SqlConstraint> BookSearchWidget::constraints()
{
	QList<SqlConstraint> results;
	SqlConstraint constraint;
	if(editionEdit->value() > 0){
		constraint.setConstraint(QString("edition%1?").arg(condition(editionModeEdit->currentIndex())));
		constraint.addBindValue(editionEdit->value());
		results << constraint;
	}
	if(pagesEdit->value() > 0){
		constraint = SqlConstraint();
		constraint.setConstraint(QString("pages%1?").arg(condition(pageModeEdit->currentIndex())));
		constraint.addBindValue(pagesEdit->value());
		results << constraint;
	}
	if(!pubPlaceEdit->text().isEmpty()){
		constraint = SqlConstraint();
		constraint.setConstraint("publicationPlace=?");
		constraint.addBindValue(pubPlaceEdit->text());
		results << constraint;
	}
	if(pubDate.isValid()){
		constraint = SqlConstraint();
		constraint.setConstraint(QString("publicationDate%1?").arg(condition(pubDateModeEdit->currentIndex())));
		constraint.addBindValue(pubDate);
		results << constraint;
	}
	return results;
}

void BookSearchWidget::dateChanged ( QDate date )
{
	pubDate = date;
}

QString BookSearchWidget::condition ( int index )
{
	switch(index){
		case 0:
			return "<=";
		case 1:
			return ">=";
		default:
			return "=";
	}
}

#include "booksearchwidget.moc"
