/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "booktest.h"
#include "booktest.moc"

#include <dbtestinit.h>
#include "book.h"
#include "books.h"
#include <esqldatabase.h>

#include <QtTest/QtTest>
#include <QSqlQuery>
#include <QSqlError>

void BookTest::initTestCase()
{
	testMedId = 0;
	
	testInit = new DBTestInit(this);
	if(!testInit->initTestDB()){
		QSKIP("Could not create test database", SkipAll);
	}
	
	Book *book = new Book();
	keys = testInit->prepareMediumKeys(book->group());
	delete book;
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
}

void BookTest::cleanupTestCase()
{
	delete testInit;
}

void BookTest::fetchMessages(QString msg)
{
	qDebug() << msg;
}

void BookTest::testExtendedProperties()
{
	Book *med = new Book();
	connectMedium(med);
	
	baseTests(med);
	
	QDate date = QDate::currentDate();
	med->setPublicationDate(date);
	QCOMPARE(med->publicationDate(), date);

	QString place = "Testplace";
	med->setPublicationPlace(place);
	QCOMPARE(med->publicationPlace(), place);

	unsigned int number = 10;
	med->setEdition(10);
	QVERIFY(med->edition() == number);

	number = 11;
	med->setPages(number);
	QVERIFY(med->pages() == number);

	delete med;
}

void BookTest::copyBook()
{
	Book *med = new Book();
	med->setEdition(10);
	med->setPages(11);
	med->setPublicationDate(QDate::currentDate());
	
	Book *copy = qobject_cast<Book*>(med->copyMedium());
	QVERIFY(copy);
	QCOMPARE(copy->edition(), med->edition());
	QCOMPARE(copy->publicationPlace(), med->publicationPlace());
	QCOMPARE(copy->publicationDate(), med->publicationDate());
	
	delete copy;
	delete med;
}

void BookTest::insertAndLoadExtendedProperties()
{
	Book *med = new Book();
	connectMedium(med);
	
	med->setTitle("'SELECT * FROM meganizer_user;");
	med->setDescription("This is greate show; INSERT INTO test test = 10;");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	
	med->setEdition(10);
	med->setPages(11);
	med->setPublicationDate(QDate::currentDate());
	
	Book *checkMed = qobject_cast<Book*>(med->copyMedium());
	QVERIFY(checkMed);
	QCOMPARE(med->store(), true);
	testMedId = med->id();
	delete med;
	
	med = new Book();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->id(), testMedId);
	QCOMPARE(med->edition(), checkMed->edition());
	QCOMPARE(med->publicationPlace(), checkMed->publicationPlace());
	QCOMPARE(med->publicationDate(), checkMed->publicationDate());
	
	delete med;
	delete checkMed;
}

void BookTest::extentedSetValueAndValue()
{
	QSKIP("Not implemented yet", SkipSingle);
}

void BookTest::connectMedium(Medium* med)
{
	connect(med, SIGNAL(error(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(warning(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(information(QString)), this, SLOT(fetchMessages(QString)));
}

QTEST_MAIN(BookTest)
