/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef ARTICLE_H
#define ARTICLE_H

#include <medium.h>

#include <QDate>

class Article : public Medium
{
	Q_OBJECT
	Q_PROPERTY(uint startPage READ startPage WRITE setStartPage)
	Q_PROPERTY(uint endPage READ endPage WRITE setEndPage)
	Q_PROPERTY(QDate publicationDate READ publicationDate WRITE setPublicationDate)
	public:
		Q_INVOKABLE Article(EQueryThread* thread = 0, Medium* newParent = 0);

#if QT_VERSION < 0x040500
		Medium* newMedium();
#endif
		bool select(QSqlRecord record);
		bool store(bool addOwner = false, bool startTransaction = true);
		bool setValue(QString attribute, QVariant value);
		QVariant value(QString attribute, int role);
		QList< ColKeyTrans > supportedAttrs();
		EditMedium* editWidget(EWidget* parent = 0);
		Medium* copyMedium();
		bool exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly = false);
		QString exportPlainText(Medium::ExportFormat format);

		/**
			* @brief The first page of the article.
			*
			* Normally it is the number of the first page of the article
			* at it's magazine.
			*/
		void setStartPage(uint page);
		uint startPage() const;
		/**
			* @brief The last page of the article.
			*
			* Normally it is the number of the last page of the article
			* at it's magazine.
			*/
		void setEndPage(uint page);
		uint endPage() const;
		/**
			* @brief The difference between startPage() and endPage().
			*/
		uint pages() const;
		/**
			* @brief Is provided to manage articles without a magazine.
			*/
		void setPublicationDate ( QDate newDate );
    QDate publicationDate() const;

	private:
		QString dbTable;
		uint myStartPage, myEndPage;
		QDate myPubDate;
};

#endif // ARTICLE_H
