/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "magazines.h"
#include "magazines.moc"

#include "magazine.h"
#include "magazinesmodel.h"
#include "magazinesearchwidget.h"
#include "article.h"
#include "articlesmodel.h"
#include "magazinesdb.h"

#include <esetupwizarddb.h>

#include <QtPlugin>

Magazines::Magazines(): MediaPlugin()
{

}

QString Magazines::group()
{
	return "magazines";
}

QString Magazines::untranslateMaterialType(QString materialType)
{
	Magazine mag;
	if(materialType == mag.name())
		return mag.group();
	Article art;
	if(materialType == art.name())
		return art.group();
	return QString();
}

QStringList Magazines::translateMaterialType(QString materialType)
{
	QStringList result;
	Magazine mag;
	if(materialType == mag.group()){
		result << mag.name() << tr ( "All periodically published media (Like Magazines)" );
		return result;
	}
	Article art;
	if(materialType == art.group()){
		result << art.name() << tr ( "Articles are small pieces of content,"
																	" which are normally collected in another medium like a Magazine" );
		return result;
	}
	return result;
}

QStringList Magazines::supportedMaterialTypes()
{
	QStringList result;
	Magazine mag;
	result << mag.group();
	Article art;
	result << art.group();
	return result;
}

bool Magazines::isSupportedMaterialType(QString materialType)
{
	if(supportedMaterialTypes().contains(materialType))
		return true;
	return false;
}

ESetupWizardObject* Magazines::wizardObject()
{
	return new ESetupWizardDB ( tr ( "Configure Database of %1" ).arg(name()), new MagazinesDB ( ), QStringList(), this );
}

ESettingsModelItem* Magazines::settingsModelItem()
{
	return NULL;
}

QStringList Magazines::supportedAttributes(QString materialType)
{
	Magazine mag;
	if(materialType == mag.group())
		return mag.supportedAttributes();
	Article art;
	if(materialType == art.group())
		return art.supportedAttributes();
	return QStringList();
}

SearchDetailsWidget* Magazines::searchWidget(QString materialType)
{
	Magazine mag;
	if(materialType == mag.group())
		return new MagazineSearchWidget();
// 	Article art;
// 	if(materialType == art.group())
// 		return ArticleSearchWidget();
	return NULL;
}

MediaModel* Magazines::newModel(QString materialType, EQueryThread* thread)
{
	Magazine mag;
	if(materialType == mag.group())
		return new MagazinesModel(thread);
	Article art;
	if(materialType == art.group())
		return new ArticlesModel(thread);
	return NULL;
}

Medium* Magazines::newMedium(QString materialType, EQueryThread* thread)
{
	Magazine *mag = new Magazine(thread);
	if(materialType == mag->group())
		return mag;
	delete mag;
	Article *art = new Article(thread);
	if(materialType == art->group())
		return art;
	delete art;
	return NULL;
}

QString Magazines::description() const
{
	return tr("Handles Magazines and Articles and their relations");
}

QIcon Magazines::icon() const
{
	return QIcon();
}

QString Magazines::name() const
{
	return tr("Magazines");
}

bool Magazines::unload()
{
	return true;
}

bool Magazines::initiate()
{
	return true;
}

Q_EXPORT_PLUGIN2 ( Meganizer_Magazines, Magazines )
