/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "magazinesmodel.h"
#include "magazinesmodel.moc"

#include "magazine.h"
#include <delegateuser.h>
#include <eaction.h>
#include <editmedium.h>

#include <QAbstractItemView>
#include <QMenu>
#include <QItemSelectionModel>

MagazinesModel::MagazinesModel(EQueryThread* queryThread, QObject* parent)
	: MediaModel(parent)
{
	setupModel(queryThread);
}

#if QT_VERSION < 0x040500
MediaModel* MagazinesModel::createNewModel()
{
	return new MagazinesModel();
}
#endif

Medium* MagazinesModel::newMedium() const
{
	return new Magazine();
}

void MagazinesModel::addItemDelegates(QAbstractItemView* view)
{
	if(!view)
    return;

	Medium *medium = newMedium();
	if(!medium)
		return;

	QStringList attrs = medium->supportedAttributes();
  view->setItemDelegateForColumn(medium->attrs("owner").column, new DelegateUser(view));
	delete medium;
}

void MagazinesModel::addCustomContextItems(QMenu* contextMenu, QModelIndexList selection)
{
	if(!contextMenu)
		return;
	
	if(selection.size() < 1)
		return;
	
	this->selection = selection;
	contextMenu->addAction(tr("New Issue"), this, SLOT(newIssue()));
}

void MagazinesModel::newIssue()
{
	foreach(QModelIndex index, selection){
		Magazine *mag = qobject_cast<Magazine*>(mediumFromIndex(index));
		if(mag){
			Magazine *newMag = qobject_cast<Magazine*>(mag->copyMedium());
			if(newMag){
				newMag->setId(0);
				newMag->setIssue("");
				newMag->setPublicationDate(QDate());
				EditMedium *widget = newMag->editWidget();
				if ( !widget ){
					emit error ( tr ( "Could no get edit wigdet" ) );
				}
				else {
					connectByPass ( this, widget );
					widget->setNewMode(this, newMag);
					if ( widget->exec() != QDialog::Accepted )
						emit error ( tr ( "Could not add medium!" ) );
					delete widget;
				}
			}
		}
	}
}
