/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "articletest.h"
#include "articletest.moc"

#include "article.h"

#include <QTest>

void ArticleTest::initTestCase()
{
// 	testMedId = 0;

	DBTestInit *testInit = new DBTestInit(this);
	if(!testInit->initTestDB())
		QSKIP("Could not create test database", SkipAll);

	Article *art = new Article();
	keys = testInit->prepareMediumKeys(art->group());
	delete art;
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
}

void ArticleTest::fetchMessages(QString msg)
{
	QTest::qWarn(msg.toLatin1());
}

void ArticleTest::testExtendedProperties()
{
	Article *med = new Article();
	connectMedium(med);
	
	baseTests(med);

	// testing init status
	QCOMPARE(med->startPage(), 0u);
	QCOMPARE(med->endPage(), 0u);
	QCOMPARE(med->pages(), 0u);
	QCOMPARE(med->publicationDate(), QDate());

	uint startPage = 11;
	uint endPage = 323;
	med->setStartPage(startPage);
	QCOMPARE(med->startPage(), startPage);
	med->setEndPage(endPage);
	QCOMPARE(med->endPage(), endPage);
	QCOMPARE(med->pages(), endPage - startPage);

	QDate pubDate = QDate::currentDate();
	pubDate.addDays(20);
	med->setPublicationDate(pubDate);
	QCOMPARE(med->publicationDate(), pubDate);

	delete med;
}

void ArticleTest::copyTrack()
{
	Article *med = new Article();
	connectMedium(med);
	
	med->setStartPage(12);
	med->setEndPage(232);
	med->setPublicationDate(QDate::currentDate());

	Article *copy = qobject_cast<Article*>(med->copyMedium());
	QVERIFY(copy != 0);
	QCOMPARE(copy->startPage(), med->startPage());
	QCOMPARE(copy->endPage(), med->endPage());
	QCOMPARE(copy->publicationDate(), med->publicationDate());

	delete copy;
	delete med;
}

void ArticleTest::insertAndLoadExtendedProperties()
{
	Article *med = new Article();
	connectMedium(med);

	med->setTitle("'SELECT * FROM user;");
	med->setDescription("This is greate show; INSERT INTO test test = 10;");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	med->setStartPage(11);
	med->setEndPage(232);
	med->setPublicationDate(QDate::currentDate());

	Article *checkMed = qobject_cast<Article*>(med->copyMedium());
	QVERIFY(checkMed);
	QCOMPARE(med->store(), true);
	QVERIFY(med->id() > 0);
	unsigned int id = med->id();
	delete med;

	med = new Article();
	connectMedium(med);
	
	QCOMPARE(med->load(id), true);
	QCOMPARE(med->id(), id);
	QCOMPARE(checkMed->startPage(), med->startPage());
	QCOMPARE(checkMed->endPage(), med->endPage());
	QCOMPARE(checkMed->publicationDate(), med->publicationDate());

	delete med;
	delete checkMed;
}

void ArticleTest::connectMedium(Medium* med)
{
	connect(med, SIGNAL(error(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(warning(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(information(QString)), this, SLOT(fetchMessages(QString)));
}

QTEST_MAIN(ArticleTest)
