/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "magazinetest.h"
#include "magazinetest.moc"

#include "magazine.h"

#include <QTest>

void MagazineTest::initTestCase()
{
// 	testMedId = 0;

	DBTestInit *testInit = new DBTestInit(this);
	if(!testInit->initTestDB())
		QSKIP("Could not create test database", SkipAll);

	Magazine *med = new Magazine();
	keys = testInit->prepareMediumKeys(med->group());
	delete med;
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
}

void MagazineTest::testExtendedProperties()
{
	Magazine *med = new Magazine();

	baseTests(med);

	// testing init status
	QCOMPARE(med->issue(), QString());
	QCOMPARE(med->publicationDate(), QDate());

	QString issue = "TEst Vol";
	med->setIssue(issue);
	QCOMPARE(med->issue(), issue);

	QDate pubDate = QDate::currentDate();
	pubDate.addDays(20);
	med->setPublicationDate(pubDate);
	QCOMPARE(med->publicationDate(), pubDate);

	delete med;
}

void MagazineTest::copyTrack()
{
	Magazine *med = new Magazine();
	med->setIssue("09/2010");
	med->setPublicationDate(QDate::currentDate());

	Magazine *copy = qobject_cast<Magazine*>(med->copyMedium());
	QVERIFY(copy);
	QCOMPARE(copy->issue(), med->issue());
	QCOMPARE(copy->publicationDate(), med->publicationDate());

	delete copy;
	delete med;
}

void MagazineTest::insertAndLoadExtendedProperties()
{
	Magazine *med = new Magazine();

	med->setTitle("'SELECT * FROM user;");
	med->setDescription("This is greate show; INSERT INTO test test = 10;");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	med->setIssue("09/2010");
	med->setPublicationDate(QDate::currentDate());

	Magazine *checkMed = qobject_cast<Magazine*>(med->copyMedium());
	QVERIFY(checkMed);
	QCOMPARE(med->store(), true);
	QVERIFY(med->id() > 0);
	unsigned int id = med->id();
	delete med;

	med = new Magazine();
	QCOMPARE(med->load(id), true);
	QCOMPARE(med->id(), id);
	QCOMPARE(checkMed->issue(), med->issue());
	QCOMPARE(checkMed->publicationDate(), med->publicationDate());

	delete med;
	delete checkMed;
}

QTEST_MAIN(MagazineTest)
