/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "moviedeviceeditwidget.h"
#include "moviedeviceeditwidget.moc"

#include "movie.h"
#include "movies.h"
#include "editdevices.h"
#include <esqldatabase.h>

#include <QSqlQueryModel>
#include <QSqlQuery>

MovieDeviceEditWidget::MovieDeviceEditWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	movie = NULL;
	setupUi(this);
	connect(deviceSearchButton, SIGNAL(clicked()), this, SLOT(newDevice()));
	connect(deviceEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(setModified()));

	deviceModel = new QSqlQueryModel(this);
	deviceEdit->setModel(deviceModel);

	loadModel();
}

bool MovieDeviceEditWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);
	if(!movie){
		emit error(tr("Could not store the device changes. (No medium)"));
		return false;
	}
	if(!modified())
		return true;
	movie->setDevice ( deviceEdit->currentText() );
	setModified(false);
	return true;
}

void MovieDeviceEditWidget::setMedium(Medium* newMedium)
{
	setEnabled(false);
	setModified(false);
	medium = newMedium;
	movie = NULL;
	if(!newMedium)
		return;
	movie = qobject_cast<Movie*>(newMedium);
	if(!movie)
		return;
	deviceEdit->setCurrentIndex(deviceEdit->findText(movie->deviceName()));
	setEnabled(true);
	setModified(false);
}

void MovieDeviceEditWidget::newDevice()
{
	EditDevices edit(deviceEdit->currentText(), this);
	connectByPass(&edit);
	if(edit.exec() != QDialog::Accepted)
		return;

	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		deviceModel->setQuery(deviceModel->query());
		if(closeDB)
			db.close();
	}
	loadModel();
	deviceEdit->setCurrentIndex(deviceEdit->findText(edit.selectedName()));
	setModified();
}

void MovieDeviceEditWidget::loadModel()
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		deviceModel->setQuery("SELECT id, name FROM "+ Movies::group() +"_devices ORDER BY name");
		deviceEdit->setModelColumn(deviceModel->record().indexOf("name"));
		if(deviceEdit->currentText().isEmpty())
			deviceEdit->setCurrentIndex(0);

		if(closeDB)
			db.close();
	}
}
