/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "movies.h"
#include "movies.moc"

#include "esetupwizardobject.h"
#include "esetupwizarddb.h"
#include "moviesdb.h"
#include "moviesmodel.h"
#include "moviesearchwidget.h"
#include "movie.h"
#include "editdevices.h"

#include <QtPlugin>
#include <QStringList>
#include <QIcon>

Movies::Movies ( ) : MediaPlugin()
{
}

Movies::~Movies()
{

}

bool Movies::initiate ()
{
	QAction *editDevicesAction = new QAction ( tr ( "Movie Devices" ), this);
	editDevicesAction->setObjectName("editDevicesAction");
	connect(editDevicesAction, SIGNAL(triggered(bool)), this, SLOT(showEditDevices()));
	emit addAction ( editDevicesAction, "fileMenu", "fileMenuSeperatorBeforeExit" );
  return true;
}

bool Movies::unload ()
{
  return true;
}

QString Movies::name () const
{
  return myName();
}

QString Movies::myName()
{
	return tr ( "Movie" );
}

QString Movies::group()
{
  return "movies";
}

QIcon Movies::icon () const
{
  return QIcon();
}

QString Movies::description () const
{
  return tr ( "Plugin is able to handle media Movies" );
}

MediaModel *Movies::newModel ( QString materialType, EQueryThread *thread )
{
	Movie movie;
	if(materialType != movie.group())
		return NULL;
	return new MoviesModel ( thread, this );
}

Medium* Movies::newMedium(QString materialType, EQueryThread* thread)
{
	Movie *movie = new Movie(thread);
	if(materialType == movie->group())
		return movie;
	delete movie;
	return NULL;
}

SearchDetailsWidget *Movies::searchWidget ( QString materialType )
{
	Q_UNUSED(materialType);
	return new MovieSearchWidget();
}

QStringList Movies::supportedAttributes ( QString materialType )
{
	Movie med;
	if(materialType != med.group())
		return QStringList();
	return med.supportedAttributes();
}

ESettingsModelItem *Movies::settingsModelItem()
{
  return NULL;
}

ESetupWizardObject *Movies::wizardObject()
{
  return new ESetupWizardDB ( tr ( "Configure Database of Movies" ), new MoviesDB ( ), QStringList(), this );
}

QStringList Movies::supportedMaterialTypes()
{
	Movie movie;
  QStringList result;
  result << movie.group();
  return result;
}

bool Movies::isSupportedMaterialType(QString materialType)
{
	if(supportedMaterialTypes().contains(materialType))
		return true;
	return false;
}

QStringList Movies::translateMaterialType(QString materialType)
{
	QStringList result;
	Movie movie;
	if(materialType != movie.group())
		return result;
	result << movie.name() << tr ( "Movies" );
	return result;
}

QString Movies::untranslateMaterialType(QString materialType)
{
	Movie movie;
	if(materialType == movie.name())
		return movie.group();
	return QString();
}

void Movies::showEditDevices()
{
	EditDevices dialog;
	connectByPass(&dialog);
	dialog.exec();
}

Q_EXPORT_PLUGIN2 ( Meganizer_Movies, Movies )
