/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "moviesdb.h"
#include "moviesdb.moc"

#include "movies.h"
#include "movie.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>

MoviesDB::MoviesDB ( QObject *parent ) : ESqlInit ( parent )
{
	setObjectName("MoviesDB");
}

void MoviesDB::scheme()
{
	setup(Movies::group(), Movies::myName(), 5);

	ESqlTable *devices = addTable ( "devices" );
	ESqlField *field = devices->appendField( "name", ESqlField::Varchar, true );
	field->setLength(50);
	devices->addIndex(field, 10);
	field = devices->appendField ( "description", ESqlField::Varchar );
	field->setLength(500);
	
	QStringList values;
	values << "" << tr("DVD") << tr("Digital Versatile Disc");
	devices->appendValues(values);
	values.clear();
	values << "" << tr("File") << tr("File on HDD");
	devices->appendValues(values);
	values.clear();
	
  ESqlTable *movies = addTable ( "movies" );
  field = movies->appendField ( "materialsId", ESqlField::Integer, true );
  field->setIsUnsigned ( true );
  movies->addForeignKey ( field->name(), "meganizer_materials", "id" );

  field = movies->appendField ( "publicationDate", ESqlField::Date );
	movies->addForeignKey ( devices );

  ESqlTable *table = addTable ( "meganizer_materialtypes", true, false );
  table->appendField ( "name", ESqlField::Varchar );

	Movie movie;
	QStringList insert;
  insert << "" << movie.group();
	table->appendValues ( insert );
	insert.clear();

  table = addTable("meganizer_materialpersonsroles", true, false);
  table->appendField("name", ESqlField::Varchar);
  table->appendField("description", ESqlField::Varchar);
  
  insert.clear();
	insert << "" << tr("Actor") << tr("The artist who perfoms in the movie");
  table->appendValues(insert);
	insert.clear();
	insert << "" << tr("Director") << tr("The director of the movie");
  table->appendValues(insert);

	addView ( "viewmovies",
						"SELECT  mats.*, movies.publicationDate, movies.devicesId"
						"  FROM "+ movies->name() +" movies"
						"  LEFT JOIN viewmaterials mats ON mats.id=movies.materialsId");

	addUpdateRoutine(2, &MoviesDB::updateRoutine1To2);
}

bool MoviesDB::updateRoutine1To2(ESqlInit* caller, QList< ESqlTable* > tables, QString& errorString)
{
	Q_UNUSED(caller);
	Q_UNUSED(tables);

	Movie movie;
	QSqlQuery query;
	// update old entry
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Movie" ));
	query.exec();
	if(!query.first())
		return false;
	unsigned int id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find old movie entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("UPDATE meganizer_materialtypes SET name=:name WHERE id=:id");
	query.bindValue(":name", movie.group());
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not update old movie entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}

	// remove newly added double entry (we must leave the old one,
	// because there may be materials with this type)
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name AND id<>:id");
	query.bindValue(":name", movie.group());
	query.bindValue(":id", id);
	query.exec();
	if(!query.first())
		return false;
	id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find new movie entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("DELETE FROM meganizer_materialtypes WHERE id=:id");
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not delete new movie entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	return true;
}

