/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "movietest.h"
#include "movietest.moc"

#include <dbtestinit.h>
#include "movie.h"
#include "movies.h"
#include <esqldatabase.h>

#include <QtTest/QtTest>
#include <QSqlQuery>
#include <QSqlError>

void MovieTest::initTestCase()
{
	testMedId = 0;
	
	testInit = new DBTestInit(this);
	if(!testInit->initTestDB()){
		QSKIP("Could not create test database", SkipAll);
	}
	
	// insert dummy genre
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open())
		QSKIP("Could not connect to database", SkipAll);
	
	QSqlQuery query;
	
	testDevice = "Testdevice";
	if(!query.exec("INSERT INTO "+ Movies::group() +"_devices (name, description) VALUES('"+ testDevice +"', 'This is only for testing purposes')"))
		QSKIP("Could not create test device", SkipAll);

	query.exec("SELECT id FROM "+ Movies::group() +"_devices WHERE name='"+ testDevice +"'");
  if(!query.first())
		QSKIP("Could not fetch device Id", SkipAll);
	testDeviceId = query.record().value("id").toUInt();

	db.close();
	
	Movie *movie = new Movie();
	keys = testInit->prepareMediumKeys(movie->group());
	delete movie;
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
}

void MovieTest::cleanupTestCase()
{
	delete testInit;
}

void MovieTest::fetchMessages(QString msg)
{
	qDebug() << msg;
}

void MovieTest::testExtendedProperties()
{
	Movie *med = new Movie();
	connectMedium(med);
	
	baseTests(med);
	
	QDate date = QDate::currentDate();
	med->setPublicationDate(date);
	QCOMPARE(med->publicationDate(), date);
	
	// testing valid device name
	med->setDevice(testDevice);
	QCOMPARE(med->device(), testDeviceId);
	QCOMPARE(med->deviceName(), testDevice);

	// testing wrong genre name
	med->setDevice(keys.invalid);
	QCOMPARE(med->deviceName(), keys.invalid);
	QVERIFY(med->device() == 0);
	// valid genre id
	med->setDevice(testDeviceId);
	QCOMPARE(med->device(), testDeviceId);
	QCOMPARE(med->deviceName(), testDevice);
	// testing wrong genre id
	med->setDevice(keys.invalidId);
	QCOMPARE(med->device(), keys.invalidId);
	QCOMPARE(med->deviceName(), QString());
	// invalid genre name, with not set id
	med->setDevice(keys.invalid, false);
	QCOMPARE(med->deviceName(), keys.invalid);
	QCOMPARE(med->device(), keys.invalidId);
	// not set name
	med->setDevice(testDeviceId, false);
	QCOMPARE(med->deviceName(), keys.invalid);
	QCOMPARE(med->device(), testDeviceId);
	
	delete med;
}

void MovieTest::copyMovie()
{
	Movie *med = new Movie();
	med->setDevice(testDeviceId);
	med->setPublicationDate(QDate::currentDate());
	
	Movie *copy = qobject_cast<Movie*>(med->copyMedium());
	QVERIFY(copy);
	QCOMPARE(copy->device(), med->device());
	QCOMPARE(copy->deviceName(), med->deviceName());
	QCOMPARE(copy->publicationDate(), med->publicationDate());
	
	delete copy;
	delete med;
}

void MovieTest::insertAndLoadExtendedProperties()
{
	Movie *med = new Movie();
	connectMedium(med);
	
	med->setTitle("'SELECT * FROM meganizer_user;");
	med->setDescription("This is greate show; INSERT INTO test test = 10;");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	
	med->setDevice(testDeviceId);
	med->setPublicationDate(QDate::currentDate());
	
	Movie *checkMed = qobject_cast<Movie*>(med->copyMedium());
	QVERIFY(checkMed);
	QCOMPARE(med->store(), true);
	testMedId = med->id();
	delete med;
	
	med = new Movie();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->id(), testMedId);
	QCOMPARE(med->device(), checkMed->device());
	QCOMPARE(med->deviceName(), checkMed->deviceName());
	QCOMPARE(med->publicationDate(), checkMed->publicationDate());
	
	delete med;
	delete checkMed;
}

void MovieTest::extentedSetValueAndValue()
{
	QSKIP("Not implemented yet", SkipSingle);
}

void MovieTest::connectMedium(Medium* med)
{
	connect(med, SIGNAL(error(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(warning(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(information(QString)), this, SLOT(fetchMessages(QString)));
}

QTEST_MAIN(MovieTest)
