/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "cd.h"
#include "cd.moc"

#include "cds.h"
#include "cdedit.h"
#include <materialpersonsmodel.h>
#include <materialperson.h>
#include "tracksmodel.h"
#include "track.h"
#include <esqldatabase.h>
#include <ewidget.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QTextTable>
#include <QTextCursor>
#include <QProgressBar>

CD::CD ( EQueryThread *thread, Medium *newParent )
	: Medium ( tr ( "Music Device" ), "musicdevice", CDs::group() +"_viewcds", thread, newParent )
{
	myIsLP = false;
	myDeviceId = 0;
  myDiscs = 0;
	TracksModel *model = new TracksModel ( thread, this );
	setChildModel(model);
	// the standard is CD mode, but if isLP() is set to true,
	// the mode is switched to the LP mode.
	// This can be done quite easily, because the data is nearly the same
	model->setMode(Track::CD);
}

#if QT_VERSION < 0x040500
Medium* CD::newMedium()
{
	return new CD();
}
#endif

bool CD::select ( QSqlRecord record )
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = false;
  if ( !db.isOpen() )
    closeDB = true;

  if ( !Medium::select ( record ) )
    return false;

  if ( !record.contains ( "discs" ) ) {
    if ( closeDB )
			if( !db.open() )
				return false;

    QSqlQuery replaceRecord;

    replaceRecord.exec
    (
      QString ( "SELECT * FROM %1_cds WHERE materialsId=%2" )
      .arg ( CDs::group() )
      .arg ( id() )
    );

    if ( !replaceRecord.first() ) {
      if ( closeDB )
        db.close();

      return false;
    }

    record = replaceRecord.record();
  }

  setDiscs ( record.value ( "discs" ).toUInt() );
	setDevice( record.value ( "devicesId").toUInt() );
	setIsLP ( record.value ( "isLP" ).toBool() );
  setPublicationDate ( record.value ( "publicationDate" ).toDate() );

  if ( closeDB )
    db.close();
  return true;
}

bool CD::prepare()
{
	if(Medium::prepare())
		return false;

	// prepare your custom stuff below
	if(device() > 0)
		return true;

	EditMedium *edit = editWidget();
	edit->setAddMode(this, false);
	if(edit->exec() != QDialog::Accepted)
		return false;
	return prepare();
}

bool CD::store ( bool addOwner, bool startTransaction )
{
	bool isInsert = false;
	if(id() == 0)
		isInsert = true;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		if( !db.open() ){
			emit error(tr("Could not connect to database!\nError: %1").arg(db.lastError().text()));
			return false;
		}

	bool transaction = false;
	if(startTransaction)
		transaction = transactionsSupported;
	if(transaction)
		db.transaction();
	
  if ( !Medium::store ( addOwner, false ) ){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if ( closeDB )
        db.close();
    return false;
	}

  // id is needed and should be set by Medium::store()
  if ( id() == 0 ){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if ( closeDB )
        db.close();
    return false;
	}

  QSqlQuery query;
	query.exec(QString("SELECT id FROM %1_cds WHERE materialsId=%2").arg(CDs::group()).arg(id()));
	bool isUpdate = false;
	if(query.first())
		isUpdate = true;
  if ( isUpdate ) {
    if
    (
      !query.prepare
      (
        QString
        (
          "UPDATE %1_cds SET"
          " discs=:discs, publicationDate=:publicationDate,"
          " devicesId=:devicesId, isLP=:isLP"
          " WHERE materialsId=:materialsId"
        )
        .arg ( CDs::group() )
      )
    ) {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not prepare update CD!\nQuery:%1\nError:\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }

  } else {
    if
    (
      !query.prepare
      (
        QString
        (
          "INSERT INTO %1_cds "
          "(materialsId, discs, publicationDate, devicesId, isLP) "
          "VALUES(:materialsId, :discs, :publicationDate, :devicesId, :isLP)"
        )
        .arg ( CDs::group() )
      )
    ) {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not prepare insert CD statement!\nQuery:%1\nError:\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }
  }

  query.bindValue ( ":materialsId", id() );
  query.bindValue ( ":discs", discs() );
  query.bindValue ( ":publicationDate", publicationDate().toString ( "yyyy-MM-dd" ) );
	query.bindValue ( ":devicesId", device() );
	query.bindValue ( ":isLP", isLP() );

 if ( !query.exec() ) {
	 if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
    if ( closeDB )
      db.close();
		emit error(tr("Could not execute SQL statement!\nQuery:%1\nError:\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
    return false;
  }
  
	if(transaction)
			if(!db.commit())
				emit error(tr("Could not store medium! Commit has failed!"));
  if ( closeDB )
    db.close();

  return true;
}

bool CD::setValue ( QString attribute, QVariant value )
{
	if(attribute == "pubDate"){
		setPublicationDate(value.toDate());
		return true;
	}

	if(attribute == "discs"){
		setDiscs(value.toUInt());
		return true;
	}
	
	if(attribute == "track"){
		Track *track = 0;
		QStringList items = value.toStringList();
		if(isLP()){
			if(items.size() != 5)
				return false;
			if(items[3].isEmpty())
				return false;
		}
		else{
			if(items.size() != 4)
				return false;
			if(items[2].isEmpty())
				return false;
		}

		if(!childs())
			return false;

		track = qobject_cast<Track*>(childs()->addEmptyMedium());
		if(!track)
			return false;
		if(isLP()){
			track->setDisc(items[0].toUInt());
			track->setSide(items[1]);
			track->setTrackNumber(items[2].toUInt());
			track->setTitle(items[3]);
			track->setLength(QTime::fromString(items[4], "hh:mm:ss"));
		}
		else{
			track->setDisc(items[0].toUInt());
			track->setTrackNumber(items[1].toUInt());
			track->setTitle(items[2]);
			track->setLength(QTime::fromString(items[3], "hh:mm:ss"));
		}
		
		// set default artist and label
		track->setManufacturer(manufacturer(), false);
		track->setManufacturer(manufacturerName(), false);
		track->setGenre(genre(), false);
		track->setGenre(genreName(), false);
		MaterialPersonsModel *model = materialPersons();
		if(model){
			if(model->rowCount() == 1){
				MaterialPerson *person = qobject_cast<MaterialPerson*>(model->personFromIndex(model->index(0,0)));
				if(person){
					if((!person->name().contains(tr("Various"))) && (!person->givenName().contains(tr("Various")))){
						model = track->materialPersons();
						if(model){
							model->addPerson(person);
						}
					}
				}
			}
		}
		return true;
	}
	
	if(attribute == "device"){
		bool check = false;
		value.toUInt(&check);
		if(check){
			setDevice(value.toUInt());
			return true;
		}
		setDevice(value.toString());
		return true;
	}
	return Medium::setValue(attribute, value);
}

QVariant CD::value ( QString attribute, int role )
{
	// formatting should be the same for al media,
	// unless there are good reasons to do it in another way
	if ( role != Qt::DisplayRole || compatibilityMode())
    return Medium::value(attribute, role);
  
	if(attribute == "pubDate")
		return publicationDate();
	
	if(attribute == "discs")
		return discs();
	
	if(attribute == "device")
		return deviceName();
	
  return Medium::value(attribute, role);
}

EditMedium *CD::editWidget(EWidget* parent)
{
  return new CDEdit ( parent );
}

void CD::setDiscs ( uint numDiscs )
{
  myDiscs = numDiscs;
}

uint CD::discs() const
{
  return myDiscs;
}

void CD::setDevice(uint deviceId, bool setName)
{
	myDeviceId = deviceId;
	if(setName){
		setDevice("", false);
		if(deviceId > 0){
			QString queryString = QString("SELECT name FROM "+ CDs::group() +"_devices WHERE id=%1").arg(deviceId);
			if(thread){
				thread->execQuery(this, "setDevice", queryString, "");
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.exec(queryString);
					if(query.first()){
						setDevice(query.record());
					}
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

void CD::setDevice(QString name, bool setId)
{
	myDeviceName = name.trimmed();
	if(setId){
		setDevice(0, false);
		if(!name.isEmpty()){
			QString queryString = "SELECT id FROM "+ CDs::group() +"_devices WHERE name=?";
			if(thread){
				QList<QVariant> values;
				values << name;
				thread->execQuery(this, "setDevice", queryString, "", values);
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.prepare(queryString);
					query.addBindValue(name);
					query.exec();
					if(query.first())
						if(query.size() == 1)
						setDevice(query.record());
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

void CD::setDevice(QSqlRecord record)
{
	if(record.contains("id"))
		setDevice(record.value("id").toUInt(), false);
	else if(record.contains("name"))
		setDevice(record.value("name").toString(), false);
}

uint CD::device() const
{
	return myDeviceId;
}

QString CD::deviceName() const
{
	return myDeviceName;
}

void CD::setIsLP(bool status)
{
	TracksModel *model = qobject_cast<TracksModel*>(childs());
	if(!model)
		return;
	
	myIsLP = status;
	if(status)
		model->setMode(Track::LP);
	else
		model->setMode(Track::CD);
}

bool CD::isLP() const
{
	return myIsLP;
}

void CD::setPublicationDate ( QDate newDate )
{
  myPublicationDate = newDate;
}

QDate CD::publicationDate() const
{
  return myPublicationDate;
}

Medium* CD::copyMedium()
{
    CD *cd = qobject_cast<CD*>(Medium::copyMedium());
		if(!cd)
			return 0;
		
		cd->setDevice(device(), false);
		cd->setDevice(deviceName(), false);
		cd->setDiscs(discs());
		cd->setPublicationDate(publicationDate());
		cd->setIsLP(isLP());
		return cd;
}

bool CD::exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly)
{
	if(format == Bibtex)
		return false;
	
	if(!table)
		return false;

	if(compatibilityMode() && format == Table)
		return Medium::exportTextTable(table, format, titleOnly);
	
	// collecting the data which should be inserted	
	QStringList values;
	if(titleOnly){
		values << attrs ( "title" ).translation;
		values << attrs ( "persons" ).translation;
		values << attrs ( "discs" ).translation;
		values << attrs ( "manufacturer" ).translation;
		values << attrs ( "min" ).translation;
		values << tr("Wish");
	}
	else{
		values << title();
		MaterialPersonsModel *model = materialPersons();
		if(model){
			QStringList authors;
			for(int i = 0; i < model->rowCount(); i++){
				authors.append
				(
					QString("%1, %2")
						.arg(model->data(model->index(i, 0)).toString())
						.arg(model->data(model->index(i, 1)).toString().left(1))
				);
			}
			values << authors.join(tr(" and "));
		}
		values << QString::number(discs());
		values << manufacturerName();
		values << minsCombined();
		if(wish())
			values << tr("Yes");
	}
	
	// insert the place at the table we need
	if(!titleOnly){
		// the first row is present at the table, because empty table can not be created
#if QT_VERSION < 0x040500
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
	}
	if(table->columns() < values.size()){
#if QT_VERSION < 0x040500
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
	}
	int row = table->rows() - 1;
	bool result = true;
	for(int i = 0; i < values.size(); i++){
		QTextTableCell cell = table->cellAt(row, i);
		QTextCursor cursor = cell.firstCursorPosition();
		cursor.insertText(values[i]);
	}
	return result;
}

QList<ColKeyTrans> CD::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();
		
  QList<ColKeyTrans> attrs;
  attrs << Medium::attrs("source", 0);
	attrs << Medium::attrs("title", 1);
  attrs << Medium::attrs("persons", 2);
	attrs << ColKeyTrans(3, "manufacturer", tr ( "Label" ));
  attrs << ColKeyTrans(4, "pubDate", tr ( "Pub. Date" ));
	attrs << ColKeyTrans(5, "discs", tr ( "Discs" ));
  attrs << Medium::attrs("min", 6);
	attrs << ColKeyTrans(7, "device", tr("Device"));
	attrs << ColKeyTrans(8, "track", tr( "Track" ));
	attrs << Medium::attrs("listPrice", 9);
	attrs << Medium::attrs("listPriceCurrency", 10);
	attrs << Medium::attrs("description", 11);
	attrs << Medium::attrs("owner", 12);
  return attrs;
}

void CD::customQueryResult(QVariant requestId, int querySize, QSqlRecord record)
{
	if(requestId.toString() == "setDevice")
		if(querySize == 1)
			setDevice(record);
}
