/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef CD_H
#define CD_H

#include "medium.h"

#include <QDate>

/**
  * Represants a CD with Tracks
	* @todo add tracks
	*/
class CD : public Medium
{
	Q_OBJECT
	Q_PROPERTY(QDate publicationDate READ publicationDate WRITE setPublicationDate)
	Q_PROPERTY(bool isLP READ isLP WRITE setIsLP)
	Q_PROPERTY(uint discs READ discs WRITE setDiscs)
  public:
    Q_INVOKABLE CD ( EQueryThread *thread = NULL, Medium *parent = NULL );

#if QT_VERSION < 0x040500
		Medium *newMedium();
#endif
    bool select( QSqlRecord record);
		bool prepare();
    bool store(bool addOwner = false, bool startTransaction = true);
		/**
			* To set Tracks for a CD, use a QStringList with:
			* @li cd number (e.g.: CD 1 or CD 2) as unsigned int
			* @li track number as unsigned int
			* @li title as QString
			* @li length as QString (may be empty, format: hh:mm:ss)
			* and for a LP (isLP() == true)
			* @li LP number (e.g.: LP 1 or LP 2) as unsigned int
			* @li Side (A or B) as string
			* @li track number as unsigned int
			* @li title as QString
			* @li length as QString (may be empty, format: hh:mm:ss)
			*/
    bool setValue ( QString attribute, QVariant value );
    QVariant value ( QString attribute, int role );
		EditMedium *editWidget(EWidget* parent = 0);

    void setDiscs ( uint numDisks );
    uint discs() const;
		void setDevice ( uint deviceId, bool setName = true );
		void setDevice ( QString name, bool setId = true );
		void setDevice ( QSqlRecord record );
		uint device() const;
		QString deviceName() const;
		void setIsLP(bool status);
		bool isLP() const;

    void setPublicationDate ( QDate newDate );
    QDate publicationDate() const;
		
		Medium* copyMedium();
		bool exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly = false);

		QList<ColKeyTrans> supportedAttrs();
		
  private:
		void customQueryResult(QVariant requestId, int querySize, QSqlRecord record);
		
		uint myDiscs, myDeviceId;
		QString myDeviceName;
    QDate myPublicationDate;
		bool myIsLP;
};

#endif // CD_H
