/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cds.h"
#include "cds.moc"

#include "esetupwizardobject.h"
#include "esetupwizarddb.h"
#include "cdsdb.h"
#include "cdsmodel.h"
#include "tracksmodel.h"
#include "cdsearchwidget.h"
#include "cd.h"
#include "track.h"
#include "editdevices.h"

#include <QtPlugin>
#include <QStringList>
#include <QIcon>

CDs::CDs (  ) : MediaPlugin()
{
}

CDs::~CDs()
{

}

bool CDs::initiate ()
{
	QAction *editDevicesAction = new QAction ( tr ( "Music Devices" ), this);
	editDevicesAction->setObjectName("editDevicesAction");
	connect(editDevicesAction, SIGNAL(triggered(bool)), this, SLOT(showEditDevices()));
	emit addAction ( editDevicesAction, "fileMenu", "fileMenuSeperatorBeforeExit" );
  return true;
}

bool CDs::unload ()
{
  return true;
}

QString CDs::name () const
{
  return myName();
}

QString CDs::myName()
{
	return tr ( "Music" );
}

QString CDs::group()
{
  return "cds";
}

QIcon CDs::icon () const
{
  return QIcon();
}

QString CDs::description () const
{
  return tr ( "Plugin is able to handle media like audio CD's or DVD's (...) and tracks" );
}

MediaModel *CDs::newModel ( QString materialType, EQueryThread *thread )
{
	CD cd;
	Track track;
	MediaModel *model = NULL;
	if(materialType == cd.group())
		model = new CDsModel ( thread, this );
	if(materialType == track.group())
		model = new TracksModel ( thread, this );
  return model;
}

Medium* CDs::newMedium(QString materialType, EQueryThread* thread)
{
	CD *cd = new CD(thread);
	if(materialType == cd->group())
		return cd;
	delete cd;
	Track *track = new Track(thread);
	if(materialType == track->group())
		return track;
	delete track;
	return NULL;
}

SearchDetailsWidget *CDs::searchWidget ( QString materialType )
{
	CD cd;
	if(materialType == cd.group())
		return new CDSearchWidget();
	return NULL;
}

QStringList CDs::supportedAttributes ( QString materialType )
{
	CD cd;
	if(materialType == cd.group())
		return cd.supportedAttributes();

	Track track;
	if(materialType == track.group())
		return track.supportedAttributes();
	
	return QStringList();
}

ESettingsModelItem *CDs::settingsModelItem()
{
  return 0;
}

ESetupWizardObject *CDs::wizardObject()
{
  return new ESetupWizardDB ( tr ( "Configure Database of Music Plugin" ), new CDsDB ( ), QStringList(), this );
}

QStringList CDs::supportedMaterialTypes()
{
	CD cd;
	Track track;
  QStringList result;
  result << cd.group();
	result << track.group();
  return result;
}

bool CDs::isSupportedMaterialType(QString materialType)
{
	if(supportedMaterialTypes().contains(materialType))
		return true;
	return false;
}

QStringList CDs::translateMaterialType(QString materialType)
{
	QStringList result;
	CD cd;
	Track track;
	if(materialType == cd.group()){
		result << cd.name() << tr ( "All medias where music is stored on" );
		return result;
	}
	if(materialType == track.group()){
		result << track.name() << tr ( "A single track" );
		return result;
	}
  return result;
}

QString CDs::untranslateMaterialType(QString materialType)
{
	CD cd;
	Track track;
	if(materialType == cd.name())
		return cd.group();
	if(materialType == track.name())
		return track.group();
  return QString();
}

void CDs::showEditDevices()
{
	EditDevices dialog;
	EObject::connectByPass(this, &dialog);
	dialog.exec();
}

Q_EXPORT_PLUGIN2 ( Meganizer_CDs, CDs )
