/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cdsdb.h"
#include "cdsdb.moc"

#include "cds.h"
#include "cd.h"
#include "track.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>

CDsDB::CDsDB ( QObject *parent ) : ESqlInit ( parent )
{
	setObjectName("CDsDB");
}

void CDsDB::scheme()
{
	setup(CDs::group(), CDs::myName(), 10);
  
	ESqlTable *devices = addTable ( "devices" );
	ESqlField *field = devices->appendField( "name", ESqlField::Varchar, true );
	field->setLength(50);
	devices->addIndex(field, 10);
	field = devices->appendField ( "description", ESqlField::Varchar );
	field->setLength(500);
	
	QStringList values;
	values << "" << tr("CD") << tr("Compact Disc");
	devices->appendValues(values);
	values.clear();
	values << "" << tr("LP") << tr("Long-Playing record");
	devices->appendValues(values);
	values.clear();
	values << "" << tr("DVD") << tr("Digital Versatile Disc");
	devices->appendValues(values);
	values.clear();
	
  ESqlTable *cds = addTable ( "cds" );
  field = cds->appendField ( "materialsId", ESqlField::Integer, true );
  field->setIsUnsigned ( true );
  cds->addForeignKey ( field->name(), "meganizer_materials", "id" );

  field = cds->appendField ( "discs", ESqlField::Integer );
  field->setIsUnsigned ( true );

  field = cds->appendField ( "publicationDate", ESqlField::Date );
	cds->addForeignKey ( devices );

	field = cds->appendField ( "isLP", ESqlField::Boolean, true );
	field->setDefaultValue(0);

	ESqlTable *tracks = addTable ( "tracks" );

  field = tracks->appendField ( "materialsId", ESqlField::Integer, true );
  field->setIsUnsigned ( true );
  tracks->addForeignKey ( field->name(), "meganizer_materials", "id" );

  field = tracks->appendField ( "length", ESqlField::Time );
	
  ESqlTable *table = addTable ( "meganizer_materialtypes", true, false );
  table->appendField ( "name", ESqlField::Varchar );

	CD cd;
	Track track;
  QStringList insert;
  insert << "" << cd.group();
	table->appendValues ( insert );
	insert.clear();
	insert << "" << track.group();
	table->appendValues ( insert );
  
  table = addTable("meganizer_materialpersonsroles", true, false);
  table->appendField("name", ESqlField::Varchar);
  table->appendField("description", ESqlField::Varchar);
  
  insert.clear();
  insert << "" << tr("Artist") << tr("The artist which performs an track");
  table->appendValues(insert);
	
	// A music device can have tracks
	table = addTable ( "ondisc" );
	
	field = table->appendField ( "materialhasmaterialsId", ESqlField::Integer, true );
  field->setIsUnsigned ( true );
  table->addForeignKey ( field->name(), "meganizer_materialhasmaterials", "id" );
	
	field = table->appendField ( "disc", ESqlField::Integer );
	field->setIsUnsigned( true );
	field->setLength(2);
	table->addIndex( field );

	// this field is for LPs, therefore it can be NULL
	field = table->appendField ( "side", ESqlField::Varchar );
	field->setLength(1);
	
	field = table->appendField ( "number", ESqlField::Integer );
	field->setIsUnsigned( true );
	field->setLength(3);
	field->setDefaultValue(0);
	table->addIndex( field );
	
	// a track can have tracks
	table = addTable ( "trackhastracks");
	field = table->appendField ( "materialhasmaterialsId", ESqlField::Integer, true );
  field->setIsUnsigned ( true );
  table->addForeignKey ( field->name(), "meganizer_materialhasmaterials", "id" );
	
	field = table->appendField("number", ESqlField::Integer);
	field->setIsUnsigned(true);
	field->setLength(3);
	field->setDefaultValue(0);
	
	addView ( "viewcds",
						"SELECT  mats.*, cds.discs, cds.publicationDate, cds.devicesId, cds.isLP"
						"  FROM "+ cds->name() +" cds"
						"  LEFT JOIN viewmaterials mats ON mats.id=cds.materialsId");
  addView ( "viewtracks",
						"SELECT  mats.*, tracks.length"
						"  FROM "+ tracks->name() +" tracks"
						"  LEFT JOIN viewmaterials mats ON mats.id=tracks.materialsId");
	
	addUpdateRoutine(5, &CDsDB::updateFrom4To5);
	addUpdateRoutine(7, &CDsDB::updateFrom6To7);
}

bool CDsDB::updateFrom4To5(ESqlInit* caller, QList<ESqlTable*> tables, QString& errorString)
{
	Q_UNUSED(tables);
	QSqlQuery query;

	// add the cd type to already present musicdevices
	// get old typeId
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Audio CD"));
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int typeId = query.record().value("id").toUInt();
	if(typeId <= 0){
		errorString.append(tr("Could not find id of Audio CD\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	
	// get device id
	query.prepare("SELECT id FROM "+ caller->label() +"_devices WHERE name=:name");
	query.bindValue(":name", tr ( "CD"));
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int deviceId = query.record().value("id").toUInt();
	if(deviceId <= 0){
		errorString.append(tr("Could not find id of Music Device\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	
	query.prepare("SELECT id FROM meganizer_materials WHERE materialtypesId=:materialtypesId");
	query.bindValue(":materialtypesId", typeId);
	query.exec();
	QSqlQuery insertQuery;
	while(query.next()){
		insertQuery.prepare("UPDATE "+ caller->label() +"_cds SET devicesId=:devicesId WHERE materialsId=:materialsId");
		insertQuery.bindValue(":devicesId", deviceId);
		insertQuery.bindValue(":materialsId", query.record().value("id").toUInt());
		if(!insertQuery.exec()){
			errorString.append(tr("Could not convert all MusicDevices to new structure!\n%1\n%2")
														.arg(query.lastError().text()).arg(query.lastQuery()));
			return false;
		}
	}
	
	// rename old type name and remove new one
	query.prepare("SELECT * FROM meganizer_materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Music Device" ));
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int newTypeId = query.record().value("id").toUInt();
	QString name = query.record().value("name").toString();
	QString description = query.record().value("description").toString();
	if(newTypeId <= 0){
		errorString.append(tr("Could not find id of type Music Device\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	// rename old type
	query.prepare("UPDATE meganizer_materialtypes SET name=:name, description=:description WHERE id=:id");
	query.bindValue(":name", name);
	query.bindValue(":description", description);
	query.bindValue(":id", typeId);
	if(!query.exec())
		return false;
	
	// remove new type
	query.prepare("DELETE FROM meganizer_materialtypes WHERE id=:id");
	query.bindValue(":id", newTypeId);
	if(!query.exec())
		return false;
	
	return true;
}

bool CDsDB::updateFrom6To7(ESqlInit* caller, QList<ESqlTable *>tables, QString &errorString)
{
	// find our _devices table
	ESqlTable *table = 0;
	foreach(ESqlTable *tab, tables){
		if(tab->name() == caller->label() +"_devices")
			table = tab;
	}
	if(!table){
		errorString.append(tr("Table not found"));
		return false;
	}

	// find values for LP
	QList<QStringList> list = table->values();
	QStringList values;
	// do not insert the id field of the LP row
	values << list[1][1];
	values << list[1][2];


	QSqlQuery query;

	// rename the old Record Description into the new LP
	query.prepare(QString("SELECT id FROM %1 WHERE name=:name AND description=:description").arg(table->name()));
	query.bindValue(":name", tr ( "Record"));
	query.bindValue(":description",  tr("Record Description"));
	query.exec();
	if(!query.first()){
		errorString.append(tr("Could not find old record description entry"));
		return false;
	}
	if(query.size() != 1){
		errorString.append(tr("Could not find old record description entry (Result size != 1)"));
		return false;
	}
	unsigned int id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find id of Record Description\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare(QString("UPDATE %1 SET name=:name, description=:description WHERE id=:id").arg(table->name()));
	query.bindValue(":name", values[0]);
	query.bindValue(":description", values[1]);
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not update entry for LPs (Old Record Description entry)\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}

	// delete newly added entry
	query.prepare(QString("SELECT id FROM %1 WHERE name=:name AND id<>:id").arg(table->name()));
	query.bindValue(":name", values[0]);
	query.bindValue(":id", id);
	query.exec();
	// the newly added entry may not be there, if the old one was correct
	if(query.size() == 1){
		if(!query.first()){
			errorString.append(tr("Could not find id of newly added entry"));
			return false;
		}
		unsigned int delId = query.record().value("id").toUInt();
		if(delId <= 0){
			errorString.append(tr("Could not find id of newly added LP entry, which should be removed\n%1\n%2")
														.arg(query.lastError().text()).arg(query.lastQuery()));
			return false;
		}
		query.prepare(QString("DELETE FROM %1 WHERE id=:id").arg(table->name()));
		query.bindValue(":id", delId);
		if(!query.exec()){
			errorString.append(tr("Could not delete newly added LP entry\n%1\n%2")
												.arg(query.lastError().text()).arg(query.lastQuery()));
			return false;
		}
	}

	// rename materialtype CD
	CD cd;
	// update old entry
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Music Device" ));
	query.exec();
	if(!query.first())
		return false;
	id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find old music device entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("UPDATE meganizer_materialtypes SET name=:name WHERE id=:id");
	query.bindValue(":name", cd.group());
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not update old music device entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}

	// remove newly added double entry (we must leave the old one,
	// because there may be materials with this type)
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name AND id<>:id");
	query.bindValue(":name", cd.group());
	query.bindValue(":id", id);
	query.exec();
	if(!query.first())
		return false;
	id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find new music device entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("DELETE FROM meganizer_materialtypes WHERE id=:id");
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not delete new music device entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}

	// rename materialtype Track
	Track track;
	// update old entry
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Track" ));
	query.exec();
	if(!query.first())
		return false;
	id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find old track entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("UPDATE meganizer_materialtypes SET name=:name WHERE id=:id");
	query.bindValue(":name", track.group());
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not update old track entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}

	// remove newly added double entry (we must leave the old one,
	// because there may be materials with this type)
	query.prepare("SELECT id FROM meganizer_materialtypes WHERE name=:name AND id<>:id");
	query.bindValue(":name", track.group());
	query.bindValue(":id", id);
	query.exec();
	if(!query.first())
		return false;
	id = query.record().value("id").toUInt();
	if(id <= 0){
		errorString.append(tr("Could not find new music device entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	query.prepare("DELETE FROM meganizer_materialtypes WHERE id=:id");
	query.bindValue(":id", id);
	if(!query.exec()){
		errorString.append(tr("Could not delete new track entry\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	return true;
}
