/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cdsearchwidget.h"

#include "cds.h"

#include <esqldatabase.h>

#include <QSqlQuery>
#include <QSqlRecord>

CDSearchWidget::CDSearchWidget ( QWidget* parent, Qt::WindowFlags flags )
    :SearchDetailsWidget ( parent, flags )
{
  setupUi ( this );
	connect(pubDateEdit, SIGNAL(dateChanged(QDate)), this, SLOT(dateChanged(QDate)));
	
	deviceEdit->setDisabled(true);
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		query.exec("SELECT name FROM "+ CDs::group() +"_devices ODER BY name");
		if(query.size() > 0)
			deviceEdit->setEnabled(true);
		while(query.next())
			deviceEdit->addItem(query.record().value("name").toString());
		if(closeDB)
			db.close();
	}
}

QString CDSearchWidget::searchTable ()
{
	return CDs::group() +"_viewcds";
}

QList<SqlConstraint> CDSearchWidget::constraints()
{
	QList<SqlConstraint> results;
	SqlConstraint constraint;
	if(discsEdit->value() > 0){
		constraint.setConstraint(QString("discs%1?").arg(condition(discsModeEdit->currentIndex())));
		constraint.addBindValue(discsEdit->value());
		results << constraint;
	}
	if(pubDate.isValid()){
		constraint = SqlConstraint();
		constraint.setConstraint(QString("publicationDate%1?").arg(condition(pubDateModeEdit->currentIndex())));
		constraint.addBindValue(pubDate);
		results << constraint;
	}
	if(!deviceEdit->currentText().isEmpty()){
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if(db.isOpen()){
			QSqlQuery query;
			query.prepare("SELECT id FROM "+ CDs::group() +"_devices WHERE name=:name");
			query.bindValue(":name", deviceEdit->currentText());
			query.exec();
			if(query.first()){
				constraint.setConstraint("devicesId=?");
				constraint.addBindValue(query.record().value("id").toUInt());
			}
			if(closeDB)
				db.close();
		}
	}
	return results;
}

void CDSearchWidget::dateChanged ( QDate date )
{
	pubDate = date;
}

QString CDSearchWidget::condition ( int index )
{
	switch(index){
		case 0:
			return "<=";
		case 1:
			return ">=";
		default:
			return "=";
	}
}

#include "cdsearchwidget.moc"
