/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cddeviceeditwidgettest.h"
#include "cddeviceeditwidgettest.moc"

#include "cddeviceeditwidget.h"
#include <esqldatabase.h>
#include "cds.h"
#include "cd.h"

#include <QTest>
#include <QSqlQuery>

bool CDDeviceEditWidgetTest::deviceModify(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget)
{
	Q_UNUSED(caller);
	QComboBox *deviceEdit = widget->findChild<QComboBox*>("deviceEdit");
	if(!deviceEdit)
		return false;

	QTest::keyClick(deviceEdit, Qt::Key_Down);
	QTest::keyClick(deviceEdit, Qt::Key_Down);
	
	QTest::qWarn(deviceEdit->currentText().toLatin1());
	return true;
}

bool CDDeviceEditWidgetTest::deviceCheck(MediumEditWidgetBaseTests* caller, MediumEditWidget* widget, Medium* med)
{
	Q_UNUSED(widget);
	
	CDDeviceEditWidgetTest *test = qobject_cast<CDDeviceEditWidgetTest*>(caller);
	if(!test)
		return false;
	
	CD *cd = qobject_cast<CD*>(med);
	if(!cd){
		QTest::qWarn("Medium is not a movie");
		return false;
	}

	if(cd->deviceName() != test->deviceName){
		QTest::qWarn("Wrong deviceName: "+ cd->deviceName().toLatin1());
		return false;
	}
	
	QTest::qWarn(cd->deviceName().toLatin1());
	if(cd->device() != test->deviceId){
		QTest::qWarn("No valid deviceId");
		return false;
	}

	return true;
}

void CDDeviceEditWidgetTest::initTestCase()
{
	MediumEditWidgetBaseTests::initTestCase();
	
	// insert dummy genre
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open())
		QSKIP("Could not connect to database", SkipAll);
	
	QSqlQuery query;
	
	deviceName = "Testdevice";
	if(!query.exec("INSERT INTO "+ CDs::group() +"_devices (name, description) VALUES('"+ deviceName +"', 'This is only for testing purposes')"))
		QSKIP("Could not create test device", SkipAll);

	query.exec("SELECT id FROM "+ CDs::group() +"_devices WHERE name='"+ deviceName +"'");
  if(!query.first())
		QSKIP("Could not fetch device Id", SkipAll);
	deviceId = query.record().value("id").toUInt();
	deviceName = "Testdevice 1";
	if(!query.exec("INSERT INTO "+ CDs::group() +"_devices (name, description) VALUES('"+ deviceName +"', 'This is only for testing purposes')"))
		QSKIP("Could not create test device", SkipAll);

	query.exec("SELECT id FROM "+ CDs::group() +"_devices WHERE name='"+ deviceName +"'");
  if(!query.first())
		QSKIP("Could not fetch device Id", SkipAll);
	deviceId = query.record().value("id").toUInt();

	db.close();
}

void CDDeviceEditWidgetTest::test()
{
	registerWidget(new CDDeviceEditWidget(),
		&CDDeviceEditWidgetTest::deviceModify,
		&CDDeviceEditWidgetTest::deviceCheck,
		new CD());
	
	testWidgets();
}

QTEST_MAIN(CDDeviceEditWidgetTest)
