/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cdtest.h"
#include "cdtest.moc"

#include <dbtestinit.h>
#include "cd.h"
#include "cds.h"
#include "track.h"
#include "tracksmodel.h"
#include <esqldatabase.h>

#include <QtTest/QtTest>
#include <QSqlQuery>
#include <QSqlError>

void CDTest::initTestCase()
{
	testMedId = 0;
	
	testInit = new DBTestInit(this);
	if(!testInit->initTestDB()){
		QSKIP("Could not create test database", SkipAll);
	}
	
	// insert dummy genre
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open())
		QSKIP("Could not connect to database", SkipAll);
	
	QSqlQuery query;
	
	// we need the track type also to test track handling
	Track *med = new Track();
	testTrackType = med->name();
	delete med;
	if(!query.exec("INSERT INTO meganizer_materialtypes (name) VALUES('"+ testTrackType +"')"))
		QSKIP("Could not create test material type", SkipAll);
	query.exec("SELECT id FROM meganizer_materialtypes WHERE name='"+ testTrackType +"'");
	if(!query.first())
		QSKIP("Could not fetch type Id", SkipAll);
	testTrackTypeId = query.record().value("id").toUInt();

	testDevice = "Testdevice";
	if(!query.exec("INSERT INTO "+ CDs::group() +"_devices (name, description) VALUES('"+ testDevice +"', 'This is only for testing purposes')"))
		QSKIP("Could not create test device", SkipAll);
	query.exec("SELECT id FROM "+ CDs::group() +"_devices WHERE name='"+ testDevice +"'");
  if(!query.first())
		QSKIP("Could not fetch device Id", SkipAll);
	testDeviceId = query.record().value("id").toUInt();

	db.close();
	
	CD *cd = new CD();
	keys = testInit->prepareMediumKeys(cd->group());
	delete cd;
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
}

void CDTest::cleanupTestCase()
{
	delete testInit;
}

void CDTest::fetchMessages(QString msg)
{
	qDebug() << msg;
}

void CDTest::testExtendedProperties()
{
	CD *med = new CD();
	connectMedium(med);
	
	baseTests(med);
	
	med->setDiscs(5);
	QVERIFY(med->discs() == 5);
	
	QDate date = QDate::currentDate();
	med->setPublicationDate(date);
	QCOMPARE(med->publicationDate(), date);
	
	// testing valid device name
	med->setDevice(testDevice);
	QCOMPARE(med->device(), testDeviceId);
	QCOMPARE(med->deviceName(), testDevice);
	// testing wrong genre name
	med->setDevice(keys.invalid);
	QCOMPARE(med->deviceName(), keys.invalid);
	QVERIFY(med->device() == 0);
	// valid genre id
	med->setDevice(testDeviceId);
	QCOMPARE(med->device(), testDeviceId);
	QCOMPARE(med->deviceName(), testDevice);
	// testing wrong genre id
	med->setDevice(keys.invalidId);
	QCOMPARE(med->device(), keys.invalidId);
	QCOMPARE(med->deviceName(), QString());
	// invalid genre name, with not set id
	med->setDevice(keys.invalid, false);
	QCOMPARE(med->deviceName(), keys.invalid);
	QCOMPARE(med->device(), keys.invalidId);
	// not set name
	med->setDevice(testDeviceId, false);
	QCOMPARE(med->deviceName(), keys.invalid);
	QCOMPARE(med->device(), testDeviceId);

	QCOMPARE(med->isLP(), false);
	med->setIsLP(true);
	QCOMPARE(med->isLP(), true);
	QCOMPARE(med->childs()->columnCount(), 6);
	med->setIsLP(false);
	QCOMPARE(med->isLP(), false);
	QCOMPARE(med->childs()->columnCount(), 5);
	
	delete med;
}

void CDTest::copyCD()
{
	CD *med = new CD();
	med->setDiscs(5);
	med->setDevice(testDeviceId);
	med->setPublicationDate(QDate::currentDate());
	
	CD *copy = qobject_cast<CD*>(med->copyMedium());
	QVERIFY(copy);
	QCOMPARE(copy->discs(), med->discs());
	QCOMPARE(copy->device(), med->device());
	QCOMPARE(copy->deviceName(), med->deviceName());
	QCOMPARE(copy->publicationDate(), med->publicationDate());
	QCOMPARE(copy->isLP(), med->isLP());
	
	delete copy;
	delete med;
}

void CDTest::insertAndLoadExtendedProperties()
{
	CD *med = new CD();
	connectMedium(med);
	
	med->setTitle("'SELECT * FROM user;");
	med->setDescription("This is greate show; INSERT INTO test test = 10;");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	
	med->setDiscs(5);
	med->setDevice(testDeviceId);
	med->setPublicationDate(QDate::currentDate());
	
	CD *checkMed = qobject_cast<CD*>(med->copyMedium());
	QVERIFY(checkMed);
	QCOMPARE(med->store(), true);
	testMedId = med->id();
	delete med;
	
	med = new CD();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->id(), testMedId);
	QCOMPARE(med->discs(), checkMed->discs());
	QCOMPARE(med->device(), checkMed->device());
	QCOMPARE(med->deviceName(), checkMed->deviceName());
	QCOMPARE(med->discs(), checkMed->discs());
	
	med->setDiscs(1);
	QCOMPARE(med->store(), true);
	delete med;
	med = new CD();
	QCOMPARE(med->load(testMedId), true);
	QCOMPARE(med->id(), testMedId);
	QVERIFY(med->discs() == 1);
	
	delete med;
	delete checkMed;
}

void CDTest::tracksHandlingCD()
{
	CD *med = new CD();
	connectMedium(med);
	
	med->setTitle("Test CD with Tracks");
	med->setDescription("Contains Tracks");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	med->setDevice(testDeviceId);
	
	TracksModel *model = qobject_cast<TracksModel*>(med->childs());
	QVERIFY(model);
	
	// QStringList cd, track, title, length (may be empty, format: hh:mm:ss)
	QList<QStringList> trackStrings;
	QStringList trackString;
	trackString << "1" << "1" << "Track 1" << "00:01:01";
	trackStrings << trackString;
	trackString.clear();
	trackString << "1" << "2" << "Track 2" << "00:02:02";
	trackStrings << trackString;
	
	foreach(QStringList trackString, trackStrings){
		QCOMPARE(med->setValue("track", trackString), true);
	}
	QCOMPARE(model->rowCount(), trackStrings.size());
	
	// we can not do more here, because we cannot use EditTrack
	// QCOMPARE(med->store(), true);
	
	// testing not inserting of removed tracks if it is a new CD
	for(int i = 0; i < model->rowCount(); i++){
		Track *track = qobject_cast<Track*>(model->mediumFromIndex(model->index(i, 0)));
		QVERIFY(track);
		track->setMarkForRemovel();
	}
	QCOMPARE(med->store(), true);
	unsigned int id = med->id();
	
	// testing of adding already present Track's
	QMap<unsigned int, Track*> trackMap;
	int num = 3;
	for(int i = 0; i < num; i++){
		Track *track = new Track();
		connectMedium(track);
	
		track->setTitle(QString("Track CD %1").arg(i));
		track->setGenre(keys.testGenreId);
		track->setManufacturer(keys.testManuId);
		track->setListPriceCurrency(keys.testCurId);
		track->setBoughtPriceCurrency(keys.testCurId);
		track->setType(testTrackTypeId);
		QCOMPARE(track->store(), true);
		model->addMedium(track->copyMedium());
		trackMap.insert(track->id(), track);
	}
	// since the previous added track's are now already stored at the db,
	// store should be possible
	QCOMPARE(med->store(), true);
	delete med;
	med = new CD();
	connectMedium(med);
	QCOMPARE(med->load(id), true);
	model = qobject_cast<TracksModel*>(med->childs());
	QCOMPARE(model->rowCount(), num);
	QCOMPARE(model->mode(), Track::CD);
	
	// testing TracksModel->value();
	for(int i = 0; i < model->rowCount(); i++){
		Medium *tmp = model->mediumFromIndex(model->index(i, 0));
		QVERIFY(tmp);
		Track *track = trackMap.value(tmp->id(), 0);
		QVERIFY(track);
		QCOMPARE(model->data(model->index(i, 0)).toUInt(), track->disc());
		QCOMPARE(model->data(model->index(i, 1)).toUInt(), track->trackNumber());
		QCOMPARE(model->data(model->index(i, 2)).toString(), track->materialPersonsCombined());
		QCOMPARE(model->data(model->index(i, 3)).toString(), track->title());
		QCOMPARE(model->data(model->index(i, 4)).toString(), track->length().toString("HH:mm:ss"));
	}

	QList<Track*> tracks = trackMap.values();
	trackMap.clear();
	qDeleteAll(tracks);
	
	delete med;
}

void CDTest::tracksHandlingLP()
{
	CD *med = new CD();
	connectMedium(med);

	med->setIsLP(true);
	med->setTitle("Test CD (LP) with Tracks");
	med->setDescription("Contains Tracks");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	med->setDevice(testDeviceId);

	TracksModel *model = qobject_cast<TracksModel*>(med->childs());
	QVERIFY(model);

	// QStringList cd, track, title, length (may be empty, format: hh:mm:ss)
	QList<QStringList> trackStrings;
	QStringList trackString;
	trackString << "1" << "A" << "1" << "Track 1" << "00:01:01";
	trackStrings << trackString;
	trackString.clear();
	trackString << "1" << "B" << "2" << "Track 2" << "00:02:02";
	trackStrings << trackString;

	foreach(QStringList trackString, trackStrings){
		QCOMPARE(med->setValue("track", trackString), true);
	}
	QCOMPARE(model->rowCount(), trackStrings.size());

	// we can not do more here, because we cannot use EditTrack
	// QCOMPARE(med->store(), true);

	// testing not inserting of removed tracks if it is a new CD
	for(int i = 0; i < model->rowCount(); i++){
		Track *track = qobject_cast<Track*>(model->mediumFromIndex(model->index(i, 0)));
		QVERIFY(track);
		track->setMarkForRemovel();
	}
	QCOMPARE(med->store(), true);
	unsigned int id = med->id();

	// testing of adding already present Track's
	QMap<unsigned int, Track*> trackMap;
	int num = 3;
	for(int i = 0; i < num; i++){
		Track *track = new Track();
		connectMedium(track);

		track->setTitle(QString("Track LP %1").arg(i));
		if(num % 2 == 0)
			track->setSide(Track::SideA);
		else
			track->setSide(Track::SideB);
		track->setGenre(keys.testGenreId);
		track->setManufacturer(keys.testManuId);
		track->setListPriceCurrency(keys.testCurId);
		track->setBoughtPriceCurrency(keys.testCurId);
		track->setType(testTrackTypeId);
		QCOMPARE(track->store(), true);
		model->addMedium(track->copyMedium());
		trackMap.insert(track->id(), track);
	}
	// since the previous added track's are now already stored at the db,
	// store should be possible
	QCOMPARE(med->store(), true);
	delete med;
	med = new CD();
	connectMedium(med);
	QCOMPARE(med->load(id), true);
	model = qobject_cast<TracksModel*>(med->childs());
	QCOMPARE(model->rowCount(), num);
	QCOMPARE(model->mode(), Track::LP);

	// testing TracksModel->value();
	for(int i = 0; i < model->rowCount(); i++){
		Medium *tmp = model->mediumFromIndex(model->index(i, 0));
		QVERIFY(tmp);
		Track *track = trackMap.value(tmp->id(), 0);
		QVERIFY(track);
		QCOMPARE(model->data(model->index(i, 0)).toUInt(), track->disc());
		if(track->side() == Track::SideA)
			QCOMPARE(model->data(model->index(i, 1)).toString(), QString("A"));
		else
			QCOMPARE(model->data(model->index(i, 1)).toString(), QString("B"));
		QCOMPARE(model->data(model->index(i, 2)).toUInt(), track->trackNumber());
		QCOMPARE(model->data(model->index(i, 3)).toString(), track->materialPersonsCombined());
		QCOMPARE(model->data(model->index(i, 4)).toString(), track->title());
		QCOMPARE(model->data(model->index(i, 5)).toString(), track->length().toString("HH:mm:ss"));
	}

	QList<Track*> tracks = trackMap.values();
	trackMap.clear();
	qDeleteAll(tracks);

	delete med;
}

void CDTest::extentedSetValueAndValue()
{
	QSKIP("Not implemented yet", SkipSingle);
}

void CDTest::connectMedium(Medium* med)
{
	connect(med, SIGNAL(error(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(warning(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(information(QString)), this, SLOT(fetchMessages(QString)));
}

QTEST_MAIN(CDTest)
