/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef TRACK_H
#define TRACK_H

#include "medium.h"

#include <QDate>
#include <QTime>
#include <QMetaType>

/**
  * Represants a Track
	* @todo allow a null manufacturer for a track?
	*/
class Track : public Medium
{
	Q_OBJECT
	Q_PROPERTY(QTime length READ length WRITE setLength);
  public:
		/**
			* @brief The mode of this track.
			*
			*/
		enum Mode { Standard = 0x01,		/**< This is a standard Track. */
								CD = 0x02,					/**< This track is belongs to a CD. Therefore cd() and trackNumber() are enabled. */
								LP = 0x04,					/**< This track is belongs to a LP. Therefore cd(), side() and trackNumber() are enabled. */
								Collection = 0x08		/**< This track belongs to another Track. Therefore trackNumber() is enabled. */
		};
		enum Side {
			NotSet,
			SideA,
			SideB
		};

    Q_INVOKABLE Track ( EQueryThread *thread = NULL, Medium *parent = NULL );

#if QT_VERSION < 0x040500
		Medium *newMedium();
#endif
    bool select( QSqlRecord record);
    bool store(bool addOwner = false, bool startTransaction = true);
		bool storeChild(Medium* parent);
		bool setValue ( QString attribute, QVariant value );
    QVariant value ( QString attribute, int role );
		EditMedium *editWidget(EWidget* parent = 0);

    void setLength ( QTime duration );
    QTime length();
		
		Medium *copyMedium();
		bool exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly = false);

		void setDisc(unsigned int num);
		unsigned int disc() const;
		void setSide(QString side);
		void setSide(Track::Side side);
		Track::Side side() const;
		void setTrackNumber(unsigned int num);
		unsigned int trackNumber() const;
		
		// handle tracks
		/**
			* @see TracksModel for explanations
			*/
		void setMode(Track::Mode mode);
		Track::Mode mode() const;

		/**
			* @brief Hides Medium::childs() to create the TracksModel here.
			*
			* We must create the model here to avoid recursions.
			*/
		MediaModel *childs();

		QList<ColKeyTrans> supportedAttrs();

  private:
    QTime myLength;
		Mode myMode;
		unsigned int myDisc, myTrackNumber;
		Side mySide;
};

Q_DECLARE_METATYPE(Track::Side);

#endif // TRACK_H
