/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "tracksmodel.h"
#include "tracksmodel.moc"

#include <delegateuser.h>
#include <esqldatabase.h>
#include "cds.h"

#include <QAbstractItemView>
#include <QSqlQuery>
#include <QProgressBar>

TracksModel::TracksModel ( EQueryThread *queryThread, QObject* parent ) : MediaModel ( parent )
{
	myMode = Track::Standard;
	setupModel(queryThread);
}

#if QT_VERSION < 0x040500
MediaModel* TracksModel::createNewModel()
{
	TracksModel *model = new TracksModel(NULL);
// 	if(myAlbumMode)
// 		model->setMode(true);
	return model;
}
#endif

Medium *TracksModel::newMedium() const
{
	Track *track = new Track();
	track->setMode(mode());
  return track;
}

bool TracksModel::loadChilds(Medium *parent)
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() )
			return false;
	if(!MediaModel::loadChilds(parent)){
		if(closeDB)
			db.close();
		return false;
	}

	// load special stuff
	foreach(Medium *med, media){
		Track *track = qobject_cast<Track*>(med);
		if(!track){
			if(closeDB)
				db.close();
			return false;
		}
		QSqlQuery query;
		if(mode() == Track::CD || mode() == Track::LP){
			// add disc and track number
			query.exec(
				QString("SELECT * FROM %1_ondisc WHERE materialhasmaterialsId=%2")
				.arg(CDs::group())
				.arg(med->materialHasMaterialsId())
			);
			if(!query.first()){
				if(closeDB)
					db.close();
				return false;
			}

			track->setDisc(query.record().value("disc").toUInt());
			track->setSide(query.record().value("side").toString());
			track->setTrackNumber(query.record().value("number").toUInt());
		}
		else if(mode() == Track::Collection){
			// add track number
			query.exec(
				QString("SELECT * FROM %1_trackhastracks WHERE materialhasmaterialsId=%2")
				.arg(CDs::group())
				.arg(med->materialHasMaterialsId())
			);
			if(!query.first()){
				if(closeDB)
					db.close();
				return false;
			}
			track->setTrackNumber(query.record().value("number").toUInt());
		}
	}
	if(closeDB)
		db.close();
	return true;
}

void TracksModel::addItemDelegates(QAbstractItemView* view)
{
  if(!view)
    return;
  
	if(mode() != Track::Standard){
		Medium *medium = newMedium();
		if(!medium)
			return;
		
		QStringList attrs = medium->supportedAttributes();
		view->setItemDelegateForColumn(medium->attrs("owner").column, new DelegateUser(view));
		delete medium;
	}
}

Qt::ItemFlags TracksModel::flags ( const QModelIndex& index ) const
{
	if(mode() == Track::CD || mode() == Track::LP){
		switch(index.column()){
			case 0:
				return MediaModel::flags ( index ) | Qt::ItemIsEditable;
			case 1:
				return MediaModel::flags ( index ) | Qt::ItemIsEditable;
		}
		if(mode() == Track::LP && index.column() == 2)
			return MediaModel::flags ( index ) | Qt::ItemIsEditable;
	}
	else if(mode() == Track::Collection){
		switch(index.column()){
			case 0:
				return MediaModel::flags ( index ) | Qt::ItemIsEditable;
		}
	}
	return MediaModel::flags ( index );
}

void TracksModel::setMode ( Track::Mode mode )
{
	myMode = mode;

	// change headerList
	int last = headerList.size();
	Medium *med = newMedium();
	if ( med )
    headerList = med->supportedAttributesTrans();
	delete med;
	emit headerDataChanged(Qt::Horizontal, 0, last);
	
	foreach(Medium *medium, media){
		Track *track = qobject_cast<Track*>(medium);
		if(track){
			track->setMode(myMode);
		}
	}
}

Track::Mode TracksModel::mode() const
{
	return myMode;
}

void TracksModel::doAddMedium(Medium* med)
{
	Track *track = qobject_cast<Track*>(med);
	if(!track)
		return;
	track->setMode(mode());
  MediaModel::doAddMedium(med);
}
