/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "categories.h"

#include "categorymodel.h"
#include "category.h"
#include <medium.h>
#include <mediamodel.h>
#include <delegateshoppingcart.h>

#include <QDebug>

Categories::Categories(QWidget* parent, Qt::WindowFlags flags)
	: EWidget(parent, flags)
{
	setupUi(this);
	model = new CategoryModel(this);
	categoryTree->setModel(model);
	connect(categoryTree, SIGNAL(clicked(QModelIndex)), this, SLOT(categoryClicked(QModelIndex)));
	mediaModel = new MediaModel(this);
	mediumView->setModel(mediaModel);
	DelegateShoppingCart *delegate = new DelegateShoppingCart(mediaModel, this);
	delegate->setMode(MediumWidget::Extended);
	mediumView->setItemDelegate(delegate);
	
	
	// fill for test purposes
	Category *books = new Category("Buecher");
	model->addCategory(books);
	Category *roman = new Category("Roman");
	model->addCategory(roman, books);
	Category *crime = new Category("Krimi");
	model->addCategory(crime, books);
	
	Category *music = new Category("Music");
	model->addCategory(music);
	model->addCategory("Pop", music);
	
	Medium *med = new Medium();
	med->load(106);
	crime->addMedium(med);
	med = new Medium();
	med->load(11);
	crime->addMedium(med);
	med = new Medium();
	med->load(12);
	roman->addMedium(med);
}

void Categories::categoryClicked(QModelIndex index)
{
	Category *cat = model->itemFromIndex(index);
	if(!cat)
		return;
	categoryLabel->setText(cat->name());
	
	if(!mediaModel->clear())
		return;
	
	foreach(Medium *med, cat->newMedia())
		mediaModel->addMedium(med->copyMedium());
}

#include "categories.moc"
