/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "category.h"

#include <medium.h>

Category::Category(QString name, Category* parent)
	: QObject()
{
	setParent(parent);
	setName(name);
}

Category::~Category()
{
	qDeleteAll(mySubCategories);
	qDeleteAll(myNewMedia);
}

QString Category::name() const
{
	return myName;
}

void Category::setName(QString newName)
{
	myName = newName;
	emit changed(this);
}

int Category::numNewMedia() const
{
	if(haveSubCategories()){
		int num = 0;
		foreach(Category *cat, subCategories())
			num += cat->numNewMedia();
		return num;
	}
	return newMedia().size();
}

QList< Medium* > Category::newMedia() const
{
	if(haveSubCategories()){
		QList<Medium*> medias;
		foreach(Category *cat, subCategories())
			medias << cat->newMedia();
		return medias;
	}
	return myNewMedia;
}

Medium* Category::newMedium(int row) const
{
	if(haveSubCategories()){
		foreach(Category *cat, subCategories()){
			Medium *med = cat->newMedium(row);
			if(med)
				return med;
			row -= cat->newMedia().size();
		}
		return 0;
	}
	
	if((row < 0) || (newMedia().size() <= row))
		return 0;
	
	return newMedia()[row];
}

void Category::addMedium(Medium* medium)
{
	myNewMedia << medium;
	emit changed(this);
}

Category* Category::parent() const
{
	return myParent;
}

void Category::setParent(Category* newParent)
{
	myParent = newParent;
	emit changed(this);
}

bool Category::haveSubCategories() const
{
	if(subCategories().size() > 0)
		return true;
	return false;
}

QList< Category* > Category::subCategories() const
{
	return mySubCategories;
}

void Category::addSubCategory(Category* newChild)
{
	newChild->setParent(this);
	mySubCategories << newChild;
	emit changed(this);
}

bool Category::operator==(const Category& other) const
{
	if(other.name() == name())
		return true;
	return false;
}

bool Category::operator!=(const Category& other) const
{
	return !operator==(other);
}

#include "category.moc"
