/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef CATEGORY_H
#define CATEGORY_H

#include <QObject>

class Medium;

#include <QString>

/**
	* @todo sum up new medias from subcategories
	* @todo set own newmedias to zero, if there are subcategories
	*/
class Category : public QObject
{
	Q_OBJECT
	public:
		Category(QString name, Category *parent = 0);
		~Category();
		
		/**
			* the name is shown at the model
			*/
		QString name() const;
		void setName(QString newName);
		
		/**
			* @return the number of new media
			*/
		int numNewMedia() const;
		/**
			* @return a list with all new media
			*/
		QList<Medium*> newMedia() const;
		/**
			* @return the new medium at position @p row
			*/
		Medium * newMedium(int row) const;
		/**
			* adds a new @p medium to the list of new media
			*/
		void addMedium(Medium *medium);
		
		/**
			* this category is shown below the parent category
			*/
		Category *parent() const;
		void setParent(Category *newParent);
		
		/**
			* @return true, if this category have at least one subcategory
			*/
		bool haveSubCategories() const;
		/**
			* @return a list of all subcategories
			*/
		QList<Category*> subCategories() const;
		/**
			* adds @p newChild to the list of sub categories
			*/
		void addSubCategory(Category *newChild);
		
		bool operator==(const Category &other) const;
		bool operator!=(const Category &other) const;
	
	private:
		QList<Medium*> myNewMedia;
		Category *myParent;
		QList<Category*> mySubCategories;
		QString myName;
		
	signals:
		void changed(Category *iam);
};

#endif // CATEGORY_H
