/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "categorymodel.h"

#include "category.h"

CategoryModel::CategoryModel(QObject* parent)
	: QAbstractItemModel(parent)
{
	Q_UNUSED(parent);
}

CategoryModel::~CategoryModel()
{
	qDeleteAll(items);
}

void CategoryModel::addCategory(QString name, Category* parent)
{
	Category *cat = new Category(name, parent);
	addCategory(cat, parent);
}

void CategoryModel::addCategory(Category* cat, Category* parent)
{
	emit layoutAboutToBeChanged();
	if(!parent){
		items << cat;
	}
	else{
		parent->addSubCategory(cat);
	}
	emit layoutChanged();
	connect(cat, SIGNAL(changed(Category*)), this, SLOT(catChanged(Category*)));
}

QModelIndex CategoryModel::indexOfItem(Category* category) const
{
	if(!category->parent()){
		if(items.indexOf(category) == -1)
			return QModelIndex();
		
		return createIndex(items.indexOf(category), 0, category);
	}
	
	Category *parent = category->parent();
	if(parent->subCategories().indexOf(category) == -1)
		return QModelIndex();
	
	return createIndex(parent->subCategories().indexOf(category), 0, category);
}

Category* CategoryModel::itemFromIndex(QModelIndex index) const
{
	if(!index.isValid())
		return 0;
	
	return static_cast<Category*>(index.internalPointer());
}

QVariant CategoryModel::data(const QModelIndex& index, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	Category *cat = itemFromIndex(index);
	if(!cat)
		return QVariant();
	
	if(cat->numNewMedia() > 0)
		return QString("%1 (%2)").arg(cat->name()).arg(cat->numNewMedia());
	return cat->name();
}

int CategoryModel::columnCount(const QModelIndex& parent) const
{
	Q_UNUSED(parent);
	return 1;
}

int CategoryModel::rowCount(const QModelIndex& parent) const
{
	Category *parentCat = itemFromIndex(parent);
	if(!parentCat)
		return items.size();
	
	return parentCat->subCategories().size();
}

QModelIndex CategoryModel::parent(const QModelIndex& child) const
{
	Category *childCat = itemFromIndex(child);
	if(!childCat)
		return QModelIndex();
	
	if(!childCat->parent())
		return QModelIndex();
	
	return indexOfItem(childCat->parent());
}

QModelIndex CategoryModel::index(int row, int column, const QModelIndex& parent) const
{
	if(column != 0)
		return QModelIndex();
	
	Category *parentCat = itemFromIndex(parent);
	if(!parentCat){
		if((row < 0) || (items.size() <= row))
			return QModelIndex();
		return createIndex(row, 0, items[row]);
	}
	
	int rows = parentCat->subCategories().size();
	if((row < 0) || (rows <= row))
			return QModelIndex();
	return createIndex(row, 0, parentCat->subCategories()[row]);
}

QVariant CategoryModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	if((orientation == Qt::Horizontal) && (section == 0))
		return tr("Category");
		
	return QVariant();
}

void CategoryModel::catChanged(Category* category)
{
	emit dataChanged(indexOfItem(category), indexOfItem(category));
}

#include "categorymodel.moc"
