/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "checkmanufacturers.h"
#include "checkmanufacturers.moc"

#include "checkmanufacturerworker.h"

#include <QTime>
#include <QCoreApplication>

CheckManufacturers::CheckManufacturers(QObject* parent): QThread(parent)
{
	worker = NULL;
	finishedStartup = false;
	setBusy(false);
	connect(this, SIGNAL(started()), this, SLOT(setStarted()));
}

CheckManufacturers::~CheckManufacturers()
{
	if(worker)
		delete worker;
}

void CheckManufacturers::run()
{
	// create worker in new thread
	worker = new CheckManufacturerWorker();
// 	qRegisterMetaType< QList<QVariant> >("QList<QVariant>");
// 	qRegisterMetaType< QSqlRecord >("QSqlRecord");
// 	qRegisterMetaType<QVariant>("QVariant");
	connect(this, SIGNAL(startChecks(bool)), worker, SLOT(runChecks(bool)));
	connect(this, SIGNAL(startSendData()), worker, SLOT(sendData()));
	connect(this, SIGNAL(stop()), worker, SLOT(cancel()));
	connect(worker, SIGNAL(finished(bool)), this, SLOT(finishedByPass(bool)));
	
	connect(worker, SIGNAL(setStep(QString)), this, SLOT(setStepByPass(QString)));
	connect(worker, SIGNAL(setOverallRange(int)), this, SLOT(setOverallRangeByPass(int)));
	connect(worker, SIGNAL(overallIncrease()), this, SLOT(overallIncreaseByPass()));
	connect(worker, SIGNAL(setMINType(QString)), this, SLOT(setMINTypeByPass(QString)));
	connect(worker, SIGNAL(setMINTypeRange(int)), this, SLOT(setMINTypeRangeByPass(int)));
	connect(worker, SIGNAL(minTypeIncrease()), this, SLOT(minTypeIncreaseByPass()));
	connect(worker, SIGNAL(log(QString)), this, SLOT(logByPass(QString)));
	
	// starting our own event loop (in our own thread)
	exec();
}

bool CheckManufacturers::isStarted()
{
	return finishedStartup;
}

bool CheckManufacturers::waitForStart()
{
	if(isStarted())
		return true;
	
	// wait for thread to be started
	QTime dieTime = QTime::currentTime().addSecs(2);
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
		if(isStarted())
			return true;
	}
	
	if(isStarted())
		return true;
	return false;
}

bool CheckManufacturers::waitForIdle()
{
	if(!isBusy())
		return true;
	
	// wait for thread to be started
	QTime dieTime = QTime::currentTime().addSecs(2);
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
		if(!isBusy())
			return true;
	}
	
	return !isBusy();
}

bool CheckManufacturers::isBusy() const
{
	return myIsBusy;
}

void CheckManufacturers::runChecks()
{
	setBusy();
	emit startChecks(false);
}

void CheckManufacturers::sendData()
{
	setBusy();
	emit startSendData();
}

void CheckManufacturers::runChecksAndSendData()
{
	setBusy();
	emit startChecks(true);
}

void CheckManufacturers::cancel()
{
	if(isStarted())
		emit stop();
}

void CheckManufacturers::setStepByPass(QString text)
{
	emit setStep(text);
}

void CheckManufacturers::setOverallRangeByPass(int max)
{
	emit setOverallRange(max);
}

void CheckManufacturers::overallIncreaseByPass()
{
	emit overallIncrease();
}

void CheckManufacturers::setMINTypeByPass(QString typeName)
{
	emit setMINType(typeName);
}

void CheckManufacturers::setMINTypeRangeByPass(int max)
{
	emit setMINTypeRange(max);
}

void CheckManufacturers::minTypeIncreaseByPass()
{
	emit minTypeIncrease();
}

void CheckManufacturers::logByPass(QString msg)
{
	emit log(msg);
}

void CheckManufacturers::finishedByPass(bool result)
{
	setBusy(false);
	emit finished(result);
}

void CheckManufacturers::setStarted()
{
	finishedStartup = true;
}

void CheckManufacturers::setBusy(bool status)
{
	myIsBusy = status;
}
