/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef CHECKMANUFACTURERS_H
#define CHECKMANUFACTURERS_H

#include <QThread>

class CheckManufacturerWorker;

/**
	* @brief The thread, which handles the worker to check manufacturer number cycles.
	* 
	* @todo should be existing circles removed, if they are not computed out of actual mins anymore?
	*    E.g. a circle for manufacturer A is computed, but afterwards, some media are deleted and that
	*    circle can not be computed anymore. In this situation, it is bad to remove the old circles.
	*    But if some wrong mins were entered and therefore wrong circle is computed, the wrong circle
	*    should be removed after correcting the mins. But how can we detect if it was wrong or not?
	*/
class CheckManufacturers : public QThread
{
	Q_OBJECT
	public:
		CheckManufacturers(QObject *parent = 0);
    ~CheckManufacturers();
		
		void run();
		bool isStarted();
		bool waitForStart();
		bool waitForIdle();
		bool isBusy() const;
		
	public slots:
		/**
			* @brief Start the checks.
			* 
			* Normally you want to start them immediatly after the thread is started,
			* therefore do something like this
			* @code
			* check = new CheckManufacturers(this);
			* connect(check, SIGNAL(started()), check, SLOT(runChecks()));
			* check->start();
			* @endcode
			*/
		void runChecks();
		void sendData();
		void runChecksAndSendData();
		void cancel();

	private slots:
		void setStepByPass(QString text);
		void setOverallRangeByPass(int max);
		void overallIncreaseByPass();
		void setMINTypeByPass(QString typeName);
		void setMINTypeRangeByPass(int max);
		void minTypeIncreaseByPass();
		void logByPass(QString msg);
		void finishedByPass(bool result);
		
		void setStarted();
		
	private:
		void setBusy(bool status = true);
		/**
			* this object run's completely at the other thread
			* therefore the communication is done through the event loop 
			* with help of signals and slots
			*/
		CheckManufacturerWorker *worker;
		bool finishedStartup, myIsBusy;
		
	signals:
		void setStep(QString text);
		void setOverallRange(int max);
		void overallIncrease();
		void setMINType(QString typeName);
		void setMINTypeRange(int max);
		void minTypeIncrease();
		void log(QString msg);
		void finished(bool result);
		
		void startChecks(bool sendData);
		void startSendData();
		void stop();
};

#endif // CHECKMANUFACTURERS_H
