/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "checkmanufacturersstatusdialog.h"
#include "checkmanufacturersstatusdialog.moc"

#include "checkmanufacturers.h"

#include <QSettings>
#include <QMessageBox>

CheckManufacturersStatusDialog::CheckManufacturersStatusDialog ( QWidget* parent, Qt::WindowFlags flags )
	: EDialog ( parent, flags )
{
	dataSend = false;
	setupUi(this);
	okButton->setEnabled(false);
	connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	setHelp("MINs Manufacturers", helpButton);
	
	check = new CheckManufacturers(this);
	// run checks as fast as possible
	connect(check, SIGNAL(started()), check, SLOT(runChecks()));
	connect(stopButton, SIGNAL(clicked(bool)), check, SLOT(cancel()));
	connect(check, SIGNAL(setOverallRange(int)), this, SLOT(setOverallRange(int)));
	connect(check, SIGNAL(overallIncrease()), this, SLOT(overallIncrease()));
	connect(check, SIGNAL(setMINType(QString)), this, SLOT(setMINType(QString)));
	connect(check, SIGNAL(setMINTypeRange(int)), this, SLOT(setMINTypeRange(int)));
	connect(check, SIGNAL(setStep(QString)), this, SLOT(setStep(QString)));
	connect(check, SIGNAL(minTypeIncrease()), this, SLOT(minTypeIncrease()));
	connect(check, SIGNAL(log(QString)), this, SLOT(log(QString)));
	connect(check, SIGNAL(finished(bool)), this, SLOT(finished(bool)));
}

CheckManufacturersStatusDialog::~CheckManufacturersStatusDialog()
{
	// stopping thread
	// cancel query first
	check->cancel();
	// stop eventloop
	check->quit();
	// wait 2000 seconds to stop the eventloop
	check->wait(2000);
	delete check;
}

int CheckManufacturersStatusDialog::exec()
{
	if(!check->isStarted()){
		setStep(tr("Starting thread"));
		check->start();
	}
	
	return EDialog::exec();
}

void CheckManufacturersStatusDialog::setOverallRange(int max)
{
	overallProgress->setRange(0, max);
	overallProgress->setValue(0);
}

void CheckManufacturersStatusDialog::overallIncrease()
{
	overallProgress->setValue(overallProgress->value() + 1);
}

void CheckManufacturersStatusDialog::setMINType(QString typeName )
{
	minTypeLabel->setText(typeName);
}

void CheckManufacturersStatusDialog::setMINTypeRange(int max)
{
	minTypeProgress->setRange(0, max);
	minTypeProgress->setValue(0);
}

void CheckManufacturersStatusDialog::setStep(QString text)
{
	stepLabel->setText(text);
	log(text);
}

void CheckManufacturersStatusDialog::minTypeIncrease()
{
	minTypeProgress->setValue(minTypeProgress->value());
}

void CheckManufacturersStatusDialog::log ( QString msg )
{
	logEdit->appendPlainText(msg);
}

void CheckManufacturersStatusDialog::finished(bool result)
{
	QString res = tr("Failed");
	if(!result){
		setStep(tr("Finished %1").arg(res));
		return;
	}
	
	if(!dataSend){
		QSettings settings;
		bool sendMINRanges = false;
		if(settings.contains("sendMINRanges")){
			sendMINRanges = settings.value("sendMINRanges").toBool();
		}
		else{
			// ask the user if he wants to enable this feature
			int result = QMessageBox::question(this, tr("Send and load MIN ranges"),
				tr("Should the calculated MIN ranges and the related manufacturer data be send"
					" to the central database at mins.engsas.de?\n"
					" If you say yes here, you also recieve those data to make adding new"
					" media faster and more easy.\n"
					" You can revoke your approval at the settings dialog at any time."
				),
				QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes
			);
			if(result == QMessageBox::Yes)
				sendMINRanges = true;
			settings.setValue("sendMINRanges", sendMINRanges);
		}
		
		if(sendMINRanges){
			check->sendData();
			dataSend = true;
			return;
		}
	}
	res = tr("Successfully");
	okButton->setEnabled(true);
}
