/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef CHECKMANUFACTURERWORKER_H
#define CHECKMANUFACTURERWORKER_H

#include <eobject.h>
#include <esqldatabase.h>

#include "manufacturerrange.h"

class ManufacturerModel;

/**
	* @brief The worker thread, which does the manufacturer number circle checks.
	* 
	* It executes also all database queries on it's own database connection and
	* therefore threaded.
	* 
	* @todo may be a manufactuers does have more than one circle?
	*/
class CheckManufacturerWorker : public EObject
{
	Q_OBJECT
	public:
		CheckManufacturerWorker(QObject *parent = 0);
		~CheckManufacturerWorker();
		
	public slots:
		/**
			* @brief Starts the checks.
			*/
		void runChecks(bool doSendData);
		/**
			* @brief Sends the calculated ranges to the database at mins.engsas.de
			*/
		void sendData();
		/**
			* @brief Aborts the check execution at runChecks().
			*/
		void cancel();
		
	private slots:
		void httpRequestFinished(ManufacturerModel* model);
		
	private:
		void prepareDB(ESqlDatabase &db);
		
		QString connectionName;
		bool abort;
		QList<ManufacturerRange> addedPrefixes;

	signals:
		void setStep(QString text);
		void setOverallRange(int max);
		void overallIncrease();
		void setMINType(QString typeName);
		void setMINTypeRange(int max);
		void minTypeIncrease();
		void log(QString msg);
		void finished(bool result);
};

#endif // CHECKMANUFACTURERWORKER_H
