/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "dbinit.h"
#include "dbinit.moc"

#include "meganizer.h"
#include <minvalidator.h>

#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>

#include <QApplication>

DBInit::DBInit ( QObject *parent,
								 QString username, QString host,
								 QString database, QString driver,
								 QString password )
	: ESqlInit ( parent, username, host, database, driver, password )
{
	setObjectName("DBinit");
}

void DBInit::scheme()
{
	setup(Meganizer::group(), "Meganizer", 14);

	// users table
	ESqlTable *users = addTable ( "users", true );
	ESqlField *field = users->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 45 );
	users->addIndex ( field );
	field = users->appendField ( "givenName", ESqlField::Varchar );
	field->setLength ( 45 );
	field = users->appendField ( "userName", ESqlField::Varchar, true );
	field->setLength ( 45 );
	users->addIndex ( field );
	field = users->appendField ( "email", ESqlField::Varchar );
	field->setLength ( 120 );

	// material type
	ESqlTable *mattypes = addTable ( "materialtypes", true );
	field = mattypes->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 45 );
	mattypes->addIndex ( field, 20 );

	// manufacturer table
	ESqlTable *manuTable = addTable ( "manufacturers", true );
	field = manuTable->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 120 );
	manuTable->addIndex ( field, 20 );
	field = manuTable->appendField ( "street", ESqlField::Varchar );
	field->setLength ( 50 );
	field = manuTable->appendField ( "number", ESqlField::Varchar );
	field->setLength ( 50 );
	field = manuTable->appendField ( "city", ESqlField::Varchar );
	field->setLength ( 50 );
	field = manuTable->appendField ( "postalCode", ESqlField::Integer );
	field->setIsUnsigned ( true );
	field = manuTable->appendField ( "postOfficeBox", ESqlField::Integer );
	field->setIsUnsigned( true );
	field = manuTable->appendField ( "country", ESqlField::Varchar );
	field->setLength ( 50 );
	field = manuTable->appendField ( "website", ESqlField::Varchar );
	field->setLength ( 120 );
	field = manuTable->appendField ( "email", ESqlField::Varchar );
	field->setLength ( 120 );
	field = manuTable->appendField ( "phone", ESqlField::Varchar );
	field->setLength ( 50 );
	field = manuTable->appendField ( "fax", ESqlField::Varchar );
	field->setLength ( 50 );

	// currencies table
	ESqlTable *currencies = addTable ( "currencies", true );
	field = currencies->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 120 );
	field = currencies->appendField ( "shortName", ESqlField::Varchar, true );
	field->setLength ( 5 );
	currencies->addIndex ( field );
	field = currencies->appendField ( "toReferenceCurrency", ESqlField::Double );
	field->setIsUnsigned ( true );
	field = currencies->appendField ( "isReference", ESqlField::Boolean );

	QStringList curDef;
	curDef << "" << "Euro" << "EUR" << "1" << "1";
	currencies->appendValues ( curDef );

	ESqlTable *genres = addTable ( "genres", true );
	field = genres->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 50 );
	genres->addIndex ( field, 10 );
	field = genres->appendField ( "description", ESqlField::Varchar );
	field->setLength ( 500 );

	// material metadata table
	ESqlTable *materials = addTable ( "materials", true );
	materials->addForeignKey ( mattypes );
	materials->addForeignKey ( manuTable );
	field = materials->appendField ( "genresId", ESqlField::Integer );
	field->setIsUnsigned ( true );
	field = materials->appendField ( "title", ESqlField::Varchar, true );
	field->setLength ( 120 );
	materials->addIndex ( field, 20 );
	field = materials->appendField ( "listPrice", ESqlField::Double );
	field->setIsUnsigned ( true );
	materials->addForeignKey ( currencies, "listPriceCurrenciesId" );
	field = materials->appendField ( "description", ESqlField::Text);
	field = materials->appendField("image", ESqlField::Blob);

	// user owns material
	ESqlTable *userownmats = addTable ( "userownmaterials", true );
	userownmats->addForeignKey ( materials );
	userownmats->addForeignKey ( users );
	field = userownmats->appendField ( "boughtPrice", ESqlField::Double, true );
	field->setIsUnsigned ( true );
	field->setDefaultValue(0);
	userownmats->addForeignKey ( currencies, "boughtPriceCurrenciesId" );
	field = userownmats->appendField ( "evaluation", ESqlField::Integer );
	field->setIsUnsigned ( true );
	field = userownmats->appendField ( "wish", ESqlField::Boolean, true );
	field->setDefaultValue(0);
	field = userownmats->appendField ( "giftOnly", ESqlField::Boolean, true );
	field->setDefaultValue(0);
	field = userownmats->appendField ( "literatureOnly", ESqlField::Boolean, true );
	field->setDefaultValue(0);

	// MIN stuff
	ESqlTable *minTypes = addTable("mintypes", true);
	field = minTypes->appendField ( "name", ESqlField::Varchar, true );
	field->setLength(200);
	minTypes->addIndex(field, 5);
	field = minTypes->appendField ( "description", ESqlField::Text );
	
	MINValidator *minValidator = new MINValidator(this);
	QStringList types = minValidator->supportedTypes();
	foreach(QString type, types){
		QStringList defType;
		defType << "" << type << minValidator->descriptionToType(type);
		minTypes->appendValues ( defType );
	}
	delete minValidator;
	
	ESqlTable *mathasmins = addTable("materialhasmins", true);
	mathasmins->addForeignKey(materials);
	mathasmins->addForeignKey(minTypes);
	field = mathasmins->appendField("min", ESqlField::Varchar, true);
	field->setLength(20);
	mathasmins->addIndex(field, 13);
	
	// stuff to detect manufacturer from min
	ESqlTable *manuhasmins = addTable ( "manufacturersmins", true );
	field = manuhasmins->appendField ( "manufacturersId", ESqlField::Integer, true );
	field->setIsUnsigned(true);
	manuhasmins->addForeignKey(field, manuTable);
	manuhasmins->addForeignKey(minTypes);
	field = manuhasmins->appendField ( "characteristic", ESqlField::Varchar, true );
	field->setLength(10);
	
  // loan stuff
	ESqlTable *loanstati = addTable ( "loanstati", true );
	field = loanstati->appendField( "name", ESqlField::Varchar, true );
	field->setLength(50);
	
	QStringList insert;
	insert << "" << "Loan Requested";
	loanstati->appendValues ( insert );
	insert.clear();
	insert << "" << "Loan Denied";
	loanstati->appendValues ( insert );
	insert.clear();
	insert << "" << "Loaned";
	loanstati->appendValues ( insert );
	insert.clear();
	insert << "" << "Return Requested";
	loanstati->appendValues ( insert );
	insert.clear();
	insert << "" << "Returned";
	loanstati->appendValues ( insert );
	insert.clear();
	insert << "" << "Request Canceled";
	loanstati->appendValues ( insert );

	ESqlTable *loanedmats = addTable ( "loanedmaterials", true ) ;
	loanedmats->addForeignKey ( materials );
	loanedmats->addForeignKey ( users, "ownerId" );
	loanedmats->addForeignKey ( users, "recipientId" );
	loanedmats->addForeignKey ( loanstati, "status" );
	field = loanedmats->appendField ( "loanedUntil", ESqlField::Timestamp, true);
	field->setDefaultValue(0); // to prevent this column from being auto updated
	field = loanedmats->appendField( "lastChange", ESqlField::Timestamp, true);
	field->setDefaultValue("CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP");

	// material persons are persons with a relation to a material
	// for example an author, a producer, an actor or artist
	ESqlTable *persons = addTable ( "materialpersons", true );
	field = persons->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 120 );
	persons->addIndex ( field, 20 );
	field = persons->appendField ( "givenName", ESqlField::Varchar );
	field->setLength ( 120 );

	// materialPersonRoles
	// roles of materialPersons such like is an actor, is the producer, ...
	ESqlTable *personsroles = addTable ( "materialpersonsroles", true );
	field = personsroles->appendField ( "name", ESqlField::Varchar, true );
	field->setLength ( 50 );
	personsroles->addIndex ( field, 10 );
	field = personsroles->appendField ( "description", ESqlField::Varchar );
	field->setLength ( 500 );

	// combining material, persons and roles
	ESqlTable *mathasper = addTable ( "materialhaspersons", true );
	mathasper->addForeignKey ( materials );
	mathasper->addForeignKey ( persons, "materialPersonsId" );
	mathasper->addForeignKey ( personsroles, "materialPersonsRolesId" );

	// material has materials a kind of "Stückliste"
	// e.g. can be used to add material track to a material album
	// 			and this way a CD with track list can be stored
	ESqlTable *mathasmat = addTable ( "materialhasmaterials", true );
	mathasmat->addForeignKey ( materials );
	mathasmat->addForeignKey ( materials, "hasMaterialsId" );

	ESqlTable *projects = addTable ( "projects" );
	field = projects->appendField ( "name", ESqlField::Varchar, true );
	field->setLength(120);
	projects->addForeignKey ( users );
	projects->addForeignKey ( materials, "resultId" );
	field = projects->appendField ( "description", ESqlField::Varchar, true );
	field->setLength( 500 );

	ESqlTable *projectscol = addTable ( "projectscollection", true );
	projectscol->addForeignKey ( projects );
	projectscol->addForeignKey ( materials );
	field = projectscol->appendField ( "comment", ESqlField::Varchar, true );
	field->setLength( 500 );

	ESqlTable *gifts = addTable ( "gifts", true );
	gifts->addForeignKey ( materials );
	gifts->addForeignKey ( users, "donerId" );
	gifts->addForeignKey ( users, "presenteeId" );
	field = gifts->appendField ( "finished", ESqlField::Timestamp, true);
	field->setDefaultValue(0); // to prevent this column from being auto updated

	ESqlTable *keywords = addTable ( "keywords", true );
	field = keywords->appendField( "keyword", ESqlField::Varchar, true );
	field->setLength(100);
	keywords->addIndex(field, 10);

	ESqlTable *mathaskeywords = addTable ( "materialhaskeywords", true );
	mathaskeywords->addForeignKey( materials );
	mathaskeywords->addForeignKey( keywords );
	
	ESqlTable *mathasfiles = addTable ( "materialhasfiles", true );
	mathasfiles->addForeignKey( materials );
	field = mathasfiles->appendField("file", ESqlField::Text, true);
	mathasfiles->addIndex(field, 20);

	addView( "viewmaterials",
					 "SELECT  mats.id,"
					 "  mats.materialtypesId, mattypes.name AS materialtypesName,"
					 "  mats.manufacturersId, manu.name AS manufacturersName,"
					 "  mats.genresId, genres.name AS genresName,"
					 "  mats.title,"
					 "  mats.listPrice, mats.listPriceCurrenciesId,"
					 "  mats.description, mats.image,"
					 "  user.usersId, user.boughtPrice, user.boughtPriceCurrenciesId,"
					 "  user.evaluation, user.wish, user.giftOnly, user.literatureOnly"
					 " FROM "+ materials->name() +" mats"
					 " LEFT JOIN "+ userownmats->name() +" user ON mats.id=user.materialsId"
					 " LEFT JOIN "+ mattypes->name() +" mattypes ON mats.materialtypesId=mattypes.id"
					 " LEFT JOIN "+ manuTable->name()  +" manu ON mats.manufacturersId=manu.id"
					 " LEFT JOIN "+ genres->name() +" genres ON mats.genresId=genres.id",
					 false );
	addView( "viewmaterialpersons",
					 "SELECT mats.*, matper.materialPersonsId, persons.name, persons.givenName"
					 "  FROM viewmaterials mats"
					 "  LEFT JOIN "+ mathasper->name() +" matper ON mats.id=matper.materialsId"
					 "  LEFT JOIN "+ persons->name() +" persons ON matper.materialPersonsId=persons.id"
					 "  WHERE matper.materialsId<>0",
					 false);
	addView( "viewprojectmaterials",
					 "SELECT proj.id AS rowId, proj.projectsId, proj.comment, mats.*"
					 "  FROM "+ projectscol->name() +" proj"
					 "  LEFT JOIN "+ materials->name() +" mats ON proj.materialsId=mats.id",
					 false );
	addView( "viewgiftmaterials",
					 "SELECT mats.*, gifts.donerId, gifts.presenteeId, gifts.finished"
					 "  FROM viewmaterials mats"
					 "  LEFT JOIN "+ gifts->name() +" gifts ON gifts.materialsId=mats.id AND mats.usersId=gifts.donerId",
					 false);
	addView( "viewmaterialskeywords",
					 "SELECT mats.id, mats.materialsId, mats.keywordsId, keywords.keyword"
					 "  FROM "+ mathaskeywords->name() +" mats"
					 "  LEFT JOIN "+ keywords->name() +" keywords ON mats.keywordsId=keywords.id",
					 false);

	// register update routines
	addUpdateRoutine(4, &DBInit::updateFrom3To4);
	addUpdateRoutine(6, &DBInit::updateFrom5To6);
	addUpdateRoutine(7, &DBInit::updateFrom6To7);
	addUpdateRoutine(8, &DBInit::updateFrom7To8);
	addUpdateRoutine(10, &DBInit::updateFrom9To10);
	addUpdateRoutine(11, &DBInit::updateFrom10To11);
}

bool DBInit::updateFrom3To4 (ESqlInit *caller, QList< ESqlTable* > tables, QString& errorString )
{
	Q_UNUSED(caller);
	Q_UNUSED(tables);
	// 1. delete useless table userhasmaterials
	QSqlQuery query;
	if(!query.exec("DROP TABLE userhasmaterials")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	return true;
}

bool DBInit::updateFrom5To6 (ESqlInit *caller, QList<ESqlTable*> tables, QString &errorString )
{
	Q_UNUSED(caller);
	Q_UNUSED(tables);
	QSqlQuery query;
	if(!query.exec("UPDATE userownmaterials SET giftOnly=0, literatureOnly=0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	
	if(!query.exec("UPDATE userownmaterials SET wish=0 WHERE wish<>1")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	
	// alter table fields
	if(!query.exec("ALTER TABLE userownmaterials ALTER wish SET DEFAULT 0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	if(!query.exec("ALTER TABLE userownmaterials ALTER giftOnly SET DEFAULT 0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	if(!query.exec("ALTER TABLE userownmaterials ALTER literatureOnly SET DEFAULT 0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	
	return true;
}

bool DBInit::updateFrom6To7(ESqlInit *caller, QList< ESqlTable* > tables, QString& errorString)
{
	Q_UNUSED(caller);
	Q_UNUSED(tables);
	// alter projectscollection table
	QSqlQuery query;
	if(!query.exec("ALTER TABLE projectscollection CHANGE COLUMN comment comment varchar(500) NULL")){
		errorString.append( query.lastError().text() + "\n" + query.lastQuery() );
		return false;
	}
	
	// move already present eans to new materialhasmins table
	MINValidator *validator = new MINValidator();
	if(!validator->isSupported("EAN")){
		errorString.append(tr("Could not find EAN-Validator!\n"
													"Loaded Validators: %1")
												.arg(validator->supportedTypes().join(", "))
											);
		delete validator;
		return false;
	}
	delete validator;
	
	query.prepare("SELECT id FROM mintypes WHERE name=:name");
	query.bindValue(":name", "EAN");
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int eanId = query.record().value("id").toUInt();
	if(eanId <= 0){
		errorString.append(tr("Could not find id of MIN EAN\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	
	query.exec("SELECT id, ean FROM materials WHERE ean IS NOT NULL AND ean<>''");
	QSqlQuery insertQuery;
	while(query.next()){
		insertQuery.prepare("INSERT INTO materialhasmins (materialsId, mintypesId, min) VALUES(:materialsId, :mintypesId, :min)");
		insertQuery.bindValue(":materialsId", query.record().value("id").toUInt());
		insertQuery.bindValue(":mintypesId", eanId);
		insertQuery.bindValue(":min", query.record().value("ean").toString());
		if(!insertQuery.exec()){
			errorString.append(tr("Could not move EAN's to new table!\n%1\n%2")
														.arg(query.lastError().text()).arg(query.lastQuery()));
			return false;
		}
	}
	
	// alter materials table
	if(!query.exec("ALTER TABLE materials DROP COLUMN ean")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	
	return true;
}

bool DBInit::updateFrom7To8(ESqlInit *caller, QList<ESqlTable*> tables, QString& errorString)
{
	Q_UNUSED(caller);
	QSqlQuery query;
	if(!query.exec("ALTER TABLE loanstati DROP COLUMN action")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	if(!query.exec("ALTER TABLE loanstati DROP COLUMN description")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	
	foreach(ESqlTable *table, tables){
		if(table->name() == "loanstati"){
			// first, delete all not needed stati
			query.prepare("DELETE FROM loanstati WHERE id > :id");
			query.bindValue(":id", table->values().size());
			if(!query.exec()){
				errorString.append( query.lastError().text() +"\n"+ query.lastQuery() );
				return false;
			}
			// rename the stati
			if(table->values().size() != 6)
				return false;
			
			for(int  i = 0; i < table->values().size(); i++){
				query.prepare("UPDATE loanstati SET name=:name WHERE id=:id");
				query.bindValue(":name", table->values()[i][1]);
				query.bindValue(":id", i + 1);
				if(!query.exec()){
					errorString.append( query.lastError().text() +"\n"+ query.lastQuery() );
					return false;
				}
			}
			return true;
		}
	}
	return false;
}

bool DBInit::updateFrom9To10(ESqlInit *caller, QList< ESqlTable* > tables, QString& errorString)
{
	Q_UNUSED(caller);
	Q_UNUSED(tables);
	QSqlQuery query;
	if(!query.exec("ALTER TABLE materialtypes DROP COLUMN description")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	return true;
}

bool DBInit::updateFrom10To11(ESqlInit *caller, QList< ESqlTable* > tables, QString& errorString)
{
	QSqlQuery query;

	// drop all doubled indexes
	caller->addInformation(" "+ tr("Cleaning up doubled indexes"));
	QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	ESqlDatabase db = ESqlDatabase::database();
	QStringList tableList = db.tables(QSql::Tables);
	if(tableList.size() < 1){
		errorString.append(tr("Could not fetch list of existing tables."));
		return false;
	}
	QRegExp key("[\\s]*KEY `([^`]+)` \\(`([^`]+)`");
	foreach(QString table, tableList){
		query.exec(QString("SHOW CREATE TABLE %1").arg(table));
		if(!query.first()){
			errorString.append(tr("Could not get create table statement of table %1").arg(table));
			return false;
		}
		QString result = query.record().value("Create Table").toString();
		int pos = 0;
		// index column to index name map
		QMultiMap<QString, QString> indexes;
		while ((pos = key.indexIn(result, pos)) != -1) {
			indexes.insert(key.cap(2), key.cap(1));
			pos += key.matchedLength();
		}
		QStringList removedKeys;
		foreach(QString index, indexes.uniqueKeys()){
			QStringList indexesForColumn = indexes.values(index);
			if(indexesForColumn.size() > 1){
				// there is more than one index for this column
				QStringList doubles = indexesForColumn.filter(QRegExp("[0-9]+$"));
				// if an foreign key index exists, the index withut a number can
				// be removed also, therefore we remove all foreign key indexes
				// from the list (can only be one per index column) and delete
				// all other indexes
				bool fkFound = false;
				foreach(QString index, indexesForColumn){
					if(index.left(3) == "fk_"){
						indexesForColumn.removeAll(index);
						fkFound = true;
						break;
					}
				}
				if(fkFound)
					doubles = indexesForColumn;
				foreach(QString doubledIndex, doubles){
					if(!query.exec(QString("DROP INDEX %1 ON %2").arg(doubledIndex).arg(table))){
						errorString.append(tr("Could not remove doubled index %1 from %2").arg(doubledIndex).arg(table));
						return false;
					}
				}
				removedKeys << doubles;
			}
		}
		caller->addInformation("   "+ tr("Processed %1 (Old: %2, New: %3)").arg(table).arg(indexes.values().count()).arg(indexes.values().count() - removedKeys.count()));
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	}

	// rename own tables
	caller->addInformation(" "+ tr("Rename old tables"));
	QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	QStringList existingTables = db.tables(QSql::Tables);
	foreach(ESqlTable *table, tables){
		if(table->name() == "dbmoduleversions")
			continue;
		query.exec("SET foreign_key_checks = 0");
		// check whether old table exists (may be old tables does not exists
		// because of different database versions)
		QString oldName = table->name().right(table->name().length() - table->name().indexOf("_") - 1);
		if(!existingTables.contains(oldName)){
			caller->addInformation("   "+ tr("Skipped new table %1").arg(table->name()));
		}
		else{
			// first delete newly added empty one
			if(!query.exec(QString("DROP TABLE %1").arg(table->name()))){
				errorString.append(tr("Could not drop new empty table %1").arg(table->name()));
				query.exec("SET foreign_key_checks = 1");
				return false;
			}
			// than rename the old one
			if(!query.exec(QString("RENAME TABLE %1 TO %2").arg(oldName).arg(table->name()))){
				errorString.append(tr("Could not rename the old table %1 to %2\n%3").arg(oldName).arg(table->name()).arg(query.lastError().text()));
				return false;
			}
			query.exec("SET foreign_key_checks = 1");

			// check foreingkeys
			// getting the actual SQL statement to create the table
			if(!query.exec(QString("SHOW CREATE TABLE %1").arg(table->name()))){
				errorString.append ( query.lastError().text() +"\n"+ query.lastQuery() );
				return false;
			}
			if(!query.first()){
				errorString.append ( query.lastError().text() +"\n"+ query.lastQuery() );
				return false;
			}
			// extracting foreignkey statements
			QStringList createTableStatement = query.record().value("Create Table").toString().split("\n");
			QStringList foreignKeyList;
			// map indexed columns to index names
			QMultiMap<QString, QString> indexes;
			QRegExp key("[\\s]*KEY `([^`]+)` \\(`([^`]+)`");
			foreach(QString statement, createTableStatement){
				if(statement.contains("CONSTRAINT")){
					statement = statement.trimmed();
					if(statement.right(1) == ",")
						statement = statement.left(statement.length() - 1);
					if(!statement.isEmpty())
						foreignKeyList.append(statement);
				}
				else if(key.indexIn(statement) > -1){
					indexes.insert(key.cap(2), key.cap(1));
				}
			}
			// comparing foreignkeys in database and local stored
			QStringList localKeys = table->sqlConstraintsString().split(",\n");
			for(int i = 0; i< localKeys.size(); i++){
				localKeys[i] = localKeys[i].trimmed();
				localKeys[i] = localKeys[i].replace ( QRegExp ( "(%db%)" ), caller->databaseName() );
				if(localKeys[i].isEmpty())
					localKeys.removeAt(i);
			}
			// check wether foreignkeys should be dropped
			foreach(QString foreignKey, foreignKeyList){
				if(!localKeys.contains(foreignKey)){
					// key is in database, but shouldn't be, remove it
					QRegExp symbol("CONSTRAINT `([\\w]+)` FOREIGN KEY");
					if(symbol.indexIn(foreignKey) == -1){
						errorString.append ( tr("Could not find foreign key symbol to drop it from database\n%1").arg(foreignKey) );
						return false;
					}
					QString fkSymbol = symbol.cap(1);
					if(fkSymbol.isEmpty()){
						errorString.append ( tr("Could not find foreign key symbol to drop it from database\n%1").arg(foreignKey) );
						return false;
					}

					if(!query.exec(QString("ALTER TABLE %1 DROP FOREIGN KEY %2").arg(table->name()).arg(fkSymbol))){
						errorString.append ( query.lastError().text() + "\n" + query.lastQuery() );
						return false;
					}
				}
			}
			// check wether foreignkeys should be added
			foreach(QString foreignKey, localKeys){
				if(!foreignKeyList.contains(foreignKey)){
					// key is not in database, but it should be
					// but before we can add the key, we must add an index manually
					QRegExp reg("CONSTRAINT[\\s]+`([^`]+)`[\\s]+FOREIGN[\\s]+KEY[\\s]+\\(`([^`]+)`\\)[\\s]+REFERENCES");
					if(reg.indexIn(foreignKey.replace("\n", "")) < 0){
						errorString.append (tr("Could not analyze missing foreign key to add index at %1")
							.arg(table->name())
						);
						return false;
					}
					QString indexName = reg.cap(1);
					QString indexColumn = reg.cap(2);
					// check wether index exists and if not
					// create it
					if(!indexes.contains(indexColumn)){
						QString indexQuery = QString("CREATE INDEX %2 ON %1 (%3)")
							.arg(table->name())
							.arg(indexName)
							.arg(indexColumn);
						if(!query.exec(indexQuery)){
							errorString.append ( tr("Could not create index to add foreignkey at %1.%2\n%3")
								.arg(table->name())
								.arg(indexName)
								.arg(query.lastError().text())
							);
							return false;
						}
					}
					query.exec("SET foreign_key_checks = 0");
					if(!query.exec(QString("ALTER TABLE %1 ADD %2").arg(table->name()).arg(foreignKey))){
						errorString.append ( query.lastError().text() + "\n" + query.lastQuery()  );
						query.exec("SET foreign_key_checks = 1");
						return false;
					}
					query.exec("SET foreign_key_checks = 1");
				}
			}
			caller->addInformation("   "+ tr("Renamed %1 to %2").arg(oldName).arg(table->name()));
		}
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	}
	return true;
}
