/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "insertmanualmedia.h"
#include "insertmanualmedia.moc"

#include <mediamodel.h>
#include <medium.h>

#include <QMessageBox>

InsertManualMedia::InsertManualMedia(QWidget* parent, Qt::WindowFlags flags)
	: EDialog(parent, flags)
{
	model = 0;
	setUnsavedOrigins(Medium::Manual);
	setupUi(this);
	connect(okButton, SIGNAL(clicked()), this, SLOT(ok()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
}
void InsertManualMedia::setUnsavedOrigins(Medium::Origins origins)
{
	this->origins = origins;
}

void InsertManualMedia::setupMedia(MediaModel* srcModel)
{
	model = srcModel->createNewModel();
	QList<Medium*> media;
	
	for(int i = 0; i < srcModel->rowCount(); i++){
		Medium *med = srcModel->mediumFromIndex(srcModel->index(i, 0));
		media.append(med);
	}
	setupMedia(media);
}

void InsertManualMedia::setupMedia(QList< Medium* > media)
{
	// copy not saved items
	if(!model)
		model = new MediaModel(0, this);
	foreach(Medium *med, media){
		if(!med)
			continue;
		if(med->id() != 0)
			continue;
		if(!origins.testFlag(med->source()))
			continue;
		
		model->addMedium(med->copyMedium());
	}
	
	model->configureView(tableView);
}

int InsertManualMedia::exec()
{
	if(model->rowCount() < 1)
		return QDialog::Accepted;
	
	return EDialog::exec();
}

void InsertManualMedia::ok()
{
	QItemSelectionModel *selModel = tableView->selectionModel();
	// selectedRows must be used, because selectedIndexes
	// returns a index for every column on a selected row
	QModelIndexList indexes = selModel->selectedRows();
	if(indexes.size() < 1){
		QMessageBox msgBox;
		msgBox.setText(tr("Don't save changed media"));
		msgBox.setInformativeText(tr("No changes will be saved!\nAre you sure?"));
		msgBox.setStandardButtons(QMessageBox::Apply | QMessageBox::Cancel);
		if(msgBox.exec() != QMessageBox::Apply)
			return;
		accept();
		return;
	}
	
	progressBar->setRange(0, indexes.size());
	foreach(QModelIndex index, indexes){
		Medium *med = model->mediumFromIndex(index);
		if(!med){
			emit error(tr("No medium found!"));
			return;
		}
		
		if(!med->store()){
			emit error(tr("Failed to store medium:\n%1").arg(med->title()));
			return;
		}
		
		model->removeMedium(med);
		
		progressBar->setValue(progressBar->value() + 1);
		QApplication::processEvents();
	}
	
	accept();
}
