/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2015 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "meganizer.h"
#include "meganizer.moc"

#include <eaction.h>
#include <emodulelistwidgetitem.h>

#include <mpluginloader.h>
#include "meganizerplugin.h"
#include "mediaplugin.h"
#include "searchplugin.h"
#include <emodulestatedialog.h>
#include "searchwidget.h"
#include "editmanufacturers.h"
#include "editgenres.h"
#include "editmaterialpersons.h"
#include "editroles.h"
#include "edituser.h"
#include "shoppingcart.h"
#include "summary.h"
#include "projects.h"
#include <edbbackupdialog.h>
#include "editmintypes.h"
#include "checkmanufacturersstatusdialog.h"
#include "eupdatenoticedialog.h"
#include "categories.h"
#include <esettingsmodelitem.h>
#include "settingsgeneral.h"
#include <edependencyitem.h>
#include <exmlparseritem.h>
#include "dbinit.h"
#include <esetupwizarddb.h>
#include "setupuser.h"
#include <ecmdlineoption.h>
#include <ecmdlineargs.h>
#include <eaboutperson.h>
#include <minvalidator.h>
#include <efileservice.h>

#include <QSettings>
#include <QMessageBox>
#include <QSqlDatabase>
#include <QDockWidget>
#include <QToolBar>
#include <QTextStream>
#include <QtPlugin>
#include <QMenu>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>

#include <stdio.h>

Meganizer::Meganizer()
	: EPart()
{
	shoppingCart = 0;
	helpCenter = 0;
	doExit = false;
	fileMenu = 0;
}

Meganizer::~Meganizer()
{
	// we must unload all widgets,
	// to remove all remaining database objects
	unloadPlugins();
	setCentralWidget(0);
	QStringList connNames = QSqlDatabase::connectionNames();
  foreach ( QString name, connNames ) {
    QSqlDatabase::removeDatabase ( name );
  }
  if(fileMenu)
		delete fileMenu;
	if(shoppingCart)
		delete shoppingCart;
}

QString Meganizer::translationDir() const
{
	return QString ( SHAREDIR ) + "/lang";
}

QString Meganizer::resourceDir() const
{
	return QString ( SHAREDIR ) + "/rcc";
}

EngSaS::CmdLineOptions Meganizer::cmdLineArgs() const
{
	EngSaS::CmdLineOptions options;
	options << new ECmdLineOption("[user] <argument>", "u", tr("username to connect to the database"));
	options << new ECmdLineOption("[password] <argument>", "p", tr("password to connect to the database"));
	options << new ECmdLineOption("[server] <argument>", "s", tr("host where the database is running on"));
	options << new ECmdLineOption("[database] <argument>", "d", tr("name of the database which should be used"));
	options << new ECmdLineOption("[driver] <argument>", "t", tr("Driver to connect to database (Should be QMYSQL)"));
	return options;
}

EPart::InitState Meganizer::init(ECmdLineArgs *args, bool standalone)
{
	if(processCommandlineArgs(args))
		return EPart::Exit;

	// setting up file menu
	QString parent = "fileMenu";
	if(!standalone){
		fileMenu = new QMenu(shortName());
		fileMenu->setObjectName("file"+ shortName());
		parent = fileMenu->objectName();
		emit addMenu(fileMenu, "fileMenu", "fileNewAction");
	}

	EAction *action = new EAction(tr("Projects"));
	action->setObjectName("fileProjectsAction");
	connect ( action, SIGNAL ( triggered()), this, SLOT(fileProjects()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	QAction *sep = new QAction(this);
	sep->setObjectName("meganizerSeperator");
	sep->setSeparator(true);
	emit addAction(sep, parent, "fileMenuSeperatorBeforeExit");

	action = new EAction(tr("Genres"));
	action->setObjectName("fileGenresAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( fileGenres()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	action = new EAction(tr("Manufacturer"), this);
	action->setObjectName("fileManufacturerAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( fileManufacturer()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	action = new EAction(tr("Material Persons"), this);
	action->setObjectName("fileMaterialPersonsAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( fileMaterialPersons()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	action = new EAction(tr("Roles"), this);
	action->setObjectName("fileRolesAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( fileRoles()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	action = new EAction("", ":icons/edituser.png", tr("User"), this, true);
	action->setObjectName("fileUserAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT(fileUser()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	action = new EAction(tr("MIN Types"), this);
	action->setObjectName("fileMinTypesAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( fileMinTypes()));
	emit addAction(action, parent, "fileMenuSeperatorBeforeExit");

	// edit menu
	QString before = "editMenuSeperatorBeforeSetupWizard";
	if(standalone){
		action = new EAction(tr("Database Backups"), this);
		action->setObjectName("editDBBackupAction");
		connect ( action, SIGNAL ( triggered() ), this, SLOT( editDBBackup()));
		emit addAction(action, "editMenu", before);
		before = action->objectName();
	}
	action = new EAction(tr("Check Manufacturer"), this);
	action->setObjectName("editCheckManufacturerAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( editCheckManufacturers()));
	emit addAction(action, "editMenu", before);

	// view menu
	QString actionName = tr("Summary");
	if(!standalone)
		QString actionName = QString("%1 %2").arg(shortName()).arg(actionName);
	action = new EAction(actionName, this);
	action->setObjectName("viewMeganizerSummary");
	connect ( action, SIGNAL ( triggered() ), this, SLOT( viewSummary()));
	emit addAction(action, "viewMenu");

	// helpMenu
	action = new EAction(tr("About Plugins"), this);
	action->setObjectName("helpAboutPluginsAction");
	connect ( action, SIGNAL ( triggered() ), this, SLOT(helpAboutPlugins()));
	emit addAction(action, "helpMenu", "helpDependenciesAction");

	loadPlugins();

#if not defined(BUILD_PACKAGE) or defined(Q_OS_WIN32)
	// check if update is available
	if(standalone){
		// show update notice hint
		// must be a pointer, because otherwise the dialog
		// is deleted, because the EUPdateNoticeDialog->replyFinished() function is called,
		// because check() is exited
		EUpdateNoticeDialog *dialog = new EUpdateNoticeDialog(QIcon(QPixmap::fromImage(icon())), website());
		connectByPass(dialog);
		dialog->check();
	}
#endif

	return EPart::Successful;
}

QString Meganizer::name() const
{
	return QString("Media Organizer");
}

QString Meganizer::shortName() const
{
	return "Meganizer";
}

QImage Meganizer::icon() const
{
	return QImage(":icons/128x128/meganizer.png");
}

QString Meganizer::description() const
{
	return tr("Meganizer organizes media collections of different types.");
}

QString Meganizer::helpDescription() const
{
  return tr("Meganizer is an epart for organizing media collections of different media types.");
}

QUrl Meganizer::website() const
{
	return QUrl(QString("%1/meganizer").arg(tr("http://en.engsas.de")));
}

QString Meganizer::versionString() const
{
	return QString ( VERSION );
}

EHelpCenterSetup Meganizer::helpSetup()
{
	EHelpCenterSetup setup;
	setup.defaultPage = "meganizer/index.html";
	setup.qhcFile = "meganizer";
	setup.directories << QString ( DOCDIR );
	return setup;
}

EDependencyItem* Meganizer::dependencies()
{
	QDir dir ( QString ( SHAREDIR ) + "/depends" );
	if(!dir.exists())
		return 0;

	// setup node for meganizer EPart
	QDomDocument doc;
	QDomNodeList node;
	QString errorStr;
	int errorLine;
	int errorColumn;
	QFile xmlFile(dir.path() +"/meganizer.xml");
	if ( !doc.setContent ( &xmlFile, true, &errorStr, &errorLine, &errorColumn ) ) {
		emit information ( tr ( "Can not convert data from %1 into an XML-DOM-Document. \n Error at line %2, Column %3: %4" )
											.arg ( xmlFile.fileName() )
											.arg ( errorLine )
											.arg ( errorColumn )
											.arg ( errorStr ) );
		return 0;
	}
	QDomElement root = doc.documentElement();

	// now adding all other files as content to the root node
	QStringList filter;
	filter.append ( "*.xml" );
	QStringList newFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
	foreach ( QString file, newFiles ) {
		if(file != "meganizer.xml"){
			QFile xmlFile ( dir.path() + "/" + file );
			// convert Data into a qdomdocument
			QDomDocument result;
			QDomNodeList node;
			QString errorStr;
			int errorLine;
			int errorColumn;

			if ( !result.setContent ( &xmlFile, true, &errorStr, &errorLine, &errorColumn ) ) {
				emit information ( tr ( "Can not convert data from %1 into an XML-DOM-Document. \n Error at line %2, Column %3: %4" )
													.arg ( xmlFile.fileName() )
													.arg ( errorLine )
													.arg ( errorColumn )
													.arg ( errorStr ) );
			}
			else{
				QDomElement rootElem = result.documentElement();
				if(rootElem.tagName().toUpper() != QString("dependency").toUpper())
					emit information(tr("Found wrong dependency file"));
				else
					root.appendChild (doc.importNode(rootElem, true));
			}
		}
	}

	// convert into an EDependencyItem
	QMap<QString, EXmlParserItem*> parser;
	EDependencyItem *item = new EDependencyItem();
	parser.insert(item->tagName().toLower(), new EXmlParserItem(item->tagName(), &EDependencyItem::create, NULL));
	if(!item->load(doc.firstChildElement("dependency"), parser))
		emit information(tr("Failed to load all dependencies."));
	return item;
}

EngSaS::SetupWizardObjects Meganizer::setupWizardObjects()
{
	EngSaS::SetupWizardObjects result;
  // check database
	DBInit *dbInit = new DBInit(this, username, host, database, driver, password);
	ESetupWizardDB *dbWizard = new ESetupWizardDB ( tr ( "Configure and Update Database of %1" ).arg(name()),
																									dbInit, SettingsGeneral::sqlDrivers(), this );
	result << dbWizard;

	// check actual user
	SetupUser *setupUser = new SetupUser ( this );
	result << setupUser;

	// check MediaPlugins
	foreach ( MediaPlugin *plugin, mediaPlugins ) {
		ESetupWizardObject *object = plugin->wizardObject();
		if ( object )
			result << object;
  }

	// check SearchPlugins
	foreach ( SearchPlugin *plugin, searchPlugins ) {
		ESetupWizardObject *object = plugin->wizardObject();

		if ( object )
			result << object;
	}

	return result;
}

EngSaS::AboutPersons Meganizer::aboutPersons()
{
	EngSaS::AboutPersons persons;
	persons << new EAboutPerson("Henri", "Girard", EAboutPerson::Translator);
	return persons;
}

ESettingsModelItem* Meganizer::settingsModelItem()
{
	ESettingsModelItem *item = new ESettingsModelItem(name(), 0);
	item->addChild(new SettingsGeneral());

	item->addChild(EFileService::settings());

	foreach(MediaPlugin *plugin, mediaPlugins)
		item->addChild(plugin->settingsModelItem());
	foreach(SearchPlugin *plugin, searchPlugins)
		item->addChild(plugin->settingsModelItem());

	return item;
}

void Meganizer::show()
{
	shoppingCart = new ShoppingCart(searchPlugins);
	EObject::connectByPass(this, shoppingCart);
	shoppingCart->init(true);

  // display search widget
  SearchWidget *searchWidget = new SearchWidget ( );
  EObject::connectByPass(this,  searchWidget );
  searchWidget->init ( mediaPlugins, searchPlugins, shoppingCart );

	viewSummary();
}

bool Meganizer::loadPlugins()
{
  //load and test plugin
	QList<MeganizerPlugin*> plugins = MPluginLoader<MeganizerPlugin>::loadPlugins(pluginProtocol);
	// devide into Media- and SearchPlugins
	foreach(MeganizerPlugin* plugin, plugins)
		if(!loadPlugin(plugin))
			return false;

  return true;
}

bool Meganizer::loadPlugin ( EModuleListWidgetItem* plugin, bool emitSetupWizard )
{
	if(!plugin)
		return false;
	EPluginLoader<MeganizerPlugin> eLoader;
  return loadPlugin ( eLoader.doLoadPlugin(plugin->file()), emitSetupWizard );
}

bool Meganizer::loadPlugin ( MeganizerPlugin *plugin, bool emitSetupWizard )
{
	if(!plugin)
		return false;
	SearchPlugin *searchPlugin = qobject_cast<SearchPlugin *> ( plugin );
	if(searchPlugin){
		searchPlugins << searchPlugin;
	}
	else{
		MediaPlugin *mediaPlugin = qobject_cast<MediaPlugin *> ( plugin );
		if(mediaPlugin)
			mediaPlugins << mediaPlugin;
		else
			return false;
	}
	connectByPass(plugin);
  if(!plugin->initiate()){
		foreach(EModuleListWidgetItem *item, pluginProtocol){
			if(item->name() == plugin->name()){
				item->setStatus ( false );
				item->setErrorMessage ( tr ( "Cannot initiate module\n %1" ).arg ( item->file() ) );
				break;
			}
		}
		information ( tr("Could not load %1! Details at Help->Plugins").arg(plugin->name()) );
		return false;
	}

	if ( emitSetupWizard )
		emit showSetupWizard();
	return true;
}

bool Meganizer::unloadPlugin ( EModuleListWidgetItem *item )
{
  // try media plugins
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
    if ( item->name() == plugin->name() ) {
      return unloadPlugin ( plugin );
    }
  }

  foreach ( SearchPlugin *plugin, searchPlugins ) {
    if ( item->name() == plugin->name() ) {
      return unloadPlugin ( plugin );
    }
  }
  return false;
}

bool Meganizer::unloadPlugin ( QObject *plugin )
{
  MediaPlugin *object = qobject_cast<MediaPlugin*> ( plugin );

  if ( object ) {
    object->unload();
    mediaPlugins.removeAll ( object );
    foreach ( EModuleListWidgetItem *item, pluginProtocol ) {
      if ( item->name() == object->name() ) {
        pluginProtocol.removeAll ( item );
        delete item;
        break;
      }
    }

    delete object;
    return true;
  }

  SearchPlugin *search = qobject_cast<SearchPlugin*> ( plugin );

  if ( search ) {
    search->unload();
    searchPlugins.removeAll ( search );
    foreach ( EModuleListWidgetItem *item, pluginProtocol ) {
      if ( item->name() == search->name() ) {
        pluginProtocol.removeAll ( item );
        delete item;
        break;
      }
    }

    delete search;
    return true;
  }

  return false;
}

bool Meganizer::unloadPlugins()
{
  bool result = true;
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
    if ( !unloadPlugin ( plugin ) )
      result = false;
  }

  foreach ( SearchPlugin *plugin, searchPlugins ) {
    if ( !unloadPlugin ( plugin ) )
      result = false;
  }

  return result;
}

bool Meganizer::reloadPlugins()
{
  if ( !unloadPlugins() )
    return false;

  if ( !loadPlugins() )
    return false;

  return true;
}

void Meganizer::fileNewMedia()
{
	Categories *cats = new Categories();
	connectByPass(cats);
	setCentralWidget(cats);
}

void Meganizer::fileProjects()
{
	Projects *projects = new Projects();
	EObject::connectByPass(this, projects);
	setCentralWidget(projects);
}

void Meganizer::fileGenres()
{
	EditGenres editGenre;
	connectByPass(&editGenre);
	editGenre.setEditMode();
	editGenre.exec();
}

void Meganizer::fileManufacturer()
{
	EditManufacturers editManu;
	editManu.setEditMode();
	connectByPass(&editManu);
	editManu.exec();
}

void Meganizer::fileMaterialPersons()
{
	EditMaterialPersons editMatPer;
	connectByPass(&editMatPer);
	editMatPer.setEditMode();
	editMatPer.exec();
}

void Meganizer::fileRoles()
{
	EditRoles editRoles;
	connectByPass(&editRoles);
	editRoles.setEditMode();
	editRoles.exec();
}

void Meganizer::fileUser()
{
	EditUser editUser;
	connectByPass(&editUser);
	editUser.setEditMode();
	editUser.exec();
}

void Meganizer::fileMinTypes()
{
	EditMinTypes editMinType;
	connectByPass (&editMinType);
	editMinType.setEditMode();
	editMinType.exec();
}

void Meganizer::editSetupWizard()
{
	emit showSetupWizard();
}

void Meganizer::editDBBackup()
{
	EDBBackupDialog dia;
	connectByPass(&dia);
	dia.exec();
}

void Meganizer::editCheckManufacturers()
{
	CheckManufacturersStatusDialog dialog;
	connectByPass(&dialog);
	dialog.exec();
}

void Meganizer::viewSummary()
{
	Summary *summary = new Summary(mediaPlugins);
	connectByPass(summary);
	setCentralWidget(summary);
}

void Meganizer::helpAboutPlugins()
{
  EModuleStateDialog moduleState;
  connectByPass(&moduleState );
	moduleState.addModuleState(pluginProtocol, tr("Media- and Searchplugins"), EModuleStateDialog::Load | EModuleStateDialog::Unload);
	MINValidator validator;
	moduleState.addModuleState(validator.loadProtocol(), tr("MIN Validators"));
  moduleState.exec();
}


void Meganizer::addActionByPass(QAction *newAction, const QString &parent, QString before)
{
	QString par = parent;
	// redirect it into our own file menu,
	// if we have one (only in not standalone mode)
	if(fileMenu && parent == "fileMenu")
		par = fileMenu->objectName();
	emit addAction(newAction, par, before);
}

bool Meganizer::processCommandlineArgs(ECmdLineArgs *args)
{
	if(!args)
		return false;

	if(args->isSet("user") && args->getOption("user").isEmpty())
		return false;
	username = args->getOption("user");
	if(args->isSet("password") && args->getOption("password").isEmpty())
		return false;
	password = args->getOption("password");
	if(args->isSet("server") && args->getOption("server").isEmpty())
		return false;
	host = args->getOption("server");
	if(args->isSet("database") && args->getOption("database").isEmpty())
		return false;
	database = args->getOption("database");
	if(args->isSet("driver") && args->getOption("driver").isEmpty())
		return false;
	driver = args->getOption("driver");

	return false;
}

Q_EXPORT_PLUGIN2 ( EngSaS_MeganizerPart, Meganizer )
