/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/**
	* @mainpage Meganizer - Media Organizer
	*
	* @section sec1 About
	*
	* Meganizer - Media Organizer is a tool to manage existing media collections. It is open to integrate
	* new media types with help of  plugins
	*
	* @subsection sec11 Project Structure
	*
	* The project is structered in a shared library called libmeganizer, which provide basic classes,
	* which are used by the main application meganizer and the MediaPlugins and SearchPlugins.
	* The application and the library itself contains only stuff, which is needed by all types of medias.
	* The application contains the main window and provides some gui stuff.
	* Each specific medium type is contributed withhelp of an MediaPlugin.
	* This plugins provide all needed stuff to handle the specific mediatype.
	* SearchPlugin 's provides the possibilitie to search media meta data and may be allows to buy them too.
	*
	* <ul>
	*  <li>Root</li>
	*  <ul>
	*    <li>doc (Contains all Documentation stuff)</li>
	*    <ul>
	*      <li>dev (Documentation related to delevopment e.g. Database model, ...)</li>
	*      <li>app (Enduser documentation)</li>
	*      <ul>
	*        <li>man (Manpages)</li>
	*        <li>manual (Usermanual in Latex, HTML, PDF and Online usage in Application)</li>
	*      </ul>
	*      <li>src (Doxygen source documentation)</li>
	*    </ul>
	*    <li>src (Contains Source files)</li>
	*    <ul>
	*      <li>@ref libmeganizer </li>
	*      <li>@ref libmeganizertest </li>
	*      <li>Meganizer</li>
	*      <li>MediaPlugin 's</li>
	*      <ul>
	*        <li>Books</li>
	*        <li>Music (CD's, Tracks, LP's, ..., entry: CDs) </li>
	*        <li>Movies</li>
	*        <li>Magazines (Magazines and Articles)</li>
	*      </ul>
	*      <li>SearchPlugin 's</li>
	*      <ul>
	*        <li>Amazon</li>
	*        <li>DemoShop (Testing purposes)</li>
	*      </ul>
	*    </ul>
	*  </ul>
	* </ul>
	*
	* @section sec2 Developing
	*
	* @subsection sec21 Prepare Environment
	*
	* Developing can be done in two ways. The first is the developing within this project and the second is the usage
	* of @ref libmeganizer and @ref libmeganizertest to extend Meganizer from outside of this project.
	* In both cases, the dependencies from this project must be there. They can be found at the dependencies.xml file
	* in the components directories. But basically those are needed:
	* @li Qt4 (Gui, SQL, Network, XML)
	* @li LibEngsas
	* @li QCA2 (for Amazon plugin)
	* @li ZBar-Qt (to recognize barcodes with webcams, In development right now)
	*
	* Further on, in both cases it is useful to install the MeganizerDesignerPlugin of @ref libmeganizer. It allows
	* the usage of all GUI elements of @ref libmeganizer within the QtDesigner. To do so, configure this project
	* with BUILD_DEV=ON and call sudo make install afterwards.
	*
	* To develop outside of this project, the header files of @ref libmeganizer and @ref libmeganizertest are also
	* needed or useful. Both are installed if BUILD_DEV is set to ON.
	*
	* @subsection sec22 Contributing Work
	*
	* If you have done some changes and you think, they may be need by other people,
	* please create a patch and send us a mail to meganizer@engsas.de, send it to the mailing list or
	* create a ticket and append the patch.
	*
  * @author Joachim Langenbach
	*/

#ifndef MEGANIZER_H
#define MEGANIZER_H

#include <epart.h>

class MediaPlugin;
class SearchPlugin;
class MeganizerPlugin;
class EModuleListWidgetItem;
class ShoppingCart;
class HelpCenter;

#include <QPointer>

class QDockWidget;
class QMenu;

/**
	* @brief The Meganizer EPart.
	*
	* This class is the root object of the whole application and can be used like any other EPart.
	*/
class Meganizer : public EPart
{
	Q_OBJECT
	Q_INTERFACES ( EPart )
	Q_PROPERTY(QString name READ shortName)
	Q_PROPERTY(QString description READ description)
	Q_PROPERTY(QImage icon READ icon)
	EPART_LIBENGSAS_VERSION
  public:
    Meganizer();
    ~Meganizer();
		
		static QString group(){ return "meganizer"; };

		QString translationDir() const;
		QString resourceDir() const;
		EngSaS::CmdLineOptions cmdLineArgs() const;
		InitState init(ECmdLineArgs *args, bool standalone = false);
		QString name() const;
		QString shortName() const;
		QImage icon() const;
    QString helpDescription() const;
		QString description() const;
		QUrl website() const;
		QString versionString() const;
		EHelpCenterSetup helpSetup();
		EDependencyItem *dependencies();
		EngSaS::SetupWizardObjects setupWizardObjects();
		EngSaS::AboutPersons aboutPersons();
		ESettingsModelItem *settingsModelItem();

	public slots:
		void show();


  private slots:
    bool loadPlugins();
    bool loadPlugin ( EModuleListWidgetItem* plugin, bool emitSetupWizard = false );
    bool loadPlugin ( MeganizerPlugin *plugin, bool emitSetupWizard = false );
    bool unloadPlugin ( EModuleListWidgetItem* plugin );
    bool unloadPlugin ( QObject *plugin );
    bool unloadPlugins();
    bool reloadPlugins();

		void fileNewMedia();
		void fileProjects();
		void fileGenres();
		void fileManufacturer();
		void fileMaterialPersons();
		void fileRoles();
		void fileUser();
		void fileMinTypes();
		void editSetupWizard();
		void editDBBackup();
		void editCheckManufacturers();
		void viewSummary();
    void helpAboutPlugins();

		inline void errorByPass(const QString &msg){ emit error(msg); };
		inline void warningByPass(const QString &msg){ emit warning(msg); };
		inline void informationByPass(const QString &msg){ emit information(msg); };
		inline void addMenuByPass(QMenu* newMenu, const QString &parent, QString before = QString()){ emit addMenu(newMenu, parent, before); };
		void addActionByPass(QAction* newAction, const QString &parent, QString before = QString());
		inline void addToolBarByPass(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea){ emit addToolBar(newToolBar, area); };
		inline void addNewDockWidgetByPass(QDockWidget *widget, const Qt::DockWidgetArea &area){ emit addNewDockWidget(widget, area); };
		inline void setCentralWidgetByPass(QWidget *centralWidget){ emit setCentralWidget(centralWidget); };
		inline void showSettingsByPass(){ emit showSettings(); };
		inline void showHelpByPass(const QString &page){ emit showHelp(page); };
		inline void showStatusWidgetByPass(QWidget* widget){ emit showStatusWidget(widget); };
		inline void reloadSettingsRequestByPass(){ emit reloadSettingsRequest(); };

  private:
    /**
			* @returns true, if the application can be exited, otherwise it returns false
			*/
		bool processCommandlineArgs(ECmdLineArgs* args);

		QList<MediaPlugin*> mediaPlugins;
    QList<SearchPlugin*> searchPlugins;
    EngSaS::ModuleListWidgetItems pluginProtocol;
		QList<QPointer<QDockWidget> > dockWidgets;
		ShoppingCart *shoppingCart;
		HelpCenter *helpCenter;
		QString username, password, host, database, driver;
		bool doExit;
		QMenu *fileMenu;

	signals:
		void showSetupWizard();
};

#endif // MEGANIZER_H
