/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef PROJECT_H
#define PROJECT_H

#include <QObject>

class Medium;
class MediaModel;

#include <QMap>
#include <QSqlRecord>

class Project : public QObject
{
	Q_OBJECT
	public:
		Project(QObject *parent = 0);
		~Project();
		
		bool store(QString *errorMsg = 0);
		bool load(QSqlRecord record);
		
		void setId(unsigned int projectId);
		unsigned int id() const;
		void setName(QString projectName);
		QString name() const;
		void setUsersId(unsigned int userId);
		unsigned int usersId() const;
		void setResultMedium(unsigned int materialsId);
		void setResultMedium(QSqlRecord record);
		void setResultMedium(Medium *medium);
		Medium *resultMedium() const;
		void setDescription(QString desc);
		QString description() const;
		Medium *addMaterial(QSqlRecord record);
		void addMaterial(Medium *medium);
		bool addMaterials();
		MediaModel *materials() const;
		void setComment(Medium *medium, QString newComment);
		QString comment(Medium *medium) const;
		
		bool operator==(const Project &project) const;
		bool operator!=(const Project &project) const;
		
	private:
		unsigned int myId, myUsersId;
		QString myName, myDescription;
		Medium *myResultMedium;
		MediaModel *myMaterials;
		QMap<Medium*, QString> comments;
		
	signals:
		void error(QString msg);
    void warning(QString msg);
    void information(QString msg);
		
		void changed(Project *project);
};

#endif // PROJECT_H
