/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "projectsmodel.h"
#include "projectsmodel.moc"

#include "project.h"
#include <mediamodel.h>
#include <medium.h>
#include <esqldatabase.h>
#include "meganizer.h"

#include <QSqlQuery>
#include <QSqlError>

ProjectsModel::ProjectsModel ( QObject* parent ) : QAbstractItemModel ( parent )
{
	Q_UNUSED(parent);
	myUsersId = 0;
	headerList << tr("Name");
}

ProjectsModel::~ProjectsModel()
{
	clear();
}

Project* ProjectsModel::addProject()
{
	Project *project = new Project(this);
	addProject(project);
	return project;
}

bool ProjectsModel::addProject ( QSqlRecord record )
{
	Project *project = addProject();
	return project->load(record);
}

bool ProjectsModel::addProject ( Project* project )
{
	if(!project)
		return false;
	connectProject(project);
	project->setUsersId(myUsersId);
	emit layoutAboutToBeChanged();
	projects << project;
	emit layoutChanged();
	return true;
}

bool ProjectsModel::store()
{
	return false;
}

bool ProjectsModel::load ( unsigned int userId )
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(!db.isOpen())
		return false;
	
	QSqlQuery query;
	
	// fetch userId if needed
	if(userId != 0)
		myUsersId = userId;
	if(myUsersId == 0){
		query.prepare("SELECT id FROM "+ Meganizer::group() +"_users WHERE userName=:userName");
		query.bindValue(":userName", db.userName());
		query.exec();
		if(query.first())
			myUsersId = query.record().value("id").toUInt();
		if(myUsersId == 0){
			if(closeDB)
				db.close();
			return false;
		}
	}
	
	// fetch project list
	query.prepare("SELECT * FROM "+ Meganizer::group() +"_projects WHERE usersId=:usersId");
	query.bindValue(":usersId", myUsersId);
	query.exec();
	bool result = true;
	while(query.next()){
		if(!addProject(query.record()))
			result = false;
	}
		
	if(closeDB)
		db.close();
	if(!result)
		return false;
	return true;
}

QModelIndex ProjectsModel::indexOfProject ( Project* project ) const
{
	if(projects.indexOf(project) == -1)
		return QModelIndex();
	
	return createIndex(projects.indexOf(project), 0, project);
}

Project* ProjectsModel::projectFromIndex ( QModelIndex index ) const
{
	if(!index.isValid())
		return 0;
	
	return static_cast<Project*>(index.internalPointer());
}

bool ProjectsModel::clear()
{
//    emit beginResetModel();
  emit layoutAboutToBeChanged();
  qDeleteAll ( projects );
  projects.clear();
  emit layoutChanged();
//    emit endResetModel();

  if ( projects.size() == 0 )
    return true;

  return false;
}

bool ProjectsModel::reload()
{
	if(!clear())
		return false;
	
	if(!load())
		return false;
	return true;
}

void ProjectsModel::addItemDelegates ( QAbstractItemView* view )
{
	if(!view)
		return;
}

void ProjectsModel::hideAndResizeColumns ( QTableView* view )
{
	if(!view)
		return;
}

QVariant ProjectsModel::data ( const QModelIndex& index, int role ) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	Project *project = projectFromIndex(index);
	if(!project)
		return QVariant();
	
	return project->name();
}

int ProjectsModel::columnCount ( const QModelIndex& parent ) const
{
	Q_UNUSED(parent);
	return 1;
}

int ProjectsModel::rowCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;
	
	return projects.size();
}

QModelIndex ProjectsModel::parent ( const QModelIndex& child ) const
{
	Q_UNUSED(child);
	return QModelIndex();
}

QModelIndex ProjectsModel::index ( int row, int column, const QModelIndex& parent ) const
{
	if(parent.isValid())
		return QModelIndex();
	
	if(column < 0)
		return QModelIndex();
	if(columnCount() <= column)
		return QModelIndex();
	
	if((row < 0) || (projects.size() <= row))
		return QModelIndex();
	
	return createIndex(row, column, projects[row]);
}

QVariant ProjectsModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
	if(role != Qt::DisplayRole)
		return QAbstractItemModel::headerData(section, orientation, role);
	
  if(orientation == Qt::Vertical)
		return section + 1;
	
	if((section < 0) || (headerList.size() <= section))
		return QVariant();
	
	return headerList[section];
}

void ProjectsModel::errorByPass ( QString msg )
{
	emit error(msg);
}

void ProjectsModel::warningByPass ( QString msg )
{
	emit warning(msg);
}

void ProjectsModel::informationByPass ( QString msg )
{
	emit information(msg);
}

void ProjectsModel::itemChanged ( Project* project )
{
	QModelIndex curIndex = indexOfProject(project);
	emit dataChanged ( curIndex, index ( curIndex.row(), columnCount() ) );
}

void ProjectsModel::connectProject ( Project* project )
{
	connect(project, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
	connect(project, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
	connect(project, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	connect(project, SIGNAL(changed(Project*)), this, SLOT(itemChanged(Project*)));
}
