/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef PROJECTSMODEL_H
#define PROJECTSMODEL_H

#include <QAbstractItemModel>

class Project;

#include <QModelIndex>
#include <QStringList>
#include <QSqlRecord>

class QAbstractItemView;
class QTableView;

class ProjectsModel : public QAbstractItemModel
{
	Q_OBJECT
	public:
		ProjectsModel(QObject *parent = 0);
		~ProjectsModel();
		
		Project *addProject();
		bool addProject(QSqlRecord record);
		bool addProject(Project *project);
		bool store();
		bool load(unsigned int userId = 0);
		QModelIndex indexOfProject(Project *project) const;
		Project *projectFromIndex(QModelIndex index) const;
		bool clear();
		bool reload();
		
		void addItemDelegates(QAbstractItemView *view);
		void hideAndResizeColumns(QTableView *view);
		
		// QT API
    QVariant data ( const QModelIndex& index, int role = Qt::DisplayRole ) const;
    int columnCount ( const QModelIndex& parent = QModelIndex() ) const;
    int rowCount ( const QModelIndex& parent = QModelIndex() ) const;
    QModelIndex parent ( const QModelIndex& child ) const;
    QModelIndex index ( int row, int column, const QModelIndex& parent = QModelIndex() ) const;
    QVariant headerData ( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
		
	private slots:
		void errorByPass(QString msg);
		void warningByPass(QString msg);
		void informationByPass(QString msg);
		
		void itemChanged(Project *project);
		
	private:
		void connectProject(Project *project);
		
		QList<Project*> projects;
		QStringList headerList;
		unsigned int myUsersId;
		
	signals:
    void error(QString msg);
    void warning(QString msg);
    void information(QString msg);
};

#endif // PROJECTSMODEL_H
