/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "resultsummary.h"
#include "resultsummary.moc"

#include <mediaplugin.h>
#include <mediamodel.h>
#include "mediumexport.h"
#include "shoppingcart.h"
#include "insertmanualmedia.h"
#include <mediumloandialog.h>
#include <mediumownerdialog.h>
#include <eaction.h>
#include <editmedium.h>
#include <esqldatabase.h>
#include <etoolbar.h>
#include "meganizer.h"
#include "checkmanufacturers.h"
#include <progressstatuswidget.h>
#include <efileinfo.h>
#include <efileservice.h>

#include <QMenu>
#include <QSortFilterProxyModel>
#include <QSqlRecord>
#include <QSqlQuery>
#include <QInputDialog>
#include <QToolButton>
#include <QSignalMapper>
#include <QTimer>
#include <QMessageBox>
#include <QSettings>

ResultSummary::ResultSummary ( QList< MediaPlugin* > newMediaPlugins, QWidget* parent, Qt::WindowFlags flags )
		: EWidget ( parent, flags )
{
	model = NULL;
	mediaPlugins = newMediaPlugins;
	setupUi ( this );
	shoppingCart = NULL;
	toolBar = NULL;
	newButton = NULL;
	newButtonAction = NULL;
	newButtonMenu = NULL;
	addNewSignalMapper = NULL;
	filesSignalMapper = NULL;
	checkManufacturersThread = NULL;
}

ResultSummary::~ResultSummary()
{
	close();

	if ( model )
		delete model;

	if ( proxyModel )
		delete proxyModel;
	
	// stopping thread
	if(checkManufacturersThread){
		checkManufacturersThread->disconnect(this, SLOT(stopCheckManufacturer()));
		checkManufacturersThread->waitForIdle();
		if(checkManufacturersThread->isBusy())
			// cancel query first
			checkManufacturersThread->cancel();
		// stop eventloop
		checkManufacturersThread->quit();
		// wait 2000 seconds to stop the eventloop
		checkManufacturersThread->wait(2000);
		delete checkManufacturersThread;
	}

	delete toolBar;
}

EngSaS::AvailableActions ResultSummary::enableActions()
{
	return EngSaS::FileNew | EngSaS::ViewReload;
}

void ResultSummary::init ( ShoppingCart *shoppingCart )
{
	this->shoppingCart = shoppingCart;

	addNewSignalMapper = new QSignalMapper(this);
	connect(addNewSignalMapper, SIGNAL(mapped(QString)), this, SLOT(addNew(QString)));
	filesSignalMapper = new QSignalMapper(this);
	connect(filesSignalMapper, SIGNAL(mapped(QString)), this, SLOT(openFile(QString)));
	
	newAction = new EAction ( "document-new", ":/icons/filenew.png" , tr ( "New" ), this );
	addSelectedAction = new EAction ( "insert-object" , "", tr ( "Add Selected" ), this );
	editAction = new EAction ( "document-properties", "", tr ( "Edit / Show Details" ), this );
	buyAction = new EAction ( tr ( "Buy" ), this );
	exportAction = new EAction ( "document-export", ":/icons/fileexport.png", tr ( "Export" ), this );
	buyAction = new EAction ( tr ( "Buy" ), this );
	loanAction = new EAction ( tr ( "Loaning" ), this );
	ownerAction = new EAction ( "", ":/icons/edituser.png", tr ( "Owner Manager" ), this );

	toolBar = new EToolBar ( this );
	toolBar->setObjectName ( "resultSummaryToolbar" );
	newButton = new QToolButton(toolBar);
	newButtonAction = toolBar->addWidget(newButton);
	newButton->setPopupMode(QToolButton::MenuButtonPopup);
	newButtonMenu = new QMenu(newButton);
	// later on, we add new special types to the button,
	// if the model is a plain MediaModel, but we build up a list
	// of possible types here
	foreach ( MediaPlugin *plugin, mediaPlugins ) {
		QStringList transTypes = plugin->supportedMaterialTypes();
		foreach ( QString type, transTypes ) {
			QStringList trans = plugin->translateMaterialType(type);
			if(trans.size() == 2){
				if(!types.contains(trans[0], Qt::CaseInsensitive))
					types << trans[0];
			}
		}
	}
	
	toolBar->addAction ( addSelectedAction );
	toolBar->addAction ( editAction );
	toolBar->addAction ( buyAction );
	toolBar->addAction ( exportAction );
	toolBar->addAction ( loanAction );
	toolBar->addAction ( ownerAction );
	emit addToolBar ( toolBar );

	addSelectedAction->setDisabled ( true );
	editAction->setDisabled ( true );
	buyAction->setDisabled ( true );
	exportAction->setDisabled(true);
	loanAction->setDisabled ( true );
	ownerAction->setDisabled ( true );

	connect ( newAction, SIGNAL ( triggered() ), addNewSignalMapper, SLOT ( map() ) );
	addNewSignalMapper->setMapping(newAction, QString());
	connect ( addSelectedAction, SIGNAL ( triggered() ), this, SLOT ( addSelected() ) );
	connect ( editAction, SIGNAL ( triggered() ), this, SLOT ( edit() ) );
	connect ( buyAction, SIGNAL ( triggered() ), this, SLOT ( buy() ) );
	connect ( exportAction, SIGNAL ( triggered() ), this, SLOT ( exportMedias() ) );
	connect ( loanAction, SIGNAL ( triggered() ), this, SLOT ( loanManager() ) );
	connect ( ownerAction, SIGNAL ( triggered() ), this, SLOT ( ownerManager() ) );
	connect ( tableView, SIGNAL ( clicked ( QModelIndex ) ), this, SLOT ( indexChanged ( QModelIndex ) ) );
	connect ( tableView, SIGNAL ( doubleClicked ( QModelIndex ) ), this, SLOT ( doubleClicked(QModelIndex)) );
	connect ( tableView, SIGNAL ( customContextMenuRequested ( QPoint ) ), this, SLOT ( contextMenu ( QPoint ) ) );

	tableView->setContextMenuPolicy ( Qt::CustomContextMenu );
	tableView->setSortingEnabled ( true );

	proxyModel = new QSortFilterProxyModel ( tableView );
	proxyModel->setDynamicSortFilter ( true );
	tableView->setModel ( proxyModel );

	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		db.open();

	if ( db.isOpen() ) {
		QSqlQuery query;
		query.exec ( QString ( "SELECT id FROM "+ Meganizer::group() +"_users WHERE username = '%1'" ).arg ( db.userName() ) );

		if ( query.first() )
			myUserId = query.record().value ( "id" ).toUInt();

		if ( closeDB )
			db.close();
	}
}

void ResultSummary::setResultset(MediaModel* newModel)
{
	indexChanged(QModelIndex());
	// delegates must be deleted, before
	// the proxyModel is resetted, because
	// the columnCount is needed!
	tableView->deleteDelegates();
	proxyModel->setSourceModel(0);
	if ( model )
		delete model;
	QList<QAction*> actions = newButtonMenu->actions();
	foreach(QAction *action, actions){
		if(action != newAction){
			newButtonMenu->removeAction(action);
			// do not delete the standard newAction
			delete action;
		}
	}

	model = newModel;
	if ( model ) {
		proxyModel->setSourceModel ( model );
		model->configureView ( tableView );
		connect(model, SIGNAL(finished()), this, SLOT(resizeView()));
		
		// create special actions
		if(QString(model->metaObject()->className()) == "MediaModel"){
			foreach(QString translatedType, types){
				QAction *action = newButtonMenu->addAction(translatedType, addNewSignalMapper, SLOT(map()));
				addNewSignalMapper->setMapping(action, translatedType);
			}
		}
	}
	newButton->setMenu(newButtonMenu);
	newButton->setDefaultAction(newAction);
	if(newButtonMenu->actions().size() < 2){
		if(toolBar->actions().contains(newButtonAction)){
			toolBar->insertAction(newButtonAction, newAction);
			toolBar->removeAction(newButtonAction);
		}
	}
	else{
		if(toolBar->actions().contains(newAction)){
			toolBar->insertAction(newAction, newButtonAction);
			toolBar->removeAction(newAction);
		}
	}
	
	// call indexChanged() to enable available buttons
	// (like export button, which may be available
	// without selected items
	indexChanged(QModelIndex());
}

bool ResultSummary::save(QFile* file, QString& error)
{
	Q_UNUSED(file);
	Q_UNUSED(error);
	save();
	return true;
}

void ResultSummary::save()
{
	// save unchanged changes
	InsertManualMedia *insertMedia = new InsertManualMedia ( this );
	connectByPass ( insertMedia );
	// 	insertMedias->setUnsavedOrigins(Medium::Manual); (It's the default actually)
	insertMedia->setupMedia ( model );
	insertMedia->exec();
	delete insertMedia;
}

bool ResultSummary::fileNew(QString& error)
{
	Q_UNUSED(error);
	addNew(QString());
	return true;
}

void ResultSummary::closeEvent ( QCloseEvent *event )
{
	save();
	EWidget::closeEvent ( event );
}

void ResultSummary::addNew(QString translatedType)
{
	// we want a special medium if possible
	Medium *medium = model->newSpecialMedium(translatedType);
	if ( !medium ) {
		emit error ( tr ( "Could not add medium" ) );
		return;
	}

	EditMedium *widget = medium->editWidget(this);
	if ( !widget ) {
		emit error ( tr ( "Could no get edit wigdet" ) );
		return;
	}
	connectByPass ( widget );
	widget->setNewMode(model, medium);

	if ( widget->exec() != QDialog::Accepted )
		emit error ( tr ( "Could not add medium!" ) );
	
	delete widget;
	startCheckManufacturerTimer();
}

void ResultSummary::addSelected()
{
	Medium *medium = model->mediumFromIndex ( curIndex );

	if ( !medium ) {
		emit error ( tr ( "Please select a medium" ) );
		return;
	}

	// medium is already in database, so the only thing we must do is
	// to store it with current user
	if(!medium->edit(EditMedium::Add, this)){
		emit error ( tr ( "Could not store medium" ) );
		return;
	}

	indexChanged ( QModelIndex() );

	startCheckManufacturerTimer();
	model->reload();
}

void ResultSummary::edit()
{
	if ( !model )
		return;

	Medium *medium = model->mediumFromIndex ( curIndex );

	if ( !medium ) {
		emit error ( tr ( "Please select a medium" ) );
		return;
	}

	if(!medium->edit(EditMedium::Edit, this)){
		emit error ( tr ( "Could not store changes" ) );
		return;
	}
	
	// must be made invalid, because all model indizes are invalid after reload
	indexChanged ( QModelIndex() );
	model->reload();
	startCheckManufacturerTimer();
}

void ResultSummary::buy()
{
	if ( !curIndex.isValid() )
		return;

	if ( !model )
		return;

	Medium *medium = model->mediumFromIndex ( curIndex );

	if ( !medium )
		return;

	if ( !shoppingCart )
		return;

	shoppingCart->addMedium ( medium );
}

void ResultSummary::indexChanged ( QModelIndex index )
{
	if ( !model )
		return;

	if ( !proxyModel )
		return;

	curIndex = proxyModel->mapToSource ( index );

	// change button state
	addSelectedAction->setEnabled ( false );

	editAction->setEnabled ( false );

	buyAction->setEnabled ( false );

	exportAction->setEnabled ( false );

	loanAction->setEnabled ( false );

	ownerAction->setEnabled ( false );

	if ( model->rowCount() > 0 )
		exportAction->setEnabled ( true );

	if ( curIndex.isValid() ) {
		Medium *medium = model->mediumFromIndex ( curIndex );

		if ( !medium )
			return;

		buyAction->setEnabled ( true );

		if ( medium->id() == 0 ) {
			// Medium is not in database, so disable edit button
			addSelectedAction->setEnabled ( true );
		} else {
			// only mediums which are not our own, can be added to our inventory
			// our own medias can only be shown
			if ( medium->user() != myUserId )
				addSelectedAction->setEnabled ( true );

			editAction->setEnabled ( true );

			loanAction->setEnabled ( true );

			ownerAction->setEnabled ( true );
		}
	}
}

void ResultSummary::doubleClicked ( QModelIndex index )
{
	Q_UNUSED(index);
	if(editAction->isEnabled())
		editAction->trigger();
	else if(addSelectedAction->isEnabled())
		addSelectedAction->trigger();
}

void ResultSummary::contextMenu ( QPoint pos )
{
	if(!model)
		return;
	
	QModelIndex index = tableView->indexAt ( pos );
	indexChanged ( index );

	QMenu *menu = new QMenu();
//   menu->setTitle(title);

	// insert specific options first
	QModelIndexList indizes;
	indizes = tableView->selectionModel()->selectedRows();
	// change the indexes from proxy to source indexes
	for(int i = 0; i < indizes.size(); i++)
		indizes[i] = proxyModel->mapToSource(indizes[i]);
	model->addCustomContextItems(menu, indizes);
	
	// add files for selected item
	Medium *med = model->mediumFromIndex(curIndex);
	QList<EFileInfo> files = med->files();
	if(files.size() > 0){
		QMenu *filesMenu = menu->addMenu(tr("Files"));
		foreach(EFileInfo info, files){
			QAction *action = filesMenu->addAction(info.name(), filesSignalMapper, SLOT(map()));
			filesSignalMapper->setMapping(action, info.id());
		}
	}
	
	menu->addAction ( newAction->actionWithText() );
	menu->addAction ( addSelectedAction->actionWithText() );
	menu->addAction ( editAction->actionWithText() );
	menu->addAction ( buyAction->actionWithText() );
	menu->addAction ( exportAction->actionWithText() );
	menu->addAction ( loanAction->actionWithText() );
	menu->addAction ( ownerAction->actionWithText() );
	menu->exec ( QCursor::pos() );
	delete menu;
}

void ResultSummary::exportMedias()
{
	if ( !model )
		return;
	
	if(!proxyModel)
		return;
	
	if(!tableView)
		return;
	
	MediumExport *medExp = new MediumExport ( this );

	connectByPass ( medExp );
	
	QModelIndexList indexes;
	if(tableView->selectionModel())
		indexes = tableView->selectionModel()->selectedRows();
	
	// change the indexes from proxy to source indexes
	for(int i = 0; i < indexes.size(); i++){
		indexes[i] = proxyModel->mapToSource(indexes[i]);
	}
	medExp->exportMedium ( model, indexes );

	medExp->deleteLater();
}

void ResultSummary::loanManager()
{
	if ( !model )
		return;

	MediumLoanDialog *loanDialog = new MediumLoanDialog ( model->mediumFromIndex ( curIndex ), this );

	connectByPass ( loanDialog );

	loanDialog->exec();

	delete loanDialog;
}

void ResultSummary::ownerManager()
{
	if ( !model )
		return;

	MediumOwnerDialog *ownerDialog = new MediumOwnerDialog ( model->mediumFromIndex ( curIndex ), this );

	connectByPass ( ownerDialog );

	ownerDialog->exec();

	delete ownerDialog;
}

void ResultSummary::startCheckManufacturer()
{
	// don't start a new instance, if another is running
	if(checkManufacturersThread)
		return;
	
	QSettings settings;
	bool sendMINRanges = false;
	if(settings.contains("sendMINRanges")){
		sendMINRanges = settings.value("sendMINRanges").toBool();
	}
	else{
		// ask the user if he wants to enable this feature
		int result = QMessageBox::question(this, tr("Send and load MIN ranges"),
			tr("Should the calculated MIN ranges and the related manufacturer data be send"
				" to the central database at mins.engsas.de?\n"
				" If you say yes here, you also recieve those data to make adding new"
				" media faster and more easy.\n"
				" You can revoke your approval at the settings dialog at any time."
			),
			QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes
		);
		if(result == QMessageBox::Yes)
			sendMINRanges = true;
		settings.setValue("sendMINRanges", sendMINRanges);
	}
	
	ProgressStatusWidget *statusWidget = new ProgressStatusWidget(this);
	checkManufacturersThread = new CheckManufacturers(this);
	
	connect(checkManufacturersThread, SIGNAL(setStep(QString)), statusWidget, SLOT(setLabel(QString)));
	connect(checkManufacturersThread, SIGNAL(setOverallRange(int)), statusWidget, SLOT(setRange(int)));
	connect(checkManufacturersThread, SIGNAL(overallIncrease()), statusWidget, SLOT(increaseValue()));
	connect(checkManufacturersThread, SIGNAL(finished(bool)), statusWidget, SLOT(removeMe()));
	connect(statusWidget, SIGNAL(canceled()), checkManufacturersThread, SLOT(cancel()));
	connect(checkManufacturersThread, SIGNAL(finished(bool)), this, SLOT(stopCheckManufacturer()));
	emit showStatusWidget(statusWidget);
	
	if(sendMINRanges)
		connect(checkManufacturersThread, SIGNAL(started()), checkManufacturersThread, SLOT(runChecksAndSendData()));
	else
		connect(checkManufacturersThread, SIGNAL(started()), checkManufacturersThread, SLOT(runChecks()));
	
	checkManufacturersThread->start();
}

void ResultSummary::stopCheckManufacturer()
{
	if(!checkManufacturersThread)
		return;
	
	// stopping thread
	// cancel query first
	checkManufacturersThread->cancel();
	// stop eventloop
	checkManufacturersThread->quit();
	// wait 2000 seconds to stop the eventloop
	checkManufacturersThread->wait(2000);
	delete checkManufacturersThread;
	checkManufacturersThread = NULL;
}

void ResultSummary::openFile(QString uniqueId)
{
	if(uniqueId.isEmpty())
		return;
	
	EFileService service;
	if(!service.serviceLoaded()){
		emit warning(tr("You have no file service selected right now. Please do that now and open the file again."));
		emit showSettings();
		return;
	}
	
	EFileInfo info = service.info(uniqueId);
	if(!info.isValid()){
		emit error(tr("No valid file selected"));
		return;
	}
	
	if(!service.open(info))
		emit error(tr("Could not open the file %1.").arg(info.name()));
	else
		emit information(tr("Opened %1").arg(info.name()));
}

void ResultSummary::startCheckManufacturerTimer()
{
	checkManufacturerTimer.stop();
	checkManufacturerTimer.singleShot(2000, this, SLOT(startCheckManufacturer()));
}

void ResultSummary::resizeView()
{
	tableView->resizeRowsToContents();
	model->hideAndResizeColumns(tableView);
}
