/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef RESULTSUMMARY_H
#define RESULTSUMMARY_H

#include <ewidget.h>
#include "ui_resultsummary.h"

class MediaPlugin;
class MediaModel;
class ShoppingCart;
class QSortFilterProxyModel;
class EAction;
class EToolBar;
class CheckManufacturers;

#include <QTimer>

class QMenu;
class QToolButton;
class QSignalMapper;

/**
	* @brief The result view in the central widget.
  */
class ResultSummary : public EWidget, private Ui::ResultSummary
{
	Q_OBJECT

	public:
		ResultSummary ( QList<MediaPlugin*> newMediaPlugins, QWidget *parent = 0, Qt::WindowFlags flags = 0 );
		~ResultSummary();
		EngSaS::AvailableActions enableActions();
		void init ( ShoppingCart *shoppingCart );
		void setResultset(MediaModel *newModel);

	public slots:
		bool save(QFile *file, QString &error);
		void save();
		bool fileNew(QString &error);

	protected:
		void closeEvent ( QCloseEvent *event );

	private slots:
		void addNew(QString translatedType);
		/**
		  * adds a medium of the model to the database
		  * it also adds the actual user as an owner of an selected medium from database
		  * @todo Add our icon for local database storage
		  */
		void addSelected();
		void edit();
		void buy();
		void indexChanged ( QModelIndex index );
		void doubleClicked ( QModelIndex index );
		void contextMenu ( QPoint pos );
		void exportMedias();
		void loanManager();
		void ownerManager();
		void startCheckManufacturer();
		void stopCheckManufacturer();
		void openFile(QString uniqueId);
		void resizeView();

	private:
		void startCheckManufacturerTimer();
		
		QWidget *parent;
		QModelIndex curIndex;
		QList<MediaPlugin*> mediaPlugins;
		MediaModel *model;
		QSortFilterProxyModel *proxyModel;
		unsigned int myUserId;
		EAction *newAction, *addSelectedAction;
		EAction *editAction, *buyAction;
		EAction *exportAction, *exportAllAction;
		EAction *loanAction, *ownerAction;
		EToolBar *toolBar;
		ShoppingCart *shoppingCart;
		QMenu *newButtonMenu;
		QToolButton *newButton;
		QAction *newButtonAction;
		QStringList types;
		QSignalMapper *addNewSignalMapper, *filesSignalMapper;
		QTimer checkManufacturerTimer;
		CheckManufacturers *checkManufacturersThread;
};

#endif // RESULTSUMMARY_H
