/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "searchwidget.h"
#include "searchwidget.moc"

#include <mediaplugin.h>
#include <searchplugin.h>
#include "resultsummary.h"
#include <mediamodel.h>
#include <searchdetailswidget.h>
#include <sqlconstraint.h>
#include <esqldatabase.h>
#include <minvalidator.h>
#include "meganizer.h"
#include <equerythread.h>

#include <QSettings>
#include <QSqlQuery>
#include <QDockWidget>
#include <QAction>
#include <QTabWidget>
#include <QSizePolicy>
#include <QCompleter>
#include <QSqlQueryModel>
#include <QShortcut>

#ifdef ZBAR
	#include <QZBar.h>
#endif

SearchWidget::SearchWidget ( QWidget* parent, Qt::WindowFlags flags )
		: EWidget ( parent, flags )
{
  ownUserId = 0;
	genreModel = 0;
	manuModel = 0;
	minValidator = 0;

	// useb like barcode here
	zbarShortCut = new QShortcut(tr("Ctrl+b"), this, SLOT(showZBarWidget()));
	
	queryThread = new EQueryThread(this);
	connect(queryThread, SIGNAL(started()), this, SLOT(setupQueryThread()));
	queryThread->start();
	
	setObjectName("SearchWidget");
  setupUi ( this );
	
	searchButton->setIcon("edit-find", ":/icons/filefind.png");

  connect ( searchButton, SIGNAL ( clicked() ), this, SLOT ( search() ) );
	
	connect ( titleEdit, SIGNAL( returnPressed() ), this, SLOT( search() ) );
	connect ( personEdit, SIGNAL( returnPressed()), this, SLOT( search() ) );
	connect ( eanEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
	connect ( genreEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
	connect ( manuEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
	connect ( keywordEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
}

SearchWidget::~SearchWidget()
{
	// stopping thread
	// cancel query first
	queryThread->cancelQuery();
	// stop eventloop
	queryThread->quit();
	// wait 2000 seconds to stop the eventloop
	queryThread->wait(2000);
	delete queryThread;
}

void SearchWidget::init ( QList< MediaPlugin* > newMediaPlugins, QList< SearchPlugin* > newSearchPlugins, ShoppingCart* shoppingCart )
{
	this->shoppingCart = shoppingCart;
	
  typeEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  genreEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  manuEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  userEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  mediaPlugins = newMediaPlugins;
  searchPlugins = newSearchPlugins;

  // add us as a left dock widget and create a entry to show and hide the dock widget
  QDockWidget *dockWidget = new QDockWidget ( tr ( "Search" ) );
	dockWidget->setObjectName("searchDockWidget");
  dockWidget->setWidget ( this );
  QAction *searchAction = new QAction ( tr ( "Search" ), this );
  emit addAction ( searchAction, "viewMenu" );
  searchAction->setCheckable ( true );
  connect ( dockWidget, SIGNAL ( visibilityChanged ( bool ) ), searchAction, SLOT ( setChecked ( bool ) ) );
  connect ( searchAction, SIGNAL ( triggered ( bool ) ), dockWidget, SLOT ( setVisible ( bool ) ) );
  emit addNewDockWidget ( dockWidget, Qt::LeftDockWidgetArea );

	QSettings settings;
	settings.beginGroup("SearchWidgetDefault");
	QString defaultType = settings.value ( "materialType" ).toString();
	QString defaultOrder = settings.value("order").toString();
	QString focused = settings.value("focused").toString();
	settings.endGroup();
  // add stuff from database
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if ( db.isOpen() ) {
    // add material types
    QSqlQuery query ( "SELECT id, name FROM "+ Meganizer::group() +"_materialtypes" );
    query.exec();
    while ( query.next() ) {
			foreach(MediaPlugin *plugin, mediaPlugins){
				QStringList result = plugin->translateMaterialType(query.value ( 1 ).toString());
				if(result.size() == 2){
					if(!result[0].isEmpty()){
						// adding the id to translated name map
						typeMap.insert ( query.value ( 0 ).toInt(), result[0]);
						typeEdit->addItem ( result[0] );
					}
				}
			}
		}
    
    // add genres
		genreModel = new QSqlQueryModel(this);
		completerModels << genreModel;
    genreModel->setQuery ( "SELECT id, name FROM "+ Meganizer::group() +"_genres ORDER BY name" );
    QCompleter *completer = new QCompleter(genreModel, this);
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		completer->setCompletionColumn(genreModel->record().indexOf("name"));
		genreEdit->setCompleter ( completer );

    // add manufacturer
		manuModel = new QSqlQueryModel(this);
		completerModels << manuModel;
    manuModel->setQuery ( "SELECT id, name FROM "+ Meganizer::group() +"_manufacturers ORDER BY name" );
    completer = new QCompleter(manuModel, this);
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		completer->setCompletionColumn(manuModel->record().indexOf("name"));
		manuEdit->setCompleter ( completer );

    // add users
    query.exec ( "SELECT id, name, givenName, userName FROM "+ Meganizer::group() +"_users" );
    while ( query.next() ) {
			if(query.record().value("userName").toString() == db.userName()){
				ownUserId = query.record().value("id").toUInt();
			}
			else{
				QString name = QString ( "%1, %2" )
						.arg ( query.record().value ( "name" ).toString() )
						.arg ( query.record().value ( "givenName" ).toString() );
				userMap.insert ( query.record().value ( "id" ).toInt(), name );
				userEdit->addItem ( name );
			}
    }
    if(!defaultOrder.isEmpty())
			userEdit->setCurrentIndex(userEdit->findText(defaultOrder));

		if(closeDB)
			db.close();
  }
	
	connect ( typeEdit, SIGNAL ( currentIndexChanged ( QString ) ), this, SLOT ( typeChanged ( QString ) ) );
	typeEdit->setCurrentIndex ( typeEdit->findText ( defaultType ) );
	if(typeEdit->currentText().isEmpty())
		typeEdit->setCurrentIndex(0);
	
	QList<QWidget*> widgets;
	widgets.append(titleEdit);
	widgets.append(personEdit);
	widgets.append(eanEdit);
	foreach(QWidget *widget, widgets){
		if(widget->objectName() == focused)
			widget->setFocus();
	}
}

void SearchWidget::typeChanged ( QString text )
{
	// remove old details widget
	if ( stackedWidget->currentWidget() ){
		disconnect(stackedWidget->currentWidget());
		stackedWidget->removeWidget ( stackedWidget->currentWidget() );
	}
	
	// add new one
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
		QString type = plugin->untranslateMaterialType(text);
		if(!type.isEmpty()){
			if ( plugin->isSupportedMaterialType ( type ) ) {
				SearchDetailsWidget *widget = plugin->searchWidget( type );
				if ( widget ){
					widget->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
					stackedWidget->addWidget ( widget );
					connect(widget, SIGNAL(startSearch()), this, SLOT(search()));
				}
				return;
			}
		}
  }
}

void SearchWidget::search()
{
	if(ownUserId == 0)
		return;
	if(!genreModel)
		return;
	
	// clear out old resultsummary
// 	emit setCentralWidget(0);
	
	QList<QWidget*> filledWidgets;
  QList<SqlConstraint> constraints;
	SqlConstraint constraint;
  if ( !titleEdit->text().isEmpty() ){
    constraint.setConstraint("title LIKE ?");
		constraint.addBindValue(QString("%%1%").arg ( titleEdit->text() ) );
		constraints << constraint;
		filledWidgets.append(titleEdit);
	}
	if(	!personEdit->text().isEmpty() ){
		QStringList personsList = personEdit->text().split(";");
		QStringList constraintList;
		QStringList bindValues;
		foreach(QString string, personsList){
			QString condition;
			QStringList names = string.split(",");
			if(names.size() == 1){
				// the given name can be a part of the name or the given name
				condition = "name LIKE ? OR givenName LIKE ?";
				bindValues << names[0]<< names[0];
			}
			else if(names.size() == 2){
				// given name and the name is entered,
				// so they must be matched both
				condition = "name LIKE ? AND givenName LIKE ?";
				bindValues << names[0] << names[1];
			}
			if(!condition.isEmpty()){
				constraintList << "id IN(SELECT materialsId FROM meganizer_materialhaspersons WHERE"
													"   materialPersonsId IN(SELECT id FROM meganizer_materialpersons WHERE "+ condition +")"
													")";
			}
		}
		constraint = SqlConstraint();
		constraint.setConstraint( constraintList.join(" OR ") );
		foreach(QString value, bindValues)
			constraint.addBindValue("%"+ value.trimmed() +"%");
		constraints << constraint;
		constraint = SqlConstraint();
		constraints << constraint;
		filledWidgets.append(personEdit);
	}
  if ( !eanEdit->text().isEmpty() ){
		if(!minValidator)
			minValidator = new MINValidator(this);

		QStringList mins;
		QString min = eanEdit->text();
		// try to identify the type of the given min
		QString minType = minValidator->type(min);
		// check all possible types
		if(!minType.isEmpty()){
			min = minValidator->strip(min, minType);
		}
		else{
			foreach(QString possibleType, minValidator->possibleTypes(min)){
				QString stripped = minValidator->strip(min, possibleType);
				if(!stripped.isEmpty()){
					min = stripped;
					minType = possibleType;
					break;
				}
			}
		}
		mins << min;
		// try to also check all converted mins
		foreach(QString type, minValidator->supportedTypes()){
			QString converted = minValidator->convert(mins[0], type, minType);
			if(!converted.isEmpty())
				mins << converted;
		}

		// prepare all mins
		QStringList conditions;
		foreach(QString min, mins)
			conditions << "min LIKE ?";
		
		constraint = SqlConstraint();
    constraint.setConstraint(QString("id IN(SELECT DISTINCT materialsId FROM "+ Meganizer::group() +"_materialhasmins WHERE %1 ORDER BY materialsId)")
															.arg(conditions.join(" OR ")) );
		foreach(QString min, mins)
			constraint.addBindValue(QString("%%1%").arg ( min ) );

		constraints << constraint;
		filledWidgets.append(eanEdit);
	}
  if( !genreEdit->text().isEmpty() ){
		unsigned int id = 0;
		for(int i = 0; i < genreModel->rowCount(); i++){
			if(genreModel->record(i).value("name").toString() == genreEdit->text()){
				id = genreModel->record(i).value("id").toUInt();
				break;
			}
		}
		if(id == 0){
			constraint = SqlConstraint();
			constraint.setConstraint("genresName LIKE ?");
			constraint.addBindValue(QString("%%1%").arg(genreEdit->text()));
			constraints << constraint;
		}
		else{
			constraint = SqlConstraint();
			constraint.setConstraint("genresId=?");
			constraint.addBindValue(id);
			constraints << constraint;
		}
	}
  if( !manuEdit->text().isEmpty() ){
		unsigned int id = 0;
		for(int i = 0; i < manuModel->rowCount(); i++){
			if(manuModel->record(i).value("name").toString() == manuEdit->text()){
				id = manuModel->record(i).value("id").toUInt();
				break;
			}
		}
		if(id == 0){
			constraint = SqlConstraint();
			constraint.setConstraint("manufacturersName LIKE ?");
			constraint.addBindValue(QString("%%1%").arg(manuEdit->text()));
			constraints << constraint;
		}
		else{
			constraint = SqlConstraint();
			constraint.setConstraint("manufacturersId=?");
			constraint.addBindValue(id);
			constraints << constraint;
		}
	}
	if( !keywordEdit->text().isEmpty() ){
		QStringList keywordList = keywordEdit->text().split(" ");
		QStringList constraintList;
		foreach(QString string, keywordList)
			constraintList <<  "id IN(SELECT materialsId FROM viewmaterialskeywords WHERE keyword=?)";
		constraint = SqlConstraint();
		constraint.setConstraint( constraintList.join(" AND ") );
		foreach(QString string, keywordList)
			constraint.addBindValue(string);
		constraints << constraint;
	}
  int userId = 0;
	switch(userEdit->currentIndex()){
		case 0:
			userId = 0;
			break;
		case 1:
			userId = -2;
			break;
		case 2:
			userId = -1;
			break;
		case 3:
			userId = ownUserId;
			break;
		default:
			userId = userMap.key(userEdit->currentText());
			break;
	}
	switch(statusEdit->currentIndex()){
		case 1:
			constraint = SqlConstraint();
			constraint.setConstraint("wish=?" );
			constraint.addBindValue(1);
			constraints << constraint;
			break;
		case 2:
			constraint = SqlConstraint();
			if(userId <= 0){
				constraint.setConstraint( "(wish=? OR (presenteeId>0 AND presenteeId<>?))" );
				constraint.addBindValue(1);
				constraint.addBindValue(ownUserId);
			}
			else{
				constraint.setConstraint( "((usersId=? AND wish=?) OR (presenteeId=? AND presenteeId<>?))" );
				constraint.addBindValue(userId);
				constraint.addBindValue(1);
				constraint.addBindValue(userId);
				constraint.addBindValue(ownUserId);
			}
			constraints << constraint;
			break;
		case 3:
			constraint = SqlConstraint();
			constraint.setConstraint("literatur" );
			constraint.addBindValue(1);
			constraints << constraint;
			break;
		case 4:
			constraint = SqlConstraint();
			constraint.setConstraint("(wish=? OR giftOnly=? OR literatureOnly=?)" );
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraints << constraint;
			break;
		case 5:
			constraint = SqlConstraint();
			constraint.setConstraint("(wish<>? AND giftOnly<>? AND literatureOnly<>?)" );
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraints << constraint;
			break;
	}
  
  MediaModel *model = 0;
  if ( typeEdit->currentIndex() == 0 ) {
    // search all
    model = new MediaModel(queryThread, this);
  }
  else {
    MediaPlugin *mediaPlugin = 0;
		QString type;
    foreach ( MediaPlugin *plugin, mediaPlugins ) {
			type = plugin->untranslateMaterialType( typeEdit->currentText() );
      if ( plugin->isSupportedMaterialType ( type ) ) {
        mediaPlugin = plugin;
        break;
      }
    }
    if ( !mediaPlugin ) {
      emit error ( tr ( "Could not find a plugin which handles the selected media type" ) );
      return;
    }

    model = mediaPlugin->newModel( type, queryThread );
  }
  if(!model){
    emit error( tr ( "No model created!" ) );
    return;
  }
  connectModel(model);
	
	// show result view
	if(!resSum){
		resSum = new ResultSummary ( mediaPlugins, this );
		connectByPass ( resSum );
		resSum->init(shoppingCart);
		emit setCentralWidget ( resSum );
	}
	resSum->setResultset(model);
	
	// start search
	SearchDetailsWidget *widget = NULL;
	if(detailsBox->isChecked())
		widget = qobject_cast<SearchDetailsWidget*>(stackedWidget->widget(0));
	model->load(constraints, searchPlugins, userId, widget);
	
	// store current stati
	// check which widget has the focuse
	QString focused;
	if(filledWidgets.size() == 1){
		focused = filledWidgets[0]->objectName();
	}
	else{
		if(titleEdit->hasFocus())
			focused = titleEdit->objectName();
		if(personEdit->hasFocus())
			focused = personEdit->objectName();
		if(eanEdit->hasFocus())
			focused = eanEdit->objectName();
	}

	QSettings settings;
	settings.beginGroup("SearchWidgetDefault");
	settings.setValue("materialType", typeEdit->currentText());
	settings.setValue("order", userEdit->currentText());
	settings.setValue("focused", focused);
	settings.endGroup();
}

void SearchWidget::reloadForeignKeys()
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
	foreach(QSqlQueryModel *queryModel, completerModels)
		queryModel->setQuery(queryModel->query().lastQuery());
	if(closeDB)
		db.close();
}

void SearchWidget::setupQueryThread()
{
	ESqlDatabase db = ESqlDatabase::database();
	queryThread->dbSetup(db.databaseName(), db.userName(), db.password(), db.hostName(), db.driverName());
}

void SearchWidget::showZBarWidget()
{
#ifdef ZBAR
	zbar::QZBar *zbar = new zbar::QZBar();
	connect(zbar, SIGNAL(decoded(int, QString)), this, SLOT(zbarDecoded(int, QString)));
	zbar->setWindowTitle(QApplication::applicationName());
	zbar->show();
#else
	emit error(tr("ZBar is not supported"));
#endif
}

#ifdef ZBAR
void SearchWidget::zbarDecoded(int type, const QString data)
{
	Q_UNUSED(type);
	eanEdit->setText(data);
	zbar::QZBar *zbar = qobject_cast<zbar::QZBar*>(sender());
	if(!zbar)
		return;
	zbar->close();
	zbar->deleteLater();
}
#endif

void SearchWidget::connectModel(QObject* model)
{
  connect(model, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
  connect(model, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
  connect(model, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	
	connect(model, SIGNAL(showStatusWidget(QWidget*)), this, SLOT(showStatusWidgetByPass(QWidget*)));
	connect(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(reloadForeignKeys()));
}
