/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "settingsgeneral.h"
#include "settingsgeneral.moc"
#include "ui_settingsgeneral.h"

#include "setupuserwidget.h"
#include <edbsettings.h>

#include <QFileDialog>
#include <QSqlDatabase>
#include <QDir>
#include <QFileInfo>

SettingsGeneral::SettingsGeneral ( QWidget *parent, Qt::WindowFlags flags )
	: ESettingsWidget ( parent, flags ),
    m_ui ( new Ui::SettingsGeneral )
{
  setName ( tr ( "General" ) );
  m_ui->setupUi ( this );

  // add database stuff
  dbSettings = new EDBSettings ( sqlDrivers(), 0, m_ui->tabView );
  connectByPass ( dbSettings );
  m_ui->tabView->addTab ( dbSettings, dbSettings->name() );

  userWidget = new SetupUserWidget ( 0, m_ui->tabView );
  connectByPass ( userWidget );
  m_ui->tabView->addTab ( userWidget, userWidget->name() );

  loadSettings();
	
	mySendMINRangesChanged = false;
	connect(m_ui->submitManufacturerMinsEdit, SIGNAL(stateChanged(int)), this, SLOT(sendMINRangesChanged()));
}

SettingsGeneral::~SettingsGeneral()
{
  delete m_ui;
}

QStringList SettingsGeneral::sqlDrivers()
{
	QStringList drivers = QSqlDatabase::drivers();
	// strip out don't supported drivers
  QStringList delSQLite = drivers.filter ( "sqlite", Qt::CaseInsensitive );
  foreach ( QString string, delSQLite ) {
    drivers.removeAll ( string );
  }
  return drivers;
}

bool SettingsGeneral::saveSettings(QString &error)
{
  qSettings.setValue ( "iconsAndText", m_ui->iconsAndTextCheckBox->isChecked() );
  qSettings.setValue ( "alternatingRowColors", m_ui->alternatingRowColors->isChecked() );
	if(mySendMINRangesChanged)
		qSettings.setValue ( "sendMINRanges", m_ui->submitManufacturerMinsEdit->isChecked() );
  qSettings.setValue ( "usedBorder", m_ui->usedBorderEdit->value() );
	qSettings.setValue ( "minUsedBorder", m_ui->minUsedBorderEdit->value() );
  
  bool result = true;

  if ( !dbSettings->saveSettings(error) )
    result = false;

  if ( !userWidget->saveSettings(error) )
    result = false;

	if ( !m_ui->eUpdateNoticeSettings->saveSettings(error) )
		result = false;
	
  return result;
}

void SettingsGeneral::changeEvent ( QEvent *e )
{
  QWidget::changeEvent ( e );

  switch ( e->type() ) {
    case QEvent::LanguageChange:
      m_ui->retranslateUi ( this );
      break;
    default:
      break;
  }
}

void SettingsGeneral::loadSettings()
{
  m_ui->iconsAndTextCheckBox->setChecked ( qSettings.value ( "iconsAndText" ).toBool() );
  m_ui->alternatingRowColors->setChecked ( qSettings.value ( "alternatingRowColors" ).toBool() );
	m_ui->submitManufacturerMinsEdit->setChecked( qSettings.value("sendMINRanges", false).toBool());
#if QT_VERSION < 0x040600
	m_ui->usedBorderEdit->setValue( qSettings.value( "usedBorder", 30 ).toDouble() );
	m_ui->minUsedBorderEdit->setValue( qSettings.value( "minUsedBorder", 80 ).toDouble() );
#else
	m_ui->usedBorderEdit->setValue( qSettings.value( "usedBorder", 30 ).toFloat() );
	m_ui->minUsedBorderEdit->setValue( qSettings.value( "minUsedBorder", 80 ).toFloat() );
#endif
}

void SettingsGeneral::sendMINRangesChanged()
{
	mySendMINRangesChanged = true;
}
