/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef SHOPPINGCART_H
#define SHOPPINGCART_H

#include <ewidget.h>

#include <searchplugin.h>

class MediaModel;
class Medium;
class DelegateShoppingCart;
class ShoppingCartListView;
class EPushButton;

#include <QModelIndex>

class QAction;
class QLabel;
class QDockWidget;

/**
  * @todo store the shopping cart on program exit
  */
class ShoppingCart : public EWidget
{
    Q_OBJECT
  public:
    ShoppingCart ( QList<SearchPlugin*> searchPlugins, QWidget *parent = 0, Qt::WindowFlags flags = 0 );
    ~ShoppingCart();
		void init(bool startHidden = false);

  public slots:
    /**
    	* @todo search for existing ShoppingCart DockWidgets
    	*/
    void addMedium ( Medium *medium );

  protected slots:
    void resizeEvent ( QResizeEvent * event );

  private slots:
    void buy();
    void resizeDelegate();
    void startSearchBuyData ( Medium *medium );
    void foundBuyData ( const BuyData &data );
    void calcSum();
    void reloadView();
    void reloadData();
    void contextMenuRequested ( QPoint pos );
    void increaseQuantity(QModelIndex newIndex = QModelIndex());
    void decreaseQuantity(QModelIndex newIndex = QModelIndex());
    void removeMedium(QModelIndex newIndex = QModelIndex());
    void removeMedium(Medium *med);
		void setOffer();
		/**
			* @todo checking other buy orders than the one with less shops as possible
			* @todo the one created with less shops, must not be the cheapest with less shops
			*/
    void optimizeBasket();

  private:
    void setupUi();
		/**
			* Selects the chepeast new or used offer of an item by this preferences:
			* 1. Custom selection
			* 2. Enough Items
			* 3. Chepeast new or valid used offer
			* @param datas list of offers, of which the cheapest should be selected (customOffer and customShop are also checked)
			*/
		BuyData selectBuyData(QList<BuyData> datas, Medium *med = 0);

    ShoppingCartListView *listView;
    MediaModel *model;
    DelegateShoppingCart *delegate;
    QTimer *resizeTimer;
    QAction *viewAction;
    QList<SearchPlugin*> searchPlugins;
    QLabel *totalLabel;
    EPushButton *buyButton, *reloadButton;
    QModelIndex index;
    QMap<Medium*, int> mapQuantity;
    QMultiMap<Medium*, BuyData> mapBuyData;
		QMap<Medium*, SearchPlugin*> customShop;
		QMap<Medium*, BuyData> customOffer;
		float usedBorder, minUsedBorder;
		QDockWidget *dock;
};

#endif // SHOPPINGCART_H
