/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2015 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "statisticssummary.h"
#include "statisticssummary.moc"

#include <esqldatabase.h>

#include "meganizer.h"
#include "mediaplugin.h"

#include <escaledimage.h>
#include <echartpie.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QPainter>
#include <QVariant>
#include <QPicture>
#include <QGridLayout>

#include <cmath>

StatisticsSummary::StatisticsSummary ( QList< MediaPlugin* > plugins, QWidget* parent )
	: EWidget ( parent )
{
	setObjectName("statisticsSummaryWidget");
	setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
	setMinimumSize(200, 200);
	
	_chartMediaTypes = new EChartPie(this);
	connectByPass(_chartMediaTypes);
	_chartUsers = new EChartPie(this);
	connectByPass(_chartUsers);
	
	QGridLayout *ml = new QGridLayout(this);
	setLayout(ml);
	ml->addWidget(_chartMediaTypes, 0, 0);
	ml->addWidget(_chartUsers, 0, 1);
	
	EChartValues valuesMediaTypes, valuesUsers;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		
		// build mapping for our media types
		query.exec("SELECT COUNT(mats.id), types.name "
							 "  FROM viewmaterials mats, "+ Meganizer::group() +"_materialtypes types"
							 "  WHERE mats.materialtypesId=types.id"
							 "  GROUP BY materialtypesId"
							 "  ORDER BY COUNT(mats.id) DESC, types.name ASC");
		while(query.next()){
			QString type;
			foreach(MediaPlugin *plugin, plugins){
				QStringList result = plugin->translateMaterialType(query.record().value("name").toString());
				if(result.size() == 2 && !result[0].isEmpty()){
					type = result[0];
					break;
				}
			}
			if(!type.isEmpty()){
				// since EChartPie ignores the y value of EChartValues, we use it
				// to store the type name in here
				valuesMediaTypes.add(query.record().value("COUNT(mats.id)").toInt(), type);
			}
		}
		
		// build user medias
		query.exec("SELECT COUNT(mats.id), users.name, users.givenName"
							 "  FROM viewmaterials mats, "+ Meganizer::group() +"_users users"
							 "  WHERE mats.usersId=users.id"
							 "  GROUP BY mats.usersId"
							 "  ORDER BY COUNT(mats.id) DESC, users.name ASC");
		while(query.next()){
			QString name = query.record().value("name").toString();
			QString givenName = query.record().value("givenName").toString();
			if(!givenName.isEmpty())
				name += ", "+ givenName.left(3);
			valuesUsers.add(query.record().value("COUNT(mats.id)").toInt(), name);
		}
		
		if(closeDB)
			db.close();
	}
	
	valuesMediaTypes = restrictNumberOfEntries(valuesMediaTypes, 5);
	valuesUsers = restrictNumberOfEntries(valuesUsers, 5);
	
	int total = 0;
	foreach(QVariant x, valuesMediaTypes.xValues()){
		total += x.toInt();
		_chartMediaTypes->addPie(x.toInt(), valuesMediaTypes.y(x).toString());
	}
	foreach(QVariant x, valuesUsers.xValues())
		_chartUsers->addPie(x.toInt(), valuesUsers.y(x).toString());
	
	_chartMediaTypes->setChartTitle(tr("Items by type (Total items: %1)").arg(total));
	_chartUsers->setChartTitle(tr("Items by user (Total items: %1)").arg(total));
}

EChartValues StatisticsSummary::restrictNumberOfEntries(const EChartValues& values, int rows)
{
	EChartValues result;
	
	// reduce rows by one, because the last row is the Others entry
	rows--;
	QVariantList xValues = values.xValues();
	int others = 0;
	for(int i = 0; i < xValues.size(); i++){
		if(i < rows){
			// just append the first rows result to result
			result.add(xValues[i], values.y(xValues[i]));
		}
		else
			others += xValues[i].toInt();
	}
	if(others > 0)
		result.add(others, tr("Others"));
	return result;
}
