/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "summary.h"
#include "summary.moc"

#include "loansummary.h"
#include "statisticssummary.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QScrollArea>

#include <cmath>

Summary::Summary (QList<MediaPlugin*> plugins, QWidget* parent, Qt::WindowFlags flags )
	: EWidget ( parent, flags )
{
	setObjectName("summaryWidget");
	setupUi(plugins);
}

void Summary::setupUi(QList<MediaPlugin*> plugins)
{
	setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
	
	// setup widgets
	LoanSummary *loanSummary = new LoanSummary(this);
	widgets << loanSummary;
	connectByPass(loanSummary);
	
	StatisticsSummary *statSummary = new StatisticsSummary(plugins, this);
	widgets << statSummary;
	connectByPass(statSummary);
	
	QHBoxLayout *mainLayout = new QHBoxLayout(this);
	setLayout(mainLayout);
	scrollArea = new QScrollArea(this);
	mainLayout->addWidget(scrollArea);
	scrollArea->setWidgetResizable(true);
	
	baseWidget = new QWidget(scrollArea);
	// to achieve independent columns, we place QVBoxLayout's
	// in a QHBoxLayout
	mainLayout = new QHBoxLayout(baseWidget);
	baseWidget->setLayout(mainLayout);
	
	// devide the items in ratio 4:3
	QVBoxLayout *column;
	int cols = floor(sqrt(float(widgets.size())/0.75));
	int itemsPerRow = ceil(float(widgets.size())/float(cols));
	int widget = 0;
	for(int i = 0; i < cols; i++){
		column = new QVBoxLayout();
		mainLayout->addLayout(column);
		for(int j = 0; j < itemsPerRow; j++){
			if(widgets.size() <= widget)
				break;
			column->addWidget(widgets[widget]);
			widget++;
		}
	}	
	
	scrollArea->setWidget(baseWidget);
}
