/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "categorytests.h"

#include <category.h>
#include <categorymodel.h>
#include <medium.h>
#include <modeltest.h>

#include <QtTest/QtTest>

void CategoryTests::categoryTests()
{
	QString name = "test";
	Category *cat = new Category(name);
	QCOMPARE(cat->name(), name);
	QVERIFY(cat->parent() == 0);
	
	name = "test 2";
	cat->setName(name);
	QCOMPARE(cat->name(), name);
	
	// testing sub categories api
	QCOMPARE(cat->haveSubCategories(), false);
	QList<Category*> cats;
	cats << new Category("Test Cat 1");
	cats << new Category("Test Cat 2");
	foreach(Category *tmp, cats)
		cat->addSubCategory(tmp);
	QCOMPARE(cat->haveSubCategories(), true);
	QCOMPARE(cat->subCategories().size(), cats.size());
	for(int i = 0; i < cats.size(); i++){
		QCOMPARE(cat->subCategories()[i]->name(), cats[i]->name());
		QVERIFY(cat->subCategories()[i]->parent());
		QCOMPARE(cat->subCategories()[i]->parent()->name(), cat->name());
	}
	
		// testing new media api
	Category *subCat = new Category("name 1.1");
	cat->addSubCategory(subCat);
	QList<Medium*> media;
	media << new Medium();
	media[0]->setTitle("Test title");
	media << new Medium();
	media[1]->setTitle("Test Title 2");
	foreach(Medium *med, media)
		subCat->addMedium(med);

	QCOMPARE(cat->numNewMedia(), media.size());
	QCOMPARE(subCat->numNewMedia(), media.size());
	for(int i = 0; i < media.size(); i++){
		QCOMPARE(cat->newMedia()[i]->title(), media[i]->title());
		QCOMPARE(subCat->newMedia()[i]->title(), media[i]->title());
		QCOMPARE(cat->newMedium(i)->title(), media[i]->title());
		QCOMPARE(subCat->newMedium(i)->title(), media[i]->title());
	}
	
	delete cat;
	
	Category cat1("test");
	Category cat2("test");
	QVERIFY(cat1 == cat2);
}

void CategoryTests::modelTest()
{
	CategoryModel *model = new CategoryModel(this);
	new ModelTest(model, this);
	
	QList<Category*> items;
	items << new Category("test 1");
	items << new Category("test 2");
	items[items.size() - 1]->addSubCategory(new Category("test 1.1"));
	items[items.size() - 1]->addSubCategory(new Category("test 1.2", items[items.size() - 1]));
	items << new Category("test 3");
	items[items.size() - 1]->addSubCategory(new Category("test 2.1", items[items.size() - 1]));
	items << new Category("test 4");
	
	// insert items
	foreach(Category *cat, items){
		model->addCategory(cat);
	}
	
	QCOMPARE(model->rowCount(), items.size());
	
	// remove a min from the beginning, the middle and the end
	if(model->rowCount() > 0)
		model->removeRow(0);
	
	if(model->rowCount() > 0)
		model->removeRow(model->rowCount() - 1);
	
	if(model->rowCount() > 2){
		int half = model->rowCount() / 2;
		model->removeRow(half);
	}
	
	int rowCount = model->rowCount();
	model->addCategory("Test");
	QCOMPARE(rowCount + 1, model->rowCount());
	
	Category *cat = new Category("Test 10");
	rowCount = model->rowCount();
	model->addCategory(cat);
	QCOMPARE(rowCount + 1, model->rowCount());
	
	rowCount = model->rowCount(model->indexOfItem(cat));
	model->addCategory("Test 10.1", cat);
	model->addCategory(new Category("Test 10.2"), cat);
	QCOMPARE(rowCount + 2, model->rowCount(model->indexOfItem(cat)));
	
}


QTEST_MAIN(CategoryTests)

#include "categorytests.moc"
