/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "eanvalidatortests.h"
#include "eanvalidatortests.moc"

#include "eanvalidator.h"

#include <QtTest/QtTest>

EANValidatorTests::EANValidatorTests(QObject* parent): QObject(parent)
{
	validator = new EANValidator();
}

EANValidatorTests::~EANValidatorTests()
{
	delete validator;
}

// void EANValidatorTests::checksum_data()
// {
// 	QTest::addColumn<QString>("ean");
// 	QTest::addColumn<int>("result");
// 	
// 	QTest::newRow("9783772373251") << "9783772373251" << 1;
// 	QTest::newRow("978377237325") << "978377237325" << 1;
// 	QTest::newRow("9783772373") << "9783772373" << -1;
// 	QTest::newRow("29033805") << "29033805" << 5;
// 	QTest::newRow("2903380") << "2903380" << 5;
// 	QTest::newRow("290338") << "290338" << -1;
// 	QTest::newRow("978377237325123123") << "978377237325123123" << -1;
// }

// void EANValidatorTests::checksum()
// {
// 	QFETCH(QString, ean);
// 	QFETCH(int, result);
// 	
// 	QCOMPARE(validator->checksum(ean), result);
// }

void EANValidatorTests::strip_data()
{
	QTest::addColumn<QString>("ean");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("9788559373257") << "9788559373257" << "9788559373257";
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << "5449000096241";
	QTest::newRow("5 449000 09623-1") << "5449000096231" << "5449000096231";
	QTest::newRow("5 449000 k9624-1") << "544  900adb _?& 41" << "54490041";
}

void EANValidatorTests::strip()
{
	QFETCH(QString, ean);
	QFETCH(QString, result);
	
	QCOMPARE(validator->strip(ean), result);
}

void EANValidatorTests::isValid_data()
{
	QTest::addColumn<QString>("ean");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << true;
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1" << false;
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1" << false;
	QTest::newRow("9788559373257") << "9788559373257" << true;
	QTest::newRow("9788559373256") << "9788559373256" << false;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("978344237492") << "978344237492" << false;
	
	// instore EAN-13
	QTest::newRow("2404105 001722") << "2404105 001722" << true;
	// in store EAN-8 (Aldi Schovit ;-) )
	QTest::newRow("29033805") << "29033805" << true;
	QTest::newRow("29023805") << "29023805" << false;
	QTest::newRow("2?./0dÄa") << "2?./0dÄa" << false;
	
	QTest::newRow("5 449000 09624-123") << "5 449000 09624-123" << false;
	QTest::newRow("123456") << "123456" << false;
}

void EANValidatorTests::isValid()
{
	QFETCH(QString, ean);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isValid(ean), result);
	
// 	QBENCHMARK{validator->isValid(ean);};
}

void EANValidatorTests::validate_data()
{
	QTest::addColumn<QString>("ean");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("5 449000 09624") << "5 449000 09624" << "5449000096241";
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << "5449000096241";
	QTest::newRow("5449000096241") << "5449000096241" << "5449000096241";
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1" << "5449000096234";
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1" << "5449000962416";
	QTest::newRow("5 449000 kk624-1") << "5 449000 kk624-1" << QString();
	QTest::newRow("5 449000 k624") << "5 449000 k624" << QString();
	QTest::newRow("5 449") << "5 449" << QString();
	QTest::newRow("5 449123123123123123") << "5 449123123123123123" << QString();
	QTest::newRow("9783548609829") << "9783548609829" << "9783548609829";
	QTest::newRow("978354860982") << "978354860982" << "9783548609829";
	QTest::newRow("9783442374922") << "9783442374922" << "9783442374922";
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("978344237492") << "978344237492" << "9783442374922";
	
	QTest::newRow("29033805") << "29033805" << "29033805";
	QTest::newRow("2903380") << "2903380" << "29033805";
	QTest::newRow("2903X805") << "2903X805" << "29038059";
	QTest::newRow("2903XX05") << "2903XX05" << QString();
}

void EANValidatorTests::validate()
{
	QFETCH(QString, ean);
	QFETCH(QString, result);
	
	QCOMPARE(validator->validate(ean), result);
}

void EANValidatorTests::isType_data()
{
	QTest::addColumn<QString>("ean");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << true;
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1" << false;
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1" << false;
	
	// instore EAN-13
	QTest::newRow("2404105 001722") << "2404105 001722" << true;
	// in store EAN-8 (Aldi Schovit ;-) )
	QTest::newRow("29033805") << "29033805" << true;
	QTest::newRow("29023805") << "29023805" << false;
	QTest::newRow("2?./0dÄa") << "2?./0dÄa" << false;
	
	QTest::newRow("5 449000 09624-123") << "5 449000 09624-123" << false;
	QTest::newRow("123456") << "123456" << false;
}

void EANValidatorTests::isType()
{
	QFETCH(QString, ean);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isType(ean), result);
}

void EANValidatorTests::mayType_data()
{
	QTest::addColumn<QString>("ean");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << true;
	QTest::newRow("5 449000 09623-1") << "5 449000 09623-1" << true;
	QTest::newRow("5 449000 k9624-1") << "5 449000 k9624-1" << true;
	QTest::newRow("9788559373257") << "9788559373257" << true;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("978344237492") << "978344237492" << true;
	
	// instore EAN-13
	QTest::newRow("2404105 001722") << "2404105 001722" << true;
	// in store EAN-8 (Aldi Schovit ;-) )
	QTest::newRow("29033805") << "29033805" << true;
	QTest::newRow("29023805") << "29023805" << true;
	QTest::newRow("2?./0dÄa") << "2?./0dÄa" << true;
	
	QTest::newRow("5 449000 09624-123") << "5 449000 09624-123" << false;
	QTest::newRow("123456") << "123456" << true;
}

void EANValidatorTests::mayType()
{
	QFETCH(QString, ean);
	QFETCH(bool, result);
	
	QCOMPARE(validator->mayType(ean), result);
}

void EANValidatorTests::commonPrefixes_data()
{
	QTest::addColumn<QStringList>("mins");
	QTest::addColumn<QStringList>("prefixes");
	
	QStringList mins, prefixes;

	// test two mins with 9 character long prefix
	mins.clear();
	mins << "9783426775899" << "9783426196816" << "9783426624081";
	prefixes.clear();
	prefixes << "9783426";
	QTest::newRow("9783426") << mins << prefixes;
	
	// test mins, with one longer possible combination and one shorter
	// possible combination, where only the shorter one is the correct one
	mins.clear();
	mins << "9783423110082" << "9783423110105" << "9783423133166" << "9783423308489";
	prefixes.clear();
	prefixes << "9783423";
	QTest::newRow("9783423") << mins << prefixes;
	
	// test mins with two different prefixes
	mins.clear();
	mins << "9783785722565" << "9783404156856" << "9783404123544" << "9783404150717" << "9783924580957" << "9783924580117";
	prefixes.clear();
	prefixes << "97834041" << "978392458";
	QTest::newRow("97834041 and 978392458") << mins << prefixes;
}

void EANValidatorTests::commonPrefixes()
{
	QFETCH(QStringList, mins);
	QFETCH(QStringList, prefixes);
	
	QCOMPARE(validator->commonPrefixes(mins), prefixes);
}

// no gui testing!
QTEST_APPLESS_MAIN(EANValidatorTests)
// QTEST_MAIN(EANValidatorTests)
