/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "isbnvalidator.h"
#include "isbnvalidator.moc"

#include <QObject>
#include <QtPlugin>
#include <QRegExp>

ISBNValidator::ISBNValidator()
{

}

QString ISBNValidator::type() const
{
	return "ISBN-10";
}

QString ISBNValidator::description() const
{
	return tr("ISBN-10 - International Standard Book Number with 10 characters");
}

bool ISBNValidator::canConvert(QString toType) const
{
	if(toType == "EAN")
		return true;
	return false;
}

QString ISBNValidator::convert(QString min, QString toType) const
{
	if(!canConvert(toType))
		return QString();

	min = strip(min);
	
	if(!isValid(min))
		return QString();
	
	// it's a ISBN-10, so prepend 978 or 979 and compute a new check number to replace last number
	// computing new checknumber is done by MINValidator and the proper plugin
  QString result = "978" + min.left ( min.length() - 1 );
	return result;
}

bool ISBNValidator::isValid(QString min) const
{
	min.replace(QRegExp("[\\-\\s]"), "");

	if(min != strip(min))
		return false;
	
	if(min.length() != 10)
		return false;

	// Only the checksum can be an X
	if(min.left(9).toUpper().contains("X"))
		return false;
	
	if(checksum(min).toUpper() == min.right(1).toUpper())
		return true;
	
	return false;
}

QString ISBNValidator::validate(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return min;

	// a MIN with 9 (ISBN without checksum) or
	// 10 characters may be a ISBN
	if(min.length() < 9 || 10 < min.length())
		return QString();
	
	if(min.length() == 10)
		min = min.left(min.length() - 1);
	min += checksum(min);
	if(isValid(min))
		return min;
	
	return QString();
}

bool ISBNValidator::isType(QString min) const
{
	if(isValid(min))
		return true;
	return false;
}

bool ISBNValidator::mayType(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return true;
	
	if(min.length() < 10)
		return true;
	
	if(isValid(validate(min)))
		return true;
	
	return false;
}

QString ISBNValidator::strip(QString min) const
{
	min = min.trimmed();
	min.replace(QRegExp("[^\\dXx]"), "");
	return min;
}

QStringList ISBNValidator::commonPrefixes(QStringList mins) const
{
	return QStringList();
}

QStringList ISBNValidator::minMaxPrefixes(QString min) const
{
	return QStringList();
}

bool ISBNValidator::operator==(const MINValidatorPlugin& plugin) const
{
	if(type() == plugin.type())
		return true;
	return false;
}

bool ISBNValidator::operator!=(const MINValidatorPlugin& plugin) const
{
	return !operator==(plugin);
}

QString ISBNValidator::checksum(QString min) const
{
	min = strip(min);

	if(min.length() == 10)
		min = min.left(min.length() - 1);
	
	if(min.length() != 9)
		return QString();
	
	int sum= 0;
	for(int i = 0; i < min.length(); i++){
		sum += (i+1) * min.mid(i, 1).toInt();
	}
	QString result = "0";
	sum = sum % 11;
	if(sum == 10){
		result = "X";
	}
	else{
		result = QString::number(sum);
	}
	return result;
}

Q_EXPORT_PLUGIN2 ( Meganizer_ISBN, ISBNValidator )
