/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "isbnvalidatortests.h"

#include "isbnvalidator.h"

#include <QtTest/QtTest>

ISBNValidatorTests::ISBNValidatorTests(QObject* parent): QObject(parent)
{
	validator = new ISBNValidator();
}

ISBNValidatorTests::~ISBNValidatorTests()
{
	delete validator;
}

void ISBNValidatorTests::strip_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << "5449000096241";
	QTest::newRow("5 449000 09623-1") << "5449000096231" << "5449000096231";
	QTest::newRow("5 449000 k9624-1") << "544  900adb _?& 41" << "54490041";
	QTest::newRow("9788559373257") << "9788559373257" << "9788559373257";
	QTest::newRow("855937325X") << "855937325X" << "855937325X";
	QTest::newRow("855937325x") << "855937325x" << "855937325x";
	QTest::newRow("85  ak 593 7325x") << "85  ak 593 7325x" << "855937325x";
}

void ISBNValidatorTests::strip()
{
	QFETCH(QString, min);
	QFETCH(QString, result);
	
	QCOMPARE(validator->strip(min), result);
}

void ISBNValidatorTests::isValid_data()
{
	QTest::addColumn<QString>("isbn");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("3772373259") << "3772373259" << true;
	QTest::newRow("37 723 732 59") << "37 723 732 59" << true;
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << true;
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << false;
	QTest::newRow("3772373258") << "3772373258" << false;
	QTest::newRow("3  7723 7325-8") << "3  7723 7325-8" << false;
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << false;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("111111111") << "111111111" << false;
	QTest::newRow("855937325X") << "855937325X" << true;
	QTest::newRow("855937325x") << "855937325x" << true;
}

void ISBNValidatorTests::isValid()
{
	QFETCH(QString, isbn);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isValid(isbn), result);
}

void ISBNValidatorTests::validate_data()
{
	QTest::addColumn<QString>("isbn");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << "3772373259";
	QTest::newRow("3-7723-7325") << "3-7723-7325" << "3772373259";
	QTest::newRow("377237325") << "377237325" << "3772373259";
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << "3772373259";
	QTest::newRow("3-7723-732X") << "3-7723-732X" << QString();
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << "3773732597";
	QTest::newRow("3-77kk-7325-9") << "3-77kk-7325-9" << QString();
	QTest::newRow("3-77k3-7325") << "3-77k3-7325" << QString();
	QTest::newRow("3-5723-7325-9") << "3-5773-7325-9" << "3577373253";
	QTest::newRow("3-5723-adbc-9") << "3-5773-adbc-9" << QString();
	QTest::newRow("12313123123213123213") << "12313123123213123213" << QString();
}

void ISBNValidatorTests::validate()
{
	QFETCH(QString, isbn);
	QFETCH(QString, result);
	
	QCOMPARE(validator->validate(isbn), result);
}

void ISBNValidatorTests::isType_data()
{
	QTest::addColumn<QString>("isbn");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("855937325x") << "855937325x" << true;
	QTest::newRow("855937325X") << "855937325X" << true;
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << false;
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << false;
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << true;
	QTest::newRow("3-7723-73-9") << "3-7723-73-9" << false;
}

void ISBNValidatorTests::isType()
{
	QFETCH(QString, isbn);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isType(isbn), result);
}

void ISBNValidatorTests::mayType_data()
{
	QTest::addColumn<QString>("ean");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("3772373259") << "3772373259" << true;
	QTest::newRow("37 723 732 59") << "37 723 732 59" << true;
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << true;
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << true;
	QTest::newRow("3772373258") << "3772373258" << true;
	QTest::newRow("3  7723 7325-8") << "3  7723 7325-8" << true;
	QTest::newRow("3-77k3-7325-9") << "3-77k3-7325-9" << true;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("111111111") << "111111111" << true;
	QTest::newRow("855937325X") << "855937325X" << true;
	QTest::newRow("855937325x") << "855937325x" << true;
	QTest::newRow("855937325asdasdx") << "85593asdvsdawerf7325x" << true;
	QTest::newRow("85593732123123123123x") << "85593732123123123123x" << false;
}

void ISBNValidatorTests::mayType()
{
	QFETCH(QString, ean);
	QFETCH(bool, result);
	
	QCOMPARE(validator->mayType(ean), result);
}

void ISBNValidatorTests::convert_data()
{
	QTest::addColumn<QString>("isbn");
	QTest::addColumn<QString>("ean");
	
	QTest::newRow("3-7723-7325-9") << "3-7723-7325-9" << "978377237325";
	QTest::newRow("3-7723-73-9") << "3-7723-73-9" << QString();
	QTest::newRow("3-7723-73123-9") << "3-7723-73123-9" << QString();
	QTest::newRow("3-7723-7325-8") << "3-7723-7325-8" << QString();
	QTest::newRow("3548609821") << "3548609821" << "978354860982";
	QTest::newRow("3442374928") << "3442374928" << "978344237492";
	QTest::newRow("855937325x") << "855937325x" << "978855937325";
	QTest::newRow("855937325X") << "855937325X" << "978855937325";
}

void ISBNValidatorTests::convert()
{
	QFETCH(QString, isbn);
	QFETCH(QString, ean);
	
	QCOMPARE(validator->convert(isbn, "EAN"), ean);
}

QTEST_APPLESS_MAIN(ISBNValidatorTests)

#include "isbnvalidatortests.moc"
