/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef ISSNVALIDATOR_H
#define ISSNVALIDATOR_H

#include <minvalidatorplugin.h>

#include <QString>

/**
	* @brief ISSN MIN Validator Plugin.
	*
	* ISSNs are used by Magazines. They can be 8 characters long
	* or 13 characters long. The 13 character version is an EAN
	* with 977 as the first 3 numbers. The characters 11 and 12
	* should be 0, unless they are used for special purposes.
	*/
class ISSNValidator : public MINValidatorPlugin
{
	Q_OBJECT
  Q_INTERFACES ( MINValidatorPlugin )
  Q_PROPERTY(QString name READ type)
  Q_PROPERTY(QString description READ description)
	public:
		ISSNValidator();
		
    QString type() const;
		QString description() const;
		bool canConvert(QString toType) const;
		QString convert(QString min, QString toType) const;
		bool isValid(QString min) const;
		QString validate(QString min) const;
		bool isType(QString min) const;
		bool mayType(QString min) const;
		QString strip(QString min) const;
		QStringList commonPrefixes(QStringList mins) const;
		QStringList minMaxPrefixes(QString min) const;
		
		bool operator== (const MINValidatorPlugin &plugin) const;
		bool operator!= (const MINValidatorPlugin &plugin) const;
		
	private:
		QString checksum(QString min) const;
};

#endif // ISSNVALIDATOR_H
