/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "issnvalidatortests.h"
#include "issnvalidatortests.moc"

#include "issnvalidator.h"

#include <QtTest/QtTest>

ISSNValidatorTests::ISSNValidatorTests(QObject* parent): QObject(parent)
{
	validator = new ISSNValidator();
}

ISSNValidatorTests::~ISSNValidatorTests()
{
	delete validator;
}

// void ISSNValidatorTests::checksum_data()
// {
// 	QTest::addColumn<QString>("ean");
// 	QTest::addColumn<int>("result");
// 	
// 	QTest::newRow("9783772373251") << "9783772373251" << 1;
// 	QTest::newRow("978377237325") << "978377237325" << 1;
// 	QTest::newRow("9783772373") << "9783772373" << -1;
// 	QTest::newRow("29033805") << "29033805" << 5;
// 	QTest::newRow("2903380") << "2903380" << 5;
// 	QTest::newRow("290338") << "290338" << -1;
// 	QTest::newRow("978377237325123123") << "978377237325123123" << -1;
// }

// void ISSNValidatorTests::checksum()
// {
// 	QFETCH(QString, ean);
// 	QFETCH(int, result);
// 	
// 	QCOMPARE(validator->checksum(ean), result);
// }

void ISSNValidatorTests::strip_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("0028-9604") << "0028-9604" << "00289604";
	QTest::newRow("00289604") << "00289604" << "00289604";
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << "1432640x";
	QTest::newRow("14& 32 - / 64kj0X") << "14& 32 - / 64kj0X" << "1432640X";
	QTest::newRow("1432640X") << "1432640X" << "1432640X";
	QTest::newRow("1432640x") << "1432640x" << "1432640x";
}

void ISSNValidatorTests::strip()
{
	QFETCH(QString, min);
	QFETCH(QString, result);
	
	QCOMPARE(validator->strip(min), result);
}

void ISSNValidatorTests::isValid_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("0028 - 9604") << "0028 - 9604" << true;
	QTest::newRow("00287604") << "00287604" << false;
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << false;
	QTest::newRow("1432640X") << "1432640X" << true;
	QTest::newRow("1432640x") << "1432640x" << true;
	QTest::newRow("14326402") << "14326402" << false;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << false;
}

void ISSNValidatorTests::isValid()
{
	QFETCH(QString, min);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isValid(min), result);
	
// 	QBENCHMARK{validator->isValid(ean);};
}

void ISSNValidatorTests::validate_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("result");

	QTest::newRow("0028 - 9604") << "0028 - 9604" << "00289604";
	QTest::newRow("00287604") << "00287604" << "00287601";
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << "1432640x";
	QTest::newRow("14& 3- / 64kj0x") << "14& 3- / 64kj0x" << QString();
	QTest::newRow("14320x") << "14340x" << QString();
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << "02896044";
}

void ISSNValidatorTests::validate()
{
	QFETCH(QString, min);
	QFETCH(QString, result);
	
	QCOMPARE(validator->validate(min), result);
}

void ISSNValidatorTests::isType_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<bool>("result");

	QTest::newRow("0028 - 9604") << "0028 - 9604" << true;
	QTest::newRow("00287604") << "00287604" << false;
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << false;
	QTest::newRow("1432640X") << "1432640X" << true;
	QTest::newRow("1432640x") << "1432640x" << true;
	QTest::newRow("14326402") << "14326402" << false;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << false;
}

void ISSNValidatorTests::isType()
{
	QFETCH(QString, min);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isType(min), result);
}

void ISSNValidatorTests::mayType_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<bool>("result");

	QTest::newRow("0028 - 9604") << "0028 - 9604" << true;
	QTest::newRow("00287604") << "00287604" << true;
	QTest::newRow("14& 32 - / 64kj0x") << "14& 32 - / 64kj0x" << true;
	QTest::newRow("1432640X") << "1432640X" << true;
	QTest::newRow("1432640x") << "1432640x" << true;
	QTest::newRow("14326402") << "14326402" << true;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("0289604") << "0289604" << true;
	QTest::newRow("02894") << "02894" << true;
	QTest::newRow("0212312894") << "028912334" << false;
}

void ISSNValidatorTests::mayType()
{
	QFETCH(QString, min);
	QFETCH(bool, result);
	
	QCOMPARE(validator->mayType(min), result);
}

QTEST_MAIN(ISSNValidatorTests)
