/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "upcvalidator.h"
#include "upcvalidator.moc"

#include <QObject>
#include <QtPlugin>
#include <QRegExp>

UPCValidator::UPCValidator()
{

}

QString UPCValidator::type() const
{
	return "UPC-A";
}

QString UPCValidator::description() const
{
	return tr("UPC-A - Universal Product Code is mostly used for Music and Movies");
}

bool UPCValidator::canConvert(QString toType) const
{
	if(toType == "EAN")
		return true;
	return false;
}

QString UPCValidator::convert(QString min, QString toType) const
{
	if(!canConvert(toType))
		return QString();

	min = strip(min);
	
	if(!isValid(min))
		return QString();
	
	// it's a UPC-A append 0 and compute a new check number to replace last number
	// computing new checknumber is done by MINValidator and the proper plugin
  QString result = "0" + min.left ( min.length() - 1 );
	return result;
}

bool UPCValidator::isValid(QString min) const
{
	min.replace(QRegExp("[\\-\\s]"), "");

	if(min != strip(min))
		return false;
	
	if(min.length() != 12)
		return false;

	if(checksum(min) == min.right(1).toInt())
		return true;
	
	return false;
}

QString UPCValidator::validate(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return min;
	
	if(min.length() < 11)
		return QString();
	
	if(min.length() > 12)
		return QString();
	
	if(min.length() == 12)
		min = min.left(min.length() - 1);
	min += QString::number(checksum(min));
	if(isValid(min))
		return min;
	
	return QString();
}

bool UPCValidator::isType(QString min) const
{
	if(isValid(min))
		return true;
	return false;
}

bool UPCValidator::mayType(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return true;
	
	if(min.length() < 12)
		return true;
	
	if(isValid(validate(min)))
		return true;
	
	return false;
}

QString UPCValidator::strip(QString min) const
{
	min = min.trimmed();
	min.replace(QRegExp("[^\\d]"), "");
	return min;
}

QStringList UPCValidator::commonPrefixes(QStringList mins) const
{
	return QStringList();
}

QStringList UPCValidator::minMaxPrefixes(QString min) const
{
	return QStringList();
}

bool UPCValidator::operator==(const MINValidatorPlugin& plugin) const
{
	if(type() == plugin.type())
		return true;
	return false;
}

bool UPCValidator::operator!=(const MINValidatorPlugin& plugin) const
{
	return !operator==(plugin);
}

int UPCValidator::checksum(QString min) const
{
	min = strip(min);
	
	// remove last digest, because checksum isn't needed
	if(min.length() == 12)
		min = min.left(min.length() - 1);
		
	if(min.length() != 11)
		return -1;
	
	int sumEven = 0;
	int sumOdd = 0;
	for(int i = 0; i < min.length(); i++){
		// because of i starts at 0, even numbers are not devideable by 2
		if(i % 2 != 0){
			sumEven += min.mid(i, 1).toInt();
		}
		else{
			sumOdd += min.mid(i, 1).toInt();
		}
	}
	int sum = 3*sumOdd + sumEven;
	if(sum < 1)
		return -1;
	int result = 10 - (sum % 10);

	return result;
}

Q_EXPORT_PLUGIN2 ( Meganizer_UPC, UPCValidator )
