/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "upcvalidatortests.h"

#include "upcvalidator.h"

#include <QtTest/QtTest>

UPCValidatorTests::UPCValidatorTests(QObject* parent): QObject(parent)
{
	validator = new UPCValidator();
}

UPCValidatorTests::~UPCValidatorTests()
{
	delete validator;
}

void UPCValidatorTests::strip_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("5 449000 09624-1") << "5 449000 09624-1" << "5449000096241";
	QTest::newRow("5 449000 09623-1") << "5449000096231" << "5449000096231";
	QTest::newRow("5 449000 k9624-1") << "544  900adb _?& 41" << "54490041";
	QTest::newRow("9788559373257") << "9788559373257" << "9788559373257";
	QTest::newRow("855937325X") << "855937325X" << "855937325";
	QTest::newRow("855937325x") << "855937325x" << "855937325";
	QTest::newRow("85  ak 593 7325x") << "85  ak 593 7325x" << "855937325";
}

void UPCValidatorTests::strip()
{
	QFETCH(QString, min);
	QFETCH(QString, result);
	
	QCOMPARE(validator->strip(min), result);
}

void UPCValidatorTests::isValid_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("012345678905") << "012345678905" << true;
	QTest::newRow("01 2345 6789 0") << "01 2345 6789 0" << false;
	QTest::newRow("01 2345-6789 0-5") << "01 2345-6789 0-5" << true;
	QTest::newRow("01 2345 6789 01") << "01 2345 6789 01" << false;
	QTest::newRow("01 2345 kk6789 05") << "01 2345 kk6789 05" << false;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("11111111115") << "11111111115" << false;
	QTest::newRow("111111111155") << "111111111155" << true;
	QTest::newRow("724354206825") << "724354206825" << true;
}

void UPCValidatorTests::isValid()
{
	QFETCH(QString, min);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isValid(min), result);
}

void UPCValidatorTests::validate_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("result");
	
	QTest::newRow("0-12345-67890-5") << "0-12345-67890-5" << "012345678905";
	QTest::newRow("0-12345-67890-7") << "0-12345-67890-7" << "012345678905";
	QTest::newRow("0-12345-67890") << "0-12345-67890" << "012345678905";
	QTest::newRow("72435420682") << "72435420682" << "724354206825";
	QTest::newRow("72435k4206825") << "72435k4206825" << "724354206825";
	QTest::newRow("7243kk206825") << "7243kk206825" << QString();
	QTest::newRow("724354206") << "724354206" << QString();
	QTest::newRow("12313123123213123213") << "12313123123213123213" << QString();
}

void UPCValidatorTests::validate()
{
	QFETCH(QString, min);
	QFETCH(QString, result);
	
	QCOMPARE(validator->validate(min), result);
}

void UPCValidatorTests::isType_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("7 24354 20682 5") << "724354206825" << true;
	QTest::newRow("0-12345-67890-5") << "0-12345-67890-5" << true;
	QTest::newRow("012345678905") << "012345678905" << true;
	QTest::newRow("012345678904") << "012345678904" << false;
	// last digit is the same as the missing checksum
	// should be false
	QTest::newRow("0123456k8905") << "0123456k8905" << false;
	QTest::newRow("012345605") << "012345905" << false;
}

void UPCValidatorTests::isType()
{
	QFETCH(QString, min);
	QFETCH(bool, result);
	
	QCOMPARE(validator->isType(min), result);
}

void UPCValidatorTests::mayType_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<bool>("result");
	
	QTest::newRow("012345678905") << "012345678905" << true;
	QTest::newRow("01 2345 6789 0") << "01 2345 6789 0" << true;
	QTest::newRow("01 2345-6789 0-5") << "01 2345-6789 0-5" << true;
	QTest::newRow("01 2345 6789 01") << "01 2345 6789 01" << true;
	QTest::newRow("01 2345 kkk789 05") << "01 2345 kkk789 05" << true;
	// checksum and last real number are the same and checksum is missing
	QTest::newRow("11111111115") << "11111111115" << true;
	QTest::newRow("111111111155") << "111111111155" << true;
	QTest::newRow("724354206825") << "724354206825" << true;
	
	QTest::newRow("0-12345-67890-7") << "0-12345-67890-7" << true;
}

void UPCValidatorTests::mayType()
{
	QFETCH(QString, min);
	QFETCH(bool, result);
	
	QCOMPARE(validator->mayType(min), result);
}

void UPCValidatorTests::convert_data()
{
	QTest::addColumn<QString>("min");
	QTest::addColumn<QString>("ean");
	
	QTest::newRow("7 24354 20682 5") << "724354206825" << "072435420682";
	QTest::newRow("0-12345-67890-5") << "0-12345-67890-5" << "001234567890";
	QTest::newRow("012345678905") << "012345678905" << "001234567890";
	QTest::newRow("012345678904") << "012345678904" << QString();
	QTest::newRow("0123456k8905") << "0123456k8905" << QString();
	QTest::newRow("012345605") << "012345905" << QString();
}

void UPCValidatorTests::convert()
{
	QFETCH(QString, min);
	QFETCH(QString, ean);
	
	QCOMPARE(validator->convert(min, "EAN"), ean);
}

QTEST_APPLESS_MAIN(UPCValidatorTests)

#include "upcvalidatortests.moc"
