/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "configamazon.h"

#include "amazon.h"

#include <mediamodel.h>
#include <medium.h>
#include <esettingsmodelitem.h>
#include <esetupwizardobject.h>
#include <connectmediasearchdialog.h>
#include "amazonsettings.h"
#include "itemsearch.h"
#include "connectmaterialtypedialog.h"
#include "purchase.h"
#include "progressstatuswidget.h"
#include "amazonsetupobject.h"
#include "minvalidator.h"

#include <QtPlugin>
#include <QSettings>
#include <QRegExp>
#include <QNetworkAccessManager>
#include <QDomDocument>
#include <QDomNode>
#include <QApplication>
#include <QDockWidget>
#include <QTextEdit>
#include <QLocale>

Amazon::Amazon (  ) : SearchPlugin()
{
  model = 0;
  http = 0;
  reply = 0;
  progDia = 0;
  currentPage = 1;
  itemSearch = 0;
  offerMedium = 0;

  logWindow = 0;
  logWidget = 0;
  reply = 0;

  //   verboseLevel = 0;
  itemLookup = false;
  offerMode = false;
}

Amazon::~Amazon()
{
// 	unload();
  delete logWindow;
  delete logWidget;
  delete progDia;
  delete itemSearch;
}

bool Amazon::initiate()
{
  return true;
}

bool Amazon::unload()
{
  delete logWindow;
	logWindow = 0;
  return true;
}

QString Amazon::name() const
{
  return tr ( "Amazon" );
}

QString Amazon::group()
{
  return "amazon";
}

QString Amazon::settingsGroup()
{
  return group();
}

QIcon Amazon::icon() const
{
  return QIcon ( ":/icons/amazon.png" );
}

QString Amazon::description() const
{
  return tr ( "Searches Amazon with Amazon Web Services" );
}

QMap<QString, QString> Amazon::supportedAttributes()
{
  QMap<QString, QString> attrs;
  // Book specials
  attrs.insert("edition", tr ( "Edition" ));
	attrs.insert("pages", tr ( "Pages" ));
	attrs.insert("pubDate", tr ( "Publication Date" ));
// 	attrs.insert("pubPlace", tr ( "Publication Place" ));
  // music specials
  attrs.insert("discs", tr ( "Number Of Discs" ));
	attrs.insert("track", tr ( "Track" ));
  // video specials
  return attrs;
}

QMap<QString, QString> Amazon::supportedCategories()
{
  QMap<QString, QString> cats;
	cats.insert("Books", tr("Books"));
	cats.insert("Music", tr("Music"));
	cats.insert("Music Tracks", tr("Music Tracks"));
	cats.insert("Video", tr("Video"));
	cats.insert("Software", tr("Software"));
	cats.insert("Magazines", tr("Magazines"));
	cats.insert("notsupported", tr("Not Supported"));
  return cats;
}

ESettingsModelItem *Amazon::settingsModelItem()
{
	AmazonSettings *settings = new AmazonSettings(this);
  return settings->toModelItem();
}

ESetupWizardObject* Amazon::wizardObject()
{
  return new AmazonSetupObject(this);
}

bool Amazon::search ( MediaModel* resultModel, QList< SqlConstraint > sqlConstraints )
{
  // wait until other fetching processes are finished
  while ( offerMedium ) {
    QApplication::processEvents();
  }

  while ( model ) {
    QApplication::processEvents();
  }

  offerMode = false;

  if ( !resultModel ) {
    emit information ( tr ( "No result model given!" ) );
    cleanUpSearch();
    return false;
  }

  model = resultModel;

  Medium *medium = resultModel->newMedium();

  if ( !prepareSearch ( medium ) ) {
    delete medium;
    cleanUpSearch();
    return false;
  }

  delete medium;

  // 	processing constraints
  foreach ( SqlConstraint constraint, sqlConstraints ) {
    if ( constraint.constraint().contains ( "materialhasmins" ) ) {
			if(constraint.bindValues().size() > 0){
				// try to get an EAN
				MINValidator validator;
				foreach(QVariant value, constraint.bindValues()){
					QString ean = validator.convert(value.toString().replace ( "%", "" ), "EAN");
					if(!ean.isEmpty()){
						itemSearch->setEan ( ean );
						break;
					}
				}
			}
    }
    else if ( constraint.constraint().contains ( "title" ) ) {
			if(constraint.bindValues().size() == 1)
				itemSearch->setTitle ( constraint.bindValues()[0].toString().replace ( "%", "" ) );
		}
		else if ( constraint.constraint().contains( "viewmaterialskeyword" ) ) {
			QList<QVariant> keywords = constraint.bindValues();
			foreach(QVariant keyword, keywords)
				itemSearch->setKeyword(keyword.toString());
		}
  }

  if ( itemSearch->isEmpty() ) {
    cleanUpSearch();
    return false;
  }

  return search();
}

void Amazon::recieveBuyData ( Medium *medium )
{
  // if offerMedium or model is not 0, another fetch is running, so we must wait until
  // the running fetch is finished
	QTime dieTime = QTime::currentTime().addSecs(5);
  while ( offerMedium ) {
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 500);
		if(QTime::currentTime() > dieTime){
			emit error(tr("Could not start recieving buy data, because of pending queries."));
			return;
		}
  }

	dieTime = QTime::currentTime().addSecs(5);
  while ( model ) {
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 500);
		if(QTime::currentTime() > dieTime){
			emit error(tr("Could not start recieving buy data, because of pending queries."));
			return;
		}
  }

  offerMode = true;

  offerMedium = medium;

	// we do not need the connection of attributes here, because
	// we do not set any special data. Also the connection
	// isn't working, because the Medium is running in compatibilityMode
  if ( !prepareSearch ( medium, true ) ) {
    cleanUpSearch();
    return;
  }

  itemSearch->setOfferMode();

  if ( medium->min("EAN").isEmpty() ) {
    itemSearch->setTitle ( medium->title() );

  } else {
    itemSearch->setEan ( medium->min("EAN") );
  }

  if ( itemSearch->isEmpty() ) {
    emit error ( tr ( "The created search request does not contain any conditions" ) );
    cleanUpSearch();
    return;
  }

  search();
}

bool Amazon::purchase ( QList< BuyData > items, QMap<Medium*, int> quantities )
{
  if ( showLogs ) {
    if ( !logWindow ) {
      logWindow = new QDockWidget ( tr ( "Amazon Log" ) );
      logWindow->setObjectName ( "amazonLogWidget" );
			QAction *logAction = new QAction ( tr ( "Amazon Log" ), logWindow );
			logAction->setObjectName("logAction");
			emit addAction ( logAction, "viewMenu" );
			logAction->setCheckable ( true );
			connect ( logWindow, SIGNAL ( visibilityChanged ( bool ) ), logAction, SLOT ( setChecked ( bool ) ) );
			connect ( logAction, SIGNAL ( triggered ( bool ) ), logWindow, SLOT ( setVisible ( bool ) ) );

      if ( !logWidget ) {
        logWidget = new QTextEdit ( logWindow );
        logWindow->setObjectName ( "amazonLogWidget" );
        connect ( this, SIGNAL ( error ( QString ) ), this, SLOT ( log ( QString ) ) );
        connect ( this, SIGNAL ( warning ( QString ) ), this, SLOT ( log ( QString ) ) );
        connect ( this, SIGNAL ( information ( QString ) ), this, SLOT ( log ( QString ) ) );
        connect ( this, SIGNAL ( logMsg ( QString ) ), this, SLOT ( log ( QString ) ) );
      }

      logWindow->setWidget ( logWidget );

      emit addNewDockWidget ( logWindow, Qt::BottomDockWidgetArea );
    }
  }

  Purchase *purchase = new Purchase ( items, quantities );

  connect ( purchase, SIGNAL ( error ( QString ) ), this, SLOT ( log ( QString ) ) );
  connect ( purchase, SIGNAL ( warning ( QString ) ), this, SLOT ( log ( QString ) ) );
  connect ( purchase, SIGNAL ( information ( QString ) ), this, SLOT ( log ( QString ) ) );
  connect ( purchase, SIGNAL ( logMsg ( QString ) ), this, SLOT ( log ( QString ) ) );

  if ( !purchase->prepareShoppingCart() )
    return false;

  if ( !purchase->checkShoppingCart() )
    return false;

  if(!purchase->checkoutClicked())
    return false;

  delete purchase;
  return true;
}

QMultiMap< QString, QString > Amazon::locales()
{
  QMultiMap<QString, QString> mapLocales;
  mapLocales.insert ( "CA", "ecs.amazonaws.ca" );
  mapLocales.insert ( "DE", "ecs.amazonaws.de" );
  mapLocales.insert ( "FR", "ecs.amazonaws.fr" );
  mapLocales.insert ( "JP", "ecs.amazonaws.jp" );
  mapLocales.insert ( "UK", "ecs.amazonaws.co.uk" );
  mapLocales.insert ( "US", "ecs.amazonaws.com" );
  return mapLocales;
}

bool Amazon::operator== ( const SearchPlugin& plugin ) const
{
  if(name() == plugin.name())
    return true;
  return false;
}

bool Amazon::operator!= ( const SearchPlugin& plugin ) const
{
  if(operator==(plugin))
    return false;
  return true;
}

void Amazon::log ( QString msg )
{
  if ( !logWidget )
    return;

  logWidget->append ( msg );
}

void Amazon::httpRequestError ( QNetworkReply::NetworkError netErr )
{
  QString msg = tr ( "Unknown" );

  if ( netErr == QNetworkReply::ConnectionRefusedError ) {
    msg = tr ( "Connection Refused" );
  }

  if ( netErr == QNetworkReply::RemoteHostClosedError ) {
    msg = tr ( "Remote host has closed the connection" );
  }

  if ( netErr == QNetworkReply::HostNotFoundError ) {
    msg = tr ( "Host not found" );
  }

  if ( netErr == QNetworkReply::TimeoutError ) {
    msg = tr ( "Timeout" );
  }

  if ( netErr == QNetworkReply::OperationCanceledError ) {
    msg = tr ( "Operation canceled" );
  }

  if ( netErr == QNetworkReply::ContentAccessDenied ) {
    msg = tr ( "Access denied" );
  }

  if ( netErr == QNetworkReply::ContentOperationNotPermittedError ) {
    msg = tr ( "Operation not permitted" );
  }

  if ( netErr == QNetworkReply::ContentNotFoundError ) {
    msg = tr ( "Page not found" );
  }

  if ( netErr == QNetworkReply::AuthenticationRequiredError ) {
    msg = tr ( "Authorisation required" );
  }

  if ( netErr == QNetworkReply::ProtocolUnknownError ) {
    msg = tr ( "Unknown protocol" );
  }

  if ( netErr == QNetworkReply::ProtocolInvalidOperationError ) {
    msg = tr ( "Protocol: Operation invalid" );
  }

  if ( netErr == QNetworkReply::UnknownNetworkError ) {
    msg = tr ( "Unknown network error" );
  }

  if ( netErr == QNetworkReply::UnknownProxyError ) {
    msg = tr ( "Unknown proxy error" );
  }

  if ( netErr == QNetworkReply::UnknownContentError ) {
    msg = tr ( "Unknown content error" );
  }

  if ( netErr == QNetworkReply::ProtocolFailure ) {
    msg = tr ( "protocol failure" );
  }

  // emit error produces a segfault, don't know why.
  emit logMsg ( tr ( "Request error\n%1\nat request\n%2." ).arg ( msg ).arg ( reply->url().toString() ) );
}

void Amazon::httpRequestFinished()
{
  if ( progDia ) {
    if ( progDia->wasCanceled() ) {
      cleanUpSearch();
      return;
    }
  }

  QString msg;

  if ( !reply ) {
    cleanUpSearch();
    return;
  }

  QNetworkReply::NetworkError netError = reply->error();

  if ( netError != QNetworkReply::NoError ) {
		if ( netError == QNetworkReply::SslHandshakeFailedError ) {
			emit logMsg( tr ( "SSL Handshake failed\n"
												"Ignore SSL-Error, but this may be a security risk!" ));
			reply->ignoreSslErrors();
		}
		else{
			httpRequestError ( netError );
			cleanUpSearch();
			return;
		}
  }

  // convert Data into a qdomdocument

  QApplication::processEvents();

  QByteArray data = reply->readAll();

  QDomDocument result;

  QDomNodeList node;

  QString errorStr;

  int errorLine;

  int errorColumn;

  if ( !result.setContent ( data, true, &errorStr, &errorLine, &errorColumn ) ) {
    emit error ( tr ( "Could not convert data into an XML-DOM-Document.\n"
                      "Error in Line %1, Column %2: %3 (s. log)" )
                 .arg ( errorLine )
                 .arg ( errorColumn )
                 .arg ( errorStr ) );
    emit logMsg ( QString ( data ) );
    emit logMsg ( tr ( "An empty file (Indicated by \"Error in Line 1, Column 1\")\n"
                       "is mostly returned, if the credentials are not correct!" ) );

    cleanUpSearch();
    return;
  }


  // first check wether the request was valid
  msg = tr ( "Checking wether request was valid" );

  emit logMsg ( msg );

  QApplication::processEvents();

	node = result.elementsByTagName ( "IsValid" );
  if ( node.size() != 1 ) {
    emit error ( tr ( "No valid document!\n(IsValid tag not found)" ) );
    emit logMsg ( result.toString() );

    cleanUpSearch();
    return;
  }
  else if ( node.item(0).toElement().text().toLower() != "true" ) {
    emit error ( tr ( "Request was not valid!" ) );
    emit logMsg ( result.toString() );

    cleanUpSearch();
    return;
  }

  // Check how many results on how much pages in what time are found
  msg = tr ( "Calculating the amount of results" );

  emit logMsg ( msg );

  QApplication::processEvents();

  totalPages = 0;

  int totalResults = 0;

	if(offerMode)
		node = result.elementsByTagName ( "TotalOffers" );
	else
		node = result.elementsByTagName ( "TotalResults" );

  if ( node.size() != 1 ) {
    msg = tr ( "No valid document!\n(Result count not found)" );
//     if(verboseLevel > 0){
//       msg += "\n\n"+ QString(data);
//     }
    emit error ( msg );
    emit logMsg ( result.toString() );

    cleanUpSearch();

    return;
  }
  else if ( ( node.item(0).toElement().text().toInt() <= 0 ) && ( !itemLookup ) ) {
    emit error ( tr ( "No results found!" ) );
    emit logMsg ( result.toString() );
//     if(verboseLevel > 0){
//       emit statusChanged("\n\n"+ QString(data));
//     }

    cleanUpSearch();

    return;
  }
  totalResults = node.item(0).toElement().text().toInt();

  if ( currentPage == 1 ) {
    int myMax = totalResults;

    if ( 10 * maxPages < totalResults ) {
      myMax = 10 * maxPages;
    }

    if ( progDia )
      progDia->setRange ( progDia->value(), myMax );
  }

	if(offerMode)
		node = result.elementsByTagName ( "TotalOfferPages" );
	else
		node = result.elementsByTagName ( "TotalPages" );

	if ( node.size() != 1 ) {
		emit error ( tr ( "No valid document!\n(Page count not found)" ) );
		emit logMsg ( result.toString() );
//     if(verboseLevel > 0){
//       emit statusChanged("\n\n"+ QString(data));
//     }
		cleanUpSearch();
		return;
	}
	else if ( ( node.item(0).toElement().text().toInt() <= 0 ) && ( !itemLookup ) ) {
		emit error ( tr ( "No results found (No pages)!" ) );
		emit logMsg ( result.toString() );
//     if(verboseLevel > 0){
//       emit statusChanged("\n\n"+ QString(data));
//     }
		cleanUpSearch();
		return;
	}
	totalPages = node.item(0).toElement().text().toInt();

	if ( currentPage == 1 ) {
		msg = tr ( "Found %1 on %2, parsing details..." )
					.arg ( tr ( "%n result(s)", "", totalResults ) )
					.arg ( tr ( "%n page(s)", "", totalPages ) );
	} else {
		if ( totalPages < maxPages ) {
			msg = tr ( "Parsing page %1 of %2" ).arg ( currentPage ).arg ( totalPages );
		} else {
			msg = tr ( "Parsing page %1 of %2 (of max. %3)" ).arg ( currentPage ).arg ( maxPages ).arg ( totalPages );
		}
	}

  emit logMsg ( msg );

  node = result.elementsByTagName ( "Item" );
	if(node.size() < 1){
		emit error(tr("No Items found!"));
				emit logMsg ( result.toString() );
//     if(verboseLevel > 0){
//       emit statusChanged("\n\n"+ QString(data));
//     }
		cleanUpSearch();
		return;
	}
	QDomElement item = node.item(0).toElement();
	

  if ( offerMode ) {
    parseOffer ( offerMedium, item );
  }
	else{
		while ( !item.isNull() ) {
			parseItem ( item );

			if ( progDia ) {
				// the check must be done here, because trying to set a value at progDia
				// after it is canceled, produces a segfault
				if ( progDia->wasCanceled() ) {
					cleanUpSearch();
					return;
				}

				progDia->setValue ( progDia->value() + 1 );
			}

			QApplication::processEvents();

			item = item.nextSiblingElement ( "Item" );
		}
	}

  if ( ( currentPage < totalPages ) && ( currentPage < maxPages ) ) {
    if ( progDia ) {
      if ( progDia->wasCanceled() ) {
        cleanUpSearch();
        return;
      }
    }

    currentPage++;
		emit logMsg ( "\n\n\n\n" + result.toString() );
    search();
    return;
  }

  cleanUpSearch();
	emit logMsg ( "\n\n\n\n" + result.toString() );
}

void Amazon::errorByPass(const QString& string)
{
	emit error(string);
}

void Amazon::warningByPass(const QString& string)
{
	emit warning(string);
}

void Amazon::informationByPass(const QString& string)
{
	emit information(string);
}

void Amazon::showHelpByPass(const QString& page)
{
	emit showHelp(page);
}

bool Amazon::prepareSearch ( Medium* medium, bool ignoreAttributeConnection )
{
  // the url is going to be appended to host later, all hosts can be used as http and https
  QSettings *settings = new QSettings();
  settings->beginGroup ( group() );
	QLocale qLocale;
	locale = qLocale.name().left(2).toUpper();
  locale   = settings->value ( "locale", locale ).toString();
  maxPages = settings->value ( "maxPages", 10 ).toInt();
  showLogs = settings->value ( "showLogs", false ).toBool();
  showLogs = settings->value ( "showLogs", false ).toBool();
  settings->endGroup();

  if ( showLogs ) {
    if ( !logWindow ) {
      logWindow = new QDockWidget ( tr ( "Amazon Log" ) );
      logWindow->setObjectName ( "amazonLogWidget" );
			QAction *logAction = new QAction ( tr ( "Amazon Log" ), logWindow );
			logAction->setObjectName("logAction");
			emit addAction ( logAction, "viewMenu" );
			logAction->setCheckable ( true );
			connect ( logWindow, SIGNAL ( visibilityChanged ( bool ) ), logAction, SLOT ( setChecked ( bool ) ) );
			connect ( logAction, SIGNAL ( triggered ( bool ) ), logWindow, SLOT ( setVisible ( bool ) ) );

      if ( !logWidget ) {
        logWidget = new QTextEdit ( logWindow );
        logWindow->setObjectName ( "amazonLogWidget" );
        connect ( this, SIGNAL ( error ( QString ) ), this, SLOT ( log ( QString ) ) );
        connect ( this, SIGNAL ( warning ( QString ) ), this, SLOT ( log ( QString ) ) );
        connect ( this, SIGNAL ( information ( QString ) ), this, SLOT ( log ( QString ) ) );
        connect ( this, SIGNAL ( logMsg ( QString ) ), this, SLOT ( log ( QString ) ) );
      }

      logWindow->setWidget ( logWidget );

      emit addNewDockWidget ( logWindow, Qt::BottomDockWidgetArea );
    }
  }

  if ( !itemSearch ) {
    itemSearch = new ItemSearch ();
    // get search category
    ConnectMaterialTypeDialog connType ( medium, this );
		connectByPass(&connType);
    if ( connType.exec() != QDialog::Accepted ) {
      return false;
    }
    if(connType.category() == "notsupported")
			return false;

    itemSearch->setIndex ( connType.category() );

    // load mappings
		if( !ignoreAttributeConnection ){
			if ( !loadAndCheckMappings ( medium ) ) {
				emit information ( tr ( "Settings are not correct" ) );
				return false;
			}
		}
  }

  if ( !itemSearch->checkQCA() ) {
    emit error ( tr ( "The signature can not be created due to missing crypting features!\n"
                      "Check your QCA2 installation (e.g. with qcatool2) and espacially the qca-ossl plugin." ) );
    return false;
  }

  // prepare progress dialog
  if ( progDia )
    delete progDia;

  progDia = new ProgressStatusWidget();
	emit showStatusWidget(progDia);
	progDia->setLabel(tr("Amazon: "));

  return true;
}

bool Amazon::search()
{
  if ( !itemSearch )
    return false;

  if ( currentPage == 1 ) {
    emit logMsg ( tr ( "Start request" ) );
    // set progress bar to undefined busy
//     emit setProgressMinMax(0,0);

  } else {
    emit logMsg ( tr ( "Fetching next page" ) );
  }

  // preparing ItemSearch
  QByteArray content;

  content.append ( itemSearch->getRequest ( currentPage ).toUtf8() );

  if ( !http ) {
    http = new QNetworkAccessManager ( this );
  }

// 	emit logMsg ( tr ( "URL: %1" ).arg ( urlCompl ) );
  emit logMsg ( tr ( "Request:\n%1" ).arg ( QString ( content ) ) );

  QNetworkRequest netRequest = itemSearch->netRequest();

  reply = http->post ( netRequest, content );

  connect ( reply, SIGNAL ( finished() ),
            this, SLOT ( httpRequestFinished() ) );

// 	connect ( reply, SIGNAL ( error ( QNetworkReply::NetworkError ) ),
// 	          this, SLOT ( httpRequestError ( QNetworkReply::NetworkError ) ) );

  return true;
}

void Amazon::parseItem ( QDomElement& item )
{
  // create a medium, which is not added to the model
  Medium *medium = model->newMedium();

  if ( !medium )
    return;
	medium->setSource(Medium::SearchPlugin);
	medium->setSourceIcon ( icon() );
	
  QDomElement child = item.firstChildElement();

  while ( !child.isNull() ) {
    if ( child.tagName() == "ItemAttributes" ) {
      parseItemAttributes ( medium, child );
    }

    if ( child.tagName() == "DetailPageURL" ) {
//       emit statusChanged("DetailPageURL: "+ child.text());
    }

    if ( child.tagName().contains ( "LargeImage" ) ) {
      QDomElement image = child.firstChildElement ( "URL" );
      medium->setImage ( image.text() );
    }

    if ( child.tagName() == "EditorialReviews" ) {
      QDomElement review = child.firstChildElement ( "EditorialReview" );
      QStringList reviewContents;

      while ( !review.isNull() ) {
        reviewContents.append ( review.firstChildElement ( "Content" ).text() );
        review = review.nextSiblingElement ( "EditorialReview" );
      }

      QString biggestString;

      foreach ( QString content, reviewContents ) {
        if ( biggestString.length() < content.length() ) {
          biggestString = content;
        }
      }

      medium->setDescription ( biggestString );
    }

    if ( child.tagName() == "Tracks" ) {
      parseTracks ( medium, child );
    }

    QApplication::processEvents();

    child = child.nextSiblingElement();
  }

  // add medium to the model if it is valid
  if ( !medium->isValid() ) {
    medium->deleteLater();
    return;
  }

  model->addMedium ( medium );
}

void Amazon::parseItemAttributes ( Medium* medium, QDomElement& element )
{
  if ( !medium )
    return;

  QDomElement child = element.firstChildElement();

  while ( !child.isNull() ) {
    if ( child.tagName() == "Title" ) {
      medium->setTitle( child.text() );
    }

    if ( child.tagName() == "Author" ) {
      QStringList person;
      person << child.text().right ( child.text().length() - child.text().lastIndexOf ( " " ) );
      person << child.text().left ( child.text().lastIndexOf ( " " ) );
      person << tr ( "Author" );
      medium->setMaterialPerson ( person[0], person[1], person[2] );
    }

    if ( child.tagName() == "Creator" ) {
      if ( child.attribute ( "Role" ) == "Herausgeber" ) {
        QStringList person;
        person << child.text().right ( child.text().length() - child.text().lastIndexOf ( " " ) );
        person << child.text().left ( child.text().lastIndexOf ( " " ) );
        person << tr ( "Editor" );
        medium->setMaterialPerson(person[0], person[1], person[2]);
      }
    }

    if ( child.tagName() == "Actor" ) {
      QStringList person;
      person << child.text().right ( child.text().length() - child.text().lastIndexOf ( " " ) );
      person << child.text().left ( child.text().lastIndexOf ( " " ) );
      person << tr ( "Actor" );
      medium->setMaterialPerson(person[0], person[1], person[2]);
    }

    if ( child.tagName() == "Artist" ) {
      QStringList person;
      person << child.text().right ( child.text().length() - child.text().lastIndexOf ( " " ) );
      person << child.text().left ( child.text().lastIndexOf ( " " ) );
      person << tr ( "Artist" );
      medium->setMaterialPerson(person[0], person[1], person[2]);
    }

    if ( child.tagName() == "Director" ) {
      QStringList person;
      person << child.text().right ( child.text().length() - child.text().lastIndexOf ( " " ) );
      person << child.text().left ( child.text().lastIndexOf ( " " ) );
      person << tr ( "Director" );
      medium->setMaterialPerson(person[0], person[1], person[2]);
    }

    if ( child.tagName() == "Edition" ) {
      QRegExp reg ( "^([0-9]+)" );

      if ( reg.indexIn ( child.text() ) > -1 )
        medium->setValue ( mapAttributes.value ( "edition" ), reg.cap ( 1 ) );
    }

		// only set the manufacturer, if it is not already present
		// (it may be already set, because it may be extracted from MIN)
    if ( (child.tagName() == "Publisher") &&  (medium->manufacturer() < 1) )
      medium->setManufacturer ( child.text() );
    if ( (child.tagName() == "Label") &&  (medium->manufacturer() < 1) )
      medium->setManufacturer ( child.text() );

    if ( child.tagName() == "PublicationDate" ) {
      medium->setValue ( mapAttributes.value ( "pubDate" ), extractDate ( child.text() ) );
    }

    if ( child.tagName() == "ReleaseDate" ) {
      medium->setValue ( mapAttributes.value ( "pubDate" ), extractDate ( child.text() ) );
    }

    if ( child.tagName() == "TheatricalReleaseDate" ) {
      medium->setValue ( mapAttributes.value ( "pubDate" ), extractDate ( child.text() ) );
    }

//     if((child.tagName() == "Feature")){
//       resultModelMovie->newDetailRating(mediaIndex, child.text());
//     }

		if ( child.tagName() == "EAN" )
			medium->setMIN ( child.text(), "EAN" );
		if ( child.tagName() == "ISBN" )
			medium->setMIN ( child.text(), "ISBN-10" );
		if ( child.tagName() == "UPC" )
			medium->setMIN ( child.text(), "UPC" );
		
		if ( child.tagName() == "NumberOfPages" )
			medium->setValue ( mapAttributes.value ( "pages" ), child.text() );

//     if(child.tagName() == "Binding"){
//       resultModel->newMediaType(mediaIndex, mediaTypeMapping.value(child.text()));
//     }

		if ( child.tagName() == "NumberOfDiscs" )
			medium->setValue ( mapAttributes.value ( "discs" ), child.text() );

		if(child.tagName() == "ListPrice"){
			QDomElement tmp = child.firstChildElement("Amount");
			QString text = tmp.text();
			text.insert(text.length() -2, ",");
			medium->setListPrice ( text.toDouble() );
			tmp = child.firstChildElement("CurrencyCode");
			medium->setListPriceCurrency ( tmp.text() );
		}
		child = child.nextSiblingElement();

		QApplication::processEvents();
  }
}

void Amazon::parseTracks ( Medium* medium, QDomElement& element )
{
  if ( !medium )
    return;

  QDomElement child = element.firstChildElement ( "Disc" );

  while ( !child.isNull() ) {
    int cdNum = child.attribute ( "Number" ).toInt();
    QDomElement track = child.firstChildElement ( "Track" );

    while ( !track.isNull() ) {
      QStringList tracks;
      tracks << QString::number ( cdNum );;
      tracks << track.attribute ( "Number" );
      tracks << track.text();
      // unkown length
      tracks << "";
      medium->setValue ( mapAttributes.value ( "track" ), tracks );
      track = track.nextSiblingElement();
    }

    child = child.nextSiblingElement();

    QApplication::processEvents();
  }
}

void Amazon::parseOffer ( Medium* medium, QDomElement& element )
{
  if ( !medium )
    return;

  QList<BuyData> buyData;

  // loading settings
  QSettings settings;

  settings.beginGroup ( group() );

  buyOrder buyMode = ( buyOrder ) settings.value ( "buyingMode", AmazonOnly ).toInt();

#if QT_VERSION < 0x040600
	float feedbackBorder = settings.value ( "minFeedbackRating" ).toDouble();
#else
  float feedbackBorder = settings.value ( "minFeedbackRating" ).toFloat();
#endif

  settings.endGroup();

  QDomElement asin = element.firstChildElement ( "ASIN" );

  if ( asin.text().isEmpty() )
    return;

//   QDomElement summary = element.firstChildElement ( "OfferSummary" );

  QDomElement offer = element.firstChildElement ( "Offers" ).firstChildElement ( "Offer" );

  while ( !offer.isNull() ) {
    QDomElement child;

    // checking merchant
    child = offer.firstChildElement ( "Merchant" );

    if ( !child.isNull() ) {
      if ( buyMode == AmazonOnly ) {
        if ( !child.firstChildElement ( "Name" ).text().contains("Amazon") ) {
          offer = offer.nextSiblingElement();
          QApplication::processEvents();
          continue;
        }

      } else {
        // check rating border for other merchants than amazon
        if ( child.firstChildElement ( "AverageFeedbackRating" ).text().toFloat() < feedbackBorder ) {
          offer = offer.nextSiblingElement();
          QApplication::processEvents();
          continue;
        }
      }
    }

    BuyData data;

    data.medium = medium;
    data.shop = this;
    data.inCatalog = true;
    
    if ( offer.firstChildElement ( "OfferAttributes" ).firstChildElement ( "Condition" ).text().toLower() != "new" )
      data.isUsed = true;

    QDomElement offerListing = offer.firstChildElement ( "OfferListing" );
    QDomElement tmp = offerListing.firstChildElement ( "OfferListingId" );
    data.customShopData = "<ASIN>"+ asin.text() +"</ASIN><OfferListingId>"+ tmp.text() +"</OfferListingId>";
    tmp = offerListing.firstChildElement ( "Price" );
    tmp = tmp.firstChildElement ( "Amount" );
    data.price = tmp.text().toFloat() / 100.0;
    tmp = tmp.nextSiblingElement ( "CurrencyCode" );
    data.currencyShortName = tmp.text();

    tmp = offerListing.firstChildElement ( "Quantity" );
    int quantity = tmp.text().toInt();

    if ( quantity == -1 )
      quantity = 999; // this is the maximum value of one item on a shopping card

    data.available = quantity;

		buyData << data;

    offer = offer.nextSiblingElement();

    QApplication::processEvents();
  }

  if ( buyData.size() < 1 )
    return;

	// found lowest new price
	BuyData resultNew, resultUsed;
	foreach ( BuyData data, buyData )
	{
		if(data.isUsed){
			if(resultUsed.price == 0)
				resultUsed = data;
			else if(resultUsed.price > data.price)
				resultUsed = data;
		}
		else{
			if(resultNew.price == 0)
				resultNew = data;
			else if ( resultNew.price > data.price )
				resultNew = data;
		}
	}

  if( resultNew.price > 0)
    emit foundBuyData ( resultNew );
  if( resultUsed.price > 0)
    emit foundBuyData ( resultUsed );
}

bool Amazon::loadAndCheckMappings ( Medium *medium )
{
  if ( !medium )
    return false;

  ConnectMediaSearchDialog *mapAttrs = new ConnectMediaSearchDialog ();
	connectByPass(mapAttrs);
  mapAttrs->setPlugins ( medium, this );

	// the result is only for special data important
	// therefore we continue even on false
  mapAttrs->exec();

  delete mapAttrs;

  // loading attribute mapping
  QSettings settings;

  settings.beginGroup ( "mapping" + group() + medium->group() );

  QStringList mediaAttrs = settings.value ( "mediaAttrs" ).toStringList();

  QStringList searchAttrs = settings.value ( "searchAttrs" ).toStringList();

  settings.endGroup();


  if ( mediaAttrs.size() != searchAttrs.size() )
    return false;

  for ( int i = 0; i < mediaAttrs.size(); i++ ) {
    mapAttributes.insert ( searchAttrs[i], mediaAttrs[i] );
  }

  return true;
}

QDate Amazon::extractDate ( QString dateString )
{
  QDate extDate = QDate::fromString ( dateString, "yyyy-MM-dd" );

  if ( extDate.isValid() )
    return extDate;

  extDate = QDate::fromString ( dateString, "yyyy-MM" );

  if ( extDate.isValid() )
    return extDate;

  extDate = QDate::fromString ( dateString, "yyyy" );

  if ( extDate.isValid() )
    return extDate;

  extDate = QDate::fromString ( dateString, "MM-yyyy" );

  if ( extDate.isValid() )
    return extDate;

  extDate = QDate::fromString ( dateString, "dd-MM-yyyy" );

  if ( extDate.isValid() )
    return extDate;

  return extDate;
}

void Amazon::cleanUpSearch()
{
  delete progDia;
  progDia = 0;

  delete itemSearch;
  itemSearch = 0;

  if ( reply )
    reply->deleteLater();

  reply = 0;

  model = 0;

  offerMedium = 0;

  currentPage = 1;

  offerMode = false;
}

void Amazon::connectByPass(QObject* object)
{
	connect ( object, SIGNAL ( error ( const QString& ) ), this, SLOT ( errorByPass ( const QString& ) ) );
  connect ( object, SIGNAL ( warning ( const QString& ) ), this, SLOT ( warningByPass ( const QString& ) ) );
  connect ( object, SIGNAL ( information ( const QString& ) ), this, SLOT ( informationByPass ( const QString& ) ) );
	
	connect ( object, SIGNAL ( showHelp( QString ) ), this, SLOT ( showHelpByPass( QString ) ) );
}

#include "amazon.moc"

Q_EXPORT_PLUGIN2 ( Meganizer_Amazon, Amazon )
