/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef AMAZON_H
#define AMAZON_H

#include "meganizerplugin.h"
#include "searchplugin.h"

class MediaModel;
class ItemSearch;
class ProgressStatusWidget;

class QNetworkAccessManager;
class QTextEdit;

#include <QString>
#include <QStringList>
#include <QMultiMap>
#include <QNetworkReply>
#include <QtPlugin>
#include <QObject>
#include <QDomElement>
#include <QIcon>

/**
	* searches amazon database with help of the Amazon Product Advertising API
	* @attention Requeires an AWS account (should be without charge)
	* @todo Use QMutexLocker for offerMedium, model and offerMode to prevent starting a new search if another is running
  */
class Amazon : public SearchPlugin
{
	Q_OBJECT
	Q_INTERFACES ( SearchPlugin MeganizerPlugin )
	Q_PROPERTY(QString name READ name)
	Q_PROPERTY(QString description READ description)
	Q_PROPERTY(QIcon icon READ icon)
	public:
		Amazon ( );
		~Amazon();

		enum buyOrder { AmazonOnly, All };

		bool initiate();
		bool unload();
		QString name() const;
		static QString group();
		QString settingsGroup();
		QIcon icon() const;
		QString description() const;
		QMap<QString, QString> supportedAttributes();
		QMap<QString, QString> supportedCategories();
		ESettingsModelItem *settingsModelItem();
		ESetupWizardObject *wizardObject();
		bool search ( MediaModel *resultModel, QList<SqlConstraint> sqlConstraints );
		void recieveBuyData ( Medium *medium );
		bool purchase ( QList<BuyData> items, QMap<Medium*, int> quantities );
		static QMultiMap<QString, QString> locales();
		bool operator== (const SearchPlugin &plugin) const;
		bool operator!= (const SearchPlugin &plugin) const;

	private slots:
		void log ( QString msg );
		void httpRequestError ( QNetworkReply::NetworkError netErr );
		void httpRequestFinished();

		void errorByPass ( const QString &string );
		void warningByPass ( const QString &string );
		void informationByPass ( const QString &string );

		void showHelpByPass( const QString &page );

	private:
		bool prepareSearch ( Medium* medium, bool igoreAttributeConnection = false );
		bool search();
		void parseItem ( QDomElement &item );
		/**
			* @todo map the roles of creators
			*/
		void parseItemAttributes ( Medium *medium, QDomElement &element );
		void parseTracks ( Medium *medium, QDomElement &element );
		/**
			* @todo cheapest isn't working, because used offers are not returned by amazon
			* @todo If a cheap item is not available with the selected quantity and the choosen seller, but with another seller it is, it is not shown green
			*/
		void parseOffer ( Medium *medium, QDomElement &element );
		bool loadAndCheckMappings ( Medium *medium );
		QDate extractDate ( QString dateString );
		void cleanUpSearch();

		void connectByPass(QObject *object);
		/**
			* map our attributes to the attributes of the MediaModel
			* supplied by search();
			*/
		QMap<QString, QString> mapAttributes;
		MediaModel *model;
		Medium *offerMedium;
		int maxPages, currentPage, totalPages;
		bool itemLookup, showLogs, offerMode;
		QString locale;
		QNetworkAccessManager *http;
		QNetworkReply *reply;
		ProgressStatusWidget *progDia;
		QDockWidget *logWindow;
		QTextEdit *logWidget;
		ItemSearch *itemSearch;

	signals:
		void foundBuyData ( const BuyData &data );
		void logMsg(QString msg);
};
#endif // AMAZON_H
