/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "amazonconntypesettings.h"
#include "amazonconntypesettings.moc"

#include "configamazon.h"
#include <mediaplugin.h>
#include <mediamodel.h>
#include "amazon.h"
#include "connectmaterialtype.h"
#include <mpluginloader.h>

AmazonConnTypeSettings::AmazonConnTypeSettings(Amazon* amazon, QWidget* parent, Qt::WindowFlags flags)
	: ESettingsWidget(parent, flags)
{
	setName(tr("Type Connections"));
	
	this->amazon = amazon;
	if(!amazon)
		return;
	
	setupUi(this);
	connMatType = qobject_cast<ConnectMaterialType*>(connectMaterialType);
	connectByPass(connMatType);

	QStringList types;
	// create list with plugins
	QList<MediaPlugin*> plugins = MPluginLoader<MediaPlugin>::loadPlugins();
  foreach ( MediaPlugin* plugin, plugins ) {
		mediaPlugins.append(plugin);
		QStringList untransTypes = plugin->supportedMaterialTypes();
		foreach(QString type, untransTypes){
			QStringList result = plugin->translateMaterialType(type);
			if(result.size() == 2 && !result[0].isEmpty())
				types << result[0];
		}
  }
  
  types.sort();
	typeEdit->addItems(types);
	
	connect(typeEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(showConnType()));
	showConnType();
}


bool AmazonConnTypeSettings::saveSettings(QString &error)
{
	return connMatType->saveSettings(error);
}

void AmazonConnTypeSettings::showConnType()
{
	if(!amazon)
		return;

	QString error;
	connMatType->saveSettings(error);
	
	QString type;
	foreach(MediaPlugin *plugin, mediaPlugins){
		type = plugin->untranslateMaterialType(typeEdit->currentText());
		if(type.isEmpty() || !plugin->supportedMaterialTypes().contains(type))
			continue;
		
		MediaModel *model = plugin->newModel(type);
		Medium *med = model->newMedium();
		connMatType->setPlugins(med->group(), amazon);
		delete med;
		delete model;
	}
}
