/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "configamazon.h"

#include "amazongeneralsettings.h"
#include "amazongeneralsettings.moc"

#include "amazon.h"

#include <QSettings>
#include <QLabel>
#include <QGridLayout>
#include <QLineEdit>
#include <QComboBox>
#include <QStringList>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QCheckBox>
#include <QLocale>

AmazonGeneralSettings::AmazonGeneralSettings ( ESetupWizardObject* object, QWidget* parent, Qt::WindowFlags flags )
    : ESetupWizardWidget ( object, parent, flags )
{
  setName ( tr ( "General" ) );

  mainLayout  = new QGridLayout ( this );
  token       = new QLineEdit ( this );
  secret			= new QLineEdit ( this );
  secret->setEchoMode ( QLineEdit::Password );
  locale      = new QComboBox ( this );
  maxPages    = new QSpinBox ( this );
  maxPages->setMinimum ( 1 );
  maxPages->setMaximum ( 400 );  // this value is defined by amazon see docu of amazon
  maxPages->setSingleStep ( 10 );
  logWindowEdit = new QCheckBox ( this );
  buyingModes = new QComboBox ( this );
  minFeedbackRating = new QDoubleSpinBox ( this );
  minFeedbackRating->setMinimum ( 0 );
  minFeedbackRating->setMaximum ( 5 );
  minFeedbackRating->setSingleStep ( 0.1 );

  mainLayout->addWidget ( new QLabel ( tr ( "AWS Access Key:" ) ), 0, 0 );
  mainLayout->addWidget ( new QLabel ( tr ( "AWS Secret:" ) ), 1, 0 );
  mainLayout->addWidget ( new QLabel ( tr ( "State:" ) ), 2, 0 );
  mainLayout->addWidget ( new QLabel ( tr ( "Max. Pages per Search:\n(10 Items per Page)" ) ), 3, 0 );
  mainLayout->addWidget ( new QLabel ( tr ( "Show Logs:" ) ), 4, 0 );
  mainLayout->addWidget ( new QLabel ( tr ( "Buying mode:" ) ), 5, 0 );
  mainLayout->addWidget ( new QLabel ( tr ( "Min. Feedback Rating:" ) ), 6, 0 );
  mainLayout->addWidget ( token, 0, 1 );
  mainLayout->addWidget ( secret, 1, 1 );
  mainLayout->addWidget ( locale, 2, 1 );
  mainLayout->addWidget ( maxPages, 3, 1 );
  mainLayout->addWidget ( logWindowEdit, 4, 1 );
  mainLayout->addWidget ( buyingModes, 5, 1 );
  mainLayout->addWidget ( minFeedbackRating, 6, 1 );

  locale->addItems ( Amazon::locales().uniqueKeys() );

  mapBuyingOrder.insert ( Amazon::AmazonOnly, tr ( "Only from amazon (Only new Items)" ) );
  mapBuyingOrder.insert ( Amazon::All, tr ( "All Merchants (New and used Items)" ) );
  buyingModes->addItems ( mapBuyingOrder.values() );

  QSettings settings;
  settings.beginGroup ( Amazon::group() );
  token->setText ( settings.value ( "token" ).toString() );
  secret->setText ( settings.value ( "secret" ).toString() );
	QLocale qLocale;
  locale->setCurrentIndex (
		locale->findText (
			settings.value ( "locale", qLocale.name().left(2).toUpper() ).toString()
		)
	);
  if ( settings.value ( "maxPages" ).toInt() < 1 ) {
    maxPages->setValue ( 10 );
  } else {
    maxPages->setValue ( settings.value ( "maxPages" ).toInt() );
  }
  logWindowEdit->setChecked ( settings.value ( "showLogs", false ).toBool() );
  buyingModes->setCurrentIndex ( buyingModes->findText ( mapBuyingOrder.value ( settings.value ( "buyingMode", Amazon::AmazonOnly ).toInt() ) ) );
#if QT_VERSION < 0x040600
	minFeedbackRating->setValue ( settings.value ( "minFeedbackRating", 4.5 ).toDouble() );
#else
  minFeedbackRating->setValue ( settings.value ( "minFeedbackRating", 4.5 ).toFloat() );
#endif
  settings.endGroup();

	mainLayout->setRowStretch(mainLayout->rowCount() - 1, 20);
  setLayout ( mainLayout );

}

AmazonGeneralSettings::~AmazonGeneralSettings()
{
  delete mainLayout;
  delete token;
  delete locale;
  delete maxPages;
  delete logWindowEdit;
}

bool AmazonGeneralSettings::saveSettings(QString& error)
{
  QSettings settings;
  settings.beginGroup ( Amazon::group() );
  settings.setValue ( "token", token->text() );
  settings.setValue ( "secret", secret->text() );
  settings.setValue ( "locale", locale->currentText() );
  settings.setValue ( "maxPages", maxPages->value() );
  settings.setValue ( "showLogs", logWindowEdit->isChecked() );
  settings.setValue ( "buyingMode", mapBuyingOrder.key ( buyingModes->currentText() ) );
  settings.setValue ( "minFeedbackRating", minFeedbackRating->value() );
  settings.endGroup();
  return true;
}
