/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "configamazon.h"
#include "connectmaterialtype.h"
#include "connectmaterialtype.moc"

#include <mediamodel.h>
#include <searchplugin.h>
#include <mediaplugin.h>
#include <mpluginloader.h>

#include <QSettings>

ConnectMaterialType::ConnectMaterialType(Medium *medium, SearchPlugin* searchPlugin, QWidget* parent,	Qt::WindowFlags flags)
	: ESettingsWidget(parent, flags)
{
	// setting up ui
	setupUi(this);
	
	setPlugins(medium, searchPlugin);
}

ConnectMaterialType::ConnectMaterialType(QWidget* parent, Qt::WindowFlags flags)
	: ESettingsWidget(parent, flags)
{
	setupUi(this);
	setPlugins(QString(), NULL);
}


void ConnectMaterialType::setPlugins(QString mediumGroup, SearchPlugin* searchPlugin)
{
	this->searchPlugin = searchPlugin;
	if(!searchPlugin)
		return;
	
	this->mediumGroup = mediumGroup;
	
	typeEdit->clear();
	typeLabel->setText
	(
		tr("Please select a category of %1, which should be searched for material type %2:")
			.arg(searchPlugin->name())
			.arg(name())
	);
	mapTypes = searchPlugin->supportedCategories();
	typeEdit->addItems(mapTypes.values());
	
	QSettings settings;
	settings.beginGroup(searchPlugin->settingsGroup());
	QString stored = settings.value(mediumGroup, QString()).toString();
	typeEdit->setCurrentIndex(typeEdit->findText(mapTypes.value(stored)));
	settings.endGroup();
}

void ConnectMaterialType::setPlugins(Medium* medium, SearchPlugin* searchPlugin)
{
	if(!medium)
		return;
	
	QString group = medium->group();
	// this means, it is a common Medium,
	// but it may know its type, if it is from database
	if(group.isEmpty() && medium->type() > 0){
		QList<MediaPlugin*> plugins = MPluginLoader<MediaPlugin>::loadPlugins();
		foreach(MediaPlugin *plugin, plugins){
			if(plugin->supportedMaterialTypes().contains(medium->typeName())){
				MediaModel *model = plugin->newModel(medium->typeName());
				if(model){
					Medium *med = model->newMedium();
					group = med->group();
					delete med;
					delete model;
				}
				break;
			}
		}
	}
	
	setPlugins(group, searchPlugin);
}

QString ConnectMaterialType::category()
{
	return mapTypes.key(typeEdit->currentText());
}

bool ConnectMaterialType::loadSettings()
{
	if(!searchPlugin)
		return false;
	
	if(mediumGroup.isEmpty())
		return false;
	
	bool result = false;
	QSettings settings;
	settings.beginGroup(searchPlugin->settingsGroup());
	QString stored = settings.value(mediumGroup, QString()).toString();
	if(typeEdit->findText(mapTypes.value(stored)) > -1)
		result = true;
	typeEdit->setCurrentIndex(typeEdit->findText(mapTypes.value(stored)));
	settings.endGroup();
	
	return result;
}

bool ConnectMaterialType::saveSettings(QString& error)
{
	if(!searchPlugin){
		error.append(tr("No search plugin given"));
		return false;
	}
	
	if(mediumGroup.isEmpty()){
		error.append(tr("Medium has empty group"));
		return false;
	}
	
	QSettings settings;
	settings.beginGroup(searchPlugin->settingsGroup());
	settings.setValue(mediumGroup, mapTypes.key(typeEdit->currentText()));
	settings.endGroup();

	return true;
}
