/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "configamazon.h"

#include "itemsearch.h"

#include "amazon.h"

#include <QSettings>
#include <QDateTime>
#include <QLocale>

ItemSearch::ItemSearch(QString awsKey, QString awsSecret)
{
	token = QCA::SecureArray(awsKey.toUtf8());
	secret = QCA::SecureArray(awsSecret.toUtf8());
	
  QSettings *settings = new QSettings();
  settings->beginGroup ( Amazon::group() );
	
	if(token.isEmpty())
		token    = QCA::SecureArray(settings->value ( "token" ).toString().toUtf8());
  // we must check it afterwards, because if an empty value is stored, the keys are not used
  if ( token.isEmpty() )
    token = QCA::SecureArray(QString ( AWSACCESSKEY ).toUtf8());
	
	if(secret.isEmpty())
		secret 	 = QCA::SecureArray(settings->value ( "secret" ).toString().toUtf8());
  if ( secret.isEmpty() )
    secret = QCA::SecureArray(QString ( AWSSECRET ).toUtf8());
  
	QLocale qLocale;
	locale = qLocale.name().left(2).toUpper();
	locale   = settings->value ( "locale", locale ).toString();
  settings->endGroup();

  offerMode = false;
  cartCreateMode = false;
  cartAddMode = false;
  cartGetMode = false;
  debuggingMode = false;
}

ItemSearch::~ItemSearch()
{
	token.clear();
	secret.clear();
}

void ItemSearch::setKeyword ( QString string )
{
  if ( string.isEmpty() )
    return;

  keywords << string;
}

void ItemSearch::setTitle ( QString string )
{
  title = string;
}

void ItemSearch::setIndex ( QString string )
{
  searchIndex = string;
}

void ItemSearch::setCreator ( QString string )
{
  creator = string;
}

void ItemSearch::setManufacturer ( QString string )
{
  manufacturer = string;
}

void ItemSearch::setEan ( QString string )
{
  ean = string;
}

bool ItemSearch::addItem ( QString offerListingId, int quantity )
{
  if ( offerItems.size() == 10 )
    return false;

  offerItems.insert ( offerListingId, quantity );
  return true;
}

void ItemSearch::setOfferMode ( bool enable )
{
  offerMode = enable;
}

void ItemSearch::setCartCreateMode ( bool enable )
{
  cartCreateMode = enable;
}

void ItemSearch::setCartAddMode ( QString cartId, QString cartHmac, bool enable )
{
  this->cartId = cartId;
  this->cartHmac = cartHmac;
  cartAddMode = enable;
}

void ItemSearch::setCartGetMode ( QString cartId, QString cartHmac, bool enable )
{
  this->cartId = cartId;
  this->cartHmac = cartHmac;
  cartGetMode = enable;
}

void ItemSearch::setDebugging ( bool enable )
{
  debuggingMode = enable;
}

QString ItemSearch::getRequest ( int page, QString timeStamp )
{
  QString result;
  result  = soapHeader(timeStamp);
  if ( ( cartAddMode ) || ( cartCreateMode ) || ( cartGetMode ) ) {
// 			result += "       <ns1:AssociateTag>"+ associateId +"</ns1:AssociateTag>\n";
    if ( ( cartAddMode ) || ( cartGetMode ) ) {
      result += "       <ns1:CartId>"+ cartId +"</ns1:CartId>\n";
      result += "       <ns1:HMAC>"+ cartHmac +"</ns1:HMAC>\n";
    }
    if ( !cartGetMode )
      result += prepareOffers();
  } else {
    result += prepareIndex();
    result += prepareTitle();
    result += prepareCreator();
    result += prepareManufacturer();
    result += prepareEan();
    result += prepareKeyword();
    // fetch offers from all merchant's
    result += "       <ns1:MerchantId>All</ns1:MerchantId>\n";
		if(offerMode)
			result += "       <ns1:OfferPage>"+ QString::number ( page ) +"</ns1:OfferPage>\n";
		else
			result += "       <ns1:ItemPage>"+ QString::number ( page ) +"</ns1:ItemPage>\n";
    result += "       <ns1:Condition>All</ns1:Condition>\n";
  }
  if ( debuggingMode ) {
    result += "       <ns1:Validate>True</ns1:Validate>\n";
  }
  result += responseGroups();
  result += soapFooter();

  return result;
}

QNetworkRequest ItemSearch::netRequest()
{
  QString urlCompl = "https://" + Amazon::locales().value ( locale ) +"/onca/soap?Service=AWSECommerceService";

  QNetworkRequest request;

  request.setUrl ( QUrl ( urlCompl ) );
  if ( cartCreateMode ) {
    request.setRawHeader ( QString ( "SOAPAction" ).toAscii(), QString ( "http://soap.amazon.com/CartCreate" ).toAscii() );
  } else if ( cartAddMode ) {
    request.setRawHeader ( QString ( "SOAPAction" ).toAscii(), QString ( "http://soap.amazon.com/CartAdd" ).toAscii() );
  } else if ( cartGetMode ) {
    request.setRawHeader ( QString ( "SOAPAction" ).toAscii(), QString ( "http://soap.amazon.com/CartGet" ).toAscii() );
  } else {
    request.setRawHeader ( QString ( "SOAPAction" ).toAscii(), QString ( "http://soap.amazon.com/ItemSearch" ).toAscii() );
  }
  request.setHeader ( QNetworkRequest::ContentTypeHeader, QVariant ( "text/xml; charset=UTF-8" ) );
  request.setRawHeader ( QString ( "Connection" ).toAscii(), QString ( "Keep-Alive" ).toAscii() );

  return request;
}

bool ItemSearch::checkQCA()
{
  // we need this algorithm, if it is not supported,
  // we cannot create a signed request
  return QCA::isSupported ( "hmac(sha256)" );
}

bool ItemSearch::isEmpty()
{
  if ( cartCreateMode )
    return false;

  if ( ( cartAddMode ) || ( cartGetMode ) ) {
    if ( cartId.isEmpty() )
      return true;
    if ( cartHmac.isEmpty() )
      return true;
    return false;
  }

  if ( !creator.isEmpty() )
    return false;

  if ( !title.isEmpty() )
    return false;

  if ( !ean.isEmpty() )
    return false;

  if ( !manufacturer.isEmpty() )
    return false;

  if ( keywords.size() > 0 )
    return false;

  return true;
}

bool ItemSearch::keysSet()
{
	if(QString::fromUtf8(token.toByteArray()) == "na" || QString::fromUtf8(secret.toByteArray()) == "na")
		return false;
	return true;
}

QString ItemSearch::prepareIndex ( bool sign )
{
  if ( searchIndex.isEmpty() )
    return QString();

  if ( sign ) {
    QString result = QString ( "SearchIndex=%1" ).arg ( searchIndex );
    result.replace ( "&", "" );
    result.replace ( ",", "%2C" );
    return result;
  }

  return QString ( "        <ns1:SearchIndex>%1</ns1:SearchIndex>\n" ).arg ( searchIndex );
}

QString ItemSearch::prepareKeyword ( bool sign )
{
  if ( keywords.size() < 1 )
    return QString();

  if ( sign ) {
    QString result = QString ( "Keywords=%1" ).arg ( searchIndex );
    result.replace ( "&", "" );
    result.replace ( ",", "%2C" );
    return result;
  }

  return QString ( "       <ns1:Keywords>%1</ns1:Keywords>\n" ).arg ( keywords.join ( " " ) );
}

QString ItemSearch::prepareTitle ( bool sign )
{
  if ( title.isEmpty() )
    return QString();

  if ( searchIndex.contains ( "Blended" ) ) {
    if ( sign )
      return QString();

		// avoid double entries
		if(!keywords.contains(title))
			setKeyword ( title );
    return QString();
  }

  if ( sign ) {
    QString result = QString ( "Title=%1" ).arg ( searchIndex );
    result.replace ( "&", "" );
    result.replace ( ",", "%2C" );
    return result;
  }

  return QString ( "        <ns1:Title>%1</ns1:Title>\n" ).arg ( title );

}

QString ItemSearch::prepareCreator ( bool sign )
{
  if ( creator.isEmpty() )
    return QString();

  if ( !searchIndex.isEmpty() )
    return QString();

  if ( searchIndex.contains ( "Blended" ) ) {
    if ( sign )
      return QString();

		if(!keywords.contains(creator))
    setKeyword ( creator );
    return QString();
  }

  if ( creator.isEmpty() )
    return QString();

  QString name;
  if ( searchIndex.contains ( "Books" ) ) {
    name = "Author";
  }
  if ( searchIndex.contains ( "Video" ) ) {
    name = "Actor";
  }
  if ( searchIndex.contains ( "Music" ) ) {
    name = "Artist";
  }
  if ( name.isEmpty() )
    return QString();

  if ( sign ) {
    QString result = QString ( "%1=%2" ).arg ( name ).arg ( searchIndex );
    result.replace ( "&", "" );
    result.replace ( ",", "%2C" );
    return result;
  }

  return QString ( "        <ns1:%1>%2</ns1:%1>\n" ).arg ( name ).arg ( creator );
}

QString ItemSearch::prepareManufacturer ( bool sign )
{
  if ( searchIndex.isEmpty() )
    return QString();

  if ( manufacturer.isEmpty() )
    return QString();

  if ( searchIndex.contains ( "Blended" ) ) {
    if ( sign )
      return QString();

		if(!keywords.contains(manufacturer))
			setKeyword ( manufacturer );
    return QString();
  }

  QString name;
  if ( searchIndex.contains ( "Books" ) ) {
    name = "Publisher";
  }
  if ( searchIndex.contains ( "Video" ) ) {
    name = "Publisher";
  }
  if ( searchIndex.contains ( "Music" ) ) {
    name = "MusicLabel";
  }
  if ( name.isEmpty() )
    return QString();

  if ( sign ) {
    QString result = QString ( "%1=%2" ).arg ( name ).arg ( searchIndex );
    result.replace ( "&", "" );
    result.replace ( ",", "%2C" );
    return result;
  }

  return QString ( "        <ns1:%1>%2</ns1:%1>\n" ).arg ( name ).arg ( manufacturer );
}

QString ItemSearch::prepareEan ( bool sign )
{
//   if(searchIndex != "Blended")
//     return QString();
  if ( ean.isEmpty() )
    return QString();

  if ( sign )
    return QString();

	if(!keywords.contains(ean))
		setKeyword ( ean );
  return QString();
}


QString ItemSearch::prepareOffers ( bool sign )
{
  if ( offerItems.size() < 1 )
    return QString();

  if ( sign )
    return QString();

  QString result;
  result  = "        <ns1:Items>\n";
  foreach ( QString offerListingId, offerItems.uniqueKeys() ) {
    result += "          <ns1:Item>\n";
    result += "            <ns1:OfferListingId>"+ offerListingId +"</ns1:OfferListingId>\n";
    result += "            <ns1:Quantity>"+ QString::number ( offerItems.value ( offerListingId ) ) +"</ns1:Quantity>\n";
    result += "          </ns1:Item>\n";
  }
  result += "        </ns1:Items>\n";

  offerItems.clear();
  return result;
}

QString ItemSearch::responseGroups ( bool sign )
{
  QString result = "Small,ItemAttributes,Images,SalesRank,Tracks,EditorialReview,Similarities";
  if ( offerMode )
		result = "OfferFull";

  if ( ( cartCreateMode ) || ( cartAddMode ) || ( cartGetMode ) )
// 		result = "Cart";
    return QString();

  if ( sign ) {
    QString result = QString ( "ResponseGroup=%1" ).arg ( searchIndex );
    result.replace ( "&", "" );
    result.replace ( ",", "%2C" );
    return result;
  }

  return QString ( "        <ns1:ResponseGroup>%1</ns1:ResponseGroup>\n" ).arg ( result );
}

QString ItemSearch::soapHeader(QString timeStamp)
{
  QString stamp = timeStamp;
	if(stamp.isEmpty())
		stamp = timestamp();
  QString sign = signature ( stamp );
  QString result;
  result  = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
  result += "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\"\n";
  result += "  xmlns:ns1=\"http://webservices.amazon.com/AWSECommerceService/2009-06-01\"\n";
  result += "  xmlns:ns2=\"http://security.amazonaws.com/doc/2007-01-01/\">\n";
  result += "  <SOAP-ENV:Header>\n";
  result += "    <ns2:AWSAccessKeyId>"+ QString::fromUtf8(token.toByteArray()) +"</ns2:AWSAccessKeyId>\n";
  result += "    <ns2:Timestamp>"+ stamp +"</ns2:Timestamp>\n";
  result += "    <ns2:Signature>"+ sign +"</ns2:Signature>\n";
  result += "  </SOAP-ENV:Header>\n";
  result += "  <SOAP-ENV:Body>\n";
  if ( cartCreateMode ) {
    result += "   <ns1:CartCreate>\n";
  } else if ( cartAddMode ) {
    result += "   <ns1:CartAdd>\n";
  } else if ( cartGetMode ) {
    result += "   <ns1:CartGet>\n";
  } else {
    result += "   <ns1:ItemSearch>\n";
  }
  result += "     <ns1:AssociateTag>"+ associateId(locale) +"</ns1:AssociateTag>\n";
  if ( ( cartAddMode ) || ( cartCreateMode ) || ( cartGetMode ) )
    result += "     <ns1:AWSAccessKeyId>"+ QString::fromUtf8(token.toByteArray()) +"</ns1:AWSAccessKeyId>\n";
  result += "     <ns1:Request>\n";

  return result;
}

QString ItemSearch::soapFooter()
{
  QString result;
  result =  "     </ns1:Request>\n";
  if ( cartCreateMode ) {
    result += "   </ns1:CartCreate>\n";
  } else if ( cartAddMode ) {
    result += "   </ns1:CartAdd>\n";
  } else if ( cartGetMode ) {
    result += "   </ns1:CartGet>\n";
  } else {
    result += "   </ns1:ItemSearch>\n";
  }
  result += "  </SOAP-ENV:Body>\n";
  result += "</SOAP-ENV:Envelope>";
  return result;
}

QString ItemSearch::timestamp()
{
  return QDateTime::currentDateTime().toString ( "yyyy-MM-ddTHH:mm:ssZ" );
}

QString ItemSearch::signature ( QString timestamp )
{
  QString signString;
  if ( cartCreateMode ) {
    signString = "CartCreate";
  } else if ( cartAddMode ) {
    signString = "CartAdd";
  } else if ( cartGetMode ) {
    signString = "CartGet";
  } else {
    signString = "ItemSearch";
  }

  signString += timestamp;

  QCA::Hash shaHash ( "sha256" );
  if ( !QCA::isSupported ( "hmac(sha256)" ) )
    return QString();

  QCA::MessageAuthenticationCode hmacObject ( "hmac(sha256)", QCA::SymmetricKey ( QString::fromUtf8(secret.toByteArray()).toAscii() ) );
  QByteArray signBytes;
  signBytes.append ( signString.toUtf8() );
  QCA::SecureArray content ( signBytes );
  hmacObject.update ( content );
  QCA::SecureArray resultArray = hmacObject.final();

  QCA::Base64 encoder;
  encoder.setup ( QCA::Encode );
  QString result = encoder.arrayToString ( resultArray );
  return result;
}

QString ItemSearch::associateId(QString locale)
{
	QMap<QString, QString> associateIds;
	associateIds.insert ( "CA", "meganizer05-20" );
  associateIds.insert ( "DE", "falaba-21" );
  associateIds.insert ( "FR", "meganizer0e-21" );
  associateIds.insert ( "JP", "" );
  associateIds.insert ( "UK", "meganizer-21" );
  associateIds.insert ( "US", "meganizer-20" );

  return associateIds.value(locale.toUpper());
}
