/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PURCHASE_H
#define PURCHASE_H

#include <ewidget.h>
#include "ui_purchase.h"

#include <searchplugin.h>

class Medium;
class MediaModel;
class DelegateShoppingCart;

#include <QNetworkReply>

class Purchase : public EWidget, private Ui::Purchase
{
	Q_OBJECT
	public:
		Purchase(QList< BuyData > items, QMap<Medium*, int> quantities, QWidget *parent = 0, Qt::WindowFlags flags = 0);
		~Purchase();
		/**
			* adds all items into the shopping cart
			*/
		bool prepareShoppingCart();
		/**
			* checks wether all items are really on the shopping cart
			* in the quantitiy the user wants
			* (the SaveForLater Cart is not meant here, all items must be on the active one!)
			*/
		bool checkShoppingCart();
		bool checkoutClicked();
	
	protected:
		void closeEvent(QCloseEvent *event);
	
	private slots:
		void finishCartAdd(QNetworkReply *reply);
		void finishCartCheck(QNetworkReply *reply);
		void buyButtonClicked();
		void cancelButtonClicked();
		void httpRequestError ( QNetworkReply::NetworkError netErr );
		
	private:
		QList<BuyData> items;
		QMap<Medium*, int> quantities;
		QString cartId, cartHmac, purchaseUrl;
		bool inProgress, abort, checkedOut, canceled;
		MediaModel *model;
		DelegateShoppingCart *delegate;
		QString errorStyleSheet;
	
	signals:
		void logMsg(QString msg);
};

#endif // PURCHASE_H
