/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "itemsearchtests.h"
#include "itemsearchtests.moc"

#include <itemsearch.h>

#include <QtTest/QtTest>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QDomDocument>

void ItemSearchTests::initTestCase()
{
	ItemSearch itemSearch;
	if(!itemSearch.checkQCA())
		QSKIP("QCA is not working here", SkipAll);
}

void ItemSearchTests::itemsearchRequest_data()
{
	QTest::addColumn<QString>("searchIndex");
	QTest::addColumn<QString>("timestamp");
	QTest::addColumn<QString>("keyword");
	QTest::addColumn<QString>("result");
	
	QString result;
	result  = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
  result += "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\"\n";
  result += "  xmlns:ns1=\"http://webservices.amazon.com/AWSECommerceService/2009-06-01\"\n";
  result += "  xmlns:ns2=\"http://security.amazonaws.com/doc/2007-01-01/\">\n";
  result += "  <SOAP-ENV:Header>\n";
  result += "    <ns2:AWSAccessKeyId>12345</ns2:AWSAccessKeyId>\n";
  result += "    <ns2:Timestamp>2010-08-15T15:18:02.000Z</ns2:Timestamp>\n";
  result += "    <ns2:Signature>VdLqICZePxTEnEmiCs0hvIMg3pUPNdSu8Qiih5W9WN8=</ns2:Signature>\n";
  result += "  </SOAP-ENV:Header>\n";
  result += "  <SOAP-ENV:Body>\n";
	result += "   <ns1:ItemSearch>\n";
	result += "     <ns1:AssociateTag>falaba-21</ns1:AssociateTag>\n";
	result += "     <ns1:Request>\n";
	result += "        <ns1:SearchIndex>Books</ns1:SearchIndex>\n";
	result += "       <ns1:Keywords>Harry Potter</ns1:Keywords>\n";
	result += "       <ns1:MerchantId>All</ns1:MerchantId>\n";
	result += "       <ns1:ItemPage>1</ns1:ItemPage>\n";
	result += "       <ns1:Condition>All</ns1:Condition>\n";
	result += "        <ns1:ResponseGroup>Small,ItemAttributes,Images,SalesRank,Tracks,EditorialReview,Similarities</ns1:ResponseGroup>\n";
	result += "     </ns1:Request>\n";
	result += "   </ns1:ItemSearch>\n";
	result += "  </SOAP-ENV:Body>\n";
	result += "</SOAP-ENV:Envelope>\n";
	
	QTest::newRow("1") << "Books" << "2010-08-15T15:18:02.000Z" << "Harry Potter" << result;
}

void ItemSearchTests::itemsearchRequest()
{
	QFETCH(QString, searchIndex);
	QFETCH(QString, timestamp);
	QFETCH(QString, keyword);
	QFETCH(QString, result);
	
	ItemSearch itemSearch("12345", "12345");

	itemSearch.setIndex(searchIndex);
	itemSearch.setKeyword(keyword);
	
	// strip out beginning spaces
	// strip out linebreaks
	QString request = itemSearch.getRequest(1, timestamp);
	request.replace(QRegExp("^\\s"), "");
	request.replace("\n", "");
	
	result.replace(QRegExp("^\\s"), "");
	result.replace("\n", "");
	QVERIFY(request == result);
}

void ItemSearchTests::itemsearch()
{
	ItemSearch itemSearch;
	if(!itemSearch.keysSet())
		QSKIP("No AWS Access key or secret set", SkipSingle);
	
	itemSearch.setIndex("Books");
	itemSearch.setTitle("Harry Potter");
	// preparing ItemSearch
  QByteArray content;
	content.append ( itemSearch.getRequest ( 1 ).toUtf8() );
	
	QNetworkAccessManager *http = new QNetworkAccessManager ( this );
	QNetworkRequest netRequest = itemSearch.netRequest();
  QNetworkReply *reply = http->post ( netRequest, content );
	
	// wait for thread to be started
	QTime dieTime = QTime::currentTime().addSecs(10);
#if QT_VERSION < 0x040600
	QSignalSpy finishSpy(reply, SIGNAL(finished()));
#endif
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
#if QT_VERSION < 0x040600
		if(finishSpy.count() > 1)
#else
		if(reply->isFinished())
#endif
			break;
	}
#if QT_VERSION < 0x040600
	QCOMPARE(finishSpy.count(), 1);
#else
	QCOMPARE(reply->isFinished(), true);
#endif
	QCOMPARE(reply->error(), QNetworkReply::NoError);
	
  QDomDocument result;
  QCOMPARE(result.setContent( reply->readAll(), true ), true );
	QDomNodeList node = result.elementsByTagName ( "IsValid" );
	QCOMPARE(node.size(), 1);
}

QTEST_MAIN(ItemSearchTests)
